#!/usr/bin/python
import numpy
import copy
from mpl_toolkits.basemap import Basemap
import matplotlib.pyplot as plt
import numpy.ma as ma
import os

#=============================================
# Set parameters

#sat = "n15"
#sat = "n18"
sat = "n19"
#sat = "aqu"
#sat = "met"

chan = "ch05"
chan = "ch06"
chan = "ch07"
chan = "ch09"

year = 2017
month = 1

#DisplayCount = True
DisplayCount = False

#=============================================

# Construct input file name (e.g. monthly_grids/amsu_n15_1998_08.dat)

month2 = "%2.2d"%month
infile = "monthly_grids/amsu_"+sat+"_"+str(year)+"_"+month2+".dat"
print "infile=",infile
#exit()

f = open(infile,"rb")

crossing_time = numpy.fromfile(f,dtype=numpy.float32,count=1)
print "crossing time = ",crossing_time

grid0 = numpy.fromfile(f,dtype=numpy.float32,count=72*144*15*4)
x0 = grid0.reshape((72,144,15,4),order="F")
x0 = ma.masked_equal(x0,-999.)

igrid0 = numpy.fromfile(f,dtype=numpy.int16,count=72*144*15*4)
ix0 = igrid0.reshape((72,144,15,4),order="F")
ix0 = ma.masked_equal(ix0,0)

f.close()

if chan=="ch05":
    lower = 210
    upper = 270
if chan=="ch06":
    lower = 195
    upper = 255
if chan=="ch07":
    lower = 185
    upper = 245
if chan=="ch09":
    lower = 180
    upper = 240

if DisplayCount:
    lower=0
    upper=50

# To view counts instead of Tb. Uncomment this block.
#x0 = ix0
#x0 = ma.masked_equal(x0,0.)
#lower = 0
#upper = 30

output_image_file = "print_grids.png"
num_plot_row = 4
num_plot_col = 4

#=================================================================
# CYLINEDR PROJECTION BLOCK
# Set up and display image

def mk_image(chan,iprint,num_plot_row, num_plot_col, plot_number):
    #x=copy.deepcopy(x0)

    if chan == "ch05":
        ichannel = 0
    elif chan == "ch06":
        ichannel = 1
    elif chan == "ch07":
        ichannel = 2
    elif chan == "ch09":
        ichannel = 3
    
    fig = plt.subplot(num_plot_row, num_plot_col, plot_number)

    map = Basemap(projection="cyl",resolution="l")

    if DisplayCount:
        map.imshow(ix0[:,:,iprint-1,ichannel], origin="lower",vmin=lower,vmax=upper)
    else:   
        map.imshow(x0[:,:,iprint-1,ichannel], origin="lower",vmin=lower,vmax=upper)

    map.drawcoastlines()
    map.drawmeridians(numpy.arange(-180,180,30))

    map.drawparallels(numpy.arange(-90,90,10))
    #plt.title(infile)
    title = str(year)+"/"+str(month2)+"         "+sat+", "+str(chan)+", print= "+str(iprint)
    plt.title(title)
    #plt.colorbar(orientation="horizontal",shrink=0.7)

#=================================================================

fg = plt.figure(figsize=(30,15))
adj = plt.subplots_adjust(hspace=0.,wspace=0.07)

for iprint in range(1,15+1):
    mk_image(chan,iprint,num_plot_row, num_plot_col, iprint)

#plt.colorbar(orientation="vertical",shrink=0.7)

# Add colorbar and display
plt.subplots_adjust(left=.01,top=.99,bottom=.009,right=0.86,wspace=None,hspace=None)
cax = plt.axes([.90,.2,.03,.6])
plt.colorbar(cax=cax)

plt.show()
