!====================================================================================================
! DIRECTORY: ver6/merge_v6/
! FILE NAME: nan.mod.f90
!
! PURPOSE: Fortran 95 module
!
! DESCRIPTION:  This set of routines is for checking for real(4) or real(8) variables containing NaN
!               and removing them. For pgf90, compile with -Kieee, won't work otherwise.
!
! CODE AUTHOR: Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: March, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
! INPUTS:
!
! OUTPUTS:
!
! MODIFICATION HISTORY:
!
!====================================================================================================


! W.D. BRASWELL (2/25/2016)

MODULE NaN
  implicit none
  private
  public isnan, set_nan_to_badvalue

  interface isnan
     module procedure isnan4, isnan8
  end interface

  interface set_nan_to_badvalue
     module procedure set_nan_to_badvalue44_1d,set_nan_to_badvalue44_2d,set_nan_to_badvalue44_3d, &
                      set_nan_to_badvalue48_1d,set_nan_to_badvalue48_2d,set_nan_to_badvalue48_3d, & 
                      set_nan_to_badvalue84_1d,set_nan_to_badvalue84_2d,set_nan_to_badvalue84_3d, &
                      set_nan_to_badvalue88_1d,set_nan_to_badvalue88_2d,set_nan_to_badvalue88_3d
  end interface
   
contains
logical function isnan4(a) ! Note NaN is 0/0
  real(4) :: a
  if(a .ne. a) then
     isnan4 = .TRUE.
  else
     isnan4 = .FALSE.
  end if
end function isnan4

logical function isnan8(a) ! Note NaN is 0/0
  real(8) :: a
  if(a .ne. a) then
     isnan8 = .TRUE.
  else
     isnan8 = .FALSE.
  end if
end function isnan8

!==========================================
! 44 SET: real(4), real(4)
subroutine set_nan_to_badvalue44_1D( x,badvalue )
  real(4) :: x(:)
  real(4) :: badvalue

  integer :: n1,i1

  n1 = size(x,1)

  do i1=1,n1
     if( isnan(x(i1)) ) x(i1) = badvalue
  end do   

end subroutine set_nan_to_badvalue44_1D

subroutine set_nan_to_badvalue44_2D( x,badvalue )
  real(4) :: x(:,:)
  real(4) :: badvalue

  integer :: n1,n2,i1,i2

  n1 = size(x,1)
  n2 = size(x,2)

  do i1=1,n1
     do i2=1,n2
        if( isnan(x(i1,i2)) ) x(i1,i2) = badvalue
     end do
  end do   

end subroutine set_nan_to_badvalue44_2D

subroutine set_nan_to_badvalue44_3D( x,badvalue )
  real(4) :: x(:,:,:)
  real(4) :: badvalue

  integer :: n1,n2,n3,i1,i2,i3

  n1 = size(x,1)
  n2 = size(x,2)
  n3 = size(x,3)

  do i1=1,n1
     do i2=1,n2
        do i3=1,n3
           if( isnan(x(i1,i2,i3)) ) x(i1,i2,i3) = badvalue
        end do   
     end do
  end do   

end subroutine set_nan_to_badvalue44_3D

!==========================================
! 48 SET: real(4), real(8)
subroutine set_nan_to_badvalue48_1D( x,badvalue )
  real(4) :: x(:)
  real(8) :: badvalue

  integer :: n1,i1

  n1 = size(x,1)

  do i1=1,n1
     if( isnan(x(i1)) ) x(i1) = badvalue
  end do   

end subroutine set_nan_to_badvalue48_1D

subroutine set_nan_to_badvalue48_2D( x,badvalue )
  real(4) :: x(:,:)
  real(8) :: badvalue

  integer :: n1,n2,i1,i2

  n1 = size(x,1)
  n2 = size(x,2)

  do i1=1,n1
     do i2=1,n2
        if( isnan(x(i1,i2)) ) x(i1,i2) = badvalue
     end do
  end do   

end subroutine set_nan_to_badvalue48_2D

subroutine set_nan_to_badvalue48_3D( x,badvalue )
  real(4) :: x(:,:,:)
  real(8) :: badvalue

  integer :: n1,n2,n3,i1,i2,i3

  n1 = size(x,1)
  n2 = size(x,2)
  n3 = size(x,3)

  do i1=1,n1
     do i2=1,n2
        do i3=1,n3
           if( isnan(x(i1,i2,i3)) ) x(i1,i2,i3) = badvalue
        end do   
     end do
  end do   

end subroutine set_nan_to_badvalue48_3D

!==========================================
! 84 SET: real(8), real(4)
subroutine set_nan_to_badvalue84_1D( x,badvalue )
  real(8) :: x(:)
  real(4) :: badvalue

  integer :: n1,i1

  n1 = size(x,1)

  do i1=1,n1
     if( isnan(x(i1)) ) x(i1) = badvalue
  end do   

end subroutine set_nan_to_badvalue84_1D

subroutine set_nan_to_badvalue84_2D( x,badvalue )
  real(8) :: x(:,:)
  real(4) :: badvalue

  integer :: n1,n2,i1,i2

  n1 = size(x,1)
  n2 = size(x,2)

  do i1=1,n1
     do i2=1,n2
        if( isnan(x(i1,i2)) ) x(i1,i2) = badvalue
     end do
  end do   

end subroutine set_nan_to_badvalue84_2D

subroutine set_nan_to_badvalue84_3D( x,badvalue )
  real(8) :: x(:,:,:)
  real(4) :: badvalue

  integer :: n1,n2,n3,i1,i2,i3

  n1 = size(x,1)
  n2 = size(x,2)
  n3 = size(x,3)

  do i1=1,n1
     do i2=1,n2
        do i3=1,n3
           if( isnan(x(i1,i2,i3)) ) x(i1,i2,i3) = badvalue
        end do   
     end do
  end do   

end subroutine set_nan_to_badvalue84_3D

!==========================================
! 88 SET: real(8), real(8)
subroutine set_nan_to_badvalue88_1D( x,badvalue )
  real(8) :: x(:)
  real(8) :: badvalue

  integer :: n1,i1

  n1 = size(x,1)

  do i1=1,n1
     if( isnan(x(i1)) ) x(i1) = badvalue
  end do   

end subroutine set_nan_to_badvalue88_1D

subroutine set_nan_to_badvalue88_2D( x,badvalue )
  real(8) :: x(:,:)
  real(8) :: badvalue

  integer :: n1,n2,i1,i2

  n1 = size(x,1)
  n2 = size(x,2)

  do i1=1,n1
     do i2=1,n2
        if( isnan(x(i1,i2)) ) x(i1,i2) = badvalue
     end do
  end do   

end subroutine set_nan_to_badvalue88_2D

subroutine set_nan_to_badvalue88_3D( x,badvalue )
  real(8) :: x(:,:,:)
  real(8) :: badvalue

  integer :: n1,n2,n3,i1,i2,i3

  n1 = size(x,1)
  n2 = size(x,2)
  n3 = size(x,3)

  do i1=1,n1
     do i2=1,n2
        do i3=1,n3
           if( isnan(x(i1,i2,i3)) ) x(i1,i2,i3) = badvalue
        end do   
     end do
  end do   

end subroutine set_nan_to_badvalue88_3D

END MODULE NaN

!$$$!=============================
!$$$! MAIN PROGRAM FOR TESTING
!$$$!=============================
!$$$program main
!$$$  use NAN
!$$$  implicit none
!$$$ 
!$$$  real(8) :: x(2,3,2),a,b
!$$$  real(4) :: badvalue
!$$$  badvalue = -999
!$$$ 
!$$$  x = 88
!$$$  a=0
!$$$  b=0
!$$$ 
!$$$  x(1,1,1) = a/b
!$$$  write(6,*) "x=",x
!$$$  call set_nan_to_badvalue(x,badvalue)
!$$$  write(6,*) "x=",x
!$$$ 
!$$$end program main
