!====================================================================================================
! NAME: Support Routines
!       Version 6.0 of the University of Alabama in Huntsville (UAH)
!       Microwave Sounding Unit(MSU/AMSU) Processing Software
!
! DESCRIPTION: These subprograms are used in the reading, unpacking, and calibration
!              of MSU orbit file data.
!
! AUTHOR(S): Roy Spencer, Danny Braswell, Earth System Science Center, University of Alabama in Huntsville
!
! CREATION DATE: April, 2015
!
! COPYRIGHT: This software and its documentation are considered to be in the public domain
! and thus are available for unrestricted public use. They are furnished "as is". The authors,
! the United States Government, and its instrumentalities, officiers, employees, and agents
! make no warrant, express or implied, as to the usefulness of the software and documentation
! for any purpose. They assume no responsibility (1) for the use of the software and
! documentation; or (2) to provide technical support to users.
!
!
! MODIFICATION HISTORY:
!
!====================================================================================================

      subroutine UNPACK_MSU_MONTHLY_FILE(monthly_file,max_num_scans,
     &   reference_time,num_scans,index_start,index_stop,
     &   sat_id,year,day,secs,time78,counts,lat,lon,
     &   cal_counts,iqual1,ksat,kday1,kday2,iprint )
      implicit none
      
    !-----------------------------  
    ! Argument list declarations
    !-----------------------------  
    
    ! Input
      character*150 monthly_file ! Name of orbit file
      integer*4 index_offset   ! Amount indices are offset when loading values into output arrays
      integer*4 max_num_scans  ! last dimension of output arrays
      integer ksat,kday1,kday2,iprint 
      
    ! Output  
      integer*4 reference_time
      integer*4 num_scans      ! Number of scans in orbit file

      integer*4 index_start    ! Indices bouding the range of indices used in loading the output arrays
      integer*4 index_stop     

      integer*4 sat_id         ! Satellite id
      integer*4 year(max_num_scans)
      integer*4 day(max_num_scans)
      integer*4 secs(max_num_scans) ! Scan time, seconds since Greenwich 0Z on current Julian day
      integer*4 time78(max_num_scans) ! Scan time, seconds since Greenwich 0Z on Jan 1, 1978
      
      integer*2 counts(4,11,max_num_scans)   ! counts for channels 1..4, 11 footprints
      integer*2 lat(11,max_num_scans)        ! lat & lon, units are tenths of a degree
      integer*2 lon(11,max_num_scans)
      integer*2 cal_counts(16,max_num_scans) ! The 16 calibration count values
      integer*2 iqual1(max_num_scans)        ! Quality check 
      
      
      INTEGER*4 jsecs,j
      INTEGER*2 idsc,jyear,jday,kounts(4,11)
                integer*4 jyear4, jday4
      integer*2 cal1lo, cal2lo, cal1hi, cal2hi   ! Set of 16 calibration related counts
      integer*2 t1a, t1b, t2a, t2b   
      integer*2 cold1, cold2, cold3, cold4
      integer*2 warm1, warm2, warm3, warm4
      INTEGER*2 inlat(11),inlon(11),iqual
      INTEGER*4 nreads,jyd,scanskip,nsave,secs0sv
       
      
      integer*2 jspot 
      integer*4 i,n,scan_number

      integer*4 year0,day0,secs0,time0
      integer*4 ref_year
      parameter(ref_year=78)
      integer*4 secs_since_year ! local ftn
      
      integer*4 ngood(12,366) !number of good scans
      integer*4 is,jd
      
c.........open monthly MSU data file.....................................
      OPEN(33,FILE=monthly_file,form="unformatted",access="stream",convert="big_endian")
      rewind(33)                                        

      write(6,119)monthly_file
  119 format(a150)
      index_start=max_num_scans+1
      index_stop=0

      nreads=0
 
c.........READ NEXT SCAN LINE OF DATA.......................................
    1     READ(33,END=999,err=999)IDSC,JYEAR,JDAY,JSECS,
     &    (KOUNTS(1,J),J=1,11),   
     &    (KOUNTS(2,J),J=1,11),(KOUNTS(3,J),J=1,11),                    
     &    (KOUNTS(4,J),J=1,11),CAL1HI,CAL1LO,CAL2HI,CAL2LO,             
     &    T1A,T1B,T2A,T2B,COLD1,WARM1,COLD2,WARM2,              
     &     COLD3,WARM3,COLD4,WARM4,                            
     &    (inLAT(J),J=1,11), (inLON(J),J=1,11),IQUAL


           if(iprint.eq.1)then
                    write(6,3324)idsc,jyear,jday,jsecs,(Kounts(2,J),j=1,11)
 3324     format(i1,i3,i4,i7,11i5)
      end if



  
      if(idsc.eq.1.and.jyear.gt.87)idsc=9
      if(idsc.ne.ksat)go to 1
      
      if(jday.lt.kday1)go to 1
cc      if(jday.gt.kday2)go to 999
      
      jyd=1000*jyear+jday
                jyear4 = jyear
                jday4 = jday
      call BAD_SCANS(IDSC,JYD,JSECS,scanskip,jyear4,jday4)

      if(scanskip.eq.1)then
         if(iprint.eq.1)write(6,*)'scanskip=1'
         go to 1
      end if

      sat_id = idsc                      ! Danny's Satellite ID
      



      if( sat_id.ne.1 .and. sat_id.ne.2 .and. sat_id.ne.3 .and.
     &    sat_id.ne.4 .and.
     &    sat_id.ne.5 .and. sat_id.ne.6 .and. sat_id.ne.7 .and.
     &    sat_id.ne.8 .and. sat_id.ne.9 )then
          if(iprint.eq.1)write(6,*)'bad sat_id'
          go to 1
      end if
      



      year0 = jyear
      day0  = jday                                             
      secs0 = jsecs        


      if(day0.eq.0) then
          if(iprint.eq.1)write(6,*)'day=0'
            go to 1 ! Added on 4/18/95 after encountering a day0=0
      end if

      nreads=nreads+1                      

! Compute index n (Note that ref_year and reference_time are different)
      time0=secs_since_year(ref_year,year0,day0,secs0)

c....on first scan of monthly file, set the reference time for array index assignment
      if(nreads.eq.1)then
          reference_time=secs_since_year(ref_year,year0,day0,0)
      end if
      n = nint((time0-reference_time)/25.6d0) + 1
      
      if(nreads.eq.1)then
           write(6,*)ref_year,year0,day0,secs0,time0,reference_time,n
      end if
      
      
      
c..RWS..fix for n not changing sometimes.....................
      if(n.eq.nsave.and.secs0sv.ne.secs0)then
         n=n+1
      end if
      nsave=n
      secs0sv=secs0
c....end of fix ............................................

c      write(6,*)year0,day0,secs0,n

c      write(6,2939)nreads,n,year0,day0,secs0,time0
c 2939 format('nreads=',i5,' n=',i5,i3,i3,' secs0=',i7,' time0=',i14)

      if(n .gt. max_num_scans .or. n .lt. 1) then
         write(6,*)'*** SCAN SKIPPED: INDEX OUT OF BOUNDS ***'
         write(6,*)'sat_id,year,day,secs,index: ',sat_id,year0,day0,secs0,n
         go to 1
      endif

      if( n .gt. index_stop ) index_stop=n
      if( n .lt. index_start) index_start=n

      year(n)   = year0
      day(n)    = day0
      secs(n)   = secs0
      time78(n) = time0
            
      iqual1(n) =iqual

c      if(jday.eq.66.or.jday.eq.67)then
c         write(6,129)idsc,jyear,jday,jsecs,n,iqual
c  129    format(i1,i3,i3,i9,i9,i6)
c      end if


      ngood(sat_id,jday)=ngood(sat_id,jday)+1


!...... Load into array...................
      cal_counts( 1,n) = cal1lo  
      cal_counts( 2,n) = cal2lo  
      cal_counts( 3,n) = cal1hi  
      cal_counts( 4,n) = cal2hi  
      cal_counts( 5,n) = t1a     
      cal_counts( 6,n) = t1b     
      cal_counts( 7,n) = t2a     
      cal_counts( 8,n) = t2b     
      cal_counts( 9,n) = cold1   
      cal_counts(10,n) = cold2   
      cal_counts(11,n) = cold3   
      cal_counts(12,n) = cold4   
      cal_counts(13,n) = warm1   
      cal_counts(14,n) = warm2   
      cal_counts(15,n) = warm3   
      cal_counts(16,n) = warm4   
      
    
      do jspot = 1, 11 
         lat(jspot,n) = inlat(jspot)          
         lon(jspot,n) = inlon(jspot)
      enddo 
    
      do jspot = 1, 11
         counts(1,jspot,n)  = kounts(1,jspot)
         counts(2,jspot,n)  = kounts(2,jspot)
         counts(3,jspot,n)  = kounts(3,jspot)
         counts(4,jspot,n)  = kounts(4,jspot)
      enddo
c      if(iprint.eq.1)then
c         write(6,129)idsc,jyear,jday,jsecs,n,iqual,
c     &       (counts(2,jspot,n),jspot=4,8),lat(6,n),lon(6,n)
c  129    format(i1,i3,i3,i9,i9,i6,5i5,2x,i5,i6)
c      end if

      go to 1


999   continue    
      close(33)

c      do jd=1,366
c         write(6,2639)jd,(ngood(is,jd),is=1,12)
c 2639    format(i3,12i5)
c      end do


      return
      END      

c*****************************************************************************
c******DANNY'S MSU CALIBRATION ROUTINE****************************************
!=======================================================================
! SUBROUTINE: calibrate()
!
! COMMENTS:
!  - Scans are assumed to be stored in ascending time order
!  - Based on calibration related check, additional scans
!    may be flagged as bad by setting their badvalue flag to .TRUE.
!  
!
!  Reference: 
!     - Spencer, Christy, Grody (1990), J. of Climate, 3,10,1111-1128
!       (hereafter called SCG90 )
!
!(1/18/2000)-  ADDED TSAN MO'S NEW CALIBRATION EQUATION AS AN OPTION..
!=======================================================================

      subroutine calibrate_msu(index_low,index_high,id,year,day,secs,
     & time78,counts,lat,lon,cal_counts,      badvalue,tb,badflag,
     & iprint,caltyp,linadj,twcoef,temp1av_array,temp2av_array,Qterm_array,temp1diff_avg,temp2diff_avg,emiss,calterm,
     & avg_gain, tw_gain)

      implicit none
      
    !------------------------------
    ! Argument list declarations
    !------------------------------

      integer*4 index_low        ! Input
      integer*4 index_high       ! Input
      integer*4 id               ! Input
      integer*4 year(110000)          ! Input
      integer*4 day(110000)           ! Input
      integer*4 secs(110000)          ! Input
      integer*4 time78(110000)        ! Input
      
      integer*2 counts(4,11,110000)   ! Input
      integer*2 lat(11,110000)        ! Input
      integer*2 lon(11,110000)        ! Input
      integer*2 cal_counts(16,110000) ! Input
c      integer*2 iqual1(110000)        ! Input
      real      badvalue         ! Input

      real      tb(4,11,110000)           ! Output: brightness temperatures (K)
                real calterm(4,11,110000)
      logical badflag(110000)        ! Output: bad scan flag

                real temp1av_array(110000), temp2av_array(110000)    !...output: warm target temps
                real Qterm_array(11,110000)   !....output Tsan Mo's Q term for later regression to rederive his "u" coefficints (ch 2 only for now)

      integer jj,jyd,iprint,caltyp
c....for caltyp=3, need for calculating radiances.....
      real*8 freq(4),wavenmbr,c1,c2,Radc,Radw,Q,Gain,clight
      real*8 RadEarth
      data freq/50.3,53.74,54.96,57.95/
      parameter(c1=1.1910659d-5)
      parameter(c2=1.438833)
      parameter(clight=2.99792458d10)
                real u2(4,12)
      data u2/0.0, 1.00, 0.0, 0.0,
     &       0.0, 1.00, 0.0, 0.0,
     &       0.0, 2.38, 0.0, 0.0,
     &       0.0, 1.16, 0.0, 0.0,     
     &       0.0, 1.78, 0.0, 0.0,
     &       0.0, 1.75, 0.0, 0.0,
     &       0.0, 1.00, 0.0, 0.0,
     &       0.0, 0.89, 0.0, 0.0,
     &       0.0, 1.17, 0.0, 0.0,
     &       4*0.,
     &       4*0.,
     &       4*0./
                real u(4,12)
      data u/7.300734,5.300233,5.757293,4.594762,
     &       7.300734,5.300233,5.757293,4.594762,
     &       5.300803,3.150422,2.844725,2.485380,
     &       7.029300,5.923696,4.505891,4.540941,
     &       4.894181,3.114963,4.092390,4.073233,
     &       5.249078,5.100740,6.623566,6.156016,
     &       5.966425,5.197594,8.772592,7.607219,
     &       6.156454,4.953639,6.845809,5.591154,
     &       8.293536,7.702598,10.795112,9.085936,
     &       4*0.,
     &       4*0.,
     &       4*0./
c...new for regressions 3/1/05.....
      real tw(2,110000)
                real emiss(9,4)


    !-------------------------
    ! Internal declarations
    ! Assume 12 satellites.
    !-------------------------
      integer*2 noaa_badvalue
      parameter(noaa_badvalue=0) ! badvalue indicator used in orbit files

      integer max_num_scans
      parameter(max_num_scans=110000)
      real temp1(max_num_scans),temp2(max_num_scans)
    
      real c(12,4,3)   ! 12 satellites, 4 channels, 3 coeffs

      real e(12,4,3)   ! 12 satellites, 4 PRTs (1a,1b,2a,2b), 3 coeffs
      
      real kk(12,2)    ! 12 satellites, 2 coeffs
      
      integer n,i,j,k


    ! Calibration count variables mapped to values in array "cal_counts"
    
      integer*2 cal1hi   ! Precision reference resistor digital counts for circuit a
      integer*2 cal1lo 
      integer*2 cal2hi   ! Precision reference resistor digital counts for circuit b
      integer*2 cal2lo 

      integer*2 t1a      ! PRT digital counts for target 1 through circuit a
      integer*2 t1b      ! PRT digital counts for target 1 through circuit b
      integer*2 t2a      ! PRT digital counts for target 2 through circuit a
      integer*2 t2b      ! PRT digital counts for target 2 through circuit b

      integer*2 cold(4)  ! Channel i counts viewing deep space (i=1...4)
      integer*2 warm(4)  ! Channel i counts viewing warm calibration target
    ! .....
      
      
      integer*2 coldav(4),cold_last(4),cold_next(4)
      integer*2 warmav(4),warm_last(4),warm_next(4)
      
      real rt1a,rt1b,rt2a,rt2b
      real rcal1lo,rcal1hi,rcal2lo,rcal2hi
      real res1a,res1b,res2a,res2b
      real temp1a,temp1b,temp2a,temp2b
      
      real rk,rc,rw
      
      logical avg  ! Used to flag for 3 scan averaging

      integer*4 dt1,dt2
      
c...following added by RWS.....
      real temp1_last,temp2_last,temp1_next,temp2_next,temp1av,temp2av
                real temp1diff, temp2diff
                real temp1diff_sum, temp2diff_sum
                real temp1diff_tot, temp2diff_tot
                real temp1diff_avg, temp2diff_avg
       integer ii
c....declarations for empirical Tb adjustments............      
      integer linadj,iovrlap
      real twcoef(9,4)
                real avg_gain(9)
                real tw_gain(9)
                real gain_new
                real backlobe
      

    !------------------------      
    ! Data initialization
    !------------------------      
    
    ! Count linearization coefficients for all satellites       
    
    ! ... satellite 1
      data (c(1,1,j),j=1,3) / .355400E+02, .930300E+00, .196660E-04/  ! channel 1
      data (c(1,2,j),j=1,3) / .268500E+02, .939964E+00, .172642E-04/  ! channel 2
      data (c(1,3,j),j=1,3) / .204700E+02, .963900E+00, .994100E-05/  ! channel 3
      data (c(1,4,j),j=1,3) / .222317E+02, .964907E+00, .965909E-05/  ! channel 4
      
    ! ... satellite 2
      data (c(2,1,j),j=1,3) / .562126E+02, .903953E+00, .213218E-04/
      data (c(2,2,j),j=1,3) / .220410E+02, .927569E+00, .167910E-04/
      data (c(2,3,j),j=1,3) / .159050E+02, .948708E+00, .167910E-04/
      data (c(2,4,j),j=1,3) / .365100E+02, .935564E+00, .169230E-04/

    ! ... satellite 3
      data (c(3,1,j),j=1,3) / 90.675280, .927477, 1.377153E-05 /
c....rws...fixed transposed digits in following line....
c      data (c(3,2,j),j=1,3) / 75.393355, .939953, 1.161055E-05 /
       data (c(3,2,j),j=1,3) / 75.393355, .939953, 1.610550E-05 /
      data (c(3,3,j),j=1,3) / 44.230915, .958181, 8.905423E-06 /
c....rws...fixed third coefficient...
c      data (c(3,4,j),j=1,3) / 41.398696, .957974, 9.013738E-06 /
      data (c(3,4,j),j=1,3) / 41.398696, .957974, 9.01374E-06 /

    ! ... satellite 4
      data (c(4,1,j),j=1,3) / .243630E+02, .984400E+00, .147800E-04/
      data (c(4,2,j),j=1,3) / .260610E+02, .943800E+00, .162600E-04/
      data (c(4,3,j),j=1,3) / .159360E+02, .963000E+00, .107400E-04/
      data (c(4,4,j),j=1,3) / .217360E+02, .959800E+00, .112400E-04/

    ! ... satellite 5
      data (c(5,1,j),j=1,3) / .135350E+01, .983370E+00, .583530E-05/
c      data (c(5,2,j),j=1,3) / .153145E+02, .975793E+00, .644100E-05/
c...these remove the NOAA-12 Tb vs warm target PRT correlation during
c...the overlap with NOAA-11
c...NOTE: the following line has Tsan Mo's "gain-shift corrected coefficients"
      data (c(5,2,j),j=1,3) / .200016E+03, .855140E+00, 2.5935E-05/
      data (c(5,3,j),j=1,3) / .949320E+01, .983649E+00, .442475E-05/
      data (c(5,4,j),j=1,3) / .269150E+02, .973525E+00, .612575E-05/

    ! ... satellite 6
      data (c(6,1,j),j=1,3) / .224400E+02, .948900E+00, .163025E-04/
      data (c(6,2,j),j=1,3) / .187100E+02, .958900E+00, .128000E-04/
      data (c(6,3,j),j=1,3) / .258700E+02, .944200E+00, .163651E-04/
      data (c(6,4,j),j=1,3) / .151400E+02, .953300E+00, .146528E-04/

    ! ... satellite 7
      data (c(7,1,j),j=1,3) / .281500E+02, .959400E+00, .111514E-04/
      data (c(7,2,j),j=1,3) / .205900E+02, .966200E+00, .966190E-05/
      data (c(7,3,j),j=1,3) / .103250E+03, .906100E+00, .193189E-04/
      data (c(7,4,j),j=1,3) / .102910E+03, .908100E+00, .186008E-04/

    ! ... satellite 8
      data (c(8,1,j),j=1,3) / .360510E+02, .955271E+00, .117040E-04/
      data (c(8,2,j),j=1,3) / .384690E+02, .954615E+00, .110622E-04/
      data (c(8,3,j),j=1,3) / .106724E+03, .907674E+00, .188402E-04/
      data (c(8,4,j),j=1,3) / .105227E+03, .908741E+00, .188795E-04/

    ! ... satellite 9
      data (c(9,1,j),j=1,3) / .715311E+02, .931977E+00, .150372E-04/
      data (c(9,2,j),j=1,3) / .585272E+02, .942423E+00, .134850E-04/
c...test..............
c      data (c(9,2,j),j=1,3) / .756000E+02, .912423E+00, .215850E-04/
      data (c(9,3,j),j=1,3) / .129766E+03, .896681E+00, .206354E-04/
      data (c(9,4,j),j=1,3) / .138310E+03, .890301E+00, .212337E-04/

    ! ... satellite 10
      data (c(10,1,j),j=1,3) / 0,0,0 /
      data (c(10,2,j),j=1,3) / 0,0,0 /
      data (c(10,3,j),j=1,3) / 0,0,0 /
      data (c(10,4,j),j=1,3) / 0,0,0 /

    ! ... satellite 11
      data (c(11,1,j),j=1,3) / 0,0,0 /
      data (c(11,2,j),j=1,3) / 0,0,0 /
      data (c(11,3,j),j=1,3) / 0,0,0 /
      data (c(11,4,j),j=1,3) / 0,0,0 /

    ! ... satellite 12
      data (c(12,1,j),j=1,3) / 0,0,0 /
      data (c(12,2,j),j=1,3) / 0,0,0 /
      data (c(12,3,j),j=1,3) / 0,0,0 /
      data (c(12,4,j),j=1,3) / 0,0,0 /



    ! Platinum resistance thermometer count-to-resistance conversion      
    ! factors (nesdis quadratic fits) (PRT's: 1a,1b,2a,2b)                       

    ! ... satellite 1
      data (e(1,1,k),k=1,3) / 29.100000, 4.259600E-01, 3.187000E-05 /  ! PRT 1a
c      data (e(1,2,k),k=1,3) / 29.050000, 4.271000E-01, 3.206000E-05 /  ! PRT 1b
      data (e(1,2,k),k=1,3) / 29.490000, 4.271000E-01, 3.206000E-05 /  ! PRT 1b
c      data (e(1,3,k),k=1,3) / 27.550000, 4.278700E-01, 3.185000E-05 /  ! PRT 2a
      data (e(1,3,k),k=1,3) / 28.612000, 4.278700E-01, 3.185000E-05 /  ! PRT 2a
      data (e(1,4,k),k=1,3) / 28.950000, 4.283600E-01, 3.175000E-05 /  ! PRT 2b
      
      
    ! ... satellite 2
c rws  data (e(2,1,k),k=1,3) / 29.317000, 4.252700E-01, 3.165629E-05 /
      data (e(2,1,k),k=1,3) / 28.947000, 4.252700E-01, 3.165629E-05 /
c rws  data (e(2,2,k),k=1,3) / 29.282000, 4.261320E-01, 3.152410E-05 /
      data (e(2,2,k),k=1,3) / 28.912000, 4.261320E-01, 3.152410E-05 /
c rws  data (e(2,3,k),k=1,3) / 29.298000, 4.260800E-01, 3.149910E-05 /
      data (e(2,3,k),k=1,3) / 28.893000, 4.260800E-01, 3.149910E-05 /
c rws  data (e(2,4,k),k=1,3) / 29.363000, 4.255765E-01, 3.176020E-05 /
      data (e(2,4,k),k=1,3) / 28.974000, 4.255765E-01, 3.176020E-05 /


    ! ... satellite 3
c.....rws....fixed bad entry in following line...................
cccccc      data (e(3,1,k),k=1,3) / 29.00320, .429363, 3.144484E-05 /
      data (e(3,1,k),k=1,3) / 29.00300, .429363, 3.144484E-05 /
c.....rws....fixed bad entry in following line...................
cccccc      data (e(3,2,k),k=1,3) / 28.45560, .428430, 3.021258E-05 /
      data (e(3,2,k),k=1,3) / 28.45600, .428430, 3.021258E-05 /
c.....rws....fixed 2 bad entries in following line...................
cccccc      data (e(3,3,k),k=1,3) / 28.51489, .428037, 3.028803E-05 /
      data (e(3,3,k),k=1,3) / 28.51500, .428040, 3.028803E-05 /
      data (e(3,4,k),k=1,3) / 29.11400, .425768, 3.159801E-05 /
      
    ! ... satellite 4
      data (e(4,1,k),k=1,3) / 28.976000, 4.269400E-01, 3.173700E-05 /
      data (e(4,2,k),k=1,3) / 28.945000, 4.262800E-01, 3.160300E-05 /
      data (e(4,3,k),k=1,3) / 28.995000, 4.262700E-01, 3.177500E-05 /
      data (e(4,4,k),k=1,3) / 28.964000, 4.249800E-01, 3.138900E-05 /

    ! ... satellite 5
      data (e(5,1,k),k=1,3) / 29.397000, 4.256600E-01, 3.238700E-05 /
      data (e(5,2,k),k=1,3) / 28.185000, 4.302200E-01, 2.920400E-05 /
      data (e(5,3,k),k=1,3) / 28.453000, 4.293800E-01, 2.974600E-05 /
      data (e(5,4,k),k=1,3) / 28.959000, 4.276380E-01, 3.033100E-05 /

    ! ... satellite 6
      data (e(6,1,k),k=1,3) / 29.485700, 4.247172E-01, 3.209600E-05 /
      data (e(6,2,k),k=1,3) / 29.347100, 4.247339E-01, 3.165123E-05 /
      data (e(6,3,k),k=1,3) / 29.312300, 4.326844E-01, 3.282283E-05 /
      data (e(6,4,k),k=1,3) / 27.971900, 4.285466E-01, 3.095837E-05 /

    ! ... satellite 7
      data (e(7,1,k),k=1,3) / 29.429400, 4.283514E-01, 3.223351E-05 /
      data (e(7,2,k),k=1,3) / 29.541600, 4.248521E-01, 3.188156E-05 /
      data (e(7,3,k),k=1,3) / 29.263900, 4.338180E-01, 3.334592E-05 /
      data (e(7,4,k),k=1,3) / 29.577800, 4.243732E-01, 3.264424E-05 /

    ! ... satellite 8
      data (e(8,1,k),k=1,3) / 29.258900, 4.245799E-01, 3.169871E-05 /
      data (e(8,2,k),k=1,3) / 29.235000, 4.289328E-01, 3.232162E-05 /
      data (e(8,3,k),k=1,3) / 29.463300, 4.250778E-01, 3.192864E-05 /
      data (e(8,4,k),k=1,3) / 29.468000, 4.275379E-01, 3.221571E-05 /

    ! ... satellite 9
      data (e(9,1,k),k=1,3) / 29.101400, 4.242637E-01, 3.162073E-05 /
      data (e(9,2,k),k=1,3) / 28.985400, 4.269018E-01, 3.170732E-05 /
      data (e(9,3,k),k=1,3) / 28.948100, 4.277704E-01, 3.185110E-05 /
      data (e(9,4,k),k=1,3) / 29.042400, 4.274570E-01, 3.191209E-05 /

    ! ... satellite 10
      data (e(10,1,k),k=1,3) / 0,0,0 /
      data (e(10,2,k),k=1,3) / 0,0,0 /
      data (e(10,3,k),k=1,3) / 0,0,0 /
      data (e(10,4,k),k=1,3) / 0,0,0 /

    ! ... satellite 11
      data (e(11,1,k),k=1,3) / 0,0,0 /
      data (e(11,2,k),k=1,3) / 0,0,0 /
      data (e(11,3,k),k=1,3) / 0,0,0 /
      data (e(11,4,k),k=1,3) / 0,0,0 /

    ! ... satellite 12
      data (e(12,1,k),k=1,3) / 0,0,0 /
      data (e(12,2,k),k=1,3) / 0,0,0 /
      data (e(12,3,k),k=1,3) / 0,0,0 /
      data (e(12,4,k),k=1,3) / 0,0,0 /

      


    ! JPL resistance-to-temperature conversion factors                    
    ! for satellites 1..12
      data (kk(i,1),i=1,12) / 12*495.6 /
      data (kk(i,2),i=1,12) / 12*107.8 /

!****** NEW TSAN MO CALIBRATION EQUATION CONSTANTS ***********
c      real u(4,12),Gain,Tlin
       real Tlin
c      data u/4*0.,4*0.,4*0.,4*0.,
c     & 0.00009001004, 0.00008738477, 0.0001181006, 0.0001088968,
c     & 4*0.,4*0.,4*0.,4*0.,4*0.,4*0.,4*0./
     
c......new a0 coefficients to adjust Tb for Target temperature cross-talk
c...(produced by regression analysis on my PC 
c  of output from msdytbav_tbtb2*.f on flood)

c      real a0(12,4)
c      data a0/12*0.,
c     & -.0283,-.0361,-.0425,-.0591,-.0328,-.0573,
c     & -.0632,-.0290,-.0591,3*0.,
c     & 12*0.,
c     & 12*0./
     
c...new lincal adjustment declarations
      real temp1sav(10)
      real temp2sav(10)
      real dtemp1,rlat,coefa,coefb,coefc,coefd,meana,meanb,meanc,meand,Tbsave
      integer it,iovr,lat10,iasc
                real tempav
                real temp1av_star, rw_star, rk_star


                Qterm_array = 0.

                temp1diff_sum = 0.
                temp1diff_tot = 0.
                temp1diff_avg = 0.
                temp2diff_sum = 0.
                temp2diff_tot = 0.
                temp2diff_avg = 0.


c***************************   TESTING!!   ***********************************
c      u = 0.



      do n=1,110000
         badflag(n)=.FALSE.
      end do

      badflag(index_high+1:) = .TRUE. ! Added by wdb

    !--------------------------------------------------------------------
    ! Some corrections to the PRT count-to-resistance conversion factors
    ! due to errors in manufactures specs (given above)
    !--------------------------------------------------------------------
c...rws...I moved these adjustments to the DATA statements so the subroutine
c...could be called multiple times 
c      e(1,2,1) = e(1,2,1) +  .440   
c      e(1,3,1) = e(1,3,1) + 1.062   
      
c      e(2,1,1) = e(2,1,1) - .370   
c      e(2,2,1) = e(2,2,1) - .370   
c      e(2,3,1) = e(2,3,1) - .405   
c      e(2,4,1) = e(2,4,1) - .389    
      

    !-----------------------------
    ! Verify index_low,index_high 
    !-----------------------------
      if(index_low  .lt. 1 .or. index_low  .gt. max_num_scans .or.
     &   index_high .lt. 1 .or. index_high .gt. max_num_scans .or.
     &   index_high .lt. index_low ) then
         write(6,*)'index_low,index_high: ',index_low,index_high
         write(6,*)'max_num_scans: ',max_num_scans
         stop '---Error - calibrate_msu(): invalid index_low or 
     & index_high---'
      endif

    !---------------
    ! Verify sat id
    !---------------
      if( id.ne.1 .and. id.ne.2 .and. id.ne.3 .and. id.ne.4 
     & .and. id.ne.5 .and.
     &    id.ne.6 .and. id.ne.7 .and. id.ne.8 .and. id .ne. 9 ) then
        write(6,2995)id 
 2995   format('invalid sat id=',i5)    
         stop 'ERROR: invalid satellite id'
      endif         
      

    !-----------------
    ! Flag bad scans
    !-----------------

      do 111 n=index_low,index_high

       ! Mark missing scans as bad and continue
         if(time78(n).eq.-999) then
            badflag(n)=.TRUE.
            goto 111
         endif
      
         do i=1,16
            if(cal_counts(i,n) .eq. 4095) then  ! Check for saturated cal count
               badflag(n) = .TRUE.
               goto 111
            endif                  
         enddo            

         cal1lo = cal_counts( 1,n)
         cal2lo = cal_counts( 2,n)
         cal1hi = cal_counts( 3,n)
         cal2hi = cal_counts( 4,n)

         cold(1) = cal_counts( 9,n)   
         cold(2) = cal_counts(10,n)   
         cold(3) = cal_counts(11,n)   
         cold(4) = cal_counts(12,n)   

         warm(1) = cal_counts(13,n)   
         warm(2) = cal_counts(14,n)   
         warm(3) = cal_counts(15,n)   
         warm(4) = cal_counts(16,n)   

  !!!!!!!!!!! THIS ALSO NEEDS CHECKS FROM MSUING.F AND MAYBE ALSO FROM READMSU_NEW.F
      
     
c         if( abs(warm(1)-cold(1)) .le. 1 .or.    ! Ocassionally one of the circuits in a
c     &       abs(warm(2)-cold(2)) .le. 1 .or.    ! a warm- cold pair will fail and the same
c     &       abs(warm(3)-cold(3)) .le. 1 .or.    ! count value is returned for both.
c     &       abs(warm(4)-cold(4)) .le. 1 ) then  ! DO I REALLY WANT ABSOLUTE VALUE HERE ?????
c            badflag(n) = .TRUE.
c            go to 111
c         end if
            
            
         if( cal1hi-cal1lo .eq. 0 .or.
     &           cal2hi-cal2lo .eq. 0 ) then
            badflag(n) = .TRUE.
         endif         


111   continue

      

    !------------------------------------------------------------------
    ! Convert PRT counts to resistance to temperature using
    ! equations A2 and A3 of SCG90.
    !
    ! Flag scans with bad PRT temps
    !
    ! Average T's from the PRT's circuits (a,b) for each target.
    !
    ! Note correction to eq A2 of SCG90: "Ctarget" should read "PRTa"
    !------------------------------------------------------------------
    
      do 222 n=index_low,index_high
      if( badflag(n)) goto 222
      
         rcal1lo = cal_counts( 1,n)  
         rcal2lo = cal_counts( 2,n)  
         rcal1hi = cal_counts( 3,n)  
         rcal2hi = cal_counts( 4,n)  
         rt1a    = cal_counts( 5,n)  
         rt1b    = cal_counts( 6,n)  
         rt2a    = cal_counts( 7,n)  
         rt2b    = cal_counts( 8,n)  
      
      
       ! Covert PRT digital counts to resistance (eq A2)
       ! Note that rcal1 is for 'a' PRT's and rcal2 is for 'b' PRT's
       ! I should rename these rcalA and rcalB.
       
         res1a = kk(id,1)+kk(id,2)*(rt1a-rcal1lo)/(rcal1hi-rcal1lo)
         res1b = kk(id,1)+kk(id,2)*(rt1b-rcal2lo)/(rcal2hi-rcal2lo)

         res2a = kk(id,1)+kk(id,2)*(rt2a-rcal1lo)/(rcal1hi-rcal1lo)
         res2b = kk(id,1)+kk(id,2)*(rt2b-rcal2lo)/(rcal2hi-rcal2lo)

       
       ! Convert PRT resistance to temperature (eq A3)
       
         temp1a = e(id,1,1)+e(id,1,2)*res1a + e(id,1,3)*res1a*res1a
         temp1b = e(id,2,1)+e(id,2,2)*res1b + e(id,2,3)*res1b*res1b

         temp2a = e(id,3,1)+e(id,3,2)*res2a + e(id,3,3)*res2a*res2a
         temp2b = e(id,4,1)+e(id,4,2)*res2b + e(id,4,3)*res2b*res2b

                        temp1diff = temp1a-temp1b
                        temp2diff = temp2a-temp2b


                        if(abs(temp1diff).lt.0.15.and.abs(temp2diff).lt.0.15)then
                           temp1diff_sum = temp1diff_sum + temp1diff
                           temp2diff_sum = temp2diff_sum + temp2diff
                           temp1diff_tot = temp1diff_tot + 1.
                           temp2diff_tot = temp2diff_tot + 1.
                                temp1diff_avg = temp1diff_sum/temp1diff_tot
                                temp2diff_avg = temp2diff_sum/temp2diff_tot
c           if(mod(n,100).eq.0)then
c            if(abs(temp1diff).gt.0.06)then
c              write(6,6226)n, temp1a,temp1b,temp1diff,temp1diff_avg,   temp2a,temp2b,temp2diff,temp2diff_avg
c 6226          format('n=',i6,'  temp1a,1b=', 3f7.2,f7.3'       temp2a,2b=',3f7.2,f7.3)
c           end if
                        end if

       ! PRT disagreement check (resistance reference circuit sometimes      
       ! has been off, causing one PRT in each warm target to be biased)    

!.....1/6/2010:  changed the PRT difference checks from 2 to 4.......................
      
         if(id.eq.1) then
            if(abs(temp1a-temp1b) .gt. 4. .or.
     &         abs(temp2a-temp2b-1.6) .gt. 4. ) then
               badflag(n) = .TRUE.
               goto 222
               
            endif   

         else   
      
            if(abs(temp1a-temp1b) .gt .4. .or.
     &         abs(temp2a-temp2b) .gt .4. ) then
               badflag(n) = .TRUE.
               goto 222
               
            endif            
               
         endif
         
       ! If routine makes it to here, compute average PRT temperature for each target         
         temp1(n) = (temp1a + temp1b)*0.5
         temp2(n) = (temp2a + temp2b)*0.5

         
222   continue



    !-------------------------------------------------------------------------
    ! Initialize brightness temperature array to bad or missing data value
    !-------------------------------------------------------------------------
      do k=index_low,index_high
      do j=1,11
      do i=1,4
         tb(i,j,k) = badvalue
      enddo
      enddo
      enddo


    !----------------------
    ! Loop over all scans
    !----------------------
      do 999 n=index_low,index_high
         if( badflag(n)) then
            goto 999
         end if
      
    ! Unload calibration counts for current scan  
      cal1lo   = cal_counts( 1,n)  
      cal2lo   = cal_counts( 2,n)  
      cal1hi   = cal_counts( 3,n)  
      cal2hi   = cal_counts( 4,n)  
      t1a      = cal_counts( 5,n)  
      t1b      = cal_counts( 6,n)  
      t2a      = cal_counts( 7,n)  
      t2b      = cal_counts( 8,n)  
      cold(1)  = cal_counts( 9,n)  
      cold(2)  = cal_counts(10,n)  
      cold(3)  = cal_counts(11,n)  
      cold(4)  = cal_counts(12,n)  
      warm(1)  = cal_counts(13,n)  
      warm(2)  = cal_counts(14,n)  
      warm(3)  = cal_counts(15,n)  
      warm(4)  = cal_counts(16,n)
c      if(mod(n,1000).eq.0)then  
c        write(6,*)cal1lo,cal2lo,cal1hi,cal2hi,t1a,t1b,t2a,t2b,cold(2),
c     &   warm(2)
c      end if


    ! Previous and next "warm,cold" are needed in averaging.
    ! Determine if averaging is allowed for current scan.
      avg = .FALSE.
      if(avg)then
      if( n .gt. index_low .and. n .lt. index_high ) then
         if(badflag(n-1).or.badflag(n+1))go to 3020
      
      
          ! Check for missing scans, if none missing store last,next warm and cold avgs
           
            dt1 = time78(n) - time78(n-1)
            dt2 = time78(n+1) - time78(n)   
            
            if( dt1 .lt. 30 .and. dt1 .gt. 20 .and.
     &          dt2 .lt. 30 .and. dt2 .gt. 20 ) then
                
cc              avg = .TRUE.
      
               cold_last(1)  = cal_counts( 9,n-1)  
               cold_last(2)  = cal_counts(10,n-1)  
               cold_last(3)  = cal_counts(11,n-1)  
               cold_last(4)  = cal_counts(12,n-1)  

               warm_last(1)  = cal_counts(13,n-1)  
               warm_last(2)  = cal_counts(14,n-1)  
               warm_last(3)  = cal_counts(15,n-1)  
               warm_last(4)  = cal_counts(16,n-1)  

               cold_next(1)  = cal_counts( 9,n+1)  
               cold_next(2)  = cal_counts(10,n+1)  
               cold_next(3)  = cal_counts(11,n+1)  
               cold_next(4)  = cal_counts(12,n+1)  

               warm_next(1)  = cal_counts(13,n+1)  
               warm_next(2)  = cal_counts(14,n+1)  
               warm_next(3)  = cal_counts(15,n+1)  
               warm_next(4)  = cal_counts(16,n+1)
               
c....following added by RWS to do 3-scan line averaging of target temperatures...
               temp1_last = temp1(n-1)  
               temp2_last = temp2(n-1)  
               temp1_next = temp1(n+1)  
               temp2_next = temp2(n+1)  
           end if           
3020      continue
      endif
      end if         


    !-------------------------------------------------------
    ! Compute 3-scan line average for cold and warm counts,
    ! provided you have valid values from 3 consecutive
    ! scanlines. Otherwise use cold and warm counts for
    ! current scanline.
    !-------------------------------------------------------

c.test...test....test....
      avg=.false.

      if(avg) then
         do i=1,4
            coldav(i) = ( cold(i) + cold_last(i) + cold_next(i) )/3 ! DO I NEED TO USE ROUND FUNCTION HERE
            warmav(i) = ( warm(i) + warm_last(i) + warm_next(i) )/3 ! or does coldav need to be real.
c....following added by RWS.........................................
            temp1av=(temp1_last+temp1(n)+temp1_next)/3.
            temp2av=(temp2_last+temp2(n)+temp2_next)/3.
         enddo   
      else
         do i=1,4
            coldav(i) = cold(i)
            warmav(i) = warm(i)
c....following added by RWS.........................................
            temp1av=temp1(n)
            temp2av=temp2(n)
         enddo   
c.....added 1/5/2011..............        
                        temp1av_array(n) = temp1av
                        temp2av_array(n) = temp2av
                
c        if(abs((temp1(n)-temp2(n))).lt.5.)then
c           temp1av_array(n) = (temp1(n)-temp2(n))
c        end if

      end if
                                        



    !--------------------------------------
    ! Compute brigtness temperatures (tb)
    !--------------------------------------
      jyd=year(n)*1000+day(n)

      do i=1,4      ! Loop over channels
         do j=1,11  ! Loop over footprints                                       
c...rws...add check for bad NOAA-9 L.O. (Tb computation might blow up)....
c         if(id.eq.7.and.jyd.ge.87066.and.i.eq.2)go to 3399

 
          ! Check for and set badvalues
            if( counts(i,j,n) .eq. noaa_badvalue ) then
               tb(i,j,n) = badvalue
            else
             ! Linearize counts                                              
             
               rk = counts(i,j,n)
c....NESDIS quadratic adjustment of radiometer counts (old method)
               if(caltyp.eq.1)then
                  rk = c(id,i,1)+rk*(c(id,i,2)+rk*c(id,i,3))
               end if  
   
             ! Compute brightness temperature by linear interpolation        
             ! between cold space and warm target.
   
               rc=coldav(i)
c....NESDIS quadratic adjustment of radiometer counts (old method)
               if(caltyp.eq.1)then                                                
                  rc = c(id,i,1)+rc*(c(id,i,2)+rc*c(id,i,3))
               end if
                                          
               rw=warmav(i)
c....NESDIS quadratic adjustment of radiometer counts (old method)
               if(caltyp.eq.1)then                                                
                  rw = c(id,i,1)+rw*(c(id,i,2)+rw*c(id,i,3))
               end if  
   
c....channels 1 and 2.......................................   
               if( i.eq.1 .or. i.eq.2 )then                                 

                  if(caltyp.eq.0.or.caltyp.eq.1)then


                      !tb(i,j,n) = ((emiss(id,i)*temp1av+(1.-emiss(id,i))*280.) -2.7)/(rw-rc)*(rk-rc) + 2.7  ! linear cal with variable target "emissivity" correction
                      ! Uncomment following line to override above line, the hot target correction. 8/14/2013 (wdb)
                      tb(i,j,n) = (temp1av -2.7)/(rw-rc)*(rk-rc) + 2.7  ! normal linear calibration


c                      calterm(i,j,n) = 100.*(rk-rc)/(rw-rc)

c                    tb(i,j,n) = 0.96*(temp1av -2.7)/(rw-rc)*(rk-rc) + 12.7  ! approximate cal sidelobe version


c                    tb(i,j,n) = (temp1av -2.7)/(rw-rc)*(rk-rc) + 2.7  ! normal linear calibration (1st cut)

                     tb(i,j,n) = tb (i,j,n) - twcoef(id,i)*(temp1av - 280.)


c                    if(i.eq.2.and.id.eq.3)then
c                       tb(i,j,n) = tb(i,j,n) - (0.00214638*temp2av*temp2av-1.1915*temp2av+165.35)
c                   end if

c                    if(i.eq.2)then
c                       temp1av_star = (temp1av - (1.-emiss(id,i))*tb(i,j,n))/emiss(id,i)
c                       rw_star = (rw - (1.-emiss(id,i))*rk)/emiss(id,i)
c                       rk_star = (rk - (1.-emiss(id,i))*rw)/emiss(id,i)
c                        tb(i,j,n) = (temp1av_star -2.7)/(rw_star-rc)*(rk_star-rc) + 2.7
c                   end if



c                   if(i.eq.2)then
c                      write(6,3759)tb(i,j,n)
c 3759                  format(21x,f7.2)
c                      gain_new = tw_gain(id)  +  ((tw_gain(id)-avg_gain(id))/30.)*(tb(i,j,n)-temp1av)
c                      if(gain_new.gt.avg_gain(id))gain_new = avg_gain(id)
c                      tb(i,j,n) = temp1av + (1./gain_new)*(rk-rw)
c                      write(6,3769)tw_gain(id), avg_gain(id),gain_new,tb(i,j,n)
c 3769                  format(4f7.2)
c                   end if

 
                   end if                  
                  if(caltyp.eq.2)then
                     Gain=(rw-rc)/(temp1av-2.7)
                     Tlin=temp1av + (rk-rw)/Gain
                     Tb(i,j,n)=Tlin + 
     &                  u(i,id)*((rk-rw)*(rk-rc))/(Gain*Gain)
                  end if
                  
                  if(caltyp.eq.3)then
                      wavenmbr=freq(i)*1.d9/clight
                      Radw=(c1*wavenmbr**3.)/(exp(c2*wavenmbr/temp1av)-1.)
                      Radc=(c1*wavenmbr**3.)/(exp(c2*wavenmbr/2.7)-1.)
                      Gain=(rw-rc)/(Radw-Radc)
                      Q=u(i,id)*((rk-rw)*(rk-rc))/(Gain*Gain)

                                                         Q = Q * u2(i,id)   !...1/12/2012  adjustment factors based upon regressions of intersatellite Tb diffs WITHOUT Q adjustment against Q adjustments
                                                         
                                                         if(i.eq.2)then
                                                            Qterm_array(j,n) = 37594.5*Q  !....Q converted to temperature increment
cc                       write(6,*)n, j, Qterm_array(j,n)
                                                         end if
                      
                                                         RadEarth=Radw + (rk-rw)/Gain + Q
                      Tb(i,j,n)=(c2*wavenmbr)/log(1.+c1*wavenmbr**3./RadEarth)
                  end if
                  
               else                                                       
c........channels 3 and 4.......................................   
                  if(caltyp.eq.0.or.caltyp.eq.1)then
                     

                     !tb(i,j,n) = ((emiss(id,i)*temp2av+(1.-emiss(id,i))*280.) -2.7)/(rw-rc)*(rk-rc) + 2.7  ! T = 2.7K for cosmic background
                     ! Uncomment following line to override above line, the hot target correction. 8/14/2013 (wdb)
                     tb(i,j,n) = (temp2av -2.7)/(rw-rc)*(rk-rc) + 2.7
                     tb(i,j,n) = tb (i,j,n) - twcoef(id,i)*(temp2av - 280.)

                                                         tw(2,n)=temp2av  !..store for later averaging and regression
                   end if
                  if(caltyp.eq.2)then
                     Gain=(rw-rc)/(temp2av-2.7)
                     Tlin=temp2av + (rk-rw)/Gain
                     Tb(i,j,n)=Tlin + 
     &                  u(i,id)*((rk-rw)*(rk-rc))/(Gain*Gain)
                  end if
                  
                  if(caltyp.eq.3)then
                      wavenmbr=freq(i)*1.d9/clight
                    Radw=(c1*wavenmbr**3.)/(exp(c2*wavenmbr/temp2av)-1.)
                      Radc=(c1*wavenmbr**3.)/(exp(c2*wavenmbr/2.7)-1.)
                      Gain=(rw-rc)/(Radw-Radc)
                      Q=u(i,id)*((rk-rw)*(rk-rc))/(Gain*Gain)
                      RadEarth=Radw + (rk-rw)/Gain + Q
                Tb(i,j,n)=(c2*wavenmbr)/log(1.+c1*wavenmbr**3./RadEarth)
                  end if
                  
               end if                                                     


            endif
 3399    continue
         enddo
      enddo 
          
 
                                                       
                                                                        
    !-----------------------------------------
    ! Check for too cold or too warm tb's
    !-----------------------------------------

c      do i = 1, 11
c         ! NEED A CHECK FOR CHANNEL 1
c         if(tb(2,i,n).lt.190. .or. tb(2,i,n).gt.270.) tb(2,i,n)=badvalue
c         if(tb(3,i,n).lt.170. .or. tb(3,i,n).gt.290.) tb(3,i,n)=badvalue
c         if(tb(4,i,n).lt.170. .or. tb(4,i,n).gt.290.) tb(4,i,n)=badvalue
c      enddo
                                                                        
       
999   continue

      END

!====================================================================
! FUNCTION: secs_since_year()
! Returns sesc since start of ref_year
! NO ERROR CHECKING IS PERFORMED 
!====================================================================

      function secs_since_year(ref_year,year0,day0,secs0)
      implicit none
      integer*4 secs_since_year
      integer*4 ref_year  ! Reference year
      integer*4 year0     ! Year of interest
      integer*4 day0      ! Julian day in year0
      integer*4 secs0     ! secs in day0
      
      integer*4 i
      integer*4 sum

    ! Count secs for each year up to current year
      sum=0
      do i=ref_year,year0-1
         if( mod(i,4).eq.0) then ! leap year
            sum=sum+366*86400
         else                          ! non leap year
            sum=sum+365*86400
         endif            
      enddo

    ! Add in secs for current year. This is same whether its a leap year or not.
      sum = sum + (day0-1)*86400+secs0

      secs_since_year = sum
      return
      END
      


c*************************************************************************
c.....CHECK FOR FOR BAD ORBIT NAVIGATION DAYS, OTHER BAD DATA PERIODS*****
c*************************************************************************   
c..this collection of bad scan lines was found by imaging individual 
c..days' global grids.  The last bad scan listed is for 93138, which
c..approx. corresponds to the last time Vanessa G. and I checked imagery.
      SUBROUTINE BAD_SCANS(IDSC,JYD,JSECS,scanskip,jyear,jday)
         integer*2 idsc
         integer jyd,jsecs,scanskip,jyear,jday
         scanskip=0      
c*************************************************************************
c.....CHECK FOR FOR BAD ORBIT NAVIGATION DAYS, OTHER BAD DATA PERIODS*****   
C...(COPIED FROM (MSGRDM3Z))                                            
      IF(IDSC.EQ.1)THEN                                                 
         IF(JYD.EQ.79131.AND.JSECS.EQ.10682) GO TO 4230                   
         IF(JYD.EQ.79330.AND.JSECS.EQ.75015) GO TO 4230                    
         IF(JYEAR.LT.80)GO TO 1099                                      
         IF(JYD.EQ.80067.AND.JSECS.GE.31039) GO TO 4230                    
         IF(JYD.EQ.80068.AND.JSECS.LE. 3007) GO TO 4230                    
         IF(JYD.EQ.80268.AND.JSECS.GE.66931) GO TO 4230                    
         IF(JYD.EQ.80269.AND.JSECS.LE. 7283) GO TO 4230                    
         IF(JYD.GT.80366)GO TO 4230                                        
         GO TO 1099                                                     
      END IF                                                            
      IF(IDSC.EQ.2)THEN                                                 
         IF(JYD.LT.79152)GO TO 4230                                        
         IF(JYD.EQ.79287.AND.JSECS.EQ.18071) GO TO 4230                    
         IF(JYD.EQ.79322.AND.JSECS.EQ.18071) GO TO 4230                    
         IF(JYEAR.LT.80)GO TO 1099                                      
         IF(JYD.EQ.80062.AND.JSECS.GE.14400) GO TO 4230                    
         IF(JYD.EQ.80063) GO TO 4230                                       
         IF(JYD.EQ.80064.AND.JSECS.LE.600) GO TO 4230                      
         IF(JYD.EQ.80067.AND.JSECS.GE.28800)GO TO 4230                     
         IF(JYD.EQ.80253.AND.JSECS.GE. 3600.AND.JSECS.                  
     &    LE.61200)GO TO 4230                                              
         IF(JYD.EQ.80329.AND.JSECS.GE.21600.AND.JSECS.                  
     &    LE.83095)GO TO 4230                                              
         IF(JYD.EQ.80033.AND.JSECS.EQ.66608) GO TO 4230                    
         IF(JYD.EQ.80053.AND.JSECS.EQ.75133) GO TO 4230                    
         IF(JYD.EQ.80232.AND.JSECS.EQ.18071) GO TO 4230                    
         IF(JYD.EQ.80329.AND.JSECS.EQ.83146) GO TO 4230                    
         IF(JYD.EQ.80329.AND.JSECS.EQ.83197) GO TO 4230                   
         IF(JYD.EQ.80329.AND.JSECS.EQ.83248) GO TO 4230                    
         IF(JYEAR.LT.81)GO TO 1099                                      
         IF(JYD.EQ.81076.AND.JSECS.EQ.49123) GO TO 4230                   
         IF(JYD.EQ.81104.AND.JSECS.EQ.79920) GO TO 4230                  
         IF(JYEAR.LT.82)GO TO 1099                                      
         IF(JYD.EQ.82257.AND.JSECS.EQ.65172) GO TO 4230                    
         IF(JYD.EQ.82257.AND.JSECS.EQ.65172) GO TO 4230                   
         IF(JYD.EQ.82153)GO TO 4230                                        
         IF(JYEAR.LT.83)GO TO 1099                                      
         IF(JYEAR.EQ.83.AND.JDAY.GT.120)GO TO 4230                         
         IF(JYEAR.EQ.85.AND.JDAY.GE.182.AND.JDAY.LE.212)GO TO 4230         
         IF(JYEAR.LT.86)GO TO 1099                                      
         IF(JYD.GT.86304)GO TO 4230                                        
         IF(JYD.EQ.86049.AND.JSECS.EQ.46629) GO TO 4230                   
         IF(JYD.EQ.86065.AND.JSECS.EQ.27839) GO TO 4230                    
         IF(JYD.EQ.86130.AND.JSECS.EQ.69261) GO TO 4230                    
         IF(JYD.EQ.86130.AND.JSECS.EQ.71156) GO TO 4230                    
         IF(JYD.EQ.86001.OR.JYD.EQ.86002)GO TO 4230                        
         GO TO 1099                                                     
      END IF 
      if(idsc.eq.3)go to 1099                                                           
      IF(IDSC.EQ.4)THEN                                                 
         IF(JYD.EQ.82302.AND.JSECS.EQ.30490) GO TO 4230                    
         IF(JYD.EQ.82304.AND.JSECS.EQ.10240) GO TO 4230                    
         IF(JYD.EQ.82304.AND.JSECS.EQ.11469) GO TO 4230                    
         IF(JYD.EQ.83276.AND.JSECS.EQ.17355) GO TO 4230                    
         IF(JYEAR.LT.84)GO TO 1099                                      
         IF(JYD.EQ.84062.AND.JSECS.LE.28800) GO TO 4230                   
         IF(JYD.EQ.84330.AND.JSECS.EQ.71164) GO TO 4230                    
         IF(JYD.EQ.84330.AND.JSECS.EQ.71190) GO TO 4230                   
         IF(JYD.EQ.84330.AND.JSECS.EQ.71241) GO TO 4230                    
         IF(JYD.EQ.84340.AND.JSECS.EQ.69091) GO TO 4230                    
         IF(JYD.EQ.84340.AND.JSECS.EQ.69142) GO TO 4230                    
         IF(JYD.EQ.84340.AND.JSECS.EQ.69168) GO TO 4230                    
         IF(JYD.EQ.84366.AND.JSECS.GE.2261.AND.JSECS.LE.2389)GO TO 4230    
         IF(JYD.GT.85059)GO TO 4230                                       
         GO TO 1099                                                     
      END IF 
      if(idsc.eq.6)go to 1099                                                           
      IF(IDSC.EQ.7)THEN                                                 
         IF(JYEAR.EQ.84.AND.JDAY.LT.350)GO TO 4230                         
         IF(JYD.EQ.84350.AND.JSECS.EQ.48569) GO TO 4230                   
         IF(JYEAR.LT.85)GO TO 1099                                      
         IF(JYD.EQ.85070.AND.JSECS.EQ.22149) GO TO 4230                    
         IF(JYD.EQ.85070.AND.JSECS.EQ.73145) GO TO 4230                    
         IF(JYD.EQ.85115.AND.JSECS.EQ.80645) GO TO 4230                    
         IF(JYD.EQ.85160.AND.JSECS.EQ.42399) GO TO 4230                    
         IF(JYD.EQ.85239.AND.JSECS.EQ.47289) GO TO 4230                    
         IF(JYD.EQ.85277.AND.JSECS.EQ.82623) GO TO 4230                    
         IF(JYEAR.LT.86)GO TO 1099                                      
         IF(JYD.EQ.86001.OR.JYD.EQ.86002)GO TO 4230                        
         IF(JYD.EQ.86008.AND.JSECS.EQ.26092) GO TO 4230                   
         IF(JYD.EQ.86008.AND.JSECS.EQ.26092) GO TO 4230                    
         IF(JYD.EQ.86150.AND.JSECS.EQ.58861) GO TO 4230                    
         IF(JYD.EQ.86151.AND.JSECS.EQ.58119) GO TO 4230                   
         IF(JYD.EQ.86151.AND.JSECS.EQ.58119) GO TO 4230                    
         IF(JYD.EQ.86319.AND.JSECS.EQ.12041) GO TO 4230                    
CC       IF(JYD.GT.86334)GO TO 4230                                        
         IF(JYD.EQ.87001.OR.JYD.EQ.87002)GO TO 4230
c....this was check for bad channel 2 data when NOAA-9 L.O. died.....                      
cc         IF(JYD.GT.87066.and.n9.ne.1)GO TO 4230                                     
         GO TO 1099                                                     
      END IF                                                            
      IF(IDSC.EQ.8)THEN                                                 
         IF(JYD.EQ.86361.AND.JSECS.EQ.69126) GO TO 4230                
         IF(JYEAR.LT.87)GO TO 1099                                      
         IF(JYD.EQ.87001.OR.JYD.EQ.87002)GO TO 4230                        
         IF(JYD.EQ.87216.AND.JSECS.EQ.47649) GO TO 4230                   
         IF(JYD.EQ.87273.AND.JSECS.EQ.55329) GO TO 4230                    
         IF(JYD.EQ.87285.AND.JSECS.EQ.38586) GO TO 4230                    
         IF(JYD.EQ.87300.AND.JSECS.EQ.63879) GO TO 4230                    
         IF(JYD.EQ.87329.AND.JSECS.EQ.38330) GO TO 4230                    
         IF(JYD.EQ.87329.AND.JSECS.EQ.39585) GO TO 4230                    
         IF(JYD.EQ.87353.AND.JSECS.EQ. 4052) GO TO 4230                    
         IF(JYD.EQ.88049.AND.JSECS.EQ.16724) GO TO 4230                    
         IF(JYD.EQ.91209.AND.JSECS.EQ.29786) GO TO 4230                    
         GO TO 1099                                                     
      END IF                                                            
      IF(IDSC.EQ.9)THEN                                                 
         IF(JYD.EQ.89012.AND.JSECS.EQ.30099) GO TO 4230                  
         IF(JYD.EQ.89031.AND.JSECS.EQ.14918) GO TO 4230                   
         IF(JYD.EQ.89365.AND.JSECS.EQ.24566) GO TO 4230                  
         IF(JYD.EQ.90154.AND.JSECS.EQ. 9433) GO TO 4230                  
         GO TO 1099                                                     
      END IF                                                            
      IF(IDSC.EQ.5)THEN                                                 
         IF(JYD.EQ.91345.AND.JSECS.EQ.24592) GO TO 4230                  
         IF(JYD.EQ.92278.AND.JSECS.EQ.32553) GO TO 4230               
         IF(JYD.EQ.93138.AND.JSECS.EQ.32553) GO TO 4230                
         GO TO 1099                                                     
      END IF
      
 4230 continue
      scanskip=1
                                                                  
 1099 CONTINUE
      return 
      end                                                                                                           

