MODULE MEDIAN_MODULE
  implicit none
  private
  public :: find_median

  interface find_median
     module procedure find_median_without_badvalues, find_median_with_badvalues
  end interface   

CONTAINS
! This routine indexes the input array arr. I.e. outputs indx(1:n) such that
! arr(indx(j)) is in ascending order for j=1..n.
subroutine indexx(n,arr,indx)
   integer n,indx(n),m,nstack
   real arr(n)
   parameter (m=7,nstack=50)
   integer i,indxt,ir,itemp,j,jstack,k,l,istack(nstack)
   real a
   do 11 j=1,n
     indx(j)=j
11 continue
   jstack=0
   l=1
   ir=n
1  if(ir-l.lt.m)then
     do 13 j=l+1,ir
       indxt=indx(j)
       a=arr(indxt)
       do 12 i=j-1,1,-1
         if(arr(indx(i)).le.a)goto 2
         indx(i+1)=indx(i)
12     continue
       i=0
2      indx(i+1)=indxt
13   continue
     if(jstack.eq.0)return
     ir=istack(jstack)
     l=istack(jstack-1)
     jstack=jstack-2
   else
     k=(l+ir)/2
     itemp=indx(k)
     indx(k)=indx(l+1)
     indx(l+1)=itemp
     if(arr(indx(l+1)).gt.arr(indx(ir)))then
       itemp=indx(l+1)
       indx(l+1)=indx(ir)
       indx(ir)=itemp
     endif
     if(arr(indx(l)).gt.arr(indx(ir)))then
       itemp=indx(l)
       indx(l)=indx(ir)
       indx(ir)=itemp
     endif
     if(arr(indx(l+1)).gt.arr(indx(l)))then
       itemp=indx(l+1)
       indx(l+1)=indx(l)
       indx(l)=itemp
     endif
     i=l+1
     j=ir
     indxt=indx(l)
     a=arr(indxt)
3    continue
       i=i+1
     if(arr(indx(i)).lt.a)goto 3
4    continue
       j=j-1
     if(arr(indx(j)).gt.a)goto 4
     if(j.lt.i)goto 5
     itemp=indx(i)
     indx(i)=indx(j)
     indx(j)=itemp
     goto 3
5    indx(l)=indx(j)
     indx(j)=indxt
     jstack=jstack+2
     if(jstack.gt.nstack)stop "Error - indexx(): nstack too small"
     if(ir-i+1.ge.j-l)then
       istack(jstack)=ir
       istack(jstack-1)=i
       ir=j-1
     else
       istack(jstack)=j-1
       istack(jstack-1)=l
       l=i
     endif
   endif
   goto 1
end subroutine indexx

subroutine find_median_without_badvalues( x, median )
  real,intent(IN)  :: x(:)
  real,intent(OUT) :: median

  integer :: n,i
  integer,allocatable :: indx(:)

  n = size(x)
  allocate(indx(n))

  call indexx(n,x,indx)

  if(n/2*2 == n) then ! N is even
     i = n/2
     median = (x(indx(i)) + x(indx(i+1)))/2.
  else ! N is odd
     i =  n/2 + 1
     median = x(indx(i))
  end if   

 deallocate(indx)

end subroutine find_median_without_badvalues

subroutine find_median_with_badvalues(x,median,badvalue)
  real,intent(IN)  :: x(:)
  real,intent(OUT) :: median
  real,intent(IN)  :: badvalue

  real,allocatable :: x2(:)
  integer,allocatable :: indx(:)
  integer :: n,i,icount

  ! Count number of elements of x /= badvalue
  n = count(x/=badvalue)
  if(n==0) then
     median = badvalue
     return
  end if  

  allocate(x2(n),indx(n))

  ! Load x2 with values /= badvalue
  icount = 0
  do i=1,size(x)
     if(x(i)/=badvalue) then
        icount = icount + 1
        x2(icount) = x(i)
     end if
  end do   

  call find_median_without_badvalues(x2,median)
  
  deallocate(x2,indx)

end subroutine find_median_with_badvalues  

end module median_module


!================================
! Main program for testing
!================================
!$$$program main
!$$$  use MEDIAN_MODULE
!$$$  implicit none
!$$$
!$$$  integer,parameter :: badvalue = -999
!$$$
!$$$  real :: x(6),median
!$$$
!$$$  x = (/5,4,3,2,1,-999/)
!$$$
!$$$  call find_median(x,median,real(badvalue))
!$$$  write(6,*) "median = ",median
!$$$
!$$$end program main


