program pmsu

!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_open_log_file
use module_read_input_options
use module_get_latlon_grids
use module_read_mon_gridded_anomaly
use module_process_files
use module_write_done_file
use module_calc_times
use module_print_summary

implicit none
 
!  Local declarations

call open_log_file

call read_input_options

call calc_times ( 0 )
call get_latlon_grids

call process_files

call print_summary

call write_done_file ( lulog )

end program
!
!***********************************************************************
!
module module_print_summary
implicit none
contains
 
subroutine print_summary
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

implicit none
 
! Procedure dummy argument declarations

! Local Declarations

write (lulog,'(/a)') ' subroutine print_summary'

write (lulog,'(a,f15.3)') ' amg_min_mt = ', amg_min_mt
write (lulog,'(a,f15.3)') ' amg_max_mt = ', amg_max_mt
write (lulog,'(a,f15.3)') ' amg_min_lt = ', amg_min_lt
write (lulog,'(a,f15.3)') ' amg_max_lt = ', amg_max_lt
write (lulog,'(a,f15.3)') ' amg_min_ls = ', amg_min_ls
write (lulog,'(a,f15.3)') ' amg_max_ls = ', amg_max_ls
write (lulog,'(a,f15.3)') ' amg_min_tp = ', amg_min_tp
write (lulog,'(a,f15.3)') ' amg_max_tp = ', amg_max_tp
write (lulog,'(a,f15.3)') ' acg_min_mt = ', acg_min_mt
write (lulog,'(a,f15.3)') ' acg_max_mt = ', acg_max_mt
write (lulog,'(a,f15.3)') ' acg_min_lt = ', acg_min_lt
write (lulog,'(a,f15.3)') ' acg_max_lt = ', acg_max_lt
write (lulog,'(a,f15.3)') ' acg_min_ls = ', acg_min_ls
write (lulog,'(a,f15.3)') ' acg_max_ls = ', acg_max_ls
write (lulog,'(a,f15.3)') ' acg_min_tp = ', acg_min_tp
write (lulog,'(a,f15.3)') ' acg_max_tp = ', acg_max_tp
write (lulog,'(a,f15.3)') ' reg_min_mt = ', reg_min_mt
write (lulog,'(a,f15.3)') ' reg_max_mt = ', reg_max_mt
write (lulog,'(a,f15.3)') ' reg_min_lt = ', reg_min_lt
write (lulog,'(a,f15.3)') ' reg_max_lt = ', reg_max_lt
write (lulog,'(a,f15.3)') ' reg_min_ls = ', reg_min_ls
write (lulog,'(a,f15.3)') ' reg_max_ls = ', reg_max_ls
write (lulog,'(a,f15.3)') ' reg_min_tp = ', reg_min_tp
write (lulog,'(a,f15.3)') ' reg_max_tp = ', reg_max_tp

write (lulog,'(a,f15.3)') ' reg_trend_min_mt = ', reg_trend_min_mt
write (lulog,'(a,f15.3)') ' reg_trend_max_mt = ', reg_trend_max_mt
write (lulog,'(a,f15.3)') ' reg_trend_min_lt = ', reg_trend_min_lt
write (lulog,'(a,f15.3)') ' reg_trend_max_lt = ', reg_trend_max_lt
write (lulog,'(a,f15.3)') ' reg_trend_min_ls = ', reg_trend_min_ls
write (lulog,'(a,f15.3)') ' reg_trend_max_ls = ', reg_trend_max_ls
write (lulog,'(a,f15.3)') ' reg_trend_min_tp = ', reg_trend_min_tp
write (lulog,'(a,f15.3)') ' reg_trend_max_tp = ', reg_trend_max_tp

end subroutine

end module
!
!***********************************************************************
!
module module_calc_times
implicit none
contains
 
subroutine calc_times ( np )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_alloc_status
use module_calc_julian_minute
use module_inv_julian_minute
use module_julian
use module_get_epoch_seconds
use module_get_cdate

use module_allocate_arrays_mon_gridded_anom
use module_allocate_arrays_day_anom
use module_allocate_arrays_mon_anom
use module_allocate_arrays_mon_regional_anom

implicit none
 
! Procedure dummy argument declarations

integer, intent(in) :: np

! Local Declarations

integer :: nyr3
integer :: nmo3
integer :: ndy3
integer :: nhr3
integer :: nmin3
integer :: jmin3
integer :: jday3

integer :: nyr4
integer :: nmo4
integer :: ndy4
integer :: nhr4
integer :: nmin4
integer :: jmin4
integer :: jmidp

integer :: jdiff
integer :: jdiffp
integer :: jdiffm
integer :: nt
integer :: jmin
integer :: jminp
integer :: istatus
integer :: kdum = 1
integer :: nsum

integer :: nyrc
integer :: nmoc
integer :: ndyc
character ( len = 19 ) :: cdate

write (lulog,'(/a)') ' ENTERING subroutine calc_times'
write (lulog,'(a,i10)') ' np=', np

nhr3 = 0
nmin3 = 0

call calc_julian_minute ( nyr_beg, nmo_beg, ndy_beg, nhr3, nmin3, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_beg )

call calc_julian_minute ( nyr_end, nmo_end, ndy_end, nhr3, nmin3, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_end )

jdiff = 24 * 60

ntmax_days = ( ( jmin_end - jmin_beg ) / jdiff ) + 1

if ( allocated ( jmin_store_days ) ) then
  deallocate ( jmin_store_days, stat = istatus )
end if

if ( allocated ( dflag_store_days ) ) then
  deallocate ( dflag_store_days, stat = istatus )
end if

allocate ( jmin_store_days(ntmax_days), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine calc_times', &
    'jmin_store_days', istatus, &
    '1D', ntmax_days, kdum, kdum, lulog )
end if

allocate ( dflag_store_days(ntmax_days), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine calc_times', &
    'dflag_store_days', istatus, &
    '1D', ntmax_days, kdum, kdum, lulog )
end if

write (lulog,'(/a)') ' subroutine calc_times'
write (lulog,'(a,i10)') ' ntmax_days=', ntmax_days
write (lulog,'(a)') ' DAILY TIMES'

jmin = jmin_beg - jdiff
nsum = 0

write (lulog,'(a)') '        nt       nyr       nmo       ndy       nhr'

do nt = 1, ntmax_days
  jmin = jmin + jdiff
  jmin_store_days(nt) = jmin
  call inv_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
    nyr_base, nmo_base, ndy_base, nhr_base, &
    lulog, jmin )
  if ( ndy3 == 1 .and. nhr3 == 0 ) then
    nsum = nsum + 1
  end if
  if ( np == 0 ) then
    write (lulog,'(2i10,3i10.2)') nt, nyr3, nmo3, ndy3, nhr3
  end if
end do

ntmax_mons = nsum

if ( allocated ( jmin_store_mons ) ) then
  deallocate ( jmin_store_mons, stat = istatus )
end if

if ( allocated ( dflag_store_mons ) ) then
  deallocate ( dflag_store_mons, stat = istatus )
end if

if ( allocated ( dflag_store_mons_reg ) ) then
  deallocate ( dflag_store_mons, stat = istatus )
end if

if ( allocated ( jday_store_mons ) ) then
  deallocate ( jday_store_mons, stat = istatus )
end if

if ( allocated ( time_bnds ) ) then
  deallocate ( time_bnds, stat = istatus )
end if

allocate ( jmin_store_mons(ntmax_mons), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine calc_times', &
    'jmin_store_mons', istatus, &
    '1D', ntmax_mons, kdum, kdum, lulog )
end if

allocate ( dflag_store_mons(ntmax_mons), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine calc_times', &
    'dflag_store_mons', istatus, &
    '1D', ntmax_mons, kdum, kdum, lulog )
end if

allocate ( dflag_store_mons_reg(ntmax_mons), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine calc_times', &
    'dflag_store_mons_reg', istatus, &
    '1D', ntmax_mons, kdum, kdum, lulog )
end if

allocate ( jday_store_mons(ntmax_mons), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine calc_times', &
    'jday_store_mons', istatus, &
    '1D', ntmax_mons, kdum, kdum, lulog )
end if

allocate ( time_bnds(2,ntmax_mons), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine calc_times', &
    'time_bnds', istatus, &
    '2D', 2, ntmax_mons, kdum, lulog )
end if

write (lulog,'(/a)') ' subroutine calc_times'
write (lulog,'(a,i10)') ' ntmax_mons=', ntmax_mons
write (lulog,'(a)') ' MONTHLY TIMES'

jmin = jmin_beg - jdiff
nsum = 0

!       write (lulog,'(a)') '        nt       nyr       nmo       ndy       nhr     jdiff'
write (lulog,'(a)') '        nt       nyr       nmo       ndy       nhr     jdiff      jmid'

do nt = 1, ntmax_days

  jmin = jmin + jdiff
  call inv_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
    nyr_base, nmo_base, ndy_base, nhr_base, &
    lulog, jmin )

  if ( ndy3 == 1 .and. nhr3 == 0 ) then
    nsum = nsum + 1
    jdiffp = jmin / ( 24 * 60 )
    if ( np == 0 ) then
      write (lulog,'(2i10,4i10.2,i10)') nsum, nyr3, nmo3, ndy3, nhr3, jdiffp, 0
    end if
    jmin_store_mons(nsum) = jmin
    time_bnds(1,nsum) = jdiffp
  end if

  jminp = jmin + jdiff

  call inv_julian_minute ( nyr4, nmo4, ndy4, nhr4, nmin4, &
    nyr_base, nmo_base, ndy_base, nhr_base, &
    lulog, jminp )

  if ( nmo4 /= nmo3 ) then

    jdiffp = jmin / ( 24 * 60 )

!   Julian times have been with respect to UTC hour and minute = 0
!   Add a day for a complete 24-h day at end of the month

    jdiffp = jdiffp + 1
    time_bnds(2,nsum) = jdiffp

    jdiffm = ( time_bnds(2,nsum) - time_bnds(1,nsum) ) / 2
    jmidp = time_bnds(1,nsum) + jdiffm
    jday_store_mons(nsum) = jmidp

    if ( np == 0 ) then
      write (lulog,'(2i10,4i10.2,i10)') nsum, nyr3, nmo3, ndy3, nhr3, jdiffp, jmidp
    end if

  end if

end do

ndmax = ntmax_days
ngmax = ntmax_mons
ntmax = ngmax

call allocate_arrays_mon_gridded_anom ( .false. )        ! ngmax
call allocate_arrays_day_anom ( .false. )                ! ndmax
call allocate_arrays_mon_anom ( .false. )                ! ntmax
call allocate_arrays_mon_regional_anom ( .false. )       ! ngmax

do nt = 1, ntmax_days
  jmin = jmin_store_days(nt)
  call inv_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
    nyr_base, nmo_base, ndy_base, nhr_base, &
    lulog, jmin )
  call julian ( nmo3, ndy3, nyr3, jday3 )
  call get_epoch_seconds ( nyr3, nmo3, ndy3, esec, lulog )
  call get_cdate ( nyr3, nmo3, ndy3, cdate, lulog )
  day_anom_store%esec(nt) = dble( esec )
  day_anom_store%cdate(nt) = cdate
end do

do nt = 1, ntmax_mons
  jmin = jmin_store_mons(nt)
  call inv_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
    nyr_base, nmo_base, ndy_base, nhr_base, &
    lulog, jmin )
  call julian ( nmo3, ndy3, nyr3, jday3 )
  call get_epoch_seconds ( nyr3, nmo3, ndy3, esec, lulog )
  call get_cdate ( nyr3, nmo3, ndy3, cdate, lulog )
  esec_monthly_hold(nt) = dble ( esec )
  esec_monthly_reg_hold(nt) = dble ( esec )
  cdate_monthly_hold(nt) = cdate
  mon_anom_store%esec(nt) = dble( esec )
  mon_anom_store%cdate(nt) = cdate
  cdate_monthly_reg_hold(nt) = cdate
end do

end subroutine

end module
!
!***********************************************************************
!
module module_findilev
implicit none
contains

subroutine findilev ( icol, nlev, itemp, nbot, ntop, kbot, ktop, ngood, &
  nrev, nstop, lulog )

implicit none

!  Procedure dummy argument declarations

integer, intent(inout), dimension (nlev) :: icol
integer, intent(in) :: nlev
integer, intent(in) :: itemp
integer, intent(inout) :: nbot
integer, intent(inout) :: ntop
integer, intent(in) :: kbot
integer, intent(in) :: ktop
integer, intent(inout) :: ngood
integer, intent(in) :: nrev
integer, intent(in) :: nstop
integer, intent(in) :: lulog

!  Local declarations

integer :: kl
integer :: km
integer :: ku
integer :: zbot
integer :: ztop

if ( nlev > 1 ) then
  kl = kbot
  ku = ktop
else
  if ( icol(1) == itemp ) then
    nbot = 1
    ntop = 1
    ngood = 1
  else
    nbot = 1
    ntop = 1
    ngood = 0
  end if
  return
end if

do

  zbot = icol(kl)
  ztop = icol(ku)

  if ( (ku-kl) <= 1 ) then
    ngood = 0
    if ( nrev == +1 ) then
      if ( itemp >= zbot .and. itemp <= ztop ) ngood = 1
    else if ( nrev == -1 ) then
      if ( itemp <= zbot .and. itemp >= ztop ) ngood = 1
    end if
    exit
  end if

  km = (ku + kl) / 2
  zbot = icol(km)

  if ( nrev == +1 ) then
    if ( itemp >= zbot ) then
      kl = km
    else
      ku = km
    end if
  else if ( nrev == -1 ) then
    if ( itemp <= zbot ) then
      kl = km
    else
      ku = km
    end if
  end if

end do

if ( ngood == 0 .and. nstop == 1 ) then
  write (lulog,'(/a)') ' subroutine findilev:  could not find levels'
  write (lulog,'(a,i10)') ' itemp=',itemp
  write (lulog,'(a,2i5)') ' kl,ku=',kl,ku
  write (lulog,'(a,f10.3)') ' zbot=',zbot
  write (lulog,'(a,f10.3)') ' ztop=',ztop
  stop
end if

nbot = kl
ntop = ku

end subroutine

end module
!
***********************************************************************
!
module module_write_done_file
implicit none
contains
 
subroutine write_done_file ( lulog )
 
!  Global data module

!  Modules for needed subroutines ...

use module_get_free_logical_unit

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: lulog

! Local Declarations

call system ( '/bin/date > DONE' )

write (lulog,'(/a)') ' subroutine write_done_file'
write (lulog,'(a)') ' Calculations completed ... DONE file written ...'

close (unit=lulog,status='keep')

end subroutine

end module
!
!***********************************************************************
!
module module_write_missing
implicit none
contains
 
subroutine write_missing ( nf, atmos_region, fname, ftype, process_ver, &
  nopen, ncid, fname_netcdf, &
  fselect, fopen, gfiles, ncid_hold )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_data_driver
use module_init_arrays_mon_gridded_anom
use module_init_arrays_day_anom
use module_init_arrays_mon_anom
use module_init_arrays_mon_regional_anom

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nf
character ( len = * ), intent(in) :: atmos_region
character ( len = * ), intent(inout) :: fname
character ( len = * ), intent(in) :: ftype
character ( len = * ), intent(in) :: process_ver
integer, intent(inout) :: nopen
integer, intent(inout) :: ncid
character ( len = * ), intent(inout) :: fname_netcdf
integer, intent(in), dimension(ncvmax) :: fselect
integer, intent(inout), dimension(ncvmax) :: fopen
character ( len = * ), intent(in), dimension(ncvmax) :: gfiles
integer, intent(inout), dimension(ncvmax) :: ncid_hold

! Local Declarations

integer :: nta
integer :: ntb
integer :: nt
integer :: nsum
integer :: ntsum
integer :: ngsum

write (lulog,'(/a/)') ' ENTERING subroutine write_missing'
call flush ( lulog )

ntsum = 0
ngsum = 0

if ( trim(ftype) == 'amg' .or. &
     trim(ftype) == 'acg' )then

  call init_arrays_mon_gridded_anom

  nsum = 0

  if ( trim(ftype) == 'amg' ) then
    do nt = 1, ntmax_mons
      if ( dflag_store_mons(nt) == 0 ) then
        nsum = nsum + 1
      end if
    end do
  else if ( trim(ftype) == 'acg' ) then
    do nt = 1, ntmax_mons
!     write (lulog,'(a,2i10)') ' nt, dflag_store_mons(nt)=', nt, dflag_store_mons(nt)
      if ( dflag_store_mons(nt) == 1 ) then
        nsum = nsum + 1
      end if
    end do
    if ( nsum /= 12 ) then
      write (lulog,'(/a)') ' subroutine write_missing'
      write (lulog,'(a,a)') ' ftype = ', trim ( ftype )
      write (lulog,'(a)') ' expecting 12 months of data ???'
      write (lulog,'(a,i10)') ' nsum=', nsum
      stop
    end if
    return
  end if

  write (lulog,'(/a)') ' subroutine write_missing'
  write (lulog,'(a)') ' before call write_data_driver'
  write (lulog,'(a,i10)') ' nsum=', nsum
  write (lulog,'(a,i10)') ' ntmax_mons=', ntmax_mons
  write (lulog,'(a,i10)') ' ntsum=', ntsum
  write (lulog,'(a,i10)') ' ngsum=', ngsum

  if ( nsum > 0 ) then
    do nt = 1, ntmax_mons
      call write_data_driver ( nf, atmos_region, fname, ntsum, ngsum, &
        nt, nt, ftype, process_ver, nopen, ncid, fname_netcdf, &
        fselect, fopen, gfiles, ncid_hold )
    end do
  end if

else if ( trim(ftype) == 'mam' ) then

  nsum = 0

  call init_arrays_mon_anom

  do nt = 1, ntmax_mons
    if ( dflag_store_mons(nt) == 0 ) then
      nsum = nsum + 1
    end if
  end do

  if ( nsum > 0 ) then
    do nt = 1, ntmax_mons
      call write_data_driver ( nf, atmos_region, fname, ntsum, ngsum, &
        nt, nt, ftype, process_ver, nopen, ncid, fname_netcdf, &
        fselect, fopen, gfiles, ncid_hold )
    end do
  end if

else if ( trim(ftype) == 'amz' ) then

  call init_arrays_day_anom

  nsum = 0

  do nt = 1, ntmax_days
    if ( dflag_store_days(nt) == 0 ) then
      nsum = nsum + 1
    end if
  end do

  if ( nsum > 0 ) then
    do nt = 1, ntmax_days
      call write_data_driver ( nf, atmos_region, fname, ntsum, ngsum, &
        nt, nt, ftype, process_ver, nopen, ncid, fname_netcdf, &
        fselect, fopen, gfiles, ncid_hold )
    end do
  end if

else if ( trim(ftype) == 'ncdc' ) then

  call init_arrays_mon_regional_anom

  nsum = 0

  do nt = 1, ntmax_mons
    if ( dflag_store_mons_reg(nt) == 0 ) then
      nsum = nsum + 1
    end if
  end do

  if ( nsum > 0 ) then
    do nt = 1, ntmax_mons
      call write_data_driver ( nf, atmos_region, fname, ntsum, ngsum, &
        nt, nt, ftype, process_ver, nopen, ncid, fname_netcdf, &
        fselect, fopen, gfiles, ncid_hold )
    end do
  end if

end if

end subroutine

end module
!
!***********************************************************************
!
module module_process_files
implicit none
contains
 
subroutine process_files
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_do_wget_command
use module_get_free_logical_unit
use module_write_error
use module_read_data_driver
use module_write_data_driver
use module_allocate_arrays_mon_gridded_anom
use module_write_missing
use module_check_for_netcdf_file_close
use module_calc_times

implicit none
 
!  Procedure dummy argument declarations

! Local Declarations

integer :: nf
integer :: nv
integer :: np
integer :: lutemp
integer :: ntsum
integer :: ngsum
integer :: ios
character ( len = 1000 ) :: fname
character ( len = 1000 ) :: fname_netcdf
integer :: npass
integer :: nfsum
integer :: ncvsum
logical :: process_data
character ( len = 500 ) :: process_web
integer :: nopen
integer :: npass_max
integer :: nend
integer :: lnblnk
integer :: ncid
character ( len = 2 ) :: atmos_region
integer :: istatus
character ( len = 256 ) :: emess
integer :: nta
integer :: ntb
logical :: done_read
integer :: ncheck

! Local Arrays

character ( len = 10 ), dimension(ncvmax) :: process_ver = cmiss
character ( len = 100 ), dimension(ncvmax) :: ftypes
integer, dimension(ncvmax) :: fselect
integer, dimension(ncvmax) :: fopen
character ( len = 1000 ), dimension(ncvmax) :: gfiles
integer, dimension(ncvmax) :: ncid_hold

call get_free_logical_unit ( lutemp )

REGION: do np = 1, 4

  process_data = .false.
  nfsum = 0
  ncvsum = 0
  process_web = ''
  process_ver = ''
  ftypes = ''
  fselect = 0
  gfiles = ''
  ncid_hold = 0
  jmin_beg_exc = 0
  jmin_end_exc = 0
  atmos_region = ''

  if ( np == 1 ) then
    process_data = process_middle_troposphere
    nfsum = middle_troposphere_nfsum
    ncvsum = process_middle_troposphere_ncvsum
    process_web = process_middle_troposphere_web
    do nv = 1, ncvsum
      process_ver(nv) = process_middle_troposphere_ver(nv)
    end do
    do nf = 1, nfsum
      ftypes(nf) = middle_troposphere_ftypes(nf)
      fselect(nf) = middle_troposphere_fselect(nf)
      gfiles(nf) = middle_troposphere_gfiles(nf)
      jmin_beg_exc(nf) = middle_troposphere_jmin_beg(nf)
      jmin_end_exc(nf) = middle_troposphere_jmin_end(nf)
    end do
    atmos_region = 'MT'
  else if ( np == 2 ) then
    process_data = process_lower_troposphere
    nfsum = lower_troposphere_nfsum
    ncvsum = process_lower_troposphere_ncvsum
    process_web = process_lower_troposphere_web
    do nv = 1, ncvsum
      process_ver(nv) = process_lower_troposphere_ver(nv)
    end do
    do nf = 1, nfsum
      ftypes(nf) = lower_troposphere_ftypes(nf)
      fselect(nf) = lower_troposphere_fselect(nf)
      gfiles(nf) = lower_troposphere_gfiles(nf)
      jmin_beg_exc(nf) = lower_troposphere_jmin_beg(nf)
      jmin_end_exc(nf) = lower_troposphere_jmin_end(nf)
    end do
    atmos_region = 'LT'
  else if ( np == 3 ) then
    process_data = process_lower_stratosphere
    nfsum = lower_stratosphere_nfsum
    ncvsum = process_lower_stratosphere_ncvsum
    process_web = process_lower_stratosphere_web
    do nv = 1, ncvsum
      process_ver(nv) = process_lower_stratosphere_ver(nv)
    end do
    do nf = 1, nfsum
      ftypes(nf) = lower_stratosphere_ftypes(nf)
      fselect(nf) = lower_stratosphere_fselect(nf)
      gfiles(nf) = lower_stratosphere_gfiles(nf)
      jmin_beg_exc(nf) = lower_stratosphere_jmin_beg(nf)
      jmin_end_exc(nf) = lower_stratosphere_jmin_end(nf)
    end do
    atmos_region = 'LS'
  else if ( np == 4 ) then
    process_data = process_tropopause
    nfsum = tropopause_nfsum
    ncvsum = process_tropopause_ncvsum
    process_web = process_tropopause_web
    do nv = 1, ncvsum
      process_ver(nv) = process_tropopause_ver(nv)
    end do
    do nf = 1, nfsum
      ftypes(nf) = tropopause_ftypes(nf)
      fselect(nf) = tropopause_fselect(nf)
      gfiles(nf) = tropopause_gfiles(nf)
      jmin_beg_exc(nf) = tropopause_jmin_beg(nf)
      jmin_end_exc(nf) = tropopause_jmin_end(nf)
    end do
    atmos_region = 'TP'
  end if

  write (lulog,'(/a)') ' subroutine process_files'
  write (lulog,'(a)') ' TOP of region LOOP'
  write (lulog,'(a,i10)') ' np =', np
  write (lulog,'(a,a)') ' atmos_region = ', trim ( atmos_region )
  write (lulog,'(a,l10)') ' process_data=', process_data
  write (lulog,'(a,i10)') ' ncvsum=', ncvsum
  write (lulog,'(a,i10)') ' nfsum=', nfsum
  call flush ( lulog )

  if ( .not.process_data ) then
    cycle
  end if

  if ( np >= 2 ) then
    call calc_times ( np )
  end if

  VERSION: do nv = 1, ncvsum

    nopen = 0
    fopen = 0
    ncid_hold = 0

    write (lulog,'(/a)') ' subroutine process_files'
    write (lulog,'(a)') ' TOP of VERSION LOOP'
    write (lulog,'(a,i10)') ' nv =', nv
    write (lulog,'(a,a)') ' atmos_region = ', trim ( atmos_region )
    write (lulog,'(a,l10)') ' process_data=', process_data
    write (lulog,'(a,i10)') ' ncvsum=', ncvsum
    write (lulog,'(a,i10)') ' nfsum=', nfsum
    write (lulog,'(a,a)') ' process_web = ', trim ( process_web )
    call flush ( lulog )

    VARIABLE: do nf = 1, nfsum

      call do_wget_command ( &
        process_ver(nv), &
        ftypes(nf), &
        process_web )

      nend = lnblnk ( ftypes(nf) )

      write (lulog,'(/a)') ' subroutine process_files'
      write (lulog,'(a)') ' TOP of VARIABLE LOOP'
      write (lulog,'(a,i10)') ' nf =', nf
      write (lulog,'(a,a)') ' atmos_region = ', trim ( atmos_region )
      write (lulog,'(a,l10)') ' process_data=', process_data
      write (lulog,'(a,i10)') ' ncvsum=', ncvsum
      write (lulog,'(a,i10)') ' nfsum=', nfsum
      write (lulog,'(a,a)') ' ftypes(nf) = ', ftypes(nf)(1:nend)
      call flush ( lulog )

      if ( ftypes(nf)(1:nend) == 'amg' ) then
      else if ( ftypes(nf)(1:nend) == 'mam' ) then
      else if ( ftypes(nf)(1:nend) == 'amz' ) then
      else if ( ftypes(nf)(1:nend) == 'ncdc' ) then
      else if ( ftypes(nf)(1:nend) == 'acg' ) then
      else
        call write_error ( lulog, &
          ' subroutine process_files', &
          ' unexpected file type ???', &
          ftypes(nf) )
      end if

      npass_max = 2
      dflag_store_days = 0
      dflag_store_mons = 0
      dflag_store_mons_reg = 0

      nta_global = 2 * max ( ntmax_days, ntmax_mons )
      ntb_global = 0

      DPASS: do npass = 1, npass_max

        open (unit=lutemp,status='old',form='formatted',file='file.list',iostat=ios)

        if ( ios /= 0 ) then
          call write_error ( lulog, &
          ' subroutine process_files', &
          ' failed to open file.list ???', &
              '' )
        end if

        ngsum = 0

        write (lulog,'(/a)') ' subroutine process_files'
        write (lulog,'(a)') ' before call write_missing'
        write (lulog,'(a,i10)') ' nv =', nv
        write (lulog,'(a,i10)') ' nf =', nf
        write (lulog,'(a,i10)') ' npass=', npass
        call flush ( lulog )

        if ( npass == npass_max ) then
          call write_missing ( nf, atmos_region, fname, ftypes(nf), process_ver(nv), &
            nopen, ncid, fname_netcdf, &
            fselect, fopen, gfiles, ncid_hold )
        end if

        write (lulog,'(/a)') ' NOLA: subroutine process_files'
        write (lulog,'(a)') ' after call write_missing'
        write (lulog,'(a,i10)') ' nv =', nv
        write (lulog,'(a,i10)') ' nf =', nf
        write (lulog,'(a,i10)') ' npass=', npass
        call flush ( lulog )

        DREAD: do
          read (lutemp,'(a)',iostat=ios) fname
          if ( ios == 0 ) then

            nend = lnblnk ( fname )
            ncheck = index ( fname(1:nend), 'pen' )

            write (lulog,'(/a)') ' NOLA: subroutine process_files'
            write (lulog,'(a)') ' BEFORE call read_data_driver'
            write (lulog,'(a,a)') ' fname = ', trim ( fname )
            write (lulog,'(a,i10)') ' nv =', nv
            write (lulog,'(a,i10)') ' nf =', nf
            write (lulog,'(a,i10)') ' npass=', npass
            write (lulog,'(a,i10)') ' npass_max=', npass_max
            write (lulog,'(a,i10)') ' nta=', nta
            write (lulog,'(a,i10)') ' ntb=', ntb
            write (lulog,'(a,l10)') ' done_read = ', done_read
            write (lulog,'(a,i10)') ' ntsum=', ntsum
            write (lulog,'(a,i10)') ' ngsum=', ngsum
            write (lulog,'(a,i10)') ' ncheck=', ncheck
            call flush ( lulog )

!           Skip experimental pentad files

            if ( ncheck > 0 ) then
              cycle
            end if

            call read_data_driver ( nv, nf, nta, ntb, npass, npass_max, fname, &
              ntsum, ngsum, ftypes(nf), done_read, atmos_region )

            write (lulog,'(/a)') ' subroutine process_files'
            write (lulog,'(a)') ' AFTER call read_data_driver'
            write (lulog,'(a,a)') ' fname = ', trim ( fname )
            write (lulog,'(a,i10)') ' nv =', nv
            write (lulog,'(a,i10)') ' nf =', nf
            write (lulog,'(a,i10)') ' npass=', npass
            write (lulog,'(a,i10)') ' npass_max=', npass_max
            write (lulog,'(a,i10)') ' nta=', nta
            write (lulog,'(a,i10)') ' ntb=', ntb
            write (lulog,'(a,l10)') ' done_read = ', done_read
            write (lulog,'(a,i10)') ' ntsum=', ntsum
            write (lulog,'(a,i10)') ' ngsum=', ngsum
            call flush ( lulog )

            if ( nta < nta_global ) then
              nta_global = nta
            end if
            if ( ntb > ntb_global ) then
              ntb_global = ntb
            end if

            if ( ntb >= nta .and. npass == npass_max ) then
              call write_data_driver ( nf, atmos_region, fname, ntsum, ngsum, &
                nta, ntb, ftypes(nf), process_ver(nv), nopen, ncid, fname_netcdf, &
                fselect, fopen, gfiles, ncid_hold )
            end if

            if ( done_read ) then
              close (unit=lutemp,status='keep')
              exit DREAD
            end if

          else
            close (unit=lutemp,status='keep')
            exit DREAD
          end if
        end do DREAD

      end do DPASS

      call check_for_netcdf_file_close ( fselect, fopen, ncid_hold, nopen, &
        fname_netcdf, nfsum, 'IN-VAR-LOOP' )

    end do VARIABLE

    call check_for_netcdf_file_close ( fselect, fopen, ncid_hold, nopen, &
      fname_netcdf, nfsum, 'OUT-VAR-LOOP' )

  end do VERSION

end do REGION

end subroutine

end module
!
!***********************************************************************
!
module module_check_for_netcdf_file_close
implicit none
contains
 
subroutine check_for_netcdf_file_close ( fselect, fopen, ncid_hold, nopen, &
  fname_netcdf, nfsum, vmode )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error

implicit none
 
! Procedure dummy argument declarations

integer, intent(inout), dimension(ncvmax) :: fselect
integer, intent(inout), dimension(ncvmax) :: fopen
integer, intent(inout), dimension(ncvmax) :: ncid_hold
integer, intent(inout) :: nopen
character ( len = * ), intent(in) :: fname_netcdf
integer, intent(in) :: nfsum
character ( len = * ), intent(in) :: vmode

! Local Declarations

integer :: nend
integer :: lnblnk
integer :: istatus
integer :: ncid
character ( len = 256 ) :: emess
integer :: nf
integer :: nff
integer :: nsum
integer :: ngood
integer :: ncsum
integer :: nclose

ngood = 0
if ( trim(vmode) == 'IN-VAR-LOOP' ) ngood = 1
if ( trim(vmode) == 'OUT-VAR-LOOP' ) ngood = 1

if ( ngood == 0 ) then
  call write_error ( lulog, &
    ' subroutine check_for_netcdf_file_close', &
    ' unexpected vmode ???', &
    trim(vmode) )
end if

ncsum = 0

do nf = 1, nfsum

  write (lulog,'(/a)') ' subroutine check_for_netcdf_file_close'
  write (lulog,'(a,2i10)') ' nf, nfsum=', nf, nfsum
  write (lulog,'(a,i10)') ' fselect(nf)=', fselect(nf)

  if ( fselect(nf) <= 0 ) then
    cycle
  end if

  if ( fopen(fselect(nf)) <= 0 ) then
    cycle
  end if

  write (lulog,'(a,i10)') ' fopen(fselect(nf))=', fopen(fselect(nf))
  write (lulog,'(a,i10)') ' ncid_hold(fselect(nf))=', ncid_hold(fselect(nf))

! See how many variables are being output to the same file

  nsum = 0

  do nff = 1, nfsum
    if ( fselect(nff) <= 0 ) then
      cycle
    end if
    if ( fopen(fselect(nff)) <= 0 ) then
      cycle
    end if
    if ( fselect(nff) == fselect(nf) ) then
      nsum = nsum + 1
    end if
  end do

  nclose = 0

  if ( trim(vmode) == 'IN-VAR-LOOP' .and. nsum == 1 ) then
    nclose = 1
  else if ( trim(vmode) == 'IN-VAR-LOOP' .and. nsum > 1 ) then
    nclose = 0
  else if ( trim(vmode) == 'OUT-VAR-LOOP' .and. nsum >= 1 ) then
    nclose = 1
  end if

  write (lulog,'(/a)') ' subroutine check_for_netcdf_file_close'
  write (lulog,'(a,2i10)') ' nf, nfsum=', nf, nfsum
  write (lulog,'(a,i10)') ' nsum=', nsum
  write (lulog,'(a,i10)') ' nclose=', nclose

  if ( nclose == 1 ) then

    ncsum = ncsum + 1
    ncid = ncid_hold(fselect(nf))

    istatus = nf90_close ( ncid )
    if ( istatus /= 0 ) then
      emess = ' ' // trim (  nf90_strerror(istatus) )
      call write_error ( lulog, &
        ' subroutine check_for_netcdf_file_close', &
        ' problem closing netCDF file ???', &
      emess )
    end if

    write (lulog,'(/a)') ' subroutine check_for_netcdf_file_close'
    nend = lnblnk ( fname_netcdf )
    if ( nend <= 0 ) then
      nend = 1
    end if
    write (lulog,'(a,a)') ' closed NetCDF file ', fname_netcdf(1:nend)

!       if ( fname_netcdf(1:nend) == 'uah_msu_tmt_monreg_v05r06_197811_197912.nc' ) then
!       write (lulog,'(/a)') ' STOPPING IN subroutine process_files'
!       call flush ( lulog )
!       stop
!       end if

    do nff = 1, nfsum
      if ( nf == nff ) then
        cycle
      end if
      if ( fselect(nff) <= 0 ) then
        cycle
      end if
      if ( fopen(fselect(nff)) <= 0 ) then
        cycle
      end if
      if ( fselect(nff) == fselect(nf) ) then
        fopen(fselect(nff)) = 0
        ncid_hold(fselect(nff)) = 0
      end if
    end do

    fopen(fselect(nf)) = 0
    ncid_hold(fselect(nf)) = 0

  end if

end do

if ( ncsum > 0 ) then
  nopen = 0
  var_desc = cmiss
  netcdf_varid_hold = 0
  netcdf_nvar_use = 0
  netcdf_dimid_hold = 0
  netcdf_size_hold = 0
  netcdf_vname_hold = ''
  netcdf_desc_hold = ''
  netcdf_ndim_use = 0
  ndaily_esec_done = 0
  ndaily_cdate_done = 0
  nmonthly_esec_done = 0
  nmonthly_cdate_done = 0
end if

end subroutine

end module
!
!***********************************************************************
!
module module_write_data_driver
implicit none
contains
 
subroutine write_data_driver ( nf, atmos_region, fname, ntsum, ngsum, &
  nta, ntb, ftype, ver, nopen, ncid, fname_netcdf, &
  fselect, fopen, gfiles, ncid_hold )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_mon_gridded_anomaly
use module_write_mon_anomaly
use module_write_daily_zonal_anomaly
use module_write_error

use module_allocate_arrays_mon_gridded_anom
use module_allocate_arrays_mon_anom
use module_allocate_arrays_day_anom
use module_allocate_arrays_mon_regional_anom
use module_write_mon_regional_anomaly

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nf
character ( len = * ), intent(in) :: atmos_region
character ( len = * ), intent(in) :: fname
integer, intent(in) :: ntsum
integer, intent(in) :: ngsum
integer, intent(in) :: nta
integer, intent(in) :: ntb
character ( len = * ), intent(in) :: ftype
character ( len = * ), intent(in) :: ver
integer, intent(inout) :: nopen
integer, intent(inout) :: ncid
character ( len = * ), intent(inout) :: fname_netcdf
integer, intent(in), dimension(ncvmax) :: fselect
integer, intent(inout), dimension(ncvmax) :: fopen
character ( len = * ), intent(in), dimension(ncvmax) :: gfiles
integer, intent(inout), dimension(ncvmax) :: ncid_hold

! Local Declarations

integer :: nend
integer :: lnblnk

nend = lnblnk ( ftype )
if ( nend <= 0 ) then
  nend = 1
end if

write (lulog,'(/a)') ' ENTERING subroutine write_data_driver'
write (lulog,'(a,i10)') ' nf=', nf
write (lulog,'(a,a)') ' ftype= ', ftype(1:nend)
write (lulog,'(a,i10)') ' ngsum=', ngsum
write (lulog,'(a,i10)') ' ntsum=', ntsum
write (lulog,'(a,i10)') ' ntmax=', ntmax
write (lulog,'(a,i10)') ' ngmax=', ngmax
write (lulog,'(a,i10)') ' ndmax=', ndmax
write (lulog,'(a,2i10)') ' nta, ntb=', nta, ntb
write (lulog,'(a,i10)') ' fselect(nf)=', fselect(nf)
write (lulog,'(a,i10)') ' fopen(nf)=', fopen(nf)
call flush ( lulog )

if ( ftype(1:nend) == 'amg' .or. &
     ftype(1:nend) == 'acg' ) then

  call write_mon_gridded_anomaly ( nf, atmos_region, fname, ntsum, ngsum, &
    nta, ntb, ftype, ver, nopen, ncid, fname_netcdf, &
    fselect, fopen, gfiles, ncid_hold )

  if ( ngsum == ngmax ) then
    call allocate_arrays_mon_gridded_anom ( .true. )
  end if

else if ( ftype(1:nend) == 'mam' ) then

  call write_mon_anomaly ( nf, atmos_region, fname, ntsum, ngsum, &
    ftype, ver, nopen, ncid, fname_netcdf, &
    fselect, fopen, gfiles, ncid_hold )

  if ( ntsum == ntmax ) then
    call allocate_arrays_mon_anom ( .true. )
  end if

else if ( ftype(1:nend) == 'amz' ) then

  call write_daily_zonal_anomaly ( nf, atmos_region, fname, ntsum, ngsum, &
    ftype, ver, nopen, ncid, fname_netcdf, &
    fselect, fopen, gfiles, ncid_hold )

  if ( ntsum == ndmax ) then
    call allocate_arrays_day_anom ( .true. )
  end if

else if ( ftype(1:nend) == 'ncdc' ) then

  call write_mon_regional_anomaly ( nf, atmos_region, fname, ntsum, ngsum, &
    nta, ntb, ftype, ver, nopen, ncid, fname_netcdf, &
    fselect, fopen, gfiles, ncid_hold )

  if ( ngsum == ngmax ) then
    call allocate_arrays_mon_regional_anom ( .true. )
  end if

else

  call write_error ( lulog, &
    ' subroutine write_data_driver', &
    ' unexpected file type ???', &
    ftype )

end if

write (lulog,'(/a)') ' LEAVING subroutine write_data_driver'
call flush ( lulog )

end subroutine

end module
!
!***********************************************************************
!
module module_setup_define_initial
implicit none
contains
 
subroutine setup_define_initial ( nv, vname, ntype, units, &
  missing_value_real, missing_value_int, ftype, &
  axis, long_name, standard_name, bounds )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_get_free_logical_unit

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
character ( len = * ), intent(inout) :: vname
integer, intent(inout) :: ntype
character ( len = * ), intent(inout) :: units
real, intent(inout) :: missing_value_real
integer, intent(inout) :: missing_value_int
character ( len = * ), intent(in) :: ftype

character ( len = * ), intent(inout) :: axis
character ( len = * ), intent(inout) :: long_name
character ( len = * ), intent(inout) :: standard_name
character ( len = * ), intent(inout) :: bounds

! Local Declarations

character ( len = 256 ) :: emess
logical :: fexist
character ( len = 1000 ) :: nrec
integer :: lutemp
integer :: nend
integer :: lnblnk
integer :: nfind
integer :: nc

new_dim_desc = cmiss
new_dim_vname = cmiss
new_dim_ndsize = imiss
dim_desc = cmiss
dim_vname = cmiss
dim_ndsize = 0
comment_name = cmiss
comment_value = cmiss

missing_value_real = rmiss
missing_value_int = imiss

axis = cmiss
long_name = cmiss
standard_name = cmiss
bounds = cmiss

if ( trim(pmode) == 'OLD_NETCDF' .and. nv == 1 ) then

  new_dim_desc(1) = 'South_North'
  new_dim_vname(1) = 'jmax'
  new_dim_ndsize(1) = jmax

  dim_desc(1) = 'South_North'
  dim_vname(1) = 'jmax'
  dim_ndsize(1) = jmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'Y'
  vname = 'GLAT'
  ntype = nf90_float
  units = 'Decimal_Degrees'

  comment_name(2) = 'Description'
  comment_value(2) = 'Latitude,_South_Latitude_(-),_North_Latitude_(+)'

  comment_name(3) = 'Grid_Details'
  comment_value(3) = 'South_to_North,_2.5_degree_grid,_1.25_degree_offset'

else if ( trim(pmode) == 'OLD_NETCDF' .and. nv == 2 ) then

  new_dim_desc(1) = 'West_East'
  new_dim_vname(1) = 'imax'
  new_dim_ndsize(1) = imax

  dim_desc(1) = 'West_East'
  dim_vname(1) = 'imax'
  dim_ndsize(1) = imax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'X'
  vname = 'GLON'
  ntype = nf90_float
  units = 'Decimal_Degrees'

  comment_name(2) = 'Description'
  comment_value(2) = 'Longitude,_West_Longitude_(-),_East_Longitude_(+)'

  comment_name(3) = 'Grid_Details'
  comment_value(3) = 'West_to_East,_2.5_degree_grid,_1.25_degree_offset'

else if ( trim(pmode) == 'OLD_NETCDF' .and. nv == 3 ) then

  new_dim_desc(1) = 'South_North_Zonal'
  new_dim_vname(1) = 'jmaxz'
  new_dim_ndsize(1) = jmaxz

  dim_desc(1) = 'South_North_Zonal'
  dim_vname(1) = 'jmaxz'
  dim_ndsize(1) = jmaxz

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'Y'
  vname = 'GLATZ'
  ntype = nf90_float
  units = 'Decimal_Degrees'

  comment_name(2) = 'Description'
  comment_value(2) = 'Zonal_Latitude,_South Latitude_(-),_North_Latitude_(+)'

  comment_name(3) = 'Grid_Details'
  comment_value(3) = 'Zonal_South_to_North,_2.5_degree_grid,_1.25_degree_offset,_-67_to_+67'

else if ( ( trim(ftype) == 'amg' .or. trim(ftype) == 'acg' ) .and. &
            trim(pmode) == 'NEW_NETCDF' .and. nv == 1 ) then

  new_dim_desc(1) = 'lon'
  new_dim_vname(1) = 'lon'
  new_dim_ndsize(1) = imax

  dim_desc(1) = 'lon'
  dim_vname(1) = 'lon'
  dim_ndsize(1) = imax

  vname = 'lon'
  ntype = nf90_float
  units = 'degrees_east'
  axis = 'X'
  long_name = 'longitude'
  standard_name = 'longitude'
  bounds = 'lon_bounds'

else if ( ( trim(ftype) == 'amg' .or. trim(ftype) == 'acg' ) .and. &
            trim(pmode) == 'NEW_NETCDF' .and. nv == 2 ) then

  new_dim_desc(1) = 'lat'
  new_dim_vname(1) = 'lat'
  new_dim_ndsize(1) = jmax

  dim_desc(1) = 'lat'
  dim_vname(1) = 'lat'
  dim_ndsize(1) = jmax

  vname = 'lat'
  ntype = nf90_float
  units = 'degrees_north'
  axis = 'Y'
  long_name = 'latitude'
  standard_name = 'latitude'
  bounds = 'lat_bounds'

else if ( ( trim(ftype) == 'amg' .or. trim(ftype) == 'acg' ) .and. &
            trim(pmode) == 'NEW_NETCDF' .and. nv == 3 ) then

  new_dim_desc(1) = 'bnds'
  new_dim_vname(1) = 'bnds'
  new_dim_ndsize(1) = 2

  new_dim_desc(2) = 'lon'
  new_dim_vname(2) = 'lon'
  new_dim_ndsize(2) = imax

  dim_desc(1) = 'bnds'
  dim_vname(1) = 'bnds'
  dim_ndsize(1) = 2

  dim_desc(2) = 'lon'
  dim_vname(2) = 'lon'
  dim_ndsize(2) = imax

  vname = 'lon_bounds'
  ntype = nf90_float
  units = cmiss
  axis = cmiss
  long_name = cmiss
  standard_name = cmiss
  bounds = cmiss

else if ( ( trim(ftype) == 'amg' .or. trim(ftype) == 'acg' ) .and. &
            trim(pmode) == 'NEW_NETCDF' .and. nv == 4 ) then

  new_dim_desc(1) = 'bnds'
  new_dim_vname(1) = 'bnds'
  new_dim_ndsize(1) = 2

  new_dim_desc(2) = 'lat'
  new_dim_vname(2) = 'lat'
  new_dim_ndsize(2) = jmax

  dim_desc(1) = 'bnds'
  dim_vname(1) = 'bnds'
  dim_ndsize(1) = 2

  dim_desc(2) = 'lat'
  dim_vname(2) = 'lat'
  dim_ndsize(2) = jmax

  vname = 'lat_bounds'
  ntype = nf90_float
  units = cmiss
  axis = cmiss
  long_name = cmiss
  standard_name = cmiss
  bounds = cmiss

else if ( trim(ftype) == 'ncdc' .and. trim(pmode) == 'NEW_NETCDF' .and. nv == 1 ) then

  new_dim_desc(1) = 'region_strlen'
  new_dim_vname(1) = 'region_strlen'
  new_dim_ndsize(1) = len ( rname(1) )

  new_dim_desc(2) = 'region'
  new_dim_vname(2) = 'region'
  new_dim_ndsize(2) = nreg_max

  dim_desc(1) = 'region_strlen'
  dim_vname(1) = 'region_strlen'
  dim_ndsize(1) = len ( rname(1) )

  dim_desc(2) = 'region'
  dim_vname(2) = 'region'
  dim_ndsize(2) = nreg_max

  vname = 'region'
  ntype = nf90_char
  units = cmiss
  axis = cmiss
  long_name = 'region'
  standard_name = 'region'
  bounds = cmiss

  comment_name(1) = 'comment'
  comment_value(1) = ''

!kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk

  inquire ( file='region_comment.txt',exist=fexist )

  if ( region_only .and. trim(region_def_style) == 'VAR' .and. fexist ) then

    call get_free_logical_unit ( lutemp )
    open (unit=lutemp,file='region_comment.txt',status='old')
    read (lutemp,'(a)') nrec
    close (unit=lutemp,status='keep')

    nend = lnblnk ( nrec )
    if ( nend < 3 ) then
      nend = lnblnk ( nrec )
      call write_error ( lulog, &
      ' subroutine setup_define_initial', &
      ' record in region_comment.txt file too short ???', &
      nrec(1:nend) )
    end if

    nfind = 0
    do nc = 1, nend
      if ( nrec(nc:nc) == '#' ) then
        nfind = nc
        exit
      end if
    end do

    if ( nfind == 0 ) then
      call write_error ( lulog, &
      ' subroutine setup_define_initial', &
      ' no # character in record in region_comment.txt file ???', &
      nrec(1:nend) )
    end if

    comment_value(1) = nrec(nfind+1:nend)

  else if ( region_only .and. trim(region_def_style) == 'VAR' .and. .not.fexist ) then

    write (lulog,'(/a)') ' subroutine setup_define_initial'
    write (lulog,'(a,a)') ' region_def_style = ', trim ( region_def_style )
    write (lulog,'(a)') ' file region_comment.txt does not exist ???'
    stop

  end if

!kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk

else

  write (emess,'(a,5x,a,5x,i10)') ' ftype, nv=', trim(ftype), nv
  call write_error ( lulog, &
    ' subroutine setup_define_initial', &
    ' unexpected variable number ???', &
    emess )

end if

end subroutine

end module
!
!***********************************************************************
!
module module_reformat_version
implicit none
contains
 
subroutine reformat_version ( ver, verp, lulog )
 
!  Global data module

!  Modules for needed subroutines ...

use module_write_error

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: ver
character ( len = * ), intent(inout) :: verp
integer, intent(in) :: lulog

! Local Declarations

integer :: nend
integer :: nc
integer :: np
integer :: lnblnk
integer :: nver
integer :: nsub

nend = lnblnk ( ver )

if ( nend <= 0 ) then
  nend = 1
end if

! ver comes from the last field in the following string
! PROCESS_MIDDLE_TROPOSPHERE yes http://vortex.nsstc.uah.edu/data/msu/t2/ v5.6

if ( nend < 4 .or. ver(1:1) /= 'v' ) then
  call write_error ( lulog, &
    ' subroutine reformat_version', &
    ' unexpected version syntax ???', &
    ver )
end if

np = 0

do nc = 2, nend
  if ( ver(nc:nc) == '.' ) then
    np = nc
    exit
  end if
end do

if ( np == 0 ) then
  call write_error ( lulog, &
    ' subroutine reformat_version', &
    ' unexpected version syntax  - no period ???', &
    ver )
end if

read (ver(2:np-1),*) nver
read (ver(np+1:nend),*) nsub

if ( nver >= 100 .or. nsub >= 100 ) then
  call write_error ( lulog, &
    ' subroutine reformat_version', &
    ' unexpected version  - version/subversion too large for format ???', &
    ver )
end if

write (verp,'(a1,i2.2,a1,i2.2)') 'v', nver, 'r', nsub

end subroutine

end module
!
!***********************************************************************
!
module module_initial_open_netcdf_file
implicit none
contains
 
subroutine initial_open_netcdf_file ( nf, atmos_region, ver, ncid, fname, ftype, gfiles )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_setup_define_initial
use module_define_netcdf_var
use module_setup_nf90_put_var_init
use module_write_error
use module_reformat_version
use module_get_free_logical_unit
use module_calc_julian_minute
use module_inv_julian_minute

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nf
character ( len = * ), intent(in) :: atmos_region
character ( len = * ), intent(in) :: ver
integer, intent(inout) :: ncid
character ( len = * ), intent(inout) :: fname
character ( len = * ), intent(in) :: ftype
character ( len = * ), intent(in), dimension(ncvmax) :: gfiles

! Local Declarations

logical :: ldebug
integer :: nend
integer :: lnblnk
integer :: lutemp

integer :: ndim_id
integer :: nvar_id
character ( len = 256 ) :: emess
character ( len = 256 ) :: emess2
integer :: nstop
integer :: nv
integer :: n1
integer :: n2

integer :: istatus
character ( len = 256 ) :: vname
character ( len = 256 ) :: vnamep
character ( len = 256 ) :: verp
character ( len = 256 ) :: regionp
character ( len = 256 ) :: units

character ( len = 256 ) :: axisp
character ( len = 256 ) :: long_namep
character ( len = 256 ) :: standard_namep
character ( len = 256 ) :: boundsp

integer :: ntype
real :: missing_value_realp
integer :: missing_value_intp
character ( len = 1000 ) :: gname
character ( len = 1000 ) :: nrec
character ( len = 1000 ) :: command

integer :: nyr_file_beg
integer :: nmo_file_beg
integer :: ndy_file_beg
integer :: nhr_file_beg
integer :: nmin_file_beg
integer :: nsec_file_beg

integer :: nyr_file_end
integer :: nmo_file_end
integer :: ndy_file_end
integer :: nhr_file_end
integer :: nmin_file_end
integer :: nsec_file_end

integer :: nhrp
integer :: nminp
integer :: jmin
integer :: jminf_beg
integer :: jminf_end
integer :: nsum
integer :: nyr_pd
integer :: nmo_pd
integer :: nt
integer :: ios
integer :: nfind
integer :: nc
integer :: nvmax
integer :: ndflag
logical :: fexist
integer :: nr
integer :: nyrc
integer :: nmoc
integer :: ndyc
integer :: nhrc
integer :: nminc
integer :: nsecc

! Local Arrays

integer, parameter :: nddmax = 10
integer, dimension(nddmax) :: start
integer, dimension(nddmax) :: count
real, pointer, dimension(:) :: r_p1d
real, pointer, dimension(:,:) :: r_p2d
character ( len = rname_max ), pointer, dimension(:) :: c_p1d

ldebug = .false.
ldebug = .true.

if ( jmin_beg_exc(nf) > 0 .and. jmin_end_exc(nf) > 0 ) then

  call inv_julian_minute ( nyr_file_beg, nmo_file_beg, ndy_file_beg, nhr, nmin, &
    nyr_base, nmo_base, ndy_base, nhr_base, &
    lulog, jmin_beg_exc(nf) )

  nhr_file_beg = 0
  nmin_file_beg = 0
  nsec_file_beg = 0

  call inv_julian_minute ( nyr_file_end, nmo_file_end, ndy_file_end, nhr, nmin, &
    nyr_base, nmo_base, ndy_base, nhr_base, &
    lulog, jmin_end_exc(nf) )

  nhr_file_end = 23
  nmin_file_end = 59
  nsec_file_end = 59

else

  nyr_file_beg = nyr_beg
  nmo_file_beg = nmo_beg
  ndy_file_beg = ndy_beg
  nhr_file_beg = 0
  nmin_file_beg = 0
  nsec_file_beg = 0

  nyr_file_end = nyr_end
  nmo_file_end = nmo_end
  ndy_file_end = ndy_end
  nhr_file_end = 23
  nmin_file_end = 59
  nsec_file_end = 59

end if

write (fdate_beg_string,'(i4,a1,5(i2.2,a1))') &
  nyr_file_beg, '-', &
  nmo_file_beg, '-', &
  ndy_file_beg, 'T', &
  nhr_file_beg, ':', &
  nmin_file_beg, ':', &
  nsec_file_beg, 'Z'

write (fdate_end_string,'(i4,a1,5(i2.2,a1))') &
  nyr_file_end, '-', &
  nmo_file_end, '-', &
  ndy_file_end, 'T', &
  nhr_file_end, ':', &
  nmin_file_end, ':', &
  nsec_file_end, 'Z'

nhrp = 0
nminp = 0

call calc_julian_minute ( nyr_file_beg, nmo_file_beg, ndy_file_beg, nhrp, nminp, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jminf_beg )

call calc_julian_minute ( nyr_file_end, nmo_file_end, ndy_file_end, nhrp, nminp, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jminf_end )

nsum = 0

do nt = 1, ntmax_mons
  jmin = jmin_store_mons(nt)
  if ( jmin < jminf_beg ) then
    cycle
  end if
  if ( jmin > jminf_end ) then
    cycle
  end if
  nsum = nsum + 1
end do

nyr_pd = nsum / 12
nmo_pd = nsum - ( nyr_pd * 12 )

nend = lnblnk( ver )

if ( trim(pmode) == 'OLD_NETCDF' ) then

  write (fname,'(a,a,a,a,a,i4,a,i2.2,a,i4,a,i2.2,a)') &
    'uah_msu_', atmos_region, '_', ver(1:nend), '_', &
    nyr_file_beg, ':', nmo_file_beg, '_to_', &
    nyr_file_end, ':', nmo_file_end, '.nc'

else if ( trim(pmode) == 'NEW_NETCDF' ) then

  if ( trim(ftype) == 'amg' ) then
    vnamep = 'monamg'
  else if ( trim(ftype) == 'ncdc' ) then
    vnamep = 'monreg'
  else if ( trim(ftype) == 'acg' ) then
    vnamep = 'monacg'
  else
    call write_error ( lulog, &
      ' subroutine initial_open_netcdf_file', &
      ' unexpected ftype ???', &
      ftype )
  end if

  if ( trim(atmos_region) == 'MT' ) then
    regionp = 'tmt'
  else if ( trim(atmos_region) == 'LT' ) then
    regionp = 'tlt'
  else if ( trim(atmos_region) == 'LS' ) then
    regionp = 'tls'
  else if ( trim(atmos_region) == 'TP' ) then
    regionp = 'ttp'
  else
    call write_error ( lulog, &
      ' subroutine initial_open_netcdf_file', &
      ' unexpected atmos_region ???', &
      atmos_region )
  end if

  call reformat_version ( ver, verp, lulog )

  read (cdate_string,'(i4,1x,5(i2.2,1x))') nyrc, nmoc, ndyc, nhrc, nminc, nsecc

! write (fname,'(a,a,a,a,a,a,a,i4,i2.2,a, i4,i2.2,a)') &
!   'uah_mlt_', trim(regionp), '_', &
!   trim(vnamep), '_', &
!   trim(verp), '_', &
!   nyr_file_beg, nmo_file_beg, '_', &
!   nyr_file_end, nmo_file_end, '.nc'

  write (fname,'(a,a,a,a,a,a,a,2(a1,i4,2i2.2,a1),a1,i4,2i2.2,a3)') &
    'uah_mlt_', &
    trim(regionp), '_', &
    trim(verp), '_', &
    trim(vnamep), '_', &
    's', nyr_file_beg, nmo_file_beg, ndy_file_beg, "_", &
    'e', nyr_file_end, nmo_file_end, ndy_file_end, "_", &
    'c', nyrc, nmoc, ndyc, ".nc"

end if

write (lulog,'(/a)') ' subroutine initial_open_netcdf_file'
write (lulog,'(a,i10)') ' nyr_file_beg=', nyr_file_beg
write (lulog,'(a,i10)') ' nmo_file_beg=', nmo_file_beg
write (lulog,'(a,i10)') ' nyr_file_end=', nyr_file_end
write (lulog,'(a,i10)') ' nmo_file_end=', nmo_file_end
nend = lnblnk ( fname )
write (lulog,'(a,a)') ' fname= ', fname(1:nend)
call flush ( lulog )

istatus = nf90_create ( fname(1:nend), nf90_clobber, ncid )

if ( istatus /= 0 ) then
  emess = ' ' // trim (  nf90_strerror(istatus) )
  call write_error ( lulog, &
    ' subroutine initial_open_netcdf_file', &
    ' problem opening new netCDF file ???', &
    emess )
end if

define_mode = .true.

if ( trim(pmode) == 'OLD_NETCDF' ) then

  nend = lnblnk ( title )

  istatus = nf90_put_att ( ncid, nf90_global, 'title', title(1:nend) )

  if ( istatus /= 0 ) then
    emess = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine initial_open_netcdf_file', &
      ' problem writing title to netCDF file ???', &
      emess )
  end if

  nend = lnblnk ( history )

  istatus = nf90_put_att ( ncid, nf90_global, 'history', history(1:nend) )

  if ( istatus /= 0 ) then
    emess = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine initial_open_netcdf_file', &
      ' problem writing history to netCDF file ???', &
      emess )
  end if

  nend = lnblnk ( contact )

  istatus = nf90_put_att ( ncid, nf90_global, 'contact', contact(1:nend) )

  if ( istatus /= 0 ) then
    emess = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine initial_open_netcdf_file', &
      ' problem writing contact to netCDF file ???', &
      emess )
  end if

else if ( trim(pmode) == 'NEW_NETCDF' ) then

  if ( trim(ftype) == 'ncdc' ) then

    call get_free_logical_unit ( lutemp )
    inquire ( file='notes.txt',exist=fexist )
    if ( fexist ) then
      open (unit=lutemp,file='notes.txt',status='old')
      close (unit=lutemp,status='delete')
    end if
    open (unit=lutemp,file='notes.txt',status='new')
    do nr = 1, note_hold_sum
      nend = lnblnk ( note_hold(nr) )
      write (lutemp,'(a)') note_hold(nr)(1:nend)
    end do
    close (unit=lutemp,status='keep')

    inquire ( file='region_def.txt',exist=fexist )
    if ( fexist ) then
      open (unit=lutemp,file='region_def.txt',status='old')
      close (unit=lutemp,status='delete')
    end if
    open (unit=lutemp,file='region_def.txt',status='new')
    do nr = 1, region_def_hold_sum
      nend = lnblnk ( region_def_hold(nr) )
      write (lutemp,'(a)') region_def_hold(nr)(1:nend)
      write (lulog,'(a,i10,5x,a)') ' nr, region_def_hold(nr)=', nr, trim ( region_def_hold(nr) )
    end do
    close (unit=lutemp,status='keep')

  end if

  gname = gfiles(nf)
  nend = lnblnk ( gname )

  write (command,'(a,1x,a,1x,a,1x,a,1x,a,1x,2i10,2x,a,2x,a,2x,a,a)') './gfile.pl', gname(1:nend), &
    cdate_string, fdate_beg_string, fdate_end_string, &
    nyr_pd, nmo_pd, &
    trim( region_def_style ), trim ( missing_data_style ), &
    trim( fname ), ' > gfile.LOG'

  nend = lnblnk ( command )

  call system ( command(1:nend) )

  call get_free_logical_unit ( lutemp )

  open (unit=lutemp,status='old',form='formatted',file='new.txt',iostat=ios)

  if ( ios /= 0 ) then
    nend = lnblnk ( gname )
    call write_error ( lulog, &
    ' subroutine initial_open_netcdf_file', &
    ' failed to open new.txt file ???', &
    gname(1:nend) )
  end if

  do
    read (lutemp,'(a)',iostat=ios) nrec
    if ( ios == 0 ) then
      nend = lnblnk ( nrec )
      if ( nend < 3 ) then
        nend = lnblnk ( gname )
        call write_error ( lulog, &
        ' subroutine initial_open_netcdf_file', &
        ' record in new.txt file too short ???', &
        nrec(1:nend) )
      end if
      nfind = 0
      do nc = 1, nend
        if ( nrec(nc:nc) == '#' ) then
          nfind = nc
          exit
        end if
      end do
      if ( nfind == 0 ) then
        call write_error ( lulog, &
        ' subroutine initial_open_netcdf_file', &
        ' no # character in record in new.txt file ???', &
        nrec(1:nend) )
      end if
      istatus = nf90_put_att ( ncid, nf90_global, nrec(1:nfind-1), nrec(nfind+1:nend) )
      if ( istatus /= 0 ) then
        write (lulog,'(/a)') ' subroutine initial_open_netcdf_file'
        write (lulog,'(a)') ' Problem writing new.txt record to NetCDF file ???'
        write (lulog,'(a,i10)') ' nfind = ', nfind
        write (lulog,'(a,a)') ' nrec(1:nfind-1) = ', nrec(1:nfind-1)
        call flush ( lulog )
        emess = ' ' // trim (  nf90_strerror(istatus) )
        call write_error ( lulog, &
          ' subroutine initial_open_netcdf_file', &
          ' problem writing new.txt record to netCDF file ???', &
          emess )
      end if
    else
      close (unit=lutemp,status='delete')
      exit
    end if
  end do

end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

if ( trim(pmode) == 'OLD_NETCDF' ) then
  nvmax = 3
else if ( trim(pmode) == 'NEW_NETCDF' ) then
  if ( trim(ftype) == 'acg' .or. trim(ftype) == 'amg' ) then
    nvmax = 4
  else if ( trim(ftype) == 'ncdc' ) then
    nvmax = 1
  else
    call write_error ( lulog, &
    ' subroutine initial_open_netcdf_file', &
    '  unexpected ftype value ???', &
    trim(ftype) )
  end if
end if

! NOLA

do nv = 1, nvmax

  write (lulog,'(/a)') ' subroutine initial_open_netcdf_file'
  write (lulog,'(a)') ' BEFORE call setup_define_initial'
  write (lulog,'(a,i10)') ' nv=', nv
  write (lulog,'(a,i10)') ' nvmax=', nvmax

  call setup_define_initial ( nv, vname, ntype, units, &
    missing_value_realp, missing_value_intp, ftype, &
    axisp, long_namep, standard_namep, boundsp )

  write (lulog,'(/a)') ' subroutine initial_open_netcdf_file'
  write (lulog,'(a)') ' AFTER call setup_define_initial'
  write (lulog,'(a)') ' BEFORE call define_netcdf_var'
  write (lulog,'(a,a)') ' vname = ', trim ( vname )
  call flush ( lulog )

  call define_netcdf_var ( ncid, nvar_id, &
    new_dim_desc, new_dim_vname, new_dim_ndsize, new_max, &
    dim_desc, dim_vname, dim_ndsize, &
    vname, ntype, units, &
    missing_value_real = missing_value_realp, &
    missing_value_int = missing_value_intp, &
    comment_namep = comment_name, &
    comment_valuep = comment_value, &
    axis = axisp, &
    long_name = long_namep, &
    standard_name = standard_namep, &
    bounds = boundsp )

  write (lulog,'(/a)') ' subroutine initial_open_netcdf_file'
  write (lulog,'(a)') ' AFTER call define_netcdf_var'
  write (lulog,'(a,a)') ' vname = ', trim ( vname )
  call flush ( lulog )

end do

istatus = nf90_enddef ( ncid  )

if ( istatus /= 0 ) then
  emess = ' ' // trim (  nf90_strerror(istatus) )
  call write_error ( lulog, &
    ' subroutine initial_open_netcdf_file', &
    ' problem setting mode to end define ???', &
    emess )
end if

define_mode = .false.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

do nv = 1, nvmax

  call setup_nf90_put_var_init ( nv, nvar_id, vname, ftype, ndflag, &
    r_p1d, r_p2d, c_p1d, &
    start, count, nddmax, glat, glon, glatz, glat_bnds, glon_bnds, rname, &
    n1, n2 )

  if ( ndflag == 1 ) then
    istatus = nf90_put_var ( ncid, nvar_id, r_p1d, start(n1:n2:1), count(n1:n2:1) )
  else if ( ndflag == 2 ) then
    istatus = nf90_put_var ( ncid, nvar_id, r_p2d, start(n1:n2:1), count(n1:n2:1) )
  else if ( ndflag == 3 ) then
    istatus = nf90_put_var ( ncid, nvar_id, c_p1d, start(n1:n2:1), count(n1:n2:1) )
  end if

  nend = lnblnk ( vname )

  if ( istatus /= 0 ) then
    write (emess,'(a,a,a)') ' problem writing ', vname(1:nend), &
      ' variable to netCDF file ???'
    emess2 = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine initial_open_netcdf_file', &
      emess, &
      emess2 )
  end if

end do

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

write (lulog,'(/a)') ' LEAVING subroutine initial_open_netcdf_file'
call flush ( lulog )

end subroutine

end module
!
!***********************************************************************
!
module module_check_date_status
implicit none
contains
 
subroutine check_date_status ( vname, nskip )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: vname
integer, intent(inout) :: nskip

! Local Declarations

integer :: nend
integer :: lnblnk
integer :: nfind1
integer :: nfind2
character ( len = 256 ) :: emess

nskip = 0

nend = lnblnk ( vname )
if ( nend <= 0 ) then
  nend = 1
end if

nfind1 = index ( vname(1:nend), 'ESEC' )
nfind2 = index ( vname(1:nend), 'CDATE' )

if ( nfind1 > 0 ) then

  nfind1 = index ( vname(1:nend), 'DESEC' )
  nfind2 = index ( vname(1:nend), 'MESEC' )

  if ( nfind1 > 0 ) then

    if ( ndaily_esec_done == 0 ) then
      ndaily_esec_done = 1
    else
      nskip = 1
    end if

  else if ( nfind2 > 0 ) then

    if ( nmonthly_esec_done == 0 ) then
      nmonthly_esec_done = 1
    else
      nskip = 1
    end if

  else

    write (emess,'(a,a)') ' vname = ', vname(1:nend)
    call write_error ( lulog, &
      ' subroutine check_date_status', &
      ' unexpected ESEC variable ???', &
      emess )

  end if

else if ( nfind2 > 0 ) then

  nfind1 = index ( vname(1:nend), 'DCDATE' )
  nfind2 = index ( vname(1:nend), 'MCDATE' )

  if ( nfind1 > 0 ) then

    if ( ndaily_cdate_done == 0 ) then
      ndaily_cdate_done = 1
    else
      nskip = 1
    end if

  else if ( nfind2 > 0 ) then

    if ( nmonthly_cdate_done == 0 ) then
      nmonthly_cdate_done = 1
    else
      nskip = 1
    end if

  else

    write (emess,'(a,a)') ' vname = ', vname(1:nend)
    call write_error ( lulog, &
      ' subroutine check_date_status', &
      ' unexpected CDATE variable ???', &
      emess )

  end if

end if

end subroutine

end module
!
!***********************************************************************
!
module module_setup_nf90_put_var_init
implicit none
contains

subroutine setup_nf90_put_var_init ( nv, nvar_id, vname, ftype, ndflag, &
  r_p1d, r_p2d, c_p1d, &
  start, count, nddmax, glatp, glonp, glatzp, glat_bndsp, glon_bndsp, rnamep, &
  n1, n2 )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_get_var_id

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
integer, intent(inout) :: nvar_id
character ( len = * ), intent(inout) :: vname
character ( len = * ), intent(in) :: ftype
integer, intent(inout) :: ndflag

real, pointer, dimension(:) :: r_p1d
real, pointer, dimension(:,:) :: r_p2d
character ( len = * ), pointer, dimension(:) :: c_p1d

integer, intent(inout), dimension(nddmax) :: start
integer, intent(inout), dimension(nddmax) :: count
integer, intent(in) :: nddmax

real, intent(in), target, dimension(jmax) :: glatp
real, intent(in), target, dimension(imax) :: glonp
real, intent(in), target, dimension(jmaxz) :: glatzp
real, intent(in), target, dimension(2,jmax) :: glat_bndsp
real, intent(in), target, dimension(2,imax) :: glon_bndsp
character ( len = rname_max ), intent(in), target, dimension(nreg_max) :: rnamep

integer, intent(inout) :: n1
integer, intent(inout) :: n2

! Local Declarations

integer :: lnblnk
integer :: nend
integer :: nstop
integer :: nsum
integer :: n
character ( len = 256 ) :: emess

nstop = 1
start = 0
count = 0

nullify ( r_p1d )
nullify ( r_p2d )
nullify ( c_p1d )

if ( trim(pmode) == 'OLD_NETCDF' .and. nv == 1 ) then

  vname = 'GLAT'
  nend = lnblnk( vname )
  start(1) = 1
  count(1) = jmax
  r_p1d => glatp
  ndflag = 1

else if ( trim(pmode) == 'OLD_NETCDF' .and. nv == 2 ) then

  vname = 'GLON'
  nend = lnblnk( vname )
  start(1) = 1
  count(1) = imax
  r_p1d => glonp
  ndflag = 1

else if ( trim(pmode) == 'OLD_NETCDF' .and. nv == 3 ) then

  vname = 'GLATZ'
  nend = lnblnk( vname )
  start(1) = 1
  count(1) = jmaxz
  r_p1d => glatzp
  ndflag = 1

else if ( ( trim(ftype) == 'amg' .or. trim(ftype) == 'acg' ) .and. &
  trim(pmode) == 'NEW_NETCDF' .and. nv == 1 ) then

  vname = 'lon'
  nend = lnblnk( vname )
  start(1) = 1
  count(1) = imax
  r_p1d => glonp
  ndflag = 1

else if ( ( trim(ftype) == 'amg' .or. trim(ftype) == 'acg' ) .and. &
  trim(pmode) == 'NEW_NETCDF' .and. nv == 2 ) then

  vname = 'lat'
  nend = lnblnk( vname )
  start(1) = 1
  count(1) = jmax
  r_p1d => glatp
  ndflag = 1

else if ( ( trim(ftype) == 'amg' .or. trim(ftype) == 'acg' ) .and. &
  trim(pmode) == 'NEW_NETCDF' .and. nv == 3 ) then

  vname = 'lon_bounds'
  nend = lnblnk( vname )

  start(1) = 1
  count(1) = 2
  start(2) = 1
  count(2) = imax

  r_p2d => glon_bndsp
  ndflag = 2

else if ( ( trim(ftype) == 'amg' .or. trim(ftype) == 'acg' ) .and. &
  trim(pmode) == 'NEW_NETCDF' .and. nv == 4 ) then

  vname = 'lat_bounds'
  nend = lnblnk( vname )

  start(1) = 1
  count(1) = 2
  start(2) = 1
  count(2) = jmax

  r_p2d => glat_bndsp
  ndflag = 2

else if ( trim(ftype) == 'ncdc' .and. trim(pmode) == 'NEW_NETCDF' .and. nv == 1 ) then

  vname = 'region'
  nend = lnblnk( vname )

  start(1) = 1
  count(1) = len ( rnamep(1) )
  start(2) = 1
  count(2) = nreg_max

  c_p1d => rnamep(1:nreg_max)
  ndflag = 3

else if ( trim(ftype) == 'ncdc' .and. trim(pmode) == 'NEW_NETCDF' .and. nv == 2 ) then

  vname = 'region'
  nend = lnblnk( vname )

  start(1) = 1
  count(1) = len ( rnamep(1) )
  start(2) = 1
  count(2) = nreg_max

  c_p1d => rnamep(1:nreg_max)
  ndflag = 4

else

  write (lulog,'(/a)') ' subroutine setup_nf90_put_var_init'
  write (lulog,'(a,a)') ' ftype = ', trim ( ftype )
  write (lulog,'(a,a)') ' pmode = ', trim ( pmode )
  write (lulog,'(a,i10/)') ' nv=',nv

  call write_error ( lulog, &
    ' subroutine setup_nf90_put_var_init', &
    ' unexpected variable number ???', &
    emess )

end if

nstop = 1
nend = lnblnk ( vname )

call get_var_id ( nvar_id, vname(1:nend), nstop )

n1 = 1

do n = nddmax, 1, -1
  if ( start(n) > 0 ) then
    n2 = n
    exit
  end if
end do

end subroutine

end module
!
!***********************************************************************
!
module module_write_var_attributes
implicit none
contains
 
subroutine write_var_attributes ( ncid, nvar_id, desc, units, &
  missing_value_real, missing_value_int, &
  scale_factor_real, scale_factor_int, &
  comment_name, comment_value, &
  axis, long_name, standard_name, bounds, &
  fillvalue, valid_min, valid_max, coordinates, cell_methods, &
  climatology )

!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: ncid
integer, intent(in) :: nvar_id
character ( len = * ), optional, intent(in) :: desc
character ( len = * ), optional, intent(in) :: units
real, optional, intent(in) :: missing_value_real
integer, optional, intent(in) :: missing_value_int
real, optional, intent(in) :: scale_factor_real
integer, optional, intent(in) :: scale_factor_int
character ( len = * ), optional, intent(in) :: comment_name
character ( len = * ), optional, intent(in) :: comment_value
character ( len = * ), intent(in), optional :: axis
character ( len = * ), intent(in), optional :: long_name
character ( len = * ), intent(in), optional :: standard_name
character ( len = * ), intent(in), optional :: bounds
real, intent(in), optional :: fillvalue
real, intent(in), optional :: valid_min
real, intent(in), optional :: valid_max
character ( len = * ), intent(in), optional :: coordinates
character ( len = * ), intent(in), optional :: cell_methods
character ( len = * ), intent(in), optional :: climatology

! Local Declarations

integer :: istatus
integer :: nend
integer :: nend1
integer :: nend2
integer :: lnblnk
character ( len = 256 ) :: emess

if ( present ( desc ) ) then
  nend = lnblnk ( desc )
  if ( nend <= 0 ) then
    nend = 1
  end if
  istatus = nf90_put_att ( ncid, nvar_id, 'description', desc(1:nend) )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' description' )
  end if
end if

if ( present ( units ) ) then
  nend = lnblnk ( units )
  if ( nend <= 0 ) then
    nend = 1
  end if
  if ( units(1:nend) /= trim(cmiss) ) then
    istatus = nf90_put_att ( ncid, nvar_id, 'units', units(1:nend) )
    if ( istatus /= 0 ) then
      call write_error ( lulog, &
        ' subroutine write_var_attributes', &
        ' problem writing var attribute to netCDF file ???', &
        ' units' )
    end if
  end if
end if

if ( present ( axis ) ) then
  nend = lnblnk ( axis )
  if ( nend <= 0 ) then
    nend = 1
  end if
  istatus = nf90_put_att ( ncid, nvar_id, 'axis', axis(1:nend) )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' axis' )
  end if
end if

if ( present ( long_name ) ) then
  nend = lnblnk ( long_name )
  if ( nend <= 0 ) then
    nend = 1
  end if
  istatus = nf90_put_att ( ncid, nvar_id, 'long_name', long_name(1:nend) )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' long_name' )
  end if
end if

if ( present ( standard_name ) ) then
  nend = lnblnk ( standard_name )
  if ( nend <= 0 ) then
    nend = 1
  end if
  istatus = nf90_put_att ( ncid, nvar_id, 'standard_name', standard_name(1:nend) )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' standard_name' )
  end if
end if

if ( present ( bounds ) ) then
  nend = lnblnk ( bounds )
  if ( nend <= 0 ) then
    nend = 1
  end if
  istatus = nf90_put_att ( ncid, nvar_id, 'bounds', bounds(1:nend) )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' bounds' )
  end if
end if

if ( present ( fillvalue ) ) then
  istatus = nf90_put_att ( ncid, nvar_id, '_FillValue', fillvalue )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' fillvalue' )
  end if
end if

if ( present ( valid_min ) ) then
  istatus = nf90_put_att ( ncid, nvar_id, 'valid_min', valid_min )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' valid_min' )
  end if
end if

if ( present ( valid_max ) ) then
  istatus = nf90_put_att ( ncid, nvar_id, 'valid_max', valid_max )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' valid_max' )
  end if
end if

if ( present ( coordinates ) ) then
  nend = lnblnk ( coordinates )
  if ( nend <= 0 ) then
    nend = 1
  end if
  istatus = nf90_put_att ( ncid, nvar_id, 'coordinates', coordinates(1:nend) )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' coordinates' )
  end if
end if

if ( present ( cell_methods ) ) then
  nend = lnblnk ( cell_methods )
  if ( nend <= 0 ) then
    nend = 1
  end if
  istatus = nf90_put_att ( ncid, nvar_id, 'cell_methods', cell_methods(1:nend) )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' cell_methods' )
  end if
end if

if ( present ( climatology ) ) then
  nend = lnblnk ( climatology )
  if ( nend <= 0 ) then
    nend = 1
  end if
  istatus = nf90_put_att ( ncid, nvar_id, 'climatology', climatology(1:nend) )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' climatology' )
  end if
end if

if ( present ( missing_value_real ) ) then
  istatus = nf90_put_att ( ncid, nvar_id, 'missing_value', missing_value_real )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' missing_value_real' )
  end if
end if

if ( present ( missing_value_int ) ) then
  istatus = nf90_put_att ( ncid, nvar_id, 'missing_value', missing_value_int )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' missing_value_int' )
  end if
end if

if ( present ( scale_factor_real ) ) then
  istatus = nf90_put_att ( ncid, nvar_id, 'scale_factor', scale_factor_real )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' scale_factor_real' )
  end if
end if

if ( present ( scale_factor_int ) ) then
  istatus = nf90_put_att ( ncid, nvar_id, 'scale_factor', scale_factor_int )
  if ( istatus /= 0 ) then
    call write_error ( lulog, &
      ' subroutine write_var_attributes', &
      ' problem writing var attribute to netCDF file ???', &
      ' scale_factor_int' )
  end if
end if

if ( present ( comment_name ) .and.  present ( comment_value ) ) then
  nend1 = lnblnk ( comment_name )
  nend2 = lnblnk ( comment_value )
  if ( nend1 > 0 .and. nend2 > 0 ) then
    istatus = nf90_put_att ( ncid, nvar_id, &
      comment_name(1:nend1), comment_value(1:nend2) )
    if ( istatus /= 0 ) then
      emess = ''
      emess = ' ' // comment_name(1:nend1) // ' ' // comment_value(1:nend2)
      call write_error ( lulog, &
        ' subroutine write_var_attributes', &
        ' problem writing comment attribute to netCDF file ???', &
        emess )
    end if
  end if
end if

end subroutine

end module
!
!***********************************************************************
!
module module_get_var_id
implicit none
contains
 
subroutine get_var_id ( nvar_id, vname, nstop )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error

implicit none
 
!  Procedure dummy argument declarations

integer, intent(inout) :: nvar_id
character ( len = * ), intent(in) :: vname
integer, intent(in) :: nstop

! Local Declarations

integer :: nend
integer :: nend1
integer :: nend2
integer :: lnblnk
integer :: nv
integer :: nfind
character ( len = 256 ) :: emess

nend = lnblnk ( vname )
nend1 = lnblnk ( cmiss )
nfind = 0
nvar_id = 0

do nv = 1, new_max
  if ( netcdf_varid_hold(nv) <= 0 ) then
    exit
  end if
  nend2 = lnblnk ( var_desc(nv) )
  if ( nend2 <= 0 ) then
    exit
  end if
  if ( var_desc(nv)(1:nend2) == cmiss(1:nend1) ) then
    exit
  end if
  if ( vname(1:nend) == var_desc(nv)(1:nend2) ) then
    nfind = 1
    nvar_id = netcdf_varid_hold(nv)
    exit
  end if
end do

if ( nstop == 1 .and. nfind == 0 ) then
  emess = ''
  write (emess,'(a,a)') ' vname = ', vname(1:nend)
  emess = trim ( emess )
  call write_error ( lulog, &
    ' subroutine get_var_id', &
    ' could not find variable ID in list ???', &
    emess )
end if

end subroutine

end module
!
!***********************************************************************
!
module module_get_netcdf_dim
implicit none
contains
 
subroutine get_netcdf_dim ( ndim_id, vname, desc, ndsize, nstop )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error

implicit none
 
!  Procedure dummy argument declarations

integer, intent(inout) :: ndim_id
character ( len = * ), intent(in) :: vname
character ( len = * ), intent(in) :: desc
integer, intent(in) :: ndsize
integer, intent(in) :: nstop

! Local Declarations

integer :: nd
integer :: nfind
integer :: nsum
character ( len = 256 ) :: emess
integer :: nend
integer :: nend1
integer :: nend2
integer :: lnblnk

nfind = 0
ndim_id = 0

nend1 = lnblnk ( vname )
if ( nend1 <= 0 ) then
  nend1 = 1
end if

nend2 = lnblnk ( desc )
if ( nend2 <= 0 ) then
  nend2 = 1
end if

do nd = 1, netcdf_ndim_use
  nsum = 0
  if ( netcdf_size_hold(nd) == ndsize ) then
    nsum = nsum + 1
  end if
  nend = lnblnk ( netcdf_vname_hold(nd) )
  if ( nend <= 0 ) then
    nend = 1
  end if
  if ( netcdf_vname_hold(nd)(1:nend) == vname(1:nend1) ) then
    nsum = nsum + 1
  end if
  nend = lnblnk ( netcdf_desc_hold(nd) )
  if ( nend <= 0 ) then
    nend = 1
  end if
  if ( netcdf_desc_hold(nd)(1:nend) == desc(1:nend2) ) then
    nsum = nsum + 1
  end if

  write (lulog,'(/a)') ' subroutine get_netcdf_dim'
  write (lulog,'(a,i10)') ' netcdf_ndim_use=', netcdf_ndim_use
  write (lulog,'(a,i10)') ' nd=', nd
  write (lulog,'(a,i10)') ' input arg: ndsize=', ndsize
  write (lulog,'(a,a)') ' input arg: vname = ', trim ( vname )
  write (lulog,'(a,a)') ' input arg: desc = ', trim ( desc )
  write (lulog,'(a,i10)') ' netcdf_size_hold(nd)=', netcdf_size_hold(nd)
  write (lulog,'(a,a)') ' netcdf_vname_hold(nd)=', trim ( netcdf_vname_hold(nd) )
  write (lulog,'(a,a)') ' netcdf_desc_hold(nd)=', trim ( netcdf_desc_hold(nd) )
  write (lulog,'(a,i10)') ' nstop=', nstop
  write (lulog,'(a,i10)') ' nsum=', nsum
  call flush ( lulog )

  if ( nsum == 3 ) then
    nfind = nd
    ndim_id = netcdf_dimid_hold(nd)
    exit
  end if
end do

if ( nstop == 1 .and. nfind == 0 ) then
  emess = ''
  write (emess,'(a,a,a)') ' desc= ', desc(1:nend2)
  emess = trim ( emess )
  call write_error ( lulog, &
    ' subroutine get_netcdf_dim', &
    ' could not find dimension ID in list ???', &
    emess )
end if

end subroutine

end module
!
!***********************************************************************
!
module module_define_netcdf_dim
implicit none
contains
 
subroutine define_netcdf_dim ( ncid, ndsize, desc, vname )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_get_netcdf_dim

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: ncid
integer, intent(in) :: ndsize
character ( len = * ), intent(in) :: desc
character ( len = * ), intent(in) :: vname

! Local Declarations

character ( len = 256 ) :: emess
character ( len = 256 ) :: emess2
integer :: istatus
integer :: nend
integer :: nend2
integer :: lnblnk
integer :: ndim_id
integer :: nstop

nend = lnblnk ( vname )
if ( nend <= 0 ) then
  nend = 1
end if

nend2 = lnblnk ( desc )
if ( nend2 <= 0 ) then
  nend2 = 1
end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! See if dimension already exists

nstop = 0
call get_netcdf_dim ( ndim_id, vname, desc, ndsize, nstop )
 
write (lulog,'(/a)') ' subroutine define_netcdf_dim'
write (lulog,'(a)') ' after call get_netcdf_dim'
write (lulog,'(a,i10)') ' ndim_id=', ndim_id
call flush ( lulog )

if ( ndim_id > 0 ) then
  return
end if
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

istatus = nf90_def_dim ( ncid, desc(1:nend2), ndsize, ndim_id )
 
write (lulog,'(/a)') ' subroutine define_netcdf_dim'
write (lulog,'(a)') ' after call nf90_def_dim'
write (lulog,'(a,i10)') ' ndim_id=', ndim_id
call flush ( lulog )

if ( istatus /= 0 ) then
  emess = ' ' // trim (  nf90_strerror(istatus) )
  emess2 = ''
  write (emess2,'(a,a,a)') ' problem writing ', vname(1:nend), ' dimension to netCDF file ???'
  emess2 = trim ( emess2 )
  call write_error ( lulog, &
    ' subroutine define_netcdf_dim', &
    emess2, &
    emess )
end if

netcdf_ndim_use = netcdf_ndim_use + 1

if ( netcdf_ndim_use > netcdf_max_dim ) then
  emess2 = ''
  write (emess2,'(a,i10,5x,a,i10)') ' netcdf_max_dim= ', netcdf_max_dim, &
    ' netcdf_ndim_use= ', netcdf_ndim_use
  emess2 = trim ( emess2 )
  call write_error ( lulog, &
    ' subroutine define_netcdf_dim', &
    ' netcdf_ndim_use > netcdf_max_dim ???', &
    emess2 )
end if

netcdf_dimid_hold(netcdf_ndim_use) = ndim_id
netcdf_size_hold(netcdf_ndim_use) = ndsize
netcdf_vname_hold(netcdf_ndim_use) = vname
netcdf_desc_hold(netcdf_ndim_use) = desc

end subroutine

end module
!
!***********************************************************************
!
module module_define_netcdf_var
implicit none
contains
 
subroutine define_netcdf_var ( ncid, nvar_id, &
  new_dim_desc, new_dim_vname, new_dim_ndsize, new_max, &
  dim_desc, dim_vname, dim_ndsize, &
  vname_new, ntype, units, &
  missing_value_real, missing_value_int, &
  scale_factor_real, scale_factor_int, &
  comment_namep, comment_valuep, &
  axis, long_name, standard_name, bounds, &
  fillvalue, valid_min, valid_max, coordinates, cell_methods, &
  climatology )

!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_define_netcdf_dim
use module_get_netcdf_dim
use module_write_var_attributes
use module_get_var_id

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: ncid
integer, intent(inout) :: nvar_id

character ( len = * ), intent(in), dimension(new_max) :: new_dim_desc
character ( len = * ), intent(in), dimension(new_max) :: new_dim_vname

integer, intent(in), dimension(new_max) :: new_dim_ndsize
integer, intent(in) :: new_max

character ( len = * ), intent(in), dimension(new_max) :: dim_desc
character ( len = * ), intent(in), dimension(new_max) :: dim_vname
integer, intent(in), dimension(new_max) :: dim_ndsize

character ( len = * ), intent(in) :: vname_new
integer, intent(in) :: ntype
character ( len = * ), intent(in) :: units

real, intent(in), optional :: missing_value_real
integer, intent(in), optional :: missing_value_int
real, intent(in), optional :: scale_factor_real
integer, intent(in), optional :: scale_factor_int
character ( len = * ), intent(in), optional, dimension(new_max) :: comment_namep
character ( len = * ), intent(in), optional, dimension(new_max) :: comment_valuep

character ( len = * ), intent(in), optional :: axis
character ( len = * ), intent(in), optional :: long_name
character ( len = * ), intent(in), optional :: standard_name
character ( len = * ), intent(in), optional :: bounds

real, intent(in), optional :: fillvalue
real, intent(in), optional :: valid_min
real, intent(in), optional :: valid_max
character ( len = * ), intent(in), optional :: coordinates
character ( len = * ), intent(in), optional :: cell_methods
character ( len = * ), intent(in), optional :: climatology

! Local Declarations

character ( len = 256 ) :: emess
integer :: ndim_id
integer :: n
integer :: nend1
integer :: nend2
integer :: nend3
integer :: lnblnk
integer :: nsum
integer :: istatus
integer :: nstop

! Local Arrays

integer, dimension(netcdf_max_dim) :: nhold

nend1 = lnblnk ( cmiss )

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! See if variable already exists

nend3 = lnblnk ( vname_new )
if ( nend3 <= 0 ) then
  nend3 = 1
end if

nstop = 0
call get_var_id ( nvar_id, vname_new, nstop )

write (lulog,'(/a)') ' subroutine define_netcdf_var'
write (lulog,'(a,a)') ' vname_new = ', trim ( vname_new )
write (lulog,'(a,i10)') ' nvar_id=', nvar_id
write (lulog,'(a,i10)') ' new_max=', new_max
call flush ( lulog )

if ( nvar_id > 0 ) then
  return
end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Define new dimensions if needed

do n = 1, new_max

write (lulog,'(/a)') ' subroutine define_netcdf_var'
write (lulog,'(a,i10)') ' n=', n
write (lulog,'(a,i10)') ' ncid=', ncid
write (lulog,'(a,i10)') ' new_dim_ndsize(n)=', new_dim_ndsize(n)
write (lulog,'(a,a)') ' new_dim_vname(n) = ', trim ( new_dim_vname(n) )
write (lulog,'(a,a)') ' new_dim_desc(n) = ', trim ( new_dim_desc(n) )
write (lulog,'(a,i10)') ' nf90_unlimited=', nf90_unlimited
call flush ( lulog )

  nend2 = lnblnk ( new_dim_desc(n) )
  if ( nend2 <= 0 ) then
    exit
  end if
  if ( new_dim_desc(n)(1:nend2) == cmiss(1:nend1) ) then
    exit
  end if
  nend3 = lnblnk ( new_dim_vname(n) )
  if ( nend3 <= 0 ) then
    exit
  end if
  if ( new_dim_ndsize(n) < 0 ) then ! do not exit for nf90_unlimited = 0
    exit
  end if

  call define_netcdf_dim ( ncid, new_dim_ndsize(n), &
    new_dim_desc(n)(1:nend2), new_dim_vname(n)(1:nend3) )
end do

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Retrieve dimension IDs for this new variable

nsum = 0
nstop = 1

do n = 1, new_max
  nend2 = lnblnk ( dim_desc(n) )
  if ( nend2 <= 0 ) then
    exit
  end if
  if ( dim_desc(n)(1:nend2) == cmiss(1:nend1) ) then
    exit
  end if
  nend3 = lnblnk ( dim_vname(n) )
  if ( nend3 <= 0 ) then
    exit
  end if
  if ( dim_ndsize(n) < 0 ) then ! do not exit for nf90_unlimited = 0
    exit
  end if
  call get_netcdf_dim ( ndim_id, dim_vname(n)(1:nend3), &
    dim_desc(n)(1:nend2), dim_ndsize(n), nstop )
  nsum = nsum + 1
  if ( nsum > netcdf_max_dim ) then
    emess = ' ' // trim ( dim_desc(n) )
    call write_error ( lulog, &
      ' subroutine define_netcdf_var', &
      ' nsum > netcdf_max_dim ???', &
      emess )
  end if
  nhold(nsum) = ndim_id
end do

if ( nsum == 0 ) then
  emess = ' ' // trim ( vname_new )
  call write_error ( lulog, &
    ' subroutine define_netcdf_var', &
    ' no dimension IDs found ???', &
    emess )
end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Define new variable

nend3 = lnblnk ( vname_new )

if ( nend3 <= 0 ) then
  nend3 = 1
end if

istatus = nf90_def_var ( ncid, vname_new(1:nend3), ntype, nhold(1:nsum), nvar_id )

if ( istatus /= 0 ) then
  emess = ' ' // trim (vname_new ) // ' : ' // trim (  nf90_strerror(istatus) )
  call write_error ( lulog, &
    ' subroutine define_netcdf_var', &
    ' problem defining variable in netCDF file ???', &
    emess )
end if

netcdf_nvar_use = netcdf_nvar_use + 1

if ( netcdf_nvar_use > new_max ) then
  emess = ' ' // trim ( vname_new )
  call write_error ( lulog, &
    ' subroutine define_netcdf_var', &
    ' netcdf_nvar_use > new_max ???', &
    emess )
end if

var_desc(netcdf_nvar_use) = vname_new(1:nend3)
netcdf_varid_hold(netcdf_nvar_use) = nvar_id

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Define attributes for new variable

nend2 = lnblnk ( units )
if ( nend2 <= 0 ) then
  nend2 = 1
end if

nend3 = lnblnk ( vname_new )
if ( nend3 <= 0 ) then
  nend3 = 1
end if

if ( trim(pmode) == 'NEW_NETCDF' ) then
  call write_var_attributes ( ncid, nvar_id, &
    units = units(1:nend2) )
else if ( trim(pmode) == 'OLD_NETCDF' ) then
  call write_var_attributes ( ncid, nvar_id, &
    desc = vname_new(1:nend3), units = units(1:nend2) )
end if

if ( present ( missing_value_real ) ) then
  if ( nint(missing_value_real) /= nint(rmiss) ) then
    call write_var_attributes ( ncid, nvar_id, &
      missing_value_real = missing_value_real )
  end if
end if

if ( present ( missing_value_int ) ) then
  if ( missing_value_int /= imiss ) then
    call write_var_attributes ( ncid, nvar_id, &
      missing_value_int = missing_value_int )
  end if
end if

if ( present ( scale_factor_real ) ) then
  if ( nint(scale_factor_real) /= nint(rmiss) ) then
    call write_var_attributes ( ncid, nvar_id, &
      scale_factor_real = scale_factor_real )
  end if
end if

if ( present ( scale_factor_int ) ) then
  if ( scale_factor_int /= imiss ) then
    call write_var_attributes ( ncid, nvar_id, &
      scale_factor_int = scale_factor_int )
  end if
end if

if ( present ( axis ) ) then
  if ( trim(axis) /= trim(cmiss) ) then
    call write_var_attributes ( ncid, nvar_id, &
      axis = axis )
  end if
end if

if ( present ( long_name ) ) then
  if ( trim(long_name) /= trim(cmiss) ) then
    call write_var_attributes ( ncid, nvar_id, &
      long_name = long_name )
  end if
end if

if ( present ( standard_name ) ) then
  if ( trim(standard_name) /= trim(cmiss) ) then
    call write_var_attributes ( ncid, nvar_id, &
      standard_name = standard_name )
  end if
end if

if ( present ( bounds ) ) then
  if ( trim(bounds) /= trim(cmiss) ) then
    call write_var_attributes ( ncid, nvar_id, &
      bounds = bounds )
  end if
end if

if ( present ( fillvalue ) ) then
  if ( nint(fillvalue) /= nint(rmiss) ) then
    call write_var_attributes ( ncid, nvar_id, &
      fillvalue = fillvalue )
  end if
end if

if ( present ( valid_min ) ) then
  if ( nint(valid_min) /= nint(rmiss) ) then
    call write_var_attributes ( ncid, nvar_id, &
      valid_min = valid_min )
  end if
end if

if ( present ( valid_max ) ) then
  if ( nint(valid_max) /= nint(rmiss) ) then
    call write_var_attributes ( ncid, nvar_id, &
      valid_max = valid_max )
  end if
end if

if ( present ( coordinates ) ) then
  if ( trim(coordinates) /= trim(cmiss) ) then
    call write_var_attributes ( ncid, nvar_id, &
      coordinates = coordinates )
  end if
end if

if ( present ( cell_methods ) ) then
  if ( trim(cell_methods) /= trim(cmiss) ) then
    call write_var_attributes ( ncid, nvar_id, &
      cell_methods = cell_methods )
  end if
end if

if ( present ( climatology ) ) then
  if ( trim(climatology) /= trim(cmiss) ) then
    call write_var_attributes ( ncid, nvar_id, &
      climatology = climatology )
  end if
end if

nend1 = lnblnk ( cmiss )

if ( present(comment_namep) .and. present(comment_valuep) ) then
  do n = 1, new_max
    nend2 = lnblnk( comment_namep(n) )
    if ( nend2 <= 0 ) then
      exit
    end if
    if ( comment_namep(n)(1:nend2) == cmiss(1:nend1) ) then
      exit
    end if
    nend3 = lnblnk( comment_valuep(n) )
    if ( nend3 <= 0 ) then
      exit
    end if
    if ( comment_valuep(n)(1:nend3) == cmiss(1:nend1) ) then
      exit
    end if
    call write_var_attributes ( ncid, nvar_id, &
      comment_name = comment_namep(n)(1:nend2), &
      comment_value = comment_valuep(n)(1:nend3) )
  end do
end if

end subroutine

end module
!
!***********************************************************************
!
module module_setup_define_netcdf_var_amg
implicit none
contains
 
subroutine setup_define_netcdf_var_amg ( nv, vname, ntype, atmos_region, units, ftype, &
  missing_value_real, missing_value_int, &
  scale_factor_real, scale_factor_int, &
  axis, long_name, standard_name, bounds, &
  fillvalue, valid_min, valid_max, coordinates, cell_methods, &
  climatology )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
character ( len = * ), intent(inout) :: vname
integer, intent(inout) :: ntype
character ( len = * ), intent(in) :: atmos_region
character ( len = * ), intent(inout) :: units
character ( len = * ), intent(in) :: ftype
real, intent(inout) :: missing_value_real
integer, intent(inout) :: missing_value_int
real, intent(inout) :: scale_factor_real
integer, intent(inout) :: scale_factor_int
character ( len = * ), intent(inout) :: axis
character ( len = * ), intent(inout) :: long_name
character ( len = * ), intent(inout) :: standard_name
character ( len = * ), intent(inout) :: bounds
real, intent(inout) :: fillvalue
real, intent(inout) :: valid_min
real, intent(inout) :: valid_max
character ( len = * ), intent(inout) :: coordinates
character ( len = * ), intent(inout) :: cell_methods
character ( len = * ), intent(inout) :: climatology

! Local Declarations

character ( len = 256 ) :: emess
character ( len = 2 ) :: atmos_region_use
integer :: nmmax

units = cmiss
new_dim_desc = cmiss
new_dim_vname = cmiss
new_dim_ndsize = imiss
dim_desc = cmiss
dim_vname = cmiss
dim_ndsize = 0
comment_name = cmiss
comment_value = cmiss

axis = cmiss
long_name = cmiss
standard_name = cmiss
bounds = cmiss

fillvalue = rmiss
valid_min = rmiss
valid_max = rmiss
coordinates = cmiss
cell_methods = cmiss
climatology = cmiss

missing_value_real = rmiss
missing_value_int = imiss
scale_factor_real = rmiss
scale_factor_int = imiss

if ( ntmax > 0 ) then
  if ( ntmax /= ngmax ) then
    write (emess,'(a,2i10)') ' ngmax, ntmax=', ngmax, ntmax
    call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_amg', &
      ' monthly times do not agree ???', &
      emess )
  end if
end if

nmmax = ngmax

if ( trim(pmode) == 'OLD_NETCDF' .and. nv == 1 ) then

  new_dim_desc(1) = 'TIME_Number_of_Months'
  new_dim_vname(1) = 'nmmax'
  new_dim_ndsize(1) = nmmax

  dim_desc(1) = 'West_East'
  dim_vname(1) = 'imax'
  dim_ndsize(1) = imax

  dim_desc(2) = 'South_North'
  dim_vname(2) = 'jmax'
  dim_ndsize(2) = jmax

  dim_desc(3) = 'TIME_Number_of_Months'
  dim_vname(3) = 'nmmax'
  dim_ndsize(3) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'XYT'
  vname = 'MGA'
  ntype = nf90_int
  units = 'Degrees C'
  missing_value_int = -9999
  scale_factor_real = 1.0e-02

  comment_name(2) = 'Description'
  comment_value(2) = 'Monthly_Gridded_Anomaly,_Hundreths_of_Degrees'

else if ( trim(pmode) == 'OLD_NETCDF' .and. nv == 2 ) then

  dim_desc(1) = 'TIME_Number_of_Months'
  dim_vname(1) = 'nmmax'
  dim_ndsize(1) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'MESEC'
  ntype = nf90_double
  units = 'seconds'

  comment_name(2) = 'Description'
  comment_value(2) = 'Seconds since 1970-01-01 00:00:00 UTC'

else if ( trim(pmode) == 'OLD_NETCDF' .and. nv == 3 ) then

  new_dim_desc(1) = 'Length_of_Character_Date_String'
  new_dim_vname(1) = '19'
  new_dim_ndsize(1) = 19

  dim_desc(1) = 'Length_of_Character_Date_String'
  dim_vname(1) = '19'
  dim_ndsize(1) = 19

  dim_desc(2) = 'TIME_Number_of_Months'
  dim_vname(2) = 'nmmax'
  dim_ndsize(2) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'CT'
  vname = 'MCDATE'
  ntype = nf90_char
  units = 'calendar date'

  comment_name(2) = 'Description'
  comment_value(2) = 'Calendar date in the form YYYY-MM-DD-HH-NN-SS'

else if ( trim(ftype) == 'amg' .and. trim(pmode) == 'NEW_NETCDF' .and. nv == 1 ) then

  new_dim_desc(1) = 'time'
  new_dim_vname(1) = 'time'
  new_dim_ndsize(1) = nf90_unlimited

  dim_desc(1) = 'time'
  dim_vname(1) = 'time'
  dim_ndsize(1) = nf90_unlimited

  vname = 'time'
  ntype = nf90_short
  units = 'days since 1978-01-01 00:00:00'

  axis = 'T'
  long_name = 'time'
  standard_name = 'time'
  bounds = 'time_bounds'

else if ( trim(ftype) == 'amg' .and. trim(pmode) == 'NEW_NETCDF' .and. nv == 2 ) then

  new_dim_desc(1) = 'bnds'
  new_dim_vname(1) = 'bnds'
  new_dim_ndsize(1) = 2

  new_dim_desc(2) = 'time'
  new_dim_vname(2) = 'time'
  new_dim_ndsize(2) = nf90_unlimited

  dim_desc(1) = 'bnds'
  dim_vname(1) = 'bnds'
  dim_ndsize(1) = 2

  dim_desc(2) = 'time'
  dim_vname(2) = 'time'
  dim_ndsize(2) = nf90_unlimited

  vname = 'time_bounds'
  ntype = nf90_short

else if ( trim(ftype) == 'amg' .and. trim(pmode) == 'NEW_NETCDF' .and. nv == 3 ) then

  new_dim_desc(1) = 'lon'
  new_dim_vname(1) = 'lon'
  new_dim_ndsize(1) = imax

  new_dim_desc(2) = 'lat'
  new_dim_vname(2) = 'lat'
  new_dim_ndsize(2) = jmax

  new_dim_desc(3) = 'time'
  new_dim_vname(3) = 'time'
  new_dim_ndsize(3) = nf90_unlimited

  dim_desc(1) = 'lon'
  dim_vname(1) = 'lon'
  dim_ndsize(1) = imax

  dim_desc(2) = 'lat'
  dim_vname(2) = 'lat'
  dim_ndsize(2) = jmax

  dim_desc(3) = 'time'
  dim_vname(3) = 'time'
  dim_ndsize(3) = nf90_unlimited

  if ( trim(atmos_region) == 'MT' ) then
    atmos_region_use = 'mt'
    long_name = 'NOAA Climate Data Record of monthly anomaly for mid-tropospheric' // &
                ' microwave brightness temperature'
  else if ( trim(atmos_region) == 'LT' ) then
    atmos_region_use = 'lt'
    long_name = 'NOAA Climate Data Record of monthly anomaly for lower tropospheric' // &
                ' microwave brightness temperature'
  else if ( trim(atmos_region) == 'LS' ) then
    atmos_region_use = 'ls'
    long_name = 'NOAA Climate Data Record of monthly anomaly for lower stratosphere' // &
                ' microwave brightness temperature'
  else if ( trim(atmos_region) == 'TP' ) then
    atmos_region_use = 'tp'
    long_name = 'NOAA Climate Data Record of monthly anomaly for tropopause' // &
                ' microwave brightness temperature'
  else
    call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_amg', &
      ' unexpected atmos_region ???', &
      trim(atmos_region) )
  end if

  vname = 'bt_anomaly_' // trim( atmos_region_use )
  ntype = nf90_float

  units = 'K'
  standard_name = 'brightness_temperature_anomaly'
  fillvalue = -99.99
  valid_min = -20.0
  valid_max = +25.0
  coordinates = 'time lat lon'
  cell_methods = 'area: mean time: mean'

! NOLA
!KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK

else if ( trim(ftype) == 'acg' .and. trim(pmode) == 'NEW_NETCDF' .and. nv == 1 ) then

  new_dim_desc(1) = 'time'
  new_dim_vname(1) = 'time'
! new_dim_ndsize(1) = nf90_unlimited
  new_dim_ndsize(1) = 12

  dim_desc(1) = 'time'
  dim_vname(1) = 'time'
! dim_ndsize(1) = nf90_unlimited
  dim_ndsize(1) = 12

  vname = 'time'
  ntype = nf90_short
  units = 'days since 1981-01-01 00:00:00'

  axis = 'T'
  long_name = 'time'
  standard_name = 'time'
  climatology = 'time_bounds'

else if ( trim(ftype) == 'acg' .and. trim(pmode) == 'NEW_NETCDF' .and. nv == 2 ) then

  new_dim_desc(1) = 'bnds'
  new_dim_vname(1) = 'bnds'
  new_dim_ndsize(1) = 2

  new_dim_desc(2) = 'time'
  new_dim_vname(2) = 'time'
! new_dim_ndsize(2) = nf90_unlimited
  new_dim_ndsize(2) = 12

  dim_desc(1) = 'bnds'
  dim_vname(1) = 'bnds'
  dim_ndsize(1) = 2

  dim_desc(2) = 'time'
  dim_vname(2) = 'time'
! dim_ndsize(2) = nf90_unlimited
  dim_ndsize(2) = 12

  vname = 'time_bounds'
  ntype = nf90_short

else if ( trim(ftype) == 'acg' .and. trim(pmode) == 'NEW_NETCDF' .and. nv == 3 ) then

  new_dim_desc(1) = 'lon'
  new_dim_vname(1) = 'lon'
  new_dim_ndsize(1) = imax

  new_dim_desc(2) = 'lat'
  new_dim_vname(2) = 'lat'
  new_dim_ndsize(2) = jmax

  new_dim_desc(3) = 'time'
  new_dim_vname(3) = 'time'
! new_dim_ndsize(3) = nf90_unlimited
  new_dim_ndsize(3) = 12

  dim_desc(1) = 'lon'
  dim_vname(1) = 'lon'
  dim_ndsize(1) = imax

  dim_desc(2) = 'lat'
  dim_vname(2) = 'lat'
  dim_ndsize(2) = jmax

  dim_desc(3) = 'time'
  dim_vname(3) = 'time'
! dim_ndsize(3) = nf90_unlimited
  dim_ndsize(3) = 12

!KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
! NOLA

  if ( trim(atmos_region) == 'MT' ) then
    atmos_region_use = 'mt'
    long_name = 'NOAA Climate Data Record of mean annual cycle of microwave brightness temperature in the middle troposphere'
    vname = 'ac_mt'
  else if ( trim(atmos_region) == 'LT' ) then
    atmos_region_use = 'lt'
    long_name = 'NOAA Climate Data Record of mean annual cycle of microwave brightness temperature in the lower troposphere'
    vname = 'ac_lt'
  else if ( trim(atmos_region) == 'LS' ) then
    atmos_region_use = 'ls'
    long_name = 'NOAA Climate Data Record of mean annual cycle of microwave brightness temperature in the lower stratosphere'
    vname = 'ac_ls'
  else if ( trim(atmos_region) == 'TP' ) then
    atmos_region_use = 'tp'
    long_name = 'NOAA Climate Data Record of mean annual cycle of microwave brightness temperature in the tropopause'
    vname = 'ac_tp'
  else
    call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_amg', &
      ' unexpected atmos_region ???', &
      trim(atmos_region) )
  end if

  ntype = nf90_float

  units = 'K'
  standard_name = 'brightness_temperature'
  fillvalue = -99.99
  valid_min = +170.0
  valid_max = +300.0
  coordinates = 'time lat lon'
  cell_methods = 'area: mean time: mean within years time: mean over years'

! NOLA-HERE
!KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK

else

  write (lulog,'(a,a)') ' ftype = ', trim ( ftype )
  write (lulog,'(a,a)') ' pmode = ', trim ( pmode )
  write (lulog,'(a,10,/a)') ' nv=', nv

  write (emess,'(a,i10)') ' nv= ', nv
  call write_error ( lulog, &
    ' subroutine setup_define_netcdf_var_amg', &
    ' unexpected variable number ???', &
    emess )

end if

end subroutine

end module
!
!***********************************************************************
!
module module_get_acg_times
implicit none
contains
 
subroutine get_acg_times ( nt, vname, nf, ftype, jday_beg, jday_end, jday_mid, mode )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_inv_julian_minute
use module_findilev
use module_calc_julian_minute

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nt
character ( len = * ), intent(in) :: vname
integer, intent(inout) :: nf
character ( len = * ), intent(in) :: ftype
integer, intent(inout) :: jday_beg
integer, intent(inout) :: jday_end
integer, intent(inout) :: jday_mid
character ( len = * ), intent(in) :: mode

! Local Declarations

integer :: jmin_begp
integer :: jmin_endp

integer :: nyr_begp
integer :: nmo_begp
integer :: ndy_begp
integer :: nhr_begp
integer :: nmin_begp

integer :: nyr_endp
integer :: nmo_endp
integer :: ndy_endp
integer :: nhr_endp
integer :: nmin_endp

integer :: nyr_file_beg
integer :: nmo_file_beg
integer :: ndy_file_beg
integer :: nhr_file_beg
integer :: nmin_file_beg

integer :: nyr_file_end
integer :: nmo_file_end
integer :: ndy_file_end
integer :: nhr_file_end
integer :: nmin_file_end

integer :: kbot
integer :: ktop
integer :: nrev
integer :: nstop
integer :: ne
integer :: nl
integer :: ntt
integer :: ngood

integer :: nyr3
integer :: nmo3
integer :: ndy3
integer :: nhr3
integer :: nmin3
integer :: jmin3
integer :: jmid
integer :: jday_mid

! time_bnds is in units of days - convert to minutes

jmin_begp = ( 24 * 60 * time_bnds(1,nt) )
jmin_endp = ( 24 * 60 * time_bnds(2,nt) )

jmid = ( jmin_endp + jmin_begp ) / 2

jday_beg = 0
jday_end = 0

if ( use_alternate ) then
  if ( jmin_beg_exc(nf) <= 0 ) then
    write (lulog,'(/a)') ' subroutine get_acg_times'
    write (lulog,'(a)') ' jmin_beg_exc(nf) not defined ???'
    write (lulog,'(a,i10)') ' nf=', nf
    write (lulog,'(a,i10)') ' jmin_beg_exc(nf)=', jmin_beg_exc(nf)
    stop
  end if
  jday_mid = ( jmid - jmin_beg_exc(nf) ) / ( 24 * 60 )
else
  jday_mid = jmid / ( 24 * 60 )
end if

if ( trim(mode) == 'time' ) then
  return
end if

call inv_julian_minute ( nyr_begp, nmo_begp, ndy_begp, nhr_begp, nmin_begp, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_begp )

call inv_julian_minute ( nyr_endp, nmo_endp, ndy_endp, nhr_endp, nmin_endp, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_endp )

call inv_julian_minute ( nyr_file_beg, nmo_file_beg, ndy_file_beg, nhr_file_beg, nmin_file_beg, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_beg_exc(nf) )

call inv_julian_minute ( nyr_file_end, nmo_file_end, ndy_file_end, nhr_file_end, nmin_file_end, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_end_exc(nf) )

nyr3 = nyr_file_beg
nmo3 = nmo_begp
ndy3 = ndy_begp
nhr3 = nhr_begp
nmin3 = nmin_begp

call calc_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_begp )

nyr3 = nyr_file_end
nmo3 = nmo_endp
if ( nmo_begp == 12 ) then
  nyr3 = nyr3 + 1
end if
ndy3 = 1
nhr3 = 0
nmin3 = 0

call calc_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_endp )

write (lulog,'(/a)') ' subroutine get_acg_times'
write (lulog,'(a,i10)') ' nyr_begp=', nyr_begp
write (lulog,'(a,i10)') ' nmo_begp=', nmo_begp
write (lulog,'(a,i10)') ' ndy_begp=', ndy_begp
write (lulog,'(a,i10)') ' nhr_begp=', nhr_begp
write (lulog,'(a,i10)') ' nmin_begp=', nmin_begp
write (lulog,'(a,i10)') ' nyr_endp=', nyr_endp
write (lulog,'(a,i10)') ' nmo_endp=', nmo_endp
write (lulog,'(a,i10)') ' ndy_endp=', ndy_endp
write (lulog,'(a,i10)') ' nhr_endp=', nhr_endp
write (lulog,'(a,i10)') ' nmin_endp=', nmin_endp

write (lulog,'(a,i10)') ' nyr_file_beg=', nyr_file_beg
write (lulog,'(a,i10)') ' nmo_file_beg=', nmo_file_beg
write (lulog,'(a,i10)') ' ndy_file_beg=', ndy_file_beg
write (lulog,'(a,i10)') ' nhr_file_beg=', nhr_file_beg
write (lulog,'(a,i10)') ' nmin_file_beg=', nmin_file_beg

write (lulog,'(a,i10)') ' nyr_file_end=', nyr_file_end
write (lulog,'(a,i10)') ' nmo_file_end=', nmo_file_end
write (lulog,'(a,i10)') ' ndy_file_end=', ndy_file_end
write (lulog,'(a,i10)') ' nhr_file_end=', nhr_file_end
write (lulog,'(a,i10)') ' nmin_file_end=', nmin_file_end

kbot = 1
ktop = ngmax
nrev = +1
nstop = 1

call findilev ( jmin_store_mons, ngmax, jmin_begp, ne, nl, &
  kbot, ktop, ngood, nrev, nstop, lulog )

if ( jmin_store_mons(ne) == jmin_begp ) then
  ntt = ne
else if ( jmin_store_mons(nl) == jmin_begp ) then
  ntt = nl
end if

jmin_begp = ( 24 * 60 * time_bnds(1,ntt) )

call inv_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_begp )

write (lulog,'(/a)') ' subroutine get_acg_times'
write (lulog,'(a)') ' BEG CLIMATOLOGY TIME'
write (lulog,'(a,i10)') ' nyr3=', nyr3
write (lulog,'(a,i10)') ' nmo3=', nmo3
write (lulog,'(a,i10)') ' ndy3=', ndy3
write (lulog,'(a,i10)') ' nhr3=', nhr3
write (lulog,'(a,i10)') ' nmin3=', nmin3

call findilev ( jmin_store_mons, ngmax, jmin_endp, ne, nl, &
  kbot, ktop, ngood, nrev, nstop, lulog )

if ( jmin_store_mons(ne) == jmin_endp ) then
  ntt = ne
else if ( jmin_store_mons(nl) == jmin_endp ) then
  ntt = nl
end if

jmin_endp = ( 24 * 60 * time_bnds(1,ntt) )

call inv_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_endp )

write (lulog,'(/a)') ' subroutine get_acg_times'
write (lulog,'(a)') ' END CLIMATOLOGY TIME'
write (lulog,'(a,i10)') ' nyr3=', nyr3
write (lulog,'(a,i10)') ' nmo3=', nmo3
write (lulog,'(a,i10)') ' ndy3=', ndy3
write (lulog,'(a,i10)') ' nhr3=', nhr3
write (lulog,'(a,i10)') ' nmin3=', nmin3

if ( use_alternate .and. jmin_beg_exc(nf) <= 0 ) then
  write (lulog,'(/a)') ' subroutine get_acg_times'
  write (lulog,'(a)') ' jmin_beg_exc(nf) not defined ???'
  write (lulog,'(a,i10)') ' nf=', nf
  write (lulog,'(a,i10)') ' jmin_beg_exc(nf)=', jmin_beg_exc(nf)
  stop
end if

if ( use_alternate ) then
  jday_beg = ( jmin_begp - jmin_beg_exc(nf) ) / ( 24 * 60 )
  jday_end = ( jmin_endp - jmin_beg_exc(nf) ) / ( 24 * 60 )
else
  jday_beg = jmin_begp / ( 24 * 60 )
  jday_end = jmin_endp / ( 24 * 60 )
end if

write (lulog,'(/a)') ' subroutine get_acg_times'
write (lulog,'(a,i10)') ' jday_beg=', jday_beg
write (lulog,'(a,i10)') ' jday_end=', jday_end
write (lulog,'(a,i10)') ' jday_mid=', jday_mid

end subroutine

end module
!
!***********************************************************************
!
module module_get_maxmin_reg
implicit none
contains
 
subroutine get_maxmin_reg ( atmos_region, dval )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: atmos_region
real, intent(in) :: dval

! Local Declarations

real :: diff

diff = abs ( abs(dval) - 99.99 )

if ( diff <= 1.0e-03 ) then
  return
end if

if ( trim(atmos_region) == 'MT' ) then
  if ( dval < reg_min_mt ) reg_min_mt = dval
  if ( dval > reg_max_mt ) reg_max_mt = dval
else if ( trim(atmos_region) == 'LT' ) then
  if ( dval < reg_min_lt ) reg_min_lt = dval
  if ( dval > reg_max_lt ) reg_max_lt = dval
else if ( trim(atmos_region) == 'LS' ) then
  if ( dval < reg_min_ls ) reg_min_ls = dval
  if ( dval > reg_max_ls ) reg_max_ls = dval
else if ( trim(atmos_region) == 'TP' ) then
  if ( dval < reg_min_tp ) reg_min_tp = dval
  if ( dval > reg_max_tp ) reg_max_tp = dval
end if

end subroutine

end module
!
!***********************************************************************
!
module module_get_maxmin_reg_trends
implicit none
contains
 
subroutine get_maxmin_reg_trends ( atmos_region, dval )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: atmos_region
real, intent(in) :: dval

! Local Declarations

real :: diff

diff = abs ( abs(dval) - 99.99 )

if ( diff <= 1.0e-03 ) then
  return
end if

if ( trim(atmos_region) == 'MT' ) then
  if ( dval < reg_trend_min_mt ) reg_trend_min_mt = dval
  if ( dval > reg_trend_max_mt ) reg_trend_max_mt = dval
else if ( trim(atmos_region) == 'LT' ) then
  if ( dval < reg_trend_min_lt ) reg_trend_min_lt = dval
  if ( dval > reg_trend_max_lt ) reg_trend_max_lt = dval
else if ( trim(atmos_region) == 'LS' ) then
  if ( dval < reg_trend_min_ls ) reg_trend_min_ls = dval
  if ( dval > reg_trend_max_ls ) reg_trend_max_ls = dval
else if ( trim(atmos_region) == 'TP' ) then
  if ( dval < reg_trend_min_tp ) reg_trend_min_tp = dval
  if ( dval > reg_trend_max_tp ) reg_trend_max_tp = dval
end if

end subroutine

end module
!
!***********************************************************************
!
module module_get_maxmin_amg
implicit none
contains
 
subroutine get_maxmin_amg ( ftype, atmos_region, dval )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: ftype
character ( len = * ), intent(in) :: atmos_region
real, intent(in) :: dval

! Local Declarations

real :: diff

diff = abs ( abs(dval) - 99.99 )

!       write (lulog,'(/a)') ' subroutine get_maxmin_amg'
!       write (lulog,'(a,f15.3)') ' dval=', dval
!       write (lulog,'(a,5x,e12.5)') ' diff=', diff
!       write (lulog,'(a,a)') ' ftype = ', trim ( ftype )
!       write (lulog,'(a,a)') ' atmos_region = ', trim ( atmos_region )

if ( diff <= 1.0e-03 ) then
  return
end if

if ( trim(ftype) == 'amg' ) then

  if ( trim(atmos_region) == 'MT' ) then
    if ( dval < amg_min_mt ) amg_min_mt = dval
    if ( dval > amg_max_mt ) amg_max_mt = dval
  else if ( trim(atmos_region) == 'LT' ) then
    if ( dval < amg_min_lt ) amg_min_lt = dval
    if ( dval > amg_max_lt ) amg_max_lt = dval
  else if ( trim(atmos_region) == 'LS' ) then
    if ( dval < amg_min_ls ) amg_min_ls = dval
    if ( dval > amg_max_ls ) amg_max_ls = dval
  else if ( trim(atmos_region) == 'TP' ) then
    if ( dval < amg_min_tp ) amg_min_tp = dval
    if ( dval > amg_max_tp ) amg_max_tp = dval
  end if

else if ( trim(ftype) == 'acg' ) then

  if ( trim(atmos_region) == 'MT' ) then
    if ( dval < acg_min_mt ) acg_min_mt = dval
    if ( dval > acg_max_mt ) acg_max_mt = dval
  else if ( trim(atmos_region) == 'LT' ) then
    if ( dval < acg_min_lt ) acg_min_lt = dval
    if ( dval > acg_max_lt ) acg_max_lt = dval
  else if ( trim(atmos_region) == 'LS' ) then
    if ( dval < acg_min_ls ) acg_min_ls = dval
    if ( dval > acg_max_ls ) acg_max_ls = dval
  else if ( trim(atmos_region) == 'TP' ) then
    if ( dval < acg_min_tp ) acg_min_tp = dval
    if ( dval > acg_max_tp ) acg_max_tp = dval
  end if

end if

!       write (lulog,'(/a)') ' subroutine get_maxmin_amg'
!       write (lulog,'(a,f15.3)') ' current amg_min_mt = ', amg_min_mt
!       write (lulog,'(a,f15.3)') ' current amg_max_mt = ', amg_max_mt

end subroutine

end module
!
!***********************************************************************
!
module module_setup_nf90_put_var_amg
implicit none
contains
 
subroutine setup_nf90_put_var_amg ( jmin_cut, nv, nt, ngsum, nttsum, nvar_id, vname, atmos_region, &
  c_p1d, r8_p1d, r4_p2d, i_p1d, i_short_p1d, i_short_p2d, start, count, nddmax, n1, n2, fillv, &
  nf, ftype, nta, ntb )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_get_var_id
use module_get_acg_times
use module_get_maxmin_amg

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: jmin_cut
integer, intent(in) :: nv
integer, intent(in) :: nt
integer, intent(in) :: ngsum
integer, intent(in) :: nttsum
integer, intent(inout) :: nvar_id
character ( len = * ), intent(inout) :: vname
character ( len = * ), intent(in) :: atmos_region
real, intent(in) :: fillv

character ( len = 19 ), pointer, intent(inout) :: c_p1d
real ( kind = double_precision ), pointer, intent(inout), dimension(:) :: r8_p1d
real, pointer, intent(inout), dimension(:,:,:) :: r4_p2d
integer, pointer, intent(inout), dimension(:,:,:) :: i_p1d
integer ( kind = half_precision ), pointer, intent(inout), dimension(:) :: i_short_p1d
integer ( kind = half_precision ), pointer, intent(inout), dimension(:,:) :: i_short_p2d
integer, intent(inout), dimension(nddmax) :: start
integer, intent(inout), dimension(nddmax) :: count
integer, intent(in) :: nddmax

integer, intent(inout) :: n1
integer, intent(inout) :: n2
integer, intent(inout) :: nf
character ( len = * ), intent(in) :: ftype
integer, intent(in) :: nta
integer, intent(in) :: ntb

! Local Declarations

integer :: lnblnk
integer :: nend
integer :: nstop
integer :: nsum
integer :: n
integer :: i
integer :: j
character ( len = 256 ) :: emess
character ( len = 2 ) :: atmos_region_use
integer :: jday_beg
integer :: jday_end
integer :: jday_mid

! Local Arrays

integer, target, dimension
integer ( kind = half_precision ), target, dimension(1) :: ishort
integer ( kind = half_precision ), target, dimension(2,1) :: ishort2

nstop = 1
start = 0
count = 0

nullify ( c_p1d )
nullify ( r8_p1d )
nullify ( r4_p2d )
nullify ( i_p1d )
nullify ( i_short_p1d )
nullify ( i_short_p2d )

if ( trim(pmode) == 'OLD_NETCDF' .and. nv == 1 ) then

  vname = 'MCDATE'
  nend = lnblnk( vname )
  start(1) = 1
  start(2) = nt
  count(1) = 19
  count(2) = 1
  c_p1d => cdate_monthly_hold(nt)

else if ( trim(pmode) == 'OLD_NETCDF' .and. nv == 2 ) then

  vname = 'MESEC'
  start(1) = nt
  count(1) = 1
  esec_temp(1) = esec_monthly_hold(nt)
  r8_p1d => esec_temp

else if ( trim(pmode) == 'OLD_NETCDF' .and. nv == 3 ) then

  vname = 'MGA'
  nend = lnblnk( vname )
  start(1) = 1
  start(2) = 1
  start(3) = nt
  count(1) = imax
  count(2) = jmax
  count(3) = 1

  do i = 1, imax
    do j = 1, jmax
      ndata_2d(i,j,1) = ndata(i,j,nttsum)
    end do
  end do

  i_p1d => ndata_2d

else if ( trim(pmode) == 'NEW_NETCDF' .and. nv == 1 ) then

  vname = 'time'
  nend = lnblnk( vname )

  if ( trim(ftype) == 'amg' ) then
    start(1) = nt
    i_short_p1d => jday_store_mons(nt:nt)
  else if ( trim(ftype) == 'acg' ) then
    start(1) = nt - nta + 1
    call get_acg_times ( nt, vname, nf, ftype, jday_beg, jday_end, jday_mid, 'time' )
    ishort(1) = jday_mid
    i_short_p1d => ishort(1:1)
  end if

  count(1) = 1

else if ( trim(pmode) == 'NEW_NETCDF' .and. nv == 2 ) then

  vname = 'time_bounds'
  nend = lnblnk( vname )

  start(1) = 1

  count(1) = 2
  count(2) = 1

  if ( trim(ftype) == 'amg' ) then
    i_short_p2d => time_bnds(1:2,nt:nt)
    start(2) = nt
  else if ( trim(ftype) == 'acg' ) then
    call get_acg_times ( nt, vname, nf, ftype, jday_beg, jday_end, jday_mid, 'time_bounds' )
    ishort2(1,1) = jday_beg
    ishort2(2,1) = jday_end
    i_short_p2d =>  ishort2(1:2,1:1)
    start(2) = nt - nta + 1
  end if

else if ( trim(pmode) == 'NEW_NETCDF' .and. nv == 3 ) then

  if ( trim(atmos_region) == 'MT' ) then
    atmos_region_use = 'mt'
  else if ( trim(atmos_region) == 'LT' ) then
    atmos_region_use = 'lt'
  else if ( trim(atmos_region) == 'LS' ) then
    atmos_region_use = 'ls'
  else if ( trim(atmos_region) == 'TP' ) then
    atmos_region_use = 'tp'
  else
    call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_amg', &
      ' unexpected atmos_region ???', &
      trim(atmos_region) )
  end if

  if ( trim(ftype) == 'amg' ) then
    vname = 'bt_anomaly_' // trim( atmos_region_use )
  else if ( trim(ftype) == 'acg' ) then
    vname = 'ac_' // trim( atmos_region_use )
  end if

  nend = lnblnk( vname )

  start(1) = 1
  start(2) = 1

  if ( trim(ftype) == 'amg' ) then
    start(3) = nt
  else if ( trim(ftype) == 'acg' ) then
    start(3) = nt - nta + 1
  end if

  count(1) = imax
  count(2) = jmax
  count(3) = 1

  do i = 1, imax
    do j = 1, jmax
      if ( ndata(i,j,nttsum) /= imiss ) then
        rdata_2d(i,j,1) = 1.0e-02 * float ( ndata(i,j,nttsum) )
        call get_maxmin_amg ( ftype, atmos_region, rdata_2d(i,j,1) )
      else
        rdata_2d(i,j,1) = fillv
      end if
    end do
  end do

! NOLA-TODAY

  if ( trim_tls_2005 .and. jmin_store_mons(nt) >= jmin_cut ) then
    do i = 1, imax
      do j = 1, jmax
        rdata_2d(i,j,1) = fillv
      end do
    end do
  end if

  r4_p2d => rdata_2d

else

  write (emess,'(a,i10)') ' nv= ', nv
  call write_error ( lulog, &
    ' subroutine setup_nf90_put_var_amg', &
    ' unexpected variable number ???', &
    emess )

end if

nsum = 0
if ( associated ( c_p1d ) ) nsum = nsum + 1
if ( associated ( r8_p1d ) ) nsum = nsum + 1
if ( associated ( r4_p2d ) ) nsum = nsum + 1
if ( associated ( i_p1d ) ) nsum = nsum + 1
if ( associated ( i_short_p1d ) ) nsum = nsum + 1
if ( associated ( i_short_p2d ) ) nsum = nsum + 1

if ( nsum /= 1 ) then
  write (emess,'(a,i10)') ' nsum= ', nsum
  call write_error ( lulog, &
    ' subroutine setup_nf90_put_var_amg', &
    ' wrong number of pointers associated ???', &
    emess )
end if

nstop = 1
nend = lnblnk ( vname )

call get_var_id ( nvar_id, vname(1:nend), nstop )

n1 = 1

do n = nddmax, 1, -1
  if ( start(n) > 0 ) then
    n2 = n
    exit
  end if
end do

end subroutine

end module
!
!***********************************************************************
!
module module_write_mon_gridded_anomaly
implicit none
contains

subroutine write_mon_gridded_anomaly ( nf, atmos_region, fname, ntsum, ngsum, &
  ntaa, ntbb, ftype, ver, nopen, ncid, fname_netcdf, &
  fselect, fopen, gfiles, ncid_hold )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_initial_open_netcdf_file
use module_setup_define_netcdf_var_amg
use module_define_netcdf_var
use module_setup_nf90_put_var_amg
use module_write_alloc_status
use module_get_var_id
use module_check_date_status
use module_check_amg_data
use module_calc_julian_minute
use module_inv_julian_minute

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nf
character ( len = * ), intent(in) :: atmos_region
character ( len = * ), intent(in) :: fname
integer, intent(in) :: ntsum
integer, intent(in) :: ngsum
integer, intent(in) :: ntaa
integer, intent(in) :: ntbb
character ( len = * ), intent(in) :: ftype
character ( len = * ), intent(in) :: ver
integer, intent(inout) :: nopen
integer, intent(inout) :: ncid
character ( len = * ), intent(inout) :: fname_netcdf
integer, intent(in), dimension(ncvmax) :: fselect
integer, intent(inout), dimension(ncvmax) :: fopen
character ( len = * ), intent(in), dimension(ncvmax) :: gfiles
integer, intent(inout), dimension(ncvmax) :: ncid_hold

! Local Declarations

logical :: ldebug
integer :: nt
integer :: nc
integer :: nv
integer :: nttsum
integer :: nta
integer :: ntb

integer :: nvar_id
integer :: istatus

integer :: kdum = 1
integer :: nstop
integer :: n1
integer :: n2
integer :: nend
integer :: lnblnk
integer :: nskip

integer :: nyr3
integer :: nmo3
integer :: ndy3
integer :: nhr3
integer :: nmin3
integer :: jmin_cut

character ( len = 256 ) :: emess
character ( len = 256 ) :: emess2
character ( len = 256 ) :: vname
character ( len = 256 ) :: units
integer :: ntype
real :: missing_value_realp
integer :: missing_value_intp
real :: scale_factor_realp
integer :: scale_factor_intp
integer :: nfirst
integer :: nvmax

character ( len = 256 ) :: axisp
character ( len = 256 ) :: long_namep
character ( len = 256 ) :: standard_namep
character ( len = 256 ) :: boundsp

real :: fillvaluep
real :: valid_minp
real :: valid_maxp
character ( len = 256 ) :: coordinatesp
character ( len = 256 ) :: cell_methodsp
character ( len = 256 ) :: climatologyp

character ( len = 19 ), pointer :: c_p1d
real ( kind = double_precision ), pointer, dimension(:) :: r8_p1d
real, pointer, dimension(:,:,:) :: r4_p2d
!       integer, pointer, dimension(:,:) :: i_p1d
integer, pointer, dimension(:,:,:) :: i_p1d
integer ( kind = half_precision ), pointer, dimension(:) :: i_short_p1d
integer ( kind = half_precision ), pointer, dimension(:,:) :: i_short_p2d

! Local Arrays

integer, parameter :: nddmax = 10
integer, dimension(nddmax) :: start
integer, dimension(nddmax) :: count
real, save, dimension(nddmax) :: fill_hold

ldebug = .false.
ldebug = .true.

if ( ntaa > 0 .and. ntbb > 0 ) then
  nta = ntaa
  ntb = ntbb
else
  nta = ngsum - ntsum + 1
  ntb = ngsum
end if

if ( ldebug ) then
  write (lulog,'(/a)') ' WRITE: subroutine write_mon_gridded_anomaly'
  write (lulog,'(a,i10)') ' nf=', nf
  write (lulog,'(a,i10)') ' ntsum=', ntsum
  write (lulog,'(a,i10)') ' ngsum=', ngsum
  write (lulog,'(a,2i10)') ' ntaa, ntbb=', ntaa, ntbb
  write (lulog,'(a,2i10)') ' nta, ntb=', nta, ntb
  write (lulog,'(a,a)') ' atmos_region= ', trim( atmos_region )
  write (lulog,'(a,a)') ' fname= ', trim( fname )
  write (lulog,'(a,a)') ' ftype= ', trim( ftype )
  write (lulog,'(a,a)') ' ver= ', trim( ver )
  write (lulog,'(a,l10)') ' check_netcdf_data=', check_netcdf_data
  do nt = nta, ntb
    write (lulog,'(a,i10,5x,a)') ' nt, cdate_monthly_hold(nt)=', nt, cdate_monthly_hold(nt)
  end do
  call flush ( lulog )
end if

nopen = fopen(fselect(nf))
nfirst = 0

if ( nopen == 0 ) then

! NOLA-HERE

  call initial_open_netcdf_file ( nf, atmos_region, ver, ncid, fname_netcdf, ftype, gfiles )
  nopen = 1
  fopen(fselect(nf)) = 1
  ncid_hold(fselect(nf)) = ncid
  nfirst = 1

end if

ncid = ncid_hold(fselect(nf))

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

nstop = 0

if ( trim(pmode) == 'OLD_NETCDF' ) then
  nvmax = 3
else if ( trim(pmode) == 'NEW_NETCDF' ) then
  nvmax = 3
end if

if ( nfirst == 1 ) then

  write (lulog,'(/a)') ' subroutine write_mon_gridded_anomaly'
  write (lulog,'(a)') ' begin defining variables ...'
  call flush ( lulog )

  if ( .not.define_mode ) then
    istatus = nf90_redef ( ncid  )
    if ( istatus /= 0 ) then
      emess = ' ' // trim (  nf90_strerror(istatus) )
      call write_error ( lulog, &
        ' subroutine write_mon_gridded_anomaly', &
        ' problem setting mode to define ???', &
        emess )
    end if
    define_mode = .true.
  end if

  if ( nvmax > nddmax ) then
    write (lulog,'(/a)') ' subroutine write_mon_gridded_anomaly'
    write (lulog,'(a)') ' nvmax > nddmax ???'
    write (lulog,'(a,i10)') ' nvmax=', nvmax
    write (lulog,'(a,i10)') ' nddmax=', nddmax
    stop
  end if

  do nv = 1, nvmax

    write (lulog,'(/a)') ' subroutine write_mon_gridded_anomaly'
    write (lulog,'(a)') ' BEFORE call setup_define_netcdf_var_amg'
    write (lulog,'(a,i10)') ' nv=', nv
    write (lulog,'(a,i10)') ' nvmax=', nvmax
    call flush ( lulog )

    call setup_define_netcdf_var_amg ( nv, vname, ntype, atmos_region, units, ftype, &
      missing_value_realp, missing_value_intp, &
      scale_factor_realp, scale_factor_intp, &
      axisp, long_namep, standard_namep, boundsp, &
      fillvaluep, valid_minp, valid_maxp, coordinatesp, cell_methodsp, &
      climatologyp )

    fill_hold(nv) = fillvaluep

    write (lulog,'(/a)') ' subroutine write_mon_gridded_anomaly'
    write (lulog,'(a)') ' AFTER call setup_define_netcdf_var_amg'
    write (lulog,'(a,f15.5)') ' fill_hold(nv)=', fill_hold(nv)
    write (lulog,'(a)') ' BEFORE call define_netcdf_var'
    write (lulog,'(a,a)') ' vname = ', trim ( vname )
    call flush ( lulog )

    call define_netcdf_var ( ncid, nvar_id, &
      new_dim_desc, new_dim_vname, new_dim_ndsize, new_max, &
      dim_desc, dim_vname, dim_ndsize, &
      vname, ntype, units, &
      missing_value_real = missing_value_realp, &
      missing_value_int = missing_value_intp, &
      scale_factor_real = scale_factor_realp, &
      scale_factor_int = scale_factor_intp, &
      comment_namep = comment_name, &
      comment_valuep = comment_value, &
      axis = axisp, &
      long_name = long_namep, &
      standard_name = standard_namep, &
      bounds = boundsp, &
      fillvalue = fillvaluep, &
      valid_min = valid_minp, &
      valid_max = valid_maxp, &
      coordinates = coordinatesp, &
      cell_methods = cell_methodsp, &
      climatology = climatologyp )

  end do

  istatus = nf90_enddef ( ncid  )

  if ( istatus /= 0 ) then
    emess = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine write_mon_gridded_anomaly', &
      ' problem setting mode to end define ???', &
      emess )
  end if

  define_mode = .false.

  write (lulog,'(/a)') ' subroutine write_mon_gridded_anomaly'
  write (lulog,'(a)') ' end defining variables ...'
  call flush ( lulog )

!  NOLA-HERE-HERE

end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

nttsum = 0
nstop = 1

if ( define_mode ) then
  istatus = nf90_enddef ( ncid  )
  if ( istatus /= 0 ) then
    emess = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine write_mon_gridded_anomaly', &
      ' problem setting mode to end define ???', &
      emess )
  end if
  define_mode = .false.
end if

nyr3 = 2005
nmo3 = 1
ndy3 = 1
nhr3 = 0
nmin3 = 0

call calc_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_cut )

do nt = nta, ntb

  nttsum = month_store(nt)

  if ( nttsum <= 0 .or. nttsum > nmonths ) then
    write (lulog,'(/a)') ' subroutine write_mon_gridded_anomaly'
    write (lulog,'(a)') ' nttsum out of bounds ???'
!   NOLA
    write (lulog,'(a,i10)') ' nttsum=', nttsum
    write (lulog,'(a,i10)') ' nmonths=', nmonths
    write (lulog,'(a,i10)') ' nta=', nta
    write (lulog,'(a,i10)') ' ntb=', ntb
    write (lulog,'(a,i10)') ' nt=', nt
    stop
  end if

  do nv = 1, nvmax

    write (lulog,'(/a)') ' subroutine write_mon_gridded_anomaly'
    write (lulog,'(a)') ' before call call setup_nf90_put_var_amg'
    write (lulog,'(a,3i10)') ' nt, nta, ntb=', nt, nta, ntb
    write (lulog,'(a,i10)') ' nv=', nv
    write (lulog,'(a,i10)') ' nttsum=', nttsum
    write (lulog,'(a,i10)') ' ngsum=', ngsum
    write (lulog,'(a,i10)') ' ngmax', ngmax
    write (lulog,'(a,i10)') ' nskip=', nskip
!   write (lulog,'(a,a)') ' vname = ', trim ( vname )
    write (lulog,'(a,f15.5)') ' fill_hold(nv)=', fill_hold(nv)

!   NOLA-HERE-HERE

    call setup_nf90_put_var_amg ( jmin_cut, nv, nt, ngsum, nttsum, nvar_id, vname, atmos_region, &
      c_p1d, r8_p1d, r4_p2d, i_p1d, i_short_p1d, i_short_p2d, start, count, nddmax, &
      n1, n2, fill_hold(nv), nf, ftype, nta, ntb )

    nskip = 0

    if ( nt == ngmax ) then
      call check_date_status ( vname, nskip )
    end if

    write (lulog,'(/a)') ' subroutine write_mon_gridded_anomaly'
    write (lulog,'(a)') ' after call call setup_nf90_put_var_amg'
    write (lulog,'(a,3i10)') ' nt, nta, ntb=', nt, nta, ntb
    write (lulog,'(a,i10)') ' nv=', nv
    write (lulog,'(a,i10)') ' nttsum=', nttsum
    write (lulog,'(a,i10)') ' ngsum=', ngsum
    write (lulog,'(a,i10)') ' ngmax', ngmax
    write (lulog,'(a,2i10)') ' n1, n2=', n1, n2
    write (lulog,'(a,i10)') ' nskip=', nskip
    write (lulog,'(a,a)') ' vname = ', trim ( vname )

    if ( associated ( c_p1d ) ) then
      write (lulog,'(a)') ' c_p1d is associated'
    else if ( associated ( r8_p1d ) ) then
      write (lulog,'(a)') ' r8_p1d is associated'
    else if ( associated ( r4_p2d ) ) then
      write (lulog,'(a)') ' r4_p2d is associated'
    else if ( associated ( i_p1d ) ) then
      write (lulog,'(a)') ' i_p1d is associated'
    else if ( associated ( i_short_p1d ) ) then
      write (lulog,'(a)') ' i_short_p1d is associated'
    else if ( associated ( i_short_p2d ) ) then
      write (lulog,'(a)') ' i_short_p2d is associated'
    end if

    call flush ( lulog )

    if ( nskip == 1 ) then
      cycle
    end if

    write (lulog,'(/a)') ' subroutine write_mon_gridded_anomaly'
    write (lulog,'(a)') ' before NetCDF write ...'
    call flush ( lulog )

    if ( associated ( c_p1d ) ) then
      istatus = nf90_put_var ( ncid, nvar_id, c_p1d, start(n1:n2:1), count(n1:n2:1) )
    else if ( associated ( r8_p1d ) ) then
      istatus = nf90_put_var ( ncid, nvar_id, r8_p1d, start(n1:n2:1), count(n1:n2:1) )
    else if ( associated ( r4_p2d ) ) then
      istatus = nf90_put_var ( ncid, nvar_id, r4_p2d, start(n1:n2:1), count(n1:n2:1) )
    else if ( associated ( i_p1d ) ) then
      istatus = nf90_put_var ( ncid, nvar_id, i_p1d, start(n1:n2:1), count(n1:n2:1) )
    else if ( associated ( i_short_p1d ) ) then
      istatus = nf90_put_var ( ncid, nvar_id, i_short_p1d, start(n1:n2:1), count(n1:n2:1) )
    else if ( associated ( i_short_p2d ) ) then
      istatus = nf90_put_var ( ncid, nvar_id, i_short_p2d, start(n1:n2:1), count(n1:n2:1) )
    end if

    if ( istatus /= 0 ) then
      nend = lnblnk ( vname )
      write (emess,'(a,a,a)') ' problem writing ', vname(1:nend), &
        ' variable to netCDF file ???'
      emess2 = ' ' // trim (  nf90_strerror(istatus) )
      call write_error ( lulog, &
        ' subroutine write_mon_gridded_anomaly', &
        emess, &
        emess2 )
    end if

    write (lulog,'(/a)') ' subroutine write_mon_gridded_anomaly'
    write (lulog,'(a)') ' after NetCDF write ...'
    call flush ( lulog )

    if ( nv == 3 .and. check_netcdf_data ) then
      call check_amg_data ( vname, fname_netcdf, cdate_monthly_hold(nt), nttsum )
    end if

  end do

end do

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

write (lulog,'(/a)') ' LEAVING subroutine write_mon_gridded_anomaly'
call flush ( lulog )

end subroutine

end module
!
!***********************************************************************
!
module module_check_amg_data
implicit none
contains
 
subroutine check_amg_data ( vname, fname_in, cdate, nttsum )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_write_alloc_status

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: vname
character ( len = * ), intent(in) :: fname_in
character ( len = 19 ), intent(in) :: cdate
integer, intent(in) :: nttsum

! Local Declarations

integer :: lnblnk
integer :: nend
character ( len = 1000 ) :: fname
character ( len = 256 ) :: emess
integer :: ncid
integer :: istatus
integer :: nvar_id
integer :: nt
integer :: ntt
integer :: kdum = 1
integer :: i
integer :: j

! Local Arrays

integer, allocatable, dimension(:,:) :: ndum2d
integer, dimension(3) :: start
integer, dimension(3) :: count

allocate ( ndum2d(imax,jmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine check_amg_data', &
    'ndum2d', istatus, &
    '2D', imax, jmax, kdum, lulog )
end if

ndum2d = 0

nend = lnblnk ( fname_in )
if ( nend <= 0 ) then
  nend = 1
end if

fname = fname_in(1:nend) // '.OLD'
nend = lnblnk ( fname )

istatus = nf90_open ( fname(1:nend), nf90_nowrite, ncid )

if ( istatus /= 0 ) then
  write (emess,'(a,a)') ' fname= ', fname(1:nend)
  call write_error ( lulog, &
    ' subroutine check_amg_data', &
    ' problem opening file ???', &
    emess )
end if

nend = lnblnk ( vname )
if ( nend <= 0 ) then
  nend = 1
end if

istatus = nf90_inq_varid ( ncid, vname(1:nend), nvar_id )

if ( istatus /= 0 ) then
  write (emess,'(a,a)') ' vname= ', vname(1:nend)
  call write_error ( lulog, &
    ' subroutine check_amg_data', &
    ' problem getting variable ID ???', &
    emess )
end if

ntt = 0

!       write (lulog,'(/a)') ' subroutine check_amg_data'
!       write (lulog,'(a)') ' date string review ...'

do nt = 1, ngmax
  nend = lnblnk ( cdate_monthly_hold(nt) )
  if ( nend <= 0 ) then
    exit
  end if
! write (lulog,'(1x,a)') cdate_monthly_hold(nt)
  if ( cdate_monthly_hold(nt) == cdate ) then
    ntt = nt
    exit
  end if
end do

if ( ntt == 0 ) then
  write (lulog,'(/a)') ' subroutine check_amg_data'
  write (lulog,'(a)') ' could not find time match ???'
  write (lulog,'(a)') ' cdate = ', cdate
  stop
end if

start(1) = 1
start(2) = 1
start(3) = ntt

count(1) = imax
count(2) = jmax
count(3) = 1

istatus = nf90_get_var ( ncid, nvar_id, ndum2d, start, count )

if ( istatus /= 0 ) then
  nend = lnblnk ( vname )
  write (emess,'(a,a)') ' vname= ', vname(1:nend)
  call write_error ( lulog, &
    ' subroutine check_amg_data', &
    ' problem reading variable ???', &
    emess )
end if

do i = 1, imax
  do j = 1, jmax
    if ( ndum2d(i,j) /= ndata(i,j,nttsum) ) then
      write (lulog,'(/a)') ' PROBLEM: subroutine check_amg_data'
      write (lulog,'(a)') ' data agreement problem ???'
      write (lulog,'(a,2i10)') ' i,j=', i,j
      write (lulog,'(a,i10)') ' nttsum=', nttsum
      write (lulog,'(a,i10)') ' ndum2d(i,j)=', ndum2d(i,j)
      write (lulog,'(a,i10)') ' ndata(i,j,nttsum)=', ndata(i,j,nttsum)
      nend = lnblnk ( fname )
      write (lulog,'(a,a)') ' fname= ', fname(1:nend)
      nend = lnblnk ( vname )
      write (lulog,'(a,a)') ' vname= ', vname(1:nend)
      write (lulog,'(a)') ' cdate = ', cdate
      stop
    end if
  end do
end do

istatus = nf90_close ( ncid )

if ( istatus /= 0 ) then
  nend = lnblnk ( fname )
  write (emess,'(a,a)') ' fname= ', fname(1:nend)
  call write_error ( lulog, &
    ' subroutine check_amg_data', &
    ' problem closing file ???', &
    emess )
end if

deallocate ( ndum2d, stat = istatus )

end subroutine

end module
!
!***********************************************************************
!
module module_setup_nf90_put_var_ma
implicit none
contains
 
subroutine setup_nf90_put_var_ma ( nv, nvar_id, vname, &
  c_p1d, r8_p1d, r_p1d, i_p1d, &
  start, count, nddmax, cdate, esec, &
  gb, nh, sh, tp, ndays, &
  gb_rm, nh_rm, sh_rm, tp_rm, ndays_rm, &
  gbd, nhd, shd, tpd, &
  n1, n2 )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_get_var_id

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
integer, intent(inout) :: nvar_id
character ( len = * ), intent(inout) :: vname

character ( len = 19 ), pointer, intent(inout), dimension(:) :: c_p1d
real ( kind = double_precision ), pointer, intent(inout), dimension(:) :: r8_p1d
real, pointer, intent(inout), dimension(:) :: r_p1d
integer, pointer, intent(inout), dimension(:) :: i_p1d
integer, intent(inout), dimension(nddmax) :: start
integer, intent(inout), dimension(nddmax) :: count
integer, intent(in) :: nddmax

character ( len = 19 ), intent(in), target, dimension(ntmax) :: cdate
real ( kind = double_precision ), intent(in), target, dimension(ntmax) :: esec
real, intent(in), target, dimension(ntmax) :: gb
real, intent(in), target, dimension(ntmax) :: nh
real, intent(in), target, dimension(ntmax) :: sh
real, intent(in), target, dimension(ntmax) :: tp
integer, intent(in), target, dimension(ntmax) :: ndays

real, intent(in), target, dimension(ntmax) :: gb_rm
real, intent(in), target, dimension(ntmax) :: nh_rm
real, intent(in), target, dimension(ntmax) :: sh_rm
real, intent(in), target, dimension(ntmax) :: tp_rm
integer, intent(in), target, dimension(ntmax) :: ndays_rm

real, intent(in), target, dimension(1) :: gbd
real, intent(in), target, dimension(1) :: nhd
real, intent(in), target, dimension(1) :: shd
real, intent(in), target, dimension(1) :: tpd

integer, intent(inout) :: n1
integer, intent(inout) :: n2

! Local Declarations

integer :: lnblnk
integer :: nend
integer :: nstop
integer :: nsum
integer :: n
character ( len = 256 ) :: emess

nstop = 1
start = 0
count = 0

nullify ( c_p1d )
nullify ( r8_p1d )
nullify ( r_p1d )
nullify ( i_p1d )

if ( nv == 1 ) then

  vname = 'MCDATE'
  nend = lnblnk( vname )
  start(1) = 1
  start(2) = 1
  count(1) = 19
  count(2) = ntmax
  c_p1d => cdate

else if ( nv == 2 ) then

  vname = 'MESEC'
  start(1) = 1
  count(1) = ntmax
  r8_p1d => esec

else if ( nv == 3 ) then

  vname = 'MA_GB'
  start(1) = 1
  count(1) = ntmax
  r_p1d => gb

else if ( nv == 4 ) then

  vname = 'MA_NH'
  start(1) = 1
  count(1) = ntmax
  r_p1d => nh

else if ( nv == 5 ) then

  vname = 'MA_SH'
  start(1) = 1
  count(1) = ntmax
  r_p1d => sh

else if ( nv == 6 ) then

  vname = 'MA_TP'
  start(1) = 1
  count(1) = ntmax
  r_p1d => tp

else if ( nv == 7 ) then

  vname = 'MA_NDAYS'
  start(1) = 1
  count(1) = ntmax
  i_p1d => ndays

else if ( nv == 8 ) then

  vname = 'MA_GB_RM'
  start(1) = 1
  count(1) = ntmax
  r_p1d => gb_rm

else if ( nv == 9 ) then

  vname = 'MA_NH_RM'
  start(1) = 1
  count(1) = ntmax
  r_p1d => nh_rm

else if ( nv == 10 ) then

  vname = 'MA_SH_RM'
  start(1) = 1
  count(1) = ntmax
  r_p1d => sh_rm

else if ( nv == 11 ) then

  vname = 'MA_TP_RM'
  start(1) = 1
  count(1) = ntmax
  r_p1d => tp_rm

else if ( nv == 12 ) then

  vname = 'MA_NDAYS_RM'
  start(1) = 1
  count(1) = ntmax
  i_p1d => ndays_rm

else if ( nv == 13 ) then

  vname = 'MA_GBD'
  start(1) = 1
  count(1) = 1
  r_p1d => gbd

else if ( nv == 14 ) then

  vname = 'MA_NHD'
  start(1) = 1
  count(1) = 1
  r_p1d => nhd

else if ( nv == 15 ) then

  vname = 'MA_SHD'
  start(1) = 1
  count(1) = 1
  r_p1d => shd

else if ( nv == 16 ) then

  vname = 'MA_TPD'
  start(1) = 1
  count(1) = 1
  r_p1d => tpd

else

  write (emess,'(a,i10)') ' nv= ', nv
  call write_error ( lulog, &
    ' subroutine setup_nf90_put_var_ma', &
    ' unexpected variable number ???', &
    emess )

end if

nsum = 0
if ( associated ( c_p1d ) ) nsum = nsum + 1
if ( associated ( r8_p1d ) ) nsum = nsum + 1
if ( associated ( r_p1d ) ) nsum = nsum + 1
if ( associated ( i_p1d ) ) nsum = nsum + 1

if ( nsum /= 1 ) then
  write (emess,'(a,i10)') ' nsum= ', nsum
  call write_error ( lulog, &
    ' subroutine setup_nf90_put_var_ma', &
    ' wrong number of pointers associated ???', &
    emess )
end if

nstop = 1
nend = lnblnk ( vname )

call get_var_id ( nvar_id, vname(1:nend), nstop )

n1 = 1

do n = nddmax, 1, -1
  if ( start(n) > 0 ) then
    n2 = n
    exit
  end if
end do

end subroutine

end module
!
!***********************************************************************
!
inout
module module_write_mon_anomaly
implicit none
contains
 
subroutine write_mon_anomaly ( nf, atmos_region, fname, ntsum, ngsum, &
  ftype, ver, nopen, ncid, fname_netcdf, &
  fselect, fopen, gfiles, ncid_hold )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_initial_open_netcdf_file
use module_define_netcdf_var
use module_write_alloc_status
use module_get_var_id
use module_setup_nf90_put_var_ma
use module_setup_define_netcdf_var_ma
use module_check_date_status
use module_check_mam_data

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nf
character ( len = * ), intent(in) :: atmos_region
character ( len = * ), intent(in) :: fname
integer, intent(in) :: ntsum
integer, intent(in) :: ngsum
character ( len = * ), intent(in) :: ftype
character ( len = * ), intent(in) :: ver
integer, intent(inout) :: nopen
integer, intent(inout) :: ncid
character ( len = * ), intent(inout) :: fname_netcdf
integer, intent(in), dimension(ncvmax) :: fselect
integer, intent(inout), dimension(ncvmax) :: fopen
character ( len = * ), intent(in), dimension(ncvmax) :: gfiles
integer, intent(inout), dimension(ncvmax) :: ncid_hold

! Local Declarations

logical :: ldebug
integer :: nt
integer :: nc
integer :: nv
integer :: nta
integer :: ntb
integer :: nttsum

integer :: nvar_id
integer :: istatus
character ( len = 256 ) :: emess
character ( len = 256 ) :: emess2
character ( len = 256 ) :: vname
character ( len = 256 ) :: units
integer :: ntype
real :: missing_value_realp
integer :: missing_value_intp

integer :: kdum = 1
integer :: nstop
integer :: n1
integer :: n2
integer :: nend
integer :: lnblnk
integer :: nskip

character ( len = 19 ), pointer, dimension(:) :: c_p1d
real ( kind = double_precision ), pointer, dimension(:) :: r8_p1d
real, pointer, dimension(:) :: r_p1d
integer, pointer, dimension(:) :: i_p1d

! Local Arrays

integer, parameter :: nddmax = 10
integer, dimension(nddmax) :: start
integer, dimension(nddmax) :: count

ldebug = .false.
ldebug = .true.

if ( ldebug ) then
  write (lulog,'(/a)') ' WRITE: subroutine write_mon_anomaly'
  write (lulog,'(a,i10)') ' ntsum=', ntsum
  write (lulog,'(a,i10)') ' ngsum=', ngsum
  write (lulog,'(a,a)') ' atmos_region= ', trim( atmos_region )
  write (lulog,'(a,a)') ' fname= ', trim( fname )
  write (lulog,'(a,a)') ' ftype= ', trim( ftype )
  write (lulog,'(a,a)') ' ver= ', trim( ver )
  write (lulog,'(a,l10)') ' check_netcdf_data=', check_netcdf_data
  nta = ngsum - ntsum + 1
  ntb = ngsum
  do nt = nta, ntb
    write (lulog,'(a,i10,5x,a)') ' nt, mon_anom_store%cdate(nt)=', &
    nt, mon_anom_store%cdate(nt)
  end do
end if

nopen = fopen(fselect(nf))

if ( nopen == 0 ) then

  call initial_open_netcdf_file ( nf, atmos_region, ver, ncid, fname_netcdf, ftype, gfiles )
  nopen = 1
  fopen(fselect(nf)) = 1
  ncid_hold(fselect(nf)) = ncid

end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

nstop = 0
call get_var_id ( nvar_id, 'MA_GB', nstop )

if ( nvar_id == 0 ) then

  if ( .not.define_mode ) then
    istatus = nf90_redef ( ncid  )
    if ( istatus /= 0 ) then
      emess = ' ' // trim (  nf90_strerror(istatus) )
      call write_error ( lulog, &
        ' subroutine write_mon_anomaly', &
        ' problem setting mode to define ???', &
        emess )
    end if
    define_mode = .true.
  end if

  do nv = 1, 16

    call setup_define_netcdf_var_ma ( nv, vname, ntype, units, &
      missing_value_realp, missing_value_intp )

    call define_netcdf_var ( ncid, nvar_id, &
      new_dim_desc, new_dim_vname, new_dim_ndsize, new_max, &
      dim_desc, dim_vname, dim_ndsize, &
      vname, ntype, units, &
      missing_value_real = missing_value_realp, &
      missing_value_int = missing_value_intp, &
      comment_namep = comment_name, &
      comment_valuep = comment_value )

  end do

  istatus = nf90_enddef ( ncid  )

  if ( istatus /= 0 ) then
    emess = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine write_mon_anomaly', &
      ' problem setting mode to end define ???', &
      emess )
  end if

  define_mode = .false.

end if

nta = ngsum - ntsum + 1
ntb = ngsum
nttsum = 0
nstop = 1

if ( define_mode ) then
  istatus = nf90_enddef ( ncid  )
  if ( istatus /= 0 ) then
    emess = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine write_mon_anomaly', &
      ' problem setting mode to end define ???', &
      emess )
  end if
  define_mode = .false.
end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

do nv = 1, 14

  call setup_nf90_put_var_ma ( nv, nvar_id, vname, &
    c_p1d, r8_p1d, r_p1d, i_p1d, &
    start, count, nddmax, &
    mon_anom_store%cdate, &
    mon_anom_store%esec, &
    mon_anom_store%gb, &
    mon_anom_store%nh, &
    mon_anom_store%sh, &
    mon_anom_store%tp, &
    mon_anom_store%ndays, &
    mon_anom_store%gb_rm, &
    mon_anom_store%nh_rm, &
    mon_anom_store%sh_rm, &
    mon_anom_store%tp_rm, &
    mon_anom_store%ndays_rm, &
    mon_anom_store%gb_decadal_trend, &
    mon_anom_store%nh_decadal_trend, &
    mon_anom_store%sh_decadal_trend, &
    mon_anom_store%tp_decadal_trend, &
    n1, n2 )

  call check_date_status ( vname, nskip )

  if ( nskip == 1 ) then
    cycle
  end if

  if ( associated ( c_p1d ) ) then
    istatus = nf90_put_var ( ncid, nvar_id, c_p1d, start(n1:n2:1), count(n1:n2:1) )
  else if ( associated ( r8_p1d ) ) then
    istatus = nf90_put_var ( ncid, nvar_id, r8_p1d, start(n1:n2:1), count(n1:n2:1) )
  else if ( associated ( r_p1d ) ) then
    istatus = nf90_put_var ( ncid, nvar_id, r_p1d, start(n1:n2:1), count(n1:n2:1) )
  else if ( associated ( i_p1d ) ) then
    istatus = nf90_put_var ( ncid, nvar_id, i_p1d, start(n1:n2:1), count(n1:n2:1) )
  end if

  nend = lnblnk ( vname )

  if ( istatus /= 0 ) then
    write (emess,'(a,a,a)') ' problem writing ', vname(1:nend), &
      ' variable to netCDF file ???'
    emess2 = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine write_mon_anomaly', &
      emess, &
      emess2 )
  end if

  if ( check_netcdf_data ) then
    call check_mam_data ( vname, fname_netcdf, r_p1d, i_p1d )
  end if

end do

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

end subroutine

end module
!
!***********************************************************************
!
module module_check_mam_data
implicit none
contains
 
subroutine check_mam_data ( vname, fname_in, r_p1d, i_p1d )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_write_alloc_status

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: vname
character ( len = * ), intent(in) :: fname_in
real, pointer, intent(inout), dimension(:) :: r_p1d
integer, pointer, intent(inout), dimension(:) :: i_p1d

! Local Declarations

integer :: lnblnk
integer :: nend
character ( len = 1000 ) :: fname
character ( len = 256 ) :: emess
integer :: ncid
integer :: istatus
integer :: nvar_id
integer :: nt
integer :: kdum = 1
integer :: nvmode
real :: diff

! Local Arrays

integer, allocatable, dimension(:) :: ndum1d
real, allocatable, dimension(:) :: rdum1d

integer, dimension(1) :: start
integer, dimension(1) :: count

nend = lnblnk ( vname )
if ( nend <= 0 ) then
  nend = 1
end if

if ( vname(1:nend) == 'MESEC' ) then
  return
end if

if ( vname(1:nend) == 'MCDATE' ) then
  return
end if

nend = lnblnk ( fname_in )
if ( nend <= 0 ) then
  nend = 1
end if

fname = fname_in(1:nend) // '.OLD'
nend = lnblnk ( fname )

istatus = nf90_open ( fname(1:nend), nf90_nowrite, ncid )

if ( istatus /= 0 ) then
  write (emess,'(a,a)') ' fname= ', fname(1:nend)
  call write_error ( lulog, &
    ' subroutine check_mam_data', &
    ' problem opening file ???', &
    emess )
end if

nend = lnblnk ( vname )
if ( nend <= 0 ) then
  nend = 1
end if

nvmode = 0

if ( vname(1:nend) == 'MA_GB' ) nvmode = 1
if ( vname(1:nend) == 'MA_NH' ) nvmode = 1
if ( vname(1:nend) == 'MA_SH' ) nvmode = 1
if ( vname(1:nend) == 'MA_TP' ) nvmode = 1
if ( vname(1:nend) == 'MA_NDAYS' ) nvmode = 2

if ( vname(1:nend) == 'MA_GB_RM' ) nvmode = 1
if ( vname(1:nend) == 'MA_NH_RM' ) nvmode = 1
if ( vname(1:nend) == 'MA_SH_RM' ) nvmode = 1
if ( vname(1:nend) == 'MA_TP_RM' ) nvmode = 1
if ( vname(1:nend) == 'MA_NDAYS_RM' ) nvmode = 2

if ( vname(1:nend) == 'MA_GBD' ) nvmode = 3
if ( vname(1:nend) == 'MA_NHD' ) nvmode = 3
if ( vname(1:nend) == 'MA_SHD' ) nvmode = 3
if ( vname(1:nend) == 'MA_TPD' ) nvmode = 3

if ( nvmode == 0 ) then
  write (lulog,'(/a)') ' subroutine check_mam_data'
  write (lulog,'(a)') ' no variable match ???'
  nend = lnblnk ( vname )
  write (lulog,'(a,a)') ' vname= ', vname(1:nend)
  stop
end if

if ( nvmode == 1 ) then

  allocate ( rdum1d(ntmax), stat = istatus )

  if ( istatus /= 0 ) then
    call write_alloc_status ( 'subroutine check_mam_data', &
      'rdum1d', istatus, &
      '1D', ntmax, kdum, kdum, lulog )
  end if

  rdum1d = 0.0

  nend = lnblnk ( vname )

  istatus = nf90_inq_varid ( ncid, vname(1:nend), nvar_id )

  if ( istatus /= 0 ) then
    write (emess,'(a,a)') ' vname= ', vname(1:nend)
    call write_error ( lulog, &
      ' subroutine check_mam_data', &
      ' problem getting variable ID ???', &
      emess )
  end if

  start(1) = 1
  count(1) = ntmax

  istatus = nf90_get_var ( ncid, nvar_id, rdum1d, start, count )

  if ( istatus /= 0 ) then
    nend = lnblnk ( vname )
    write (emess,'(a,a)') ' vname= ', vname(1:nend)
    call write_error ( lulog, &
      ' subroutine check_mam_data', &
      ' problem reading variable ???', &
      emess )
  end if

  do nt = 1, ntmax
    diff = abs ( rdum1d(nt) - r_p1d(nt) )
    if ( diff >= 1.0e-06 ) then
      write (lulog,'(/a)') ' PROBLEM: subroutine check_mam_data'
      write (lulog,'(a)') ' data agreement problem ???'
      write (lulog,'(a,i10)') ' nt=', nt
      write (lulog,'(a,5x,e12.5)') ' rdum1d(nt)=', rdum1d(nt)
      write (lulog,'(a,5x,e12.5)') ' r_p1d(nt)=', r_p1d(nt)
      write (lulog,'(a,5x,e12.5)') ' diff=', diff
      nend = lnblnk ( fname )
      write (lulog,'(a,a)') ' fname= ', fname(1:nend)
      nend = lnblnk ( vname )
      write (lulog,'(a,a)') ' vname= ', vname(1:nend)
      stop
    end if
  end do

  deallocate ( rdum1d, stat = istatus )

else if ( nvmode == 2 ) then

  allocate ( ndum1d(ntmax), stat = istatus )

  if ( istatus /= 0 ) then
    call write_alloc_status ( 'subroutine check_mam_data', &
      'ndum1d', istatus, &
      '1D', ntmax, kdum, kdum, lulog )
  end if

  ndum1d = 0

  nend = lnblnk ( vname )

  istatus = nf90_inq_varid ( ncid, vname(1:nend), nvar_id )

  if ( istatus /= 0 ) then
    write (emess,'(a,a)') ' vname= ', vname(1:nend)
    call write_error ( lulog, &
      ' subroutine check_mam_data', &
      ' problem getting variable ID ???', &
      emess )
  end if

  start(1) = 1
  count(1) = ntmax

  istatus = nf90_get_var ( ncid, nvar_id, ndum1d, start, count )

  if ( istatus /= 0 ) then
    nend = lnblnk ( vname )
    write (emess,'(a,a)') ' vname= ', vname(1:nend)
    call write_error ( lulog, &
      ' subroutine check_mam_data', &
      ' problem reading variable ???', &
      emess )
  end if

  do nt = 1, ntmax
    if ( ndum1d(nt) /= i_p1d(nt) ) then
      write (lulog,'(/a)') ' PROBLEM: subroutine check_mam_data'
      write (lulog,'(a)') ' data agreement problem ???'
      write (lulog,'(a,i10)') ' nt=', nt
      write (lulog,'(a,i10)') ' ndum1d(nt)=', ndum1d(nt)
      write (lulog,'(a,i10)') ' i_p1d(nt)=', i_p1d(nt)
      nend = lnblnk ( fname )
      write (lulog,'(a,a)') ' fname= ', fname(1:nend)
      nend = lnblnk ( vname )
      write (lulog,'(a,a)') ' vname= ', vname(1:nend)
      stop
    end if
  end do

  deallocate ( ndum1d, stat = istatus )

else if ( nvmode == 3 ) then

  allocate ( rdum1d(1), stat = istatus )

  if ( istatus /= 0 ) then
    call write_alloc_status ( 'subroutine check_mam_data', &
      'rdum1d', istatus, &
      '1D', kdum, kdum, kdum, lulog )
  end if

  rdum1d = 0.0

  nend = lnblnk ( vname )

  istatus = nf90_inq_varid ( ncid, vname(1:nend), nvar_id )

  if ( istatus /= 0 ) then
    write (emess,'(a,a)') ' vname= ', vname(1:nend)
    call write_error ( lulog, &
      ' subroutine check_mam_data', &
      ' problem getting variable ID ???', &
      emess )
  end if

  start(1) = 1
  count(1) = 1

  istatus = nf90_get_var ( ncid, nvar_id, rdum1d, start, count )

  if ( istatus /= 0 ) then
    nend = lnblnk ( vname )
    write (emess,'(a,a)') ' vname= ', vname(1:nend)
    call write_error ( lulog, &
      ' subroutine check_mam_data', &
      ' problem reading variable ???', &
      emess )
  end if

  do nt = 1, 1
    diff = abs ( rdum1d(nt) - r_p1d(nt) )
    if ( diff >= 1.0e-06 ) then
      write (lulog,'(/a)') ' PROBLEM: subroutine check_mam_data'
      write (lulog,'(a)') ' data agreement problem ???'
      write (lulog,'(a,i10)') ' nt=', nt
      write (lulog,'(a,5x,e12.5)') ' rdum1d(nt)=', rdum1d(nt)
      write (lulog,'(a,5x,e12.5)') ' r_p1d(nt)=', r_p1d(nt)
      write (lulog,'(a,5x,e12.5)') ' diff=', diff
      nend = lnblnk ( fname )
      write (lulog,'(a,a)') ' fname= ', fname(1:nend)
      nend = lnblnk ( vname )
      write (lulog,'(a,a)') ' vname= ', vname(1:nend)
      stop
    end if
  end do

  deallocate ( rdum1d, stat = istatus )

end if

istatus = nf90_close ( ncid )

if ( istatus /= 0 ) then
  nend = lnblnk ( fname )
  write (emess,'(a,a)') ' fname= ', fname(1:nend)
  call write_error ( lulog, &
    ' subroutine check_mam_data', &
    ' problem closing file ???', &
    emess )
end if

end subroutine

end module
!
!***********************************************************************
!
module module_setup_define_netcdf_var_ma
implicit none
contains
 
subroutine setup_define_netcdf_var_ma ( nv, vname, ntype, units, &
  missing_value_real, missing_value_int )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
character ( len = * ), intent(inout) :: vname
integer, intent(inout) :: ntype
character ( len = * ), intent(inout) :: units
real, intent(inout) :: missing_value_real
integer, intent(inout) :: missing_value_int

! Local Declarations

character ( len = 256 ) :: emess
integer :: nmmax

new_dim_desc = cmiss
new_dim_vname = cmiss
new_dim_ndsize = 0
dim_desc = cmiss
dim_vname = cmiss
dim_ndsize = 0
comment_name = cmiss
comment_value = cmiss

missing_value_real = rmiss
missing_value_int = imiss

if ( ngmax > 0 ) then
  if ( ntmax /= ngmax ) then
    write (emess,'(a,2i10)') ' ntmax, ngmax=', ntmax, ngmax
    call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_ma', &
      ' monthly times do not agree ???', &
      emess )
  end if
end if

nmmax = ntmax

if ( nv == 1 ) then

  new_dim_desc(1) = 'TIME_Number_of_Months'
  new_dim_vname(1) = 'nmmax'
  new_dim_ndsize(1) = nmmax

  dim_desc(1) = 'TIME_Number_of_Months'
  dim_vname(1) = 'nmmax'
  dim_ndsize(1) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'MA_GB'
  ntype = nf90_float
  units = 'Degrees C'
  missing_value_real = -99.990

  comment_name(2) = 'Description'
  comment_value(2) = 'Monthly_Anomaly,_Global'

else if ( nv == 2 ) then

  dim_desc(1) = 'TIME_Number_of_Months'
  dim_vname(1) = 'nmmax'
  dim_ndsize(1) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'MA_NH'
  ntype = nf90_float
  units = 'Degrees C'
  missing_value_real = -99.990

  comment_name(2) = 'Description'
  comment_value(2) = 'Monthly_Anomaly,_Northern_Hemisphere'

else if ( nv == 3 ) then

  dim_desc(1) = 'TIME_Number_of_Months'
  dim_vname(1) = 'nmmax'
  dim_ndsize(1) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'MA_SH'
  ntype = nf90_float
  units = 'Degrees C'
  missing_value_real = -99.990

  comment_name(2) = 'Description'
  comment_value(2) = 'Monthly_Anomaly,_Southern_Hemisphere'

else if ( nv == 4 ) then

  dim_desc(1) = 'TIME_Number_of_Months'
  dim_vname(1) = 'nmmax'
  dim_ndsize(1) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'MA_TP'
  ntype = nf90_float
  units = 'Degrees C'
  missing_value_real = -99.990

  comment_name(2) = 'Description'
  comment_value(2) = 'Monthly_Anomaly,_Tropics'

else if ( nv == 5 ) then

  dim_desc(1) = 'TIME_Number_of_Months'
  dim_vname(1) = 'nmmax'
  dim_ndsize(1) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'MA_NDAYS'
  ntype = nf90_int
  units = 'Days'
  missing_value_int = -9999

  comment_name(2) = 'Description'
  comment_value(2) = 'Number_of_Days_for_Monthly_Anomaly'

else if ( nv == 6 ) then

  dim_desc(1) = 'TIME_Number_of_Months'
  dim_vname(1) = 'nmmax'
  dim_ndsize(1) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'MESEC'
  ntype = nf90_double
  units = 'seconds'

  comment_name(2) = 'Description'
  comment_value(2) = 'Seconds since 1970-01-01 00:00:00 UTC'

else if ( nv == 7 ) then

  new_dim_desc(1) = 'Length_of_Character_Date_String'
  new_dim_vname(1) = '19'
  new_dim_ndsize(1) = 19

  dim_desc(1) = 'Length_of_Character_Date_String'
  dim_vname(1) = '19'
  dim_ndsize(1) = 19

  dim_desc(2) = 'TIME_Number_of_Months'
  dim_vname(2) = 'nmmax'
  dim_ndsize(2) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'CT'
  vname = 'MCDATE'
  ntype = nf90_char
  units = 'calendar date'

  comment_name(2) = 'Description'
  comment_value(2) = 'Calendar date in the form YYYY-MM-DD-HH-NN-SS'

else if ( nv == 8 ) then

  dim_desc(1) = 'TIME_Number_of_Months'
  dim_vname(1) = 'nmmax'
  dim_ndsize(1) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'MA_GB_RM'
  ntype = nf90_float
  units = 'Degrees C'
  missing_value_real = -99.990

  comment_name(2) = 'Description'
  comment_value(2) = 'Monthly_Anomaly,_Global,_Running_Mean'

else if ( nv == 9 ) then

  dim_desc(1) = 'TIME_Number_of_Months'
  dim_vname(1) = 'nmmax'
  dim_ndsize(1) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'MA_NH_RM'
  ntype = nf90_float
  units = 'Degrees C'
  missing_value_real = -99.990

  comment_name(2) = 'Description'
  comment_value(2) = 'Monthly_Anomaly,_Northern_Hemisphere,_Running_Mean'

else if ( nv == 10 ) then

  dim_desc(1) = 'TIME_Number_of_Months'
  dim_vname(1) = 'nmmax'
  dim_ndsize(1) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'MA_SH_RM'
  ntype = nf90_float
  units ='Degrees C'
  missing_value_real = -99.990

  comment_name(2) = 'Description'
  comment_value(2) = 'Monthly_Anomaly,_Southern_Hemisphere,_Running_Mean'

else if ( nv == 11 ) then

  dim_desc(1) = 'TIME_Number_of_Months'
  dim_vname(1) = 'nmmax'
  dim_ndsize(1) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'MA_TP_RM'
  ntype = nf90_float
  units = 'Degrees C'
  missing_value_real = -99.990

  comment_name(2) = 'Description'
  comment_value(2) = 'Monthly_Anomaly,_Tropics,_Running_Mean'

else if ( nv == 12 ) then

  dim_desc(1) = 'TIME_Number_of_Months'
  dim_vname(1) = 'nmmax'
  dim_ndsize(1) = nmmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'MA_NDAYS_RM'
  ntype = nf90_float
  units = 'Days'
  missing_value_int = -9999

  comment_name(2) = 'Description'
  comment_value(2) = 'Number_of_Days_for_Monthly_Anomaly,_Running_Mean'

else if ( nv == 13 ) then

  new_dim_desc(1) = 'scalar'
  new_dim_vname(1) = 'scalar'
  new_dim_ndsize(1) = 1

  dim_desc(1) = 'scalar'
  dim_vname(1) = 'scalar'
  dim_ndsize(1) = 1
  vname = 'MA_GBD'
  ntype = nf90_float
  units = 'Degrees C'

  comment_name(2) = 'Description'
  comment_value(2) = 'Monthly_Anomaly,_Global,_Decadal_Trend'

else if ( nv == 14 ) then

  dim_desc(1) = 'scalar'
  dim_vname(1) = 'scalar'
  dim_ndsize(1) = 1
  vname = 'MA_NHD'
  ntype = nf90_float
  units = 'Degrees C'

  comment_name(2) = 'Description'
  comment_value(2) = 'Monthly_Anomaly,_Northern_Hemisphere,_Decadal_Trend'

else if ( nv == 15 ) then

  dim_desc(1) = 'scalar'
  dim_vname(1) = 'scalar'
  dim_ndsize(1) = 1
  vname = 'MA_SHD'
  ntype = nf90_float
  units = 'Degrees C'

  comment_name(2) = 'Description'
  comment_value(2) = 'Monthly_Anomaly,_Southern_Hemisphere,_Decadal_Trend'

else if ( nv == 16 ) then

  dim_desc(1) = 'scalar'
  dim_vname(1) = 'scalar'
  dim_ndsize(1) = 1
  vname = 'MA_TPD'
  ntype = nf90_float
  units = 'Degrees C'

  comment_name(2) = 'Description'
  comment_value(2) = 'Monthly_Anomaly,_Tropics,_Decadal_Trend'

else

  write (emess,'(a,i10)') ' nv= ', nv
  call write_error ( lulog, &
    ' subroutine setup_define_netcdf_var_ma', &
    ' unexpected variable number ???', &
    emess )

end if

end subroutine

end module
!
!***********************************************************************
!
module module_write_daily_zonal_anomaly
implicit none
contains

subroutine write_daily_zonal_anomaly ( nf, atmos_region, fname, ntsum, ngsum, &
  ftype, ver, nopen, ncid, fname_netcdf, &
  fselect, fopen, gfiles, ncid_hold )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_initial_open_netcdf_file
use module_define_netcdf_var
use module_write_alloc_status
use module_get_var_id
use module_setup_define_netcdf_var_dza
use module_setup_nf90_put_var_dza
use module_check_date_status
use module_check_amz_data

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nf
character ( len = * ), intent(in) :: atmos_region
character ( len = * ), intent(in) :: fname
integer, intent(in) :: ntsum
integer, intent(in) :: ngsum
character ( len = * ), intent(in) :: ftype
character ( len = * ), intent(in) :: ver
integer, intent(inout) :: nopen
integer, intent(inout) :: ncid
character ( len = * ), intent(inout) :: fname_netcdf
integer, intent(in), dimension(ncvmax) :: fselect
integer, intent(inout), dimension(ncvmax) :: fopen
character ( len = * ), intent(in), dimension(ncvmax) :: gfiles
integer, intent(inout), dimension(ncvmax) :: ncid_hold

! Local Declarations

logical :: ldebug
integer :: nt
integer :: nc
integer :: nv
integer :: nta
integer :: ntb
integer :: nttsum

integer :: nvar_id
integer :: istatus
character ( len = 256 ) :: emess
character ( len = 256 ) :: emess2
character ( len = 256 ) :: vname
character ( len = 256 ) :: units
integer :: ntype
real :: missing_value_realp
integer :: missing_value_intp
real :: scale_factor_realp
integer :: scale_factor_intp

integer :: kdum = 1
integer :: nstop
integer :: n1
integer :: n2
integer :: nend
integer :: lnblnk
integer :: nskip

character ( len = 19 ), pointer, dimension(:) :: c_p1d
real ( kind = double_precision ), pointer, dimension(:) :: r8_p1d
real, pointer, dimension(:) :: r_p1d
integer, pointer, dimension(:) :: i_p1d

! Local Arrays

integer, parameter :: nddmax = 10
integer, dimension(nddmax) :: start
integer, dimension(nddmax) :: count

ldebug = .false.
ldebug = .true.

if ( ldebug ) then
  write (lulog,'(/a)') ' WRITE: subroutine write_daily_zonal_anomaly'
  write (lulog,'(a,i10)') ' ntsum=', ntsum
  write (lulog,'(a,i10)') ' ngsum=', ngsum
  write (lulog,'(a,i10)') ' ntmax=', ntmax
  write (lulog,'(a,i10)') ' ngmax=', ngmax
  write (lulog,'(a,i10)') ' ndmax=', ndmax
  write (lulog,'(a,a)') ' atmos_region= ', trim( atmos_region )
  write (lulog,'(a,a)') ' fname= ', trim( fname )
  write (lulog,'(a,a)') ' ftype= ', trim( ftype )
  write (lulog,'(a,a)') ' ver= ', trim( ver )
  write (lulog,'(a,l10)') ' check_netcdf_data=', check_netcdf_data
  nta = ngsum - ntsum + 1
  ntb = ngsum
  write (lulog,'(a,2i10)') ' nta, ntb=', nta, ntb
  call flush ( lulog )
  do nt = nta, ntb
    write (lulog,'(a,i10,5x,a)') ' nt, day_anom_store%cdate(nt)=', &
    nt, day_anom_store%cdate(nt)
  end do
end if

nopen = fopen(fselect(nf))

if ( nopen == 0 ) then

  call initial_open_netcdf_file ( nf, atmos_region, ver, ncid, fname_netcdf, ftype, gfiles )
  nopen = 1
  fopen(fselect(nf)) = 1
  ncid_hold(fselect(nf)) = ncid

end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

nstop = 0
call get_var_id ( nvar_id, 'DZA_GB', nstop )

if ( nvar_id == 0 ) then

  if ( .not.define_mode ) then
    istatus = nf90_redef ( ncid  )
    if ( istatus /= 0 ) then
      emess = ' ' // trim (  nf90_strerror(istatus) )
      call write_error ( lulog, &
        ' subroutine write_daily_zonal_anomaly', &
        ' problem setting mode to define ???', &
        emess )
    end if
    define_mode = .true.
  end if

  do nv = 1, 6

    call setup_define_netcdf_var_dza ( nv, vname, ntype, units, &
      missing_value_realp, missing_value_intp, &
      scale_factor_realp, scale_factor_intp )

    call define_netcdf_var ( ncid, nvar_id, &
      new_dim_desc, new_dim_vname, new_dim_ndsize, new_max, &
      dim_desc, dim_vname, dim_ndsize, &
      vname, ntype, units, &
      missing_value_real = missing_value_realp, &
      missing_value_int = missing_value_intp, &
      scale_factor_real = scale_factor_realp, &
      scale_factor_int = scale_factor_intp, &
      comment_namep = comment_name, &
      comment_valuep = comment_value )

  end do

  istatus = nf90_enddef ( ncid  )

  if ( istatus /= 0 ) then
    emess = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine write_daily_zonal_anomaly', &
      ' problem setting mode to end define ???', &
      emess )
  end if

  define_mode = .false.

end if

nta = ngsum - ntsum + 1
ntb = ngsum
nttsum = 0
nstop = 1

if ( define_mode ) then
  istatus = nf90_enddef ( ncid  )
  if ( istatus /= 0 ) then
    emess = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine write_daily_zonal_anomaly', &
      ' problem setting mode to end define ???', &
      emess )
  end if
  define_mode = .false.
end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

do nv = 1, 6

  call setup_nf90_put_var_dza ( nv, nvar_id, vname, &
    c_p1d, r8_p1d, r_p1d, i_p1d, &
    start, count, nddmax, &
    day_anom_store%cdate, &
    day_anom_store%esec, &
    day_anom_store%gb, &
    day_anom_store%nh, &
    day_anom_store%sh, &
    day_anom_store%tp, &
    n1, n2 )

  call check_date_status ( vname, nskip )

  if ( nskip == 1 ) then
    cycle
  end if
 
  if ( associated ( c_p1d ) ) then
    istatus = nf90_put_var ( ncid, nvar_id, c_p1d, start(n1:n2:1), count(n1:n2:1) )
  else if ( associated ( r8_p1d ) ) then
    istatus = nf90_put_var ( ncid, nvar_id, r8_p1d, start(n1:n2:1), count(n1:n2:1) )
  else if ( associated ( r_p1d ) ) then
    istatus = nf90_put_var ( ncid, nvar_id, r_p1d, start(n1:n2:1), count(n1:n2:1) )
  else if ( associated ( i_p1d ) ) then
    istatus = nf90_put_var ( ncid, nvar_id, i_p1d, start(n1:n2:1), count(n1:n2:1) )
  end if

  nend = lnblnk ( vname )

  if ( istatus /= 0 ) then
    write (emess,'(a,a,a)') ' problem writing ', vname(1:nend), &
      ' variable to netCDF file ???'
    emess2 = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine write_daily_zonal_anomaly', &
      emess, &
      emess2 )
  end if

  if ( check_netcdf_data ) then
    call check_amz_data ( vname, fname_netcdf, r_p1d, i_p1d )
  end if

end do

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!       KGD

end subroutine

end module
!
!***********************************************************************
!
module module_check_amz_data
implicit none
contains
 
subroutine check_amz_data ( vname, fname_in, r_p1d, i_p1d )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_write_alloc_status

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: vname
character ( len = * ), intent(in) :: fname_in
real, pointer, intent(inout), dimension(:) :: r_p1d
integer, pointer, intent(inout), dimension(:) :: i_p1d

! Local Declarations

integer :: lnblnk
integer :: nend
character ( len = 1000 ) :: fname
character ( len = 256 ) :: emess
integer :: ncid
integer :: istatus
integer :: nvar_id
integer :: nt
integer :: kdum = 1
integer :: nvmode
real :: diff

! Local Arrays

integer, allocatable, dimension(:) :: ndum1d
real, allocatable, dimension(:) :: rdum1d

integer, dimension(1) :: start
integer, dimension(1) :: count

nend = lnblnk ( vname )
if ( nend <= 0 ) then
  nend = 1
end if

if ( vname(1:nend) == 'DESEC' ) then
  return
end if

if ( vname(1:nend) == 'DCDATE' ) then
  return
end if

nend = lnblnk ( fname_in )
if ( nend <= 0 ) then
  nend = 1
end if

fname = fname_in(1:nend) // '.OLD'
nend = lnblnk ( fname )

istatus = nf90_open ( fname(1:nend), nf90_nowrite, ncid )

if ( istatus /= 0 ) then
  write (emess,'(a,a)') ' fname= ', fname(1:nend)
  call write_error ( lulog, &
    ' subroutine check_amz_data', &
    ' problem opening file ???', &
    emess )
end if

nend = lnblnk ( vname )
if ( nend <= 0 ) then
  nend = 1
end if

nvmode = 0

if ( vname(1:nend) == 'DZA_GB' ) nvmode = 2
if ( vname(1:nend) == 'DZA_NH' ) nvmode = 2
if ( vname(1:nend) == 'DZA_SH' ) nvmode = 2
if ( vname(1:nend) == 'DZA_TP' ) nvmode = 2

if ( nvmode == 0 ) then
  write (lulog,'(/a)') ' subroutine check_amz_data'
  write (lulog,'(a)') ' no variable match ???'
  nend = lnblnk ( vname )
  write (lulog,'(a,a)') ' vname= ', vname(1:nend)
  stop
end if

if ( nvmode == 1 ) then

else if ( nvmode == 2 ) then

  allocate ( ndum1d(ndmax), stat = istatus )

  if ( istatus /= 0 ) then
    call write_alloc_status ( 'subroutine check_amz_data', &
      'ndum1d', istatus, &
      '1D', ndmax, kdum, kdum, lulog )
  end if

  ndum1d = 0

  nend = lnblnk ( vname )

  istatus = nf90_inq_varid ( ncid, vname(1:nend), nvar_id )

  if ( istatus /= 0 ) then
    write (emess,'(a,a)') ' vname= ', vname(1:nend)
    call write_error ( lulog, &
      ' subroutine check_amz_data', &
      ' problem getting variable ID ???', &
      emess )
  end if

  start(1) = 1
  count(1) = ndmax

  istatus = nf90_get_var ( ncid, nvar_id, ndum1d, start, count )

  if ( istatus /= 0 ) then
    nend = lnblnk ( vname )
    write (emess,'(a,a)') ' vname= ', vname(1:nend)
    call write_error ( lulog, &
      ' subroutine check_amz_data', &
      ' problem reading variable ???', &
      emess )
  end if

  do nt = 1, ndmax
    if ( ndum1d(nt) /= i_p1d(nt) ) then
      write (lulog,'(/a)') ' PROBLEM: subroutine check_amz_data'
      write (lulog,'(a)') ' data agreement problem ???'
      write (lulog,'(a,i10)') ' nt=', nt
      write (lulog,'(a,i10)') ' ndum1d(nt)=', ndum1d(nt)
      write (lulog,'(a,i10)') ' i_p1d(nt)=', i_p1d(nt)
      nend = lnblnk ( fname )
      write (lulog,'(a,a)') ' fname= ', fname(1:nend)
      nend = lnblnk ( vname )
      write (lulog,'(a,a)') ' vname= ', vname(1:nend)
      stop
    end if
  end do

  deallocate ( ndum1d, stat = istatus )

else if ( nvmode == 3 ) then

end if

istatus = nf90_close ( ncid )

if ( istatus /= 0 ) then
  nend = lnblnk ( fname )
  write (emess,'(a,a)') ' fname= ', fname(1:nend)
  call write_error ( lulog, &
    ' subroutine check_amz_data', &
    ' problem closing file ???', &
    emess )
end if

end subroutine

end module
!
!***********************************************************************
!
module module_setup_define_netcdf_var_dza
implicit none
contains
 
subroutine setup_define_netcdf_var_dza ( nv, vname, ntype, units, &
  missing_value_real, missing_value_int, &
  scale_factor_real, scale_factor_int )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
character ( len = * ), intent(inout) :: vname
integer, intent(inout) :: ntype
character ( len = * ), intent(inout) :: units
real, intent(inout) :: missing_value_real
integer, intent(inout) :: missing_value_int
real, intent(inout) :: scale_factor_real
integer, intent(inout) :: scale_factor_int

! Local Declarations

character ( len = 256 ) :: emess

new_dim_desc = cmiss
new_dim_vname = cmiss
new_dim_ndsize = 0
dim_desc = cmiss
dim_vname = cmiss
dim_ndsize = 0
comment_name = cmiss
comment_value = cmiss

missing_value_real = rmiss
missing_value_int = imiss
scale_factor_real = rmiss
scale_factor_int = imiss

if ( nv == 1 ) then

  new_dim_desc(1) = 'TIME_Number_of_Days'
  new_dim_vname(1) = 'ndmax'
  new_dim_ndsize(1) = ndmax

  dim_desc(1) = 'TIME_Number_of_Days'
  dim_vname(1) = 'ndmax'
  dim_ndsize(1) = ndmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'DZA_GB'
  ntype = nf90_int
  units = 'Degrees C'
  missing_value_int = -9999
  scale_factor_real = 1.0e-03

  comment_name(2) = 'Description'
  comment_value(2) = 'Daily_Zonal_Anomaly,_Global,_Thousandths_of_Degree'

else if ( nv == 2 ) then

  dim_desc(1) = 'TIME_Number_of_Days'
  dim_vname(1) = 'ndmax'
  dim_ndsize(1) = ndmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'DZA_NH'
  ntype = nf90_int
  units = 'Degrees C'
  missing_value_int = -9999
  scale_factor_real = 1.0e-03

  comment_name(2) = 'Description'
  comment_value(2) = 'Daily_Zonal_Anomaly,_Northern_Hemisphere,_Thousandths_of_Degree'

else if ( nv == 3 ) then

  dim_desc(1) = 'TIME_Number_of_Days'
  dim_vname(1) = 'ndmax'
  dim_ndsize(1) = ndmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'DZA_SH'
  ntype = nf90_int
  units = 'Degrees C'
  missing_value_int = -9999
  scale_factor_real = 1.0e-03

  comment_name(2) = 'Description'
  comment_value(2) = 'Daily_Zonal_Anomaly,_Southern_Hemisphere,_Thousandths_of_Degree'

else if ( nv == 4 ) then

  dim_desc(1) = 'TIME_Number_of_Days'
  dim_vname(1) = 'ndmax'
  dim_ndsize(1) = ndmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'DZA_TP'
  ntype = nf90_int
  units = 'Degrees C'
  missing_value_int = -9999
  scale_factor_real = 1.0e-03

  comment_name(2) = 'Description'
  comment_value(2) = 'Daily_Zonal_Anomaly,_Tropics,_Thousandths_of_Degree'

else if ( nv == 5 ) then

  dim_desc(1) = 'TIME_Number_of_Days'
  dim_vname(1) = 'ndmax'
  dim_ndsize(1) = ndmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'T'
  vname = 'DESEC'
  ntype = nf90_double
  units = 'seconds'

  comment_name(2) = 'Description'
  comment_value(2) = 'Seconds since 1970-01-01 00:00:00 UTC'

else if ( nv == 6 ) then

  new_dim_desc(1) = 'Length_of_Character_Date_String'
  new_dim_vname(1) = '19'
  new_dim_ndsize(1) = 19

  dim_desc(1) = 'Length_of_Character_Date_String'
  dim_vname(1) = '19'
  dim_ndsize(1) = 19

  dim_desc(2) = 'TIME_Number_of_Days'
  dim_vname(2) = 'ndmax'
  dim_ndsize(2) = ndmax

  comment_name(1) = 'MemoryOrder'
  comment_value(1) = 'CT'
  vname = 'DCDATE'
  ntype = nf90_char
  units = 'calendar date'

  comment_name(2) = 'Description'
  comment_value(2) = 'Calendar date in the form YYYY-MM-DD-HH-NN-SS'

else

  write (emess,'(a,i10)') ' nv= ', nv
  call write_error ( lulog, &
    ' subroutine setup_define_netcdf_var_dza', &
    ' unexpected variable number ???', &
    emess )

end if

end subroutine

end module
!
!***********************************************************************
!
module module_setup_nf90_put_var_dza
implicit none
contains
 
subroutine setup_nf90_put_var_dza ( nv, nvar_id, vname, &
  c_p1d, r8_p1d, r_p1d, i_p1d, &
  start, count, nddmax, cdate, esec, &
  gb, nh, sh, tp, &
  n1, n2 )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_get_var_id

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
integer, intent(inout) :: nvar_id
character ( len = * ), intent(inout) :: vname

character ( len = 19 ), pointer, intent(inout), dimension(:) :: c_p1d
real ( kind = double_precision ), pointer, intent(inout), dimension(:) :: r8_p1d
real, pointer, intent(inout), dimension(:) :: r_p1d
integer, pointer, intent(inout), dimension(:) :: i_p1d
integer, intent(inout), dimension(nddmax) :: start
integer, intent(inout), dimension(nddmax) :: count
integer, intent(in) :: nddmax

character ( len = 19 ), intent(in), target, dimension(ntmax) :: cdate
real ( kind = double_precision ), intent(in), target, dimension(ntmax) :: esec
integer, intent(in), target, dimension(ntmax) :: gb
integer, intent(in), target, dimension(ntmax) :: nh
integer, intent(in), target, dimension(ntmax) :: sh
integer, intent(in), target, dimension(ntmax) :: tp

integer, intent(inout) :: n1
integer, intent(inout) :: n2

! Local Declarations

integer :: lnblnk
integer :: nend
integer :: nstop
integer :: nsum
integer :: n
character ( len = 256 ) :: emess

nstop = 1
start = 0
count = 0

nullify ( c_p1d )
nullify ( r8_p1d )
nullify ( r_p1d )
nullify ( i_p1d )

if ( nv == 1 ) then

  vname = 'DCDATE'
  nend = lnblnk( vname )
  start(1) = 1
  start(2) = 1
  count(1) = 19
  count(2) = ndmax
  c_p1d => cdate

else if ( nv == 2 ) then

  vname = 'DESEC'
  start(1) = 1
  count(1) = ndmax
  r8_p1d => esec

else if ( nv == 3 ) then

  vname = 'DZA_GB'
  start(1) = 1
  count(1) = ndmax
  i_p1d => gb

else if ( nv == 4 ) then

  vname = 'DZA_NH'
  start(1) = 1
  count(1) = ndmax
  i_p1d => nh

else if ( nv == 5 ) then

  vname = 'DZA_SH'
  start(1) = 1
  count(1) = ndmax
  i_p1d => sh

else if ( nv == 6 ) then

  vname = 'DZA_TP'
  start(1) = 1
  count(1) = ndmax
  i_p1d => tp

else

  write (emess,'(a,i10)') ' nv= ', nv
  call write_error ( lulog, &
    ' subroutine setup_nf90_put_var_dza', &
    ' unexpected variable number ???', &
    emess )

end if

nsum = 0
if ( associated ( c_p1d ) ) nsum = nsum + 1
if ( associated ( r8_p1d ) ) nsum = nsum + 1
if ( associated ( r_p1d ) ) nsum = nsum + 1
if ( associated ( i_p1d ) ) nsum = nsum + 1

if ( nsum /= 1 ) then
  write (emess,'(a,i10)') ' nsum= ', nsum
  call write_error ( lulog, &
    ' subroutine setup_nf90_put_var_dza', &
    ' wrong number of pointers associated ???', &
    emess )
end if

nstop = 1
nend = lnblnk ( vname )

call get_var_id ( nvar_id, vname(1:nend), nstop )

n1 = 1

do n = nddmax, 1, -1
  if ( start(n) > 0 ) then
    n2 = n
    exit
  end if
end do

end subroutine

end module
!
!***********************************************************************
!
module module_write_mon_regional_anomaly
implicit none
contains

subroutine write_mon_regional_anomaly ( nf, atmos_region, fname, ntsum, ngsum, &
  ntaa, ntbb, ftype, ver, nopen, ncid, fname_netcdf, &
  fselect, fopen, gfiles, ncid_hold )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_initial_open_netcdf_file
use module_setup_define_netcdf_var_reg
use module_define_netcdf_var
use module_setup_nf90_put_var_reg
use module_write_alloc_status
use module_get_var_id
use module_check_date_status

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nf
character ( len = * ), intent(in) :: atmos_region
character ( len = * ), intent(in) :: fname
integer, intent(in) :: ntsum
integer, intent(in) :: ngsum
integer, intent(in) :: ntaa
integer, intent(in) :: ntbb
character ( len = * ), intent(in) :: ftype
character ( len = * ), intent(in) :: ver
integer, intent(inout) :: nopen
integer, intent(inout) :: ncid
character ( len = * ), intent(inout) :: fname_netcdf
integer, intent(in), dimension(ncvmax) :: fselect
integer, intent(inout), dimension(ncvmax) :: fopen
character ( len = * ), intent(in), dimension(ncvmax) :: gfiles
integer, intent(inout), dimension(ncvmax) :: ncid_hold

! Local Declarations

logical :: ldebug
integer :: nt
integer :: nc
integer :: nv
integer :: nttsum
integer :: nta
integer :: ntb

integer :: nvar_id
integer :: istatus

integer :: kdum = 1
integer :: nstop
integer :: n1
integer :: n2
integer :: nend
integer :: lnblnk
integer :: nskip

character ( len = 256 ) :: emess
character ( len = 256 ) :: emess2
character ( len = 256 ) :: vname
character ( len = 256 ) :: units
integer :: ntype
real :: missing_value_realp
integer :: missing_value_intp
real :: scale_factor_realp
integer :: scale_factor_intp
integer :: nfirst
integer :: nvmax

character ( len = 256 ) :: axisp
character ( len = 256 ) :: long_namep
character ( len = 256 ) :: standard_namep
character ( len = 256 ) :: boundsp

real :: fillvaluep
real :: valid_minp
real :: valid_maxp
character ( len = 256 ) :: coordinatesp
character ( len = 256 ) :: cell_methodsp

character ( len = 19 ), pointer :: c_p1d
real ( kind = double_precision ), pointer, dimension(:) :: r8_p1d
real, pointer, dimension(:,:,:) :: r4_p2d
!       integer, pointer, dimension(:,:) :: i_p1d
integer, pointer, dimension(:,:,:) :: i_p1d
integer ( kind = half_precision ), pointer, dimension(:) :: i_short_p1d
integer ( kind = half_precision ), pointer, dimension(:,:) :: i_short_p2d
real, pointer, dimension(:) :: r4_p1d

! Local Arrays

integer, parameter :: nddmax = 10
integer, dimension(nddmax) :: start
integer, dimension(nddmax) :: count

write (lulog,'(/a)') ' ENTERING subroutine write_mon_regional_anomaly'
call flush ( lulog )

ldebug = .false.
ldebug = .true.

if ( ntaa > 0 .and. ntbb > 0 ) then
  nta = ntaa
  ntb = ntbb
else
  nta = ngsum - ntsum + 1
  ntb = ngsum
end if

if ( ldebug ) then
  write (lulog,'(/a)') ' WRITE: subroutine write_mon_regional_anomaly'
  write (lulog,'(a,i10)') ' nf=', nf
  write (lulog,'(a,i10)') ' ntsum=', ntsum
  write (lulog,'(a,i10)') ' ngsum=', ngsum
  write (lulog,'(a,2i10)') ' ntaa, ntbb=', ntaa, ntbb
  write (lulog,'(a,2i10)') ' nta, ntb=', nta, ntb
  write (lulog,'(a,a)') ' atmos_region= ', trim( atmos_region )
  write (lulog,'(a,a)') ' fname= ', trim( fname )
  write (lulog,'(a,a)') ' ftype= ', trim( ftype )
  write (lulog,'(a,a)') ' ver= ', trim( ver )
  write (lulog,'(a,l10)') ' check_netcdf_data=', check_netcdf_data
  call flush ( lulog )
  do nt = nta, ntb
    write (lulog,'(a,i10,5x,a)') ' nt, cdate_monthly_reg_hold(nt)=', nt, cdate_monthly_reg_hold(nt)
  end do
  call flush ( lulog )
end if

nopen = fopen(fselect(nf))
nfirst = 0

if ( nopen == 0 ) then

  call initial_open_netcdf_file ( nf, atmos_region, ver, ncid, fname_netcdf, ftype, gfiles )
  nopen = 1
  fopen(fselect(nf)) = 1
  ncid_hold(fselect(nf)) = ncid
  nfirst = 1

end if

ncid = ncid_hold(fselect(nf))

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

nstop = 0

if ( trim(pmode) == 'OLD_NETCDF' ) then
  write (emess,'(a,a)') ' pmode = ', trim ( pmode )
  call write_error ( lulog, &
    ' subroutine write_mon_regional_anomaly', &
    ' unexpected pmode ???', &
    trim ( emess ) )
else if ( trim(pmode) == 'NEW_NETCDF' ) then
  nvmax = 4
end if

if ( nfirst == 1 ) then

  write (lulog,'(/a)') ' subroutine write_mon_regional_anomaly'
  write (lulog,'(a)') ' begin defining variables ...'
  call flush ( lulog )

  if ( .not.define_mode ) then
    istatus = nf90_redef ( ncid  )
    if ( istatus /= 0 ) then
      emess = ' ' // trim (  nf90_strerror(istatus) )
      call write_error ( lulog, &
        ' subroutine write_mon_regional_anomaly', &
        ' problem setting mode to define ???', &
        emess )
    end if
    define_mode = .true.
  end if

  do nv = 1, nvmax

    call setup_define_netcdf_var_reg ( nv, vname, ntype, atmos_region, units, &
      missing_value_realp, missing_value_intp, &
      scale_factor_realp, scale_factor_intp, &
      axisp, long_namep, standard_namep, boundsp, &
      fillvaluep, valid_minp, valid_maxp, coordinatesp, cell_methodsp )

    call define_netcdf_var ( ncid, nvar_id, &
      new_dim_desc, new_dim_vname, new_dim_ndsize, new_max, &
      dim_desc, dim_vname, dim_ndsize, &
      vname, ntype, units, &
      missing_value_real = missing_value_realp, &
      missing_value_int = missing_value_intp, &
      scale_factor_real = scale_factor_realp, &
      scale_factor_int = scale_factor_intp, &
      comment_namep = comment_name, &
      comment_valuep = comment_value, &
      axis = axisp, &
      long_name = long_namep, &
      standard_name = standard_namep, &
      bounds = boundsp, &
      fillvalue = fillvaluep, &
      valid_min = valid_minp, &
      valid_max = valid_maxp, &
      coordinates = coordinatesp, &
      cell_methods = cell_methodsp )

  end do

  istatus = nf90_enddef ( ncid  )

  if ( istatus /= 0 ) then
    emess = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine write_mon_regional_anomaly', &
      ' problem setting mode to end define ???', &
      emess )
  end if

  define_mode = .false.

  write (lulog,'(/a)') ' subroutine write_mon_regional_anomaly'
  write (lulog,'(a)') ' end defining variables ...'
  call flush ( lulog )

end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

nttsum = 0
nstop = 1

if ( define_mode ) then
  istatus = nf90_enddef ( ncid  )
  if ( istatus /= 0 ) then
    emess = ' ' // trim (  nf90_strerror(istatus) )
    call write_error ( lulog, &
      ' subroutine write_mon_regional_anomaly', &
      ' problem setting mode to end define ???', &
      emess )
  end if
  define_mode = .false.
end if

do nt = nta, ntb

  nttsum = nttsum + 1

  do nv = 1, nvmax
 
    call setup_nf90_put_var_reg ( nv, nt, ngsum, nttsum, nvar_id, vname, atmos_region, &
      c_p1d, r8_p1d, r4_p2d, r4_p1d, i_p1d, i_short_p1d, i_short_p2d, start, count, nddmax, &
      n1, n2 )

    nskip = 0

    if ( nt == ngmax ) then
      call check_date_status ( vname, nskip )
    end if

!   Trends variable is not a function of time

    if ( nv == nvmax ) then
      if ( nfirst == 1 ) then
        if ( nt == nta ) then
          nskip = 0
        else
          nskip = 1
        end if
      else if ( nfirst == 0 ) then
        nskip = 1
      end if
    end if

    write (lulog,'(/a)') ' subroutine write_mon_regional_anomaly'
    write (lulog,'(a,3i10)') ' nt, nta, ntb=', nt, nta, ntb
    write (lulog,'(a,i10)') ' nv=', nv
    write (lulog,'(a,i10)') ' nvmax=', nvmax
    write (lulog,'(a,i10)') ' nttsum=', nttsum
    write (lulog,'(a,i10)') ' ngsum=', ngsum
    write (lulog,'(a,i10)') ' ngmax', ngmax
    write (lulog,'(a,2i10)') ' n1, n2=', n1, n2
    write (lulog,'(a,i10)') ' nskip=', nskip
    call flush ( lulog )

    if ( nskip == 1 ) then
      cycle
    end if

    write (lulog,'(/a)') ' subroutine write_mon_regional_anomaly'
    write (lulog,'(a)') ' before NetCDF write ...'
    call flush ( lulog )

    if ( associated ( c_p1d ) ) then
      istatus = nf90_put_var ( ncid, nvar_id, c_p1d, start(n1:n2:1), count(n1:n2:1) )
    else if ( associated ( r8_p1d ) ) then
      istatus = nf90_put_var ( ncid, nvar_id, r8_p1d, start(n1:n2:1), count(n1:n2:1) )
    else if ( associated ( r4_p2d ) ) then
      istatus = nf90_put_var ( ncid, nvar_id, r4_p2d, start(n1:n2:1), count(n1:n2:1) )
    else if ( associated ( i_p1d ) ) then
      istatus = nf90_put_var ( ncid, nvar_id, i_p1d, start(n1:n2:1), count(n1:n2:1) )
    else if ( associated ( i_short_p1d ) ) then
      istatus = nf90_put_var ( ncid, nvar_id, i_short_p1d, start(n1:n2:1), count(n1:n2:1) )
    else if ( associated ( i_short_p2d ) ) then
      istatus = nf90_put_var ( ncid, nvar_id, i_short_p2d, start(n1:n2:1), count(n1:n2:1) )
    end if

    write (lulog,'(/a)') ' subroutine write_mon_regional_anomaly'
    write (lulog,'(a)') ' after NetCDF write ...'
    call flush ( lulog )

    if ( istatus /= 0 ) then
      nend = lnblnk ( vname )
      write (emess,'(a,a,a)') ' problem writing ', vname(1:nend), &
        ' variable to netCDF file ???'
      emess2 = ' ' // trim (  nf90_strerror(istatus) )
      call write_error ( lulog, &
        ' subroutine write_mon_regional_anomaly', &
        emess, &
        emess2 )
    end if

  end do

end do

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

write (lulog,'(/a)') ' LEAVING subroutine write_mon_regional_anomaly'
call flush ( lulog )

end subroutine

end module
!
!***********************************************************************
!
module module_setup_define_netcdf_var_reg
implicit none
contains
 
subroutine setup_define_netcdf_var_reg ( nv, vname, ntype, atmos_region, units, &
  missing_value_real, missing_value_int, &
  scale_factor_real, scale_factor_int, &
  axis, long_name, standard_name, bounds, &
  fillvalue, valid_min, valid_max, coordinates, cell_methods )
 
!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_get_free_logical_unit

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
character ( len = * ), intent(inout) :: vname
integer, intent(inout) :: ntype
character ( len = * ), intent(in) :: atmos_region
character ( len = * ), intent(inout) :: units
real, intent(inout) :: missing_value_real
integer, intent(inout) :: missing_value_int
real, intent(inout) :: scale_factor_real
integer, intent(inout) :: scale_factor_int
character ( len = * ), intent(inout) :: axis
character ( len = * ), intent(inout) :: long_name
character ( len = * ), intent(inout) :: standard_name
character ( len = * ), intent(inout) :: bounds
real, intent(inout) :: fillvalue
real, intent(inout) :: valid_min
real, intent(inout) :: valid_max
character ( len = * ), intent(inout) :: coordinates
character ( len = * ), intent(inout) :: cell_methods

! Local Declarations

character ( len = 256 ) :: emess
character ( len = 2 ) :: atmos_region_use
integer :: nmmax
logical :: fexist
integer :: lutemp
character ( len = 1000 ) :: nrec
integer :: nend
integer :: nendp
integer :: lnblnk
integer :: nfind
integer :: nc

units = cmiss
new_dim_desc = cmiss
new_dim_vname = cmiss
new_dim_ndsize = imiss
dim_desc = cmiss
dim_vname = cmiss
dim_ndsize = 0
comment_name = cmiss
comment_value = cmiss

axis = cmiss
long_name = cmiss
standard_name = cmiss
bounds = cmiss

fillvalue = rmiss
valid_min = rmiss
valid_max = rmiss
coordinates = cmiss
cell_methods = cmiss

missing_value_real = rmiss
missing_value_int = imiss
scale_factor_real = rmiss
scale_factor_int = imiss

if ( ntmax > 0 ) then
  if ( ntmax /= ngmax ) then
    write (emess,'(a,2i10)') ' ngmax, ntmax=', ngmax, ntmax
    call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_reg', &
      ' monthly times do not agree ???', &
      emess )
  end if
end if

nmmax = ngmax

if ( trim(pmode) == 'NEW_NETCDF' .and. nv == 1 ) then

  new_dim_desc(1) = 'time'
  new_dim_vname(1) = 'time'
  new_dim_ndsize(1) = nf90_unlimited

  dim_desc(1) = 'time'
  dim_vname(1) = 'time'
  dim_ndsize(1) = nf90_unlimited

  vname = 'time'
  ntype = nf90_short
  units = 'days since 1978-01-01 00:00:00'

  axis = 'T'
  long_name = 'time'
  standard_name = 'time'
  bounds = 'time_bounds'

else if ( trim(pmode) == 'NEW_NETCDF' .and. nv == 2 ) then

  new_dim_desc(1) = 'bnds'
  new_dim_vname(1) = 'bnds'
  new_dim_ndsize(1) = 2

  new_dim_desc(2) = 'time'
  new_dim_vname(2) = 'time'
  new_dim_ndsize(2) = nf90_unlimited

  dim_desc(1) = 'bnds'
  dim_vname(1) = 'bnds'
  dim_ndsize(1) = 2

  dim_desc(2) = 'time'
  dim_vname(2) = 'time'
  dim_ndsize(2) = nf90_unlimited

  vname = 'time_bounds'
  ntype = nf90_short

else if ( trim(pmode) == 'NEW_NETCDF' .and. nv == 3 ) then

  new_dim_desc(1) = 'region'
  new_dim_vname(1) = 'region'
  new_dim_ndsize(1) = nreg_max

  new_dim_desc(2) = 'time'
  new_dim_vname(2) = 'time'
  new_dim_ndsize(2) = nf90_unlimited

  dim_desc(1) = 'region'
  dim_vname(1) = 'region'
  dim_ndsize(1) = nreg_max

  dim_desc(2) = 'time'
  dim_vname(2) = 'time'
  dim_ndsize(2) = nf90_unlimited

  if ( trim(atmos_region) == 'MT' ) then
    atmos_region_use = 'mt'
    long_name = 'NOAA Climate Data Record of regional monthly anomaly for mid-tropospheric' // &
                ' microwave brightness temperature'
  else if ( trim(atmos_region) == 'LT' ) then
    atmos_region_use = 'lt'
    long_name = 'NOAA Climate Data Record of regional monthly anomaly for lower tropospheric' // &
                ' microwave brightness temperature'
  else if ( trim(atmos_region) == 'LS' ) then
    atmos_region_use = 'ls'
    long_name = 'NOAA Climate Data Record of regional monthly anomaly for lower stratosphere' // &
                ' microwave brightness temperature'
  else if ( trim(atmos_region) == 'TP' ) then
    atmos_region_use = 'tp'
    long_name = 'NOAA Climate Data Record of regional monthly anomaly for tropopause' // &
                ' microwave brightness temperature'
  else
    call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_reg', &
      ' unexpected atmos_region ???', &
      trim(atmos_region) )
  end if

  vname = 'regional_bt_anomaly_' // trim( atmos_region_use )
  ntype = nf90_float

  units = 'K'
  standard_name = 'brightness_temperature_anomaly'
  fillvalue = -99.99
  valid_min = -15.0
  valid_max = +15.0
  coordinates = 'time region'

  cell_methods = 'area: mean (over region from monthly gridded values)'

  comment_name = 'comment'
  comment_value = ''

  inquire ( file='region_comment.txt',exist=fexist )

  if ( .not.region_only .and. trim(region_def_style) == 'VAR' .and. fexist ) then

    call get_free_logical_unit ( lutemp )
    open (unit=lutemp,file='region_comment.txt',status='old')
    read (lutemp,'(a)') nrec
    close (unit=lutemp,status='keep')

    nend = lnblnk ( nrec )
    if ( nend < 3 ) then
      nend = lnblnk ( nrec )
      call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_reg', &
      ' record in region_comment.txt file too short ???', &
      nrec(1:nend) )
    end if

    nfind = 0
    do nc = 1, nend
      if ( nrec(nc:nc) == '#' ) then
        nfind = nc
        exit
      end if
    end do

    if ( nfind == 0 ) then
      call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_reg', &
      ' no # character in record in region_comment.txt file ???', &
      nrec(1:nend) )
    end if

    comment_value = nrec(nfind+1:nend)

  else if ( .not.region_only .and. trim(region_def_style) == 'VAR' .and. .not.fexist ) then

    write (lulog,'(/a)') ' subroutine setup_define_netcdf_var_reg'
    write (lulog,'(a,a)') ' region_def_style = ', trim ( region_def_style )
    write (lulog,'(a)') ' file region_comment.txt does not exist ???'
    stop

  end if

  inquire ( file='missing_comment.txt',exist=fexist )

  if ( trim(missing_data_style) == 'VAR' .and. fexist ) then

    call get_free_logical_unit ( lutemp )
    open (unit=lutemp,file='missing_comment.txt',status='old')
    read (lutemp,'(a)') nrec
    close (unit=lutemp,status='keep')

    nend = lnblnk ( nrec )
    if ( nend < 3 ) then
      nend = lnblnk ( nrec )
      call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_reg', &
      ' record in missing_comment.txt file too short ???', &
      nrec(1:nend) )
    end if

    nfind = 0
    do nc = 1, nend
      if ( nrec(nc:nc) == '#' ) then
        nfind = nc
        exit
      end if
    end do

    if ( nfind == 0 ) then
      call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_reg', &
      ' no # character in record in missing_comment.txt file ???', &
      nrec(1:nend) )
    end if

    nendp = lnblnk ( comment_value )

    if ( nendp <= 0 ) then
      comment_value = nrec(nfind+1:nend)
    else
      comment_value = comment_value(1:nendp) // ' ' // nrec(nfind+1:nend)
    end if

  else if ( trim(missing_data_style) == 'VAR' .and. .not.fexist ) then

    write (lulog,'(/a)') ' subroutine setup_define_netcdf_var_reg'
    write (lulog,'(a,a)') ' missing_data_style = ', trim ( missing_data_style )
    write (lulog,'(a)') ' file missing_comment.txt does not exist ???'
    stop

  end if

  nendp = lnblnk ( comment_value )

  if ( nendp <= 0 ) then
    comment_name = cmiss
    comment_value = cmiss
  end if

else if ( trim(pmode) == 'NEW_NETCDF' .and. nv == 4 ) then

  new_dim_desc(1) = 'region'
  new_dim_vname(1) = 'region'
  new_dim_ndsize(1) = nreg_max

  dim_desc(1) = 'region'
  dim_vname(1) = 'region'
  dim_ndsize(1) = nreg_max

  if ( trim(atmos_region) == 'MT' ) then
    atmos_region_use = 'mt'
    long_name = 'Trends of NOAA Climate Data Record of regional monthly anomaly for mid-tropospheric' // &
                ' microwave brightness temperature'
  else if ( trim(atmos_region) == 'LT' ) then
    atmos_region_use = 'lt'
    long_name = 'Trends of NOAA Climate Data Record of regional monthly anomaly for lower tropospheric' // &
                ' microwave brightness temperature'
  else if ( trim(atmos_region) == 'LS' ) then
    atmos_region_use = 'ls'
    long_name = 'Trends of NOAA Climate Data Record of regional monthly anomaly for lower stratosphere' // &
                ' microwave brightness temperature'
  else if ( trim(atmos_region) == 'TP' ) then
    atmos_region_use = 'tp'
    long_name = 'Trends of NOAA Climate Data Record of regional monthly anomaly for tropopause' // &
                ' microwave brightness temperature'
  else
    call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_reg', &
      ' unexpected atmos_region ???', &
      trim(atmos_region) )
  end if

  vname = 'regional_bt_anomaly_' // trim( atmos_region_use ) // '_trends'
  ntype = nf90_float

  units = 'K'
  standard_name = 'brightness_temperature_anomaly_trends'
  fillvalue = -99.99
  valid_min = -1.0
  valid_max = +1.0
  coordinates = 'region'

  comment_name = 'comment'
  comment_value = ''

  inquire ( file='region_comment.txt',exist=fexist )

  if ( .not.region_only .and. trim(region_def_style) == 'VAR' .and. fexist ) then

    call get_free_logical_unit ( lutemp )
    open (unit=lutemp,file='region_comment.txt',status='old')
    read (lutemp,'(a)') nrec
    close (unit=lutemp,status='keep')

    nend = lnblnk ( nrec )
    if ( nend < 3 ) then
      nend = lnblnk ( nrec )
      call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_reg', &
      ' record in region_comment.txt file too short ???', &
      nrec(1:nend) )
    end if

    nfind = 0
    do nc = 1, nend
      if ( nrec(nc:nc) == '#' ) then
        nfind = nc
        exit
      end if
    end do

    if ( nfind == 0 ) then
      call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_reg', &
      ' no # character in record in region_comment.txt file ???', &
      nrec(1:nend) )
    end if

    comment_value = nrec(nfind+1:nend)

  else if ( .not.region_only .and. trim(region_def_style) == 'VAR' .and. .not.fexist ) then

    write (lulog,'(/a)') ' subroutine setup_define_netcdf_var_reg'
    write (lulog,'(a,a)') ' region_def_style = ', trim ( region_def_style )
    write (lulog,'(a)') ' file region_comment.txt does not exist ???'
    stop

  end if

  inquire ( file='missing_comment.txt',exist=fexist )

  if ( trim(missing_data_style) == 'VAR' .and. fexist ) then

    call get_free_logical_unit ( lutemp )
    open (unit=lutemp,file='missing_comment.txt',status='old')
    read (lutemp,'(a)') nrec
    close (unit=lutemp,status='keep')

    nend = lnblnk ( nrec )
    if ( nend < 3 ) then
      nend = lnblnk ( nrec )
      call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_reg', &
      ' record in missing_comment.txt file too short ???', &
      nrec(1:nend) )
    end if

    nfind = 0
    do nc = 1, nend
      if ( nrec(nc:nc) == '#' ) then
        nfind = nc
        exit
      end if
    end do

    if ( nfind == 0 ) then
      call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_reg', &
      ' no # character in record in missing_comment.txt file ???', &
      nrec(1:nend) )
    end if

    nendp = lnblnk ( comment_value )

    if ( nendp <= 0 ) then
      comment_value = nrec(nfind+1:nend)
    else
      comment_value = comment_value(1:nendp) // ' ' // nrec(nfind+1:nend)
    end if

  else if ( trim(missing_data_style) == 'VAR' .and. .not.fexist ) then

    write (lulog,'(/a)') ' subroutine setup_define_netcdf_var_reg'
    write (lulog,'(a,a)') ' missing_data_style = ', trim ( missing_data_style )
    write (lulog,'(a)') ' file missing_comment.txt does not exist ???'
    stop

  end if

  nendp = lnblnk ( comment_value )

  if ( nendp <= 0 ) then
    comment_name = cmiss
    comment_value = cmiss
  end if


!kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk

else

  write (emess,'(a,i10)') ' nv= ', nv
  call write_error ( lulog, &
    ' subroutine setup_define_netcdf_var_reg', &
    ' unexpected variable number ???', &
    emess )

end if

end subroutine

end module
!
!***********************************************************************
!
module module_setup_nf90_put_var_reg
implicit none
contains
 
subroutine setup_nf90_put_var_reg ( nv, nt, ngsum, nttsum, nvar_id, vname, atmos_region, &
  c_p1d, r8_p1d, r4_p2d, r4_p1d, i_p1d, i_short_p1d, i_short_p2d, start, count, nddmax, n1, n2 )

!  Global data module

use netcdf
use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_error
use module_get_var_id
use module_get_maxmin_reg
use module_get_maxmin_reg_trends

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
integer, intent(in) :: nt
integer, intent(in) :: ngsum
integer, intent(in) :: nttsum
integer, intent(inout) :: nvar_id
character ( len = * ), intent(inout) :: vname
character ( len = * ), intent(in) :: atmos_region

character ( len = 19 ), pointer, intent(inout) :: c_p1d
real ( kind = double_precision ), pointer, intent(inout), dimension(:) :: r8_p1d
real, pointer, intent(inout), dimension(:,:,:) :: r4_p2d
real, pointer, intent(inout), dimension(:) :: r4_p1d
integer, pointer, intent(inout), dimension(:,:,:) :: i_p1d
integer ( kind = half_precision ), pointer, intent(inout), dimension(:) :: i_short_p1d
integer ( kind = half_precision ), pointer, intent(inout), dimension(:,:) :: i_short_p2d
integer, intent(inout), dimension(nddmax) :: start
integer, intent(inout), dimension(nddmax) :: count
integer, intent(in) :: nddmax

integer, intent(inout) :: n1
integer, intent(inout) :: n2

! Local Declarations

integer :: lnblnk
integer :: nend
integer :: nstop
integer :: nsum
integer :: n
integer :: nr
integer :: i
integer :: j
character ( len = 256 ) :: emess
character ( len = 2 ) :: atmos_region_use

! Local Arrays

nstop = 1
start = 0
count = 0

nullify ( c_p1d )
nullify ( r8_p1d )
nullify ( r4_p2d )
nullify ( r4_p1d )
nullify ( i_p1d )
nullify ( i_short_p1d )
nullify ( i_short_p2d )

if ( trim(pmode) == 'NEW_NETCDF' .and. nv == 1 ) then

  vname = 'time'
  nend = lnblnk( vname )

  start(1) = nt
  count(1) = 1
  i_short_p1d => jday_store_mons(nt:nt)

else if ( trim(pmode) == 'NEW_NETCDF' .and. nv == 2 ) then

  vname = 'time_bounds'
  nend = lnblnk( vname )

  start(1) = 1
  count(1) = 2

  start(2) = nt
  count(2) = 1

  i_short_p2d => time_bnds(1:2,nt:nt)

else if ( trim(pmode) == 'NEW_NETCDF' .and. nv == 3 ) then

  if ( trim(atmos_region) == 'MT' ) then
    atmos_region_use = 'mt'
  else if ( trim(atmos_region) == 'LT' ) then
    atmos_region_use = 'lt'
  else if ( trim(atmos_region) == 'LS' ) then
    atmos_region_use = 'ls'
  else if ( trim(atmos_region) == 'TP' ) then
    atmos_region_use = 'tp'
  else
    call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_reg', &
      ' unexpected atmos_region ???', &
      trim(atmos_region) )
  end if

  vname = 'regional_bt_anomaly_' // trim( atmos_region_use )
  nend = lnblnk( vname )

  start(1) = 1
  start(2) = nt

  count(1) = nreg_max
  count(2) = 1

  do nr = 1, nreg_max
    if ( nint(reg_data_2d(nr,nttsum)) /= nint(rmiss) ) then
      reg_rdata_2d(nr,1,1) = reg_data_2d(nr,nttsum)
!     NOLA-TODAY
      call get_maxmin_reg ( atmos_region, reg_rdata_2d(nr,1,1) )
    else
      reg_rdata_2d(nr,1,1) = rmiss
    end if
  end do

  r4_p2d => reg_rdata_2d

!kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk

else if ( trim(pmode) == 'NEW_NETCDF' .and. nv == 4 ) then

  if ( trim(atmos_region) == 'MT' ) then
    atmos_region_use = 'mt'
  else if ( trim(atmos_region) == 'LT' ) then
    atmos_region_use = 'lt'
  else if ( trim(atmos_region) == 'LS' ) then
    atmos_region_use = 'ls'
  else if ( trim(atmos_region) == 'TP' ) then
    atmos_region_use = 'tp'
  else
    call write_error ( lulog, &
      ' subroutine setup_define_netcdf_var_reg', &
      ' unexpected atmos_region ???', &
      trim(atmos_region) )
  end if

  vname = 'regional_bt_anomaly_' // trim( atmos_region_use ) // '_trends'
  nend = lnblnk( vname )

  start(1) = 1
  count(1) = nreg_max

  do nr = 1, nreg_max
    call get_maxmin_reg_trends ( atmos_region, thold(nr) )
  end do

  r4_p1d => thold

!kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk

else

  write (emess,'(a,i10)') ' nv= ', nv
  call write_error ( lulog, &
    ' subroutine setup_nf90_put_var_reg', &
    ' unexpected variable number ???', &
    emess )

end if

nsum = 0
if ( associated ( c_p1d ) ) nsum = nsum + 1
if ( associated ( r8_p1d ) ) nsum = nsum + 1
if ( associated ( r4_p2d ) ) nsum = nsum + 1
if ( associated ( r4_p1d ) ) nsum = nsum + 1
if ( associated ( i_p1d ) ) nsum = nsum + 1
if ( associated ( i_short_p1d ) ) nsum = nsum + 1
if ( associated ( i_short_p2d ) ) nsum = nsum + 1

if ( nsum /= 1 ) then
  write (emess,'(a,i10)') ' nsum= ', nsum
  call write_error ( lulog, &
    ' subroutine setup_nf90_put_var_reg', &
    ' wrong number of pointers associated ???', &
    emess )
end if

nstop = 1
nend = lnblnk ( vname )

call get_var_id ( nvar_id, vname(1:nend), nstop )

n1 = 1

do n = nddmax, 1, -1
  if ( start(n) > 0 ) then
    n2 = n
    exit
  end if
end do

end subroutine

end module
!
!***********************************************************************
!
module module_read_data_driver
implicit none
contains
 
subroutine read_data_driver ( nv, nf, nta, ntb, npassg, npassg_max, fname, ntsum, ngsum, ftype, &
  done_read, atmos_region )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_read_mon_gridded_anomaly
use module_read_mon_anomaly
use module_read_daily_zonal_anomaly
use module_write_error
use module_read_mon_reg_anomaly

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
integer, intent(in) :: nf
integer, intent(inout) :: nta
integer, intent(inout) :: ntb
integer, intent(in) :: npassg
integer, intent(in) :: npassg_max
character ( len = * ), intent(in) :: fname
integer, intent(inout) :: ntsum
integer, intent(inout) :: ngsum
character ( len = * ), intent(in) :: ftype
logical, intent(inout) :: done_read
character ( len = * ), intent(in) :: atmos_region

! Local Declarations

integer :: nend
integer :: lnblnk

nend = lnblnk ( ftype )
if ( nend <= 0 ) then
  nend = 1
end if

if ( ftype(1:nend) == 'amg' .or. &
     ftype(1:nend) == 'acg' ) then

  call read_mon_gridded_anomaly ( nv, nf, nta, ntb, npassg, npassg_max, fname, &
    ntsum, ngsum, done_read, ftype )

else if ( ftype(1:nend) == 'mam' ) then

  call read_mon_anomaly ( nv, nf, nta, ntb, npassg, npassg_max, fname, ntsum, ngsum )

else if ( ftype(1:nend) == 'amz' ) then

  call read_daily_zonal_anomaly ( nv, nf, nta, ntb, npassg, npassg_max, fname, ntsum )

else if ( ftype(1:nend) == 'ncdc' ) then
 
  call read_mon_reg_anomaly ( nv, nf, nta, ntb, npassg, npassg_max, fname, ntsum, ngsum, &
    done_read, atmos_region )

else

  call write_error ( lulog, &
    ' subroutine read_data_driver', &
    ' unexpected file type ???', &
    ftype )

end if

end subroutine

end module
!
!***********************************************************************
!
module module_do_wget_command
implicit none
contains
 
subroutine do_wget_command ( cv, ftype, data_src )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_get_free_logical_unit
use module_do_chmod

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: cv
character ( len = * ), intent(in) :: ftype
character ( len = * ), intent(in) :: data_src

! Local Declarations

character ( len = 1000 ) :: command
integer :: nend1
integer :: nend2
integer :: nend3
integer :: nend4
integer :: nend5
integer :: nend
integer :: lnblnk
integer :: lutemp

nend5 = lnblnk ( data_dir )
nend1 = lnblnk ( ftype )
nend2 = lnblnk ( cv )
nend3 = lnblnk ( data_src )
nend4 = lnblnk ( run_dir )

call get_free_logical_unit ( lutemp )

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Create and run script to use wget to download data to the specified data directory

open (unit=lutemp,status='unknown',form='formatted',file='run_wget.csh')

write (lutemp,'(a)') '#!/bin/csh'
write (lutemp,'(a)') ' '
write (lutemp,'(a,a,a)') 'rm  ', data_dir(1:nend5), '*.*'
write (lutemp,'(a,a)') 'cd ', data_dir(1:nend5)
write (lutemp,'(a,a,a,a,a,a,a)') &
'/usr/bin/wget -e robots=off -nd -nH -r -l1 --no-parent -A ', &
'"*', &
ftype(1:nend1), &
'*', &
cv(2:nend2), &
'" ', &
data_src(1:nend3)

close (unit=lutemp,status='keep')

call do_chmod ( run_dir, 'run_wget.csh' )

command = ''
nend4 = lnblnk ( run_dir )
write (command,'(a,a)') run_dir(1:nend4), 'run_wget.csh >& run_wget.LOG'
nend = lnblnk ( command )

call system ( command(1:nend) )

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Create and run Perl script to get list of files downloaded by wget script

open (unit=lutemp,status='unknown',form='formatted',file='get_list.pl')

write (lutemp,'(a)') '#!/usr/bin/perl -w'
write (lutemp,'(a)') 'use strict;'
write (lutemp,'(a)') '{'
write (lutemp,'(a)') 'my @fmove;'
write (lutemp,'(a)') 'my $nfiles;'
write (lutemp,'(a)') 'my $nf;'
write (lutemp,'(a,a,a,a,a)') '@fmove = glob "', data_dir(1:nend5), '*', ftype(1:nend1), '*";'
write (lutemp,'(a)') '$nfiles = $#fmove;'
write (lutemp,'(a)') 'open FILE, "> file.list";'
write (lutemp,'(a)') 'for ( $nf = 0; $nf <= $nfiles; $nf++ ) {'
write (lutemp,'(a,a)') 'print FILE $fmove[$nf], ', ' "\\n";'
write (lutemp,'(a)') '}'
write (lutemp,'(a)') 'close FILE;'
write (lutemp,'(a)') '}'

close (unit=lutemp,status='keep')

call do_chmod ( run_dir, 'get_list.pl' )

command = ''
nend4 = lnblnk ( run_dir )
write (command,'(a,a)') run_dir(1:nend4), 'get_list.pl >& get_list.LOG'
nend = lnblnk ( command )

call system ( command(1:nend) )

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!       write (lulog,'(/a)') ' STOPPING IN subroutine do_wget_command'
!       stop

end subroutine

end module
!
!***********************************************************************
!
module module_do_chmod
implicit none
contains
 
subroutine do_chmod ( dir, fname )
 
!  Global data module

!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: dir
character ( len = * ), intent(in) :: fname

! Local Declarations

integer :: nend1
integer :: nend2
integer :: nend3
integer :: lnblnk
character ( len = 1000 ) :: command

nend1 = lnblnk ( dir )
nend2 = lnblnk ( fname )

command = ''
write (command,'(a,a,a)') 'chmod u+x ', dir(1:nend1), fname(1:nend2)
nend3 = lnblnk ( command )

call system( command(1:nend3) )

end subroutine

end module
!
!***********************************************************************
!
module module_get_latlon_grids
implicit none
contains
 
subroutine get_latlon_grids
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

! Local Declarations

integer :: i
integer :: j

dlatlon = 2.50
offset = 0.50 * dlatlon

glon(1) = -180.0 + offset

do i = 2, imax
  glon(i) = glon(i-1) + dlatlon
end do

glat(1) = -90.0 + offset

do j = 2, jmax
  glat(j) = glat(j-1) + dlatlon
end do

glatz(1) = -82.50

do j = 2, jmaxz
  glatz(j) = glatz(j-1) + dlatlon
end do

do j = 1, jmax
  glat_bnds(1,j) = glat(j) - offset
  glat_bnds(2,j) = glat(j) + offset
end do

do i = 1, imax
  glon_bnds(1,i) = glon(i) - offset
  glon_bnds(2,i) = glon(i) + offset
end do

end subroutine

end module
!
!***********************************************************************
!
module module_read_mon_anomaly
implicit none
contains
 
subroutine read_mon_anomaly ( nv, nf, nta, ntb, npassg, npassg_max, fname, ntsum, ngsum )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_get_free_logical_unit
use module_write_error
use module_store_mon_data
use module_handle_special_records
use module_init_arrays_mon_anom

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
integer, intent(in) :: nf
integer, intent(inout) :: nta
integer, intent(inout) :: ntb
integer, intent(in) :: npassg
integer, intent(in) :: npassg_max
character ( len = * ), intent(in) :: fname
integer, intent(inout) :: ntsum
integer, intent(inout) :: ngsum

! Local Declarations

logical :: ldebug
integer :: lutemp
integer :: nend
integer :: lnblnk
integer :: ios
integer :: nr
integer :: n
integer :: nstrt
character ( len = 1000 ) :: nrec
character ( len = 1000 ) :: desc1
character ( len = 1000 ) :: desc2
integer :: ncycle_flag
integer :: ndec_flag

! Local Arrays

integer, parameter :: ndval = 200

real, dimension ( ndval ) :: rhold
integer, dimension ( ndval ) :: ihold
character ( len = 500 ), dimension ( ndval ) :: chold

ldebug = .true.
ldebug = .false.

nend = lnblnk ( fname )
if ( nend <= 0 ) then
  nend = 1
end if

write (lulog,'(/a)') ' ENTERING subroutine read_mon_anomaly'
write (lulog,'(a,a)') ' fname= ', fname(1:nend)
call flush ( lulog )

call get_free_logical_unit ( lutemp )

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  
!MONTHLY MEANS OF MID-TROPOSPHERE MT5.5
!  
!ANNUAL CYCLE BASED ON  81001-110365              12-MON RUNNING MEAN
!  YEAR  MON  GLOBAL     NH      SH    TRPC  NO.DAYS   GLOBAL     NH      SH    TRPC  DAYS
!  1978   12  -0.129  -0.211  -0.047  -0.180   31.    -99.990 -99.990 -99.990 -99.990    42.
!  1979    1  -0.108  -0.310   0.094  -0.135   31.    -99.990 -99.990 -99.990 -99.990    73.
!  1979    2  -0.096  -0.135  -0.057  -0.146   28.    -99.990 -99.990 -99.990 -99.990   101.
!  1979    3  -0.073  -0.091  -0.054  -0.116   31.    -99.990 -99.990 -99.990 -99.990   132.
!  1979    4  -0.090  -0.132  -0.048  -0.003   30.    -99.990 -99.990 -99.990 -99.990   162.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

nend = lnblnk ( fname )
if ( nend <= 0 ) then
  nend = 1
end if

open (unit=lutemp,status='old',form='formatted',file=fname(1:nend),iostat=ios)

if ( ios /= 0 ) then
  call write_error ( lulog, &
  ' subroutine read_mon_anomaly', &
  ' failed to open monthly anomaly file ???', &
  fname(1:nend) )
end if

nstrt = 1
nr = 0
ntsum = 0
ngsum = 0

nta = 2 * ntmax_mons
ntb = 0

call init_arrays_mon_anom

do

  read (lutemp,'(a)',iostat=ios) nrec

  if ( ios == 0 ) then

    nr = nr + 1

    call handle_special_records ( nrec, nr, desc1, desc2, ncycle_flag, ndec_flag )

    if ( ncycle_flag == 1 ) then
      cycle
    end if

!  1978   12  -0.129  -0.211  -0.047  -0.180   31.    -99.990 -99.990 -99.990 -99.990    42.
!23456712345123456781234567812345678123456781234567891234567812345678123456781234567812345678

    ihold = imiss
    rhold = rmiss
    nend = lnblnk( nrec )

    if ( ndec_flag == 0 ) then
      read (nrec(1:nend),'(i7,i5,4f8.3,f9.1,5f8.3)') &
        ihold(1), ihold(2), (rhold(n),n=1,10)
    else if ( ndec_flag == 1 ) then
      read (nrec(1:nend),'(14x,4f8.3)') (rhold(n),n=1,4)
    end if

    call store_mon_data ( nv, nf, nta, ntb, ndec_flag, ihold, rhold, ndval, ntsum, &
      ngsum, npassg, npassg_max )

  else

    close (unit=lutemp,status='keep')
    exit

  end if

end do

end subroutine

end module
!
!***********************************************************************
!
module module_handle_special_records
implicit none
contains
 
subroutine handle_special_records ( nrec, nr, desc1, desc2, ncycle_flag, ndec_flag )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: nrec
integer, intent(in) :: nr
character ( len = * ), intent(inout) :: desc1
character ( len = * ), intent(inout) :: desc2
integer, intent(inout) :: ncycle_flag
integer, intent(inout) :: ndec_flag

! Local Declarations

integer :: nend
integer :: lnblnk
integer :: nfind

ncycle_flag = 0

if ( nr == 1 .or. nr == 3 ) then
  ncycle_flag = 1
  return
end if

if ( nr == 2 ) then
  desc1 = nrec
else if ( nr == 4 ) then
  desc2 = nrec
end if

if ( nr <= 5 ) then
  ncycle_flag = 1
  return
end if

nend = lnblnk ( nrec )

if ( nend <= 0 ) then
  ncycle_flag = 1
  return
end if

nfind = index ( nrec(1:nend) , 'DECADAL' )

if ( nfind == 0 ) then
  ndec_flag = 0
else
  ndec_flag = 1
end if

end subroutine

end module
!
!***********************************************************************
!
module module_store_mon_data
implicit none
contains
 
subroutine store_mon_data ( nv, nf, nta, ntb, ndec_flag, ihold, rhold, ndval, ntsum, &
  ngsum, npassg, npassg_max )

!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_calc_julian_minute
use module_findilev

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
integer, intent(in) :: nf
integer, intent(inout) :: nta
integer, intent(inout) :: ntb
integer, intent(in) :: ndec_flag
integer, intent(in), dimension(ndval) :: ihold
real, intent(in), dimension(ndval) :: rhold
integer, intent(in) :: ndval
integer, intent(inout) :: ntsum
integer, intent(inout) :: ngsum
integer, intent(in) :: npassg
integer, intent(in) :: npassg_max

! Local Declarations

integer :: nyrc
integer :: nmoc
integer :: ndyc
integer :: nhrc
integer :: nminc
integer :: jminc
integer :: ngood
integer :: ngood2
integer :: jmin_beg_use
integer :: jmin_end_use

integer :: ne
integer :: nl
integer :: kbot
integer :: ktop
integer :: nrev
integer :: nstop

logical :: ldebug

ldebug = .false.
!       ldebug = .true.

if ( jmin_beg_exc(nf) > 0 .and. jmin_end_exc(nf) > 0 ) then
  jmin_beg_use = jmin_beg_exc(nf)
  jmin_end_use = jmin_end_exc(nf)
else
  jmin_beg_use = jmin_beg_date
  jmin_end_use = jmin_end_date
end if

if ( ndec_flag == 0 ) then

  nyrc = ihold(1)
  nmoc = ihold(2)
  ndyc = 1
  nhrc = 0
  nminc = 0

!kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk

  call calc_julian_minute ( nyrc, nmoc, ndyc, nhrc, nminc, &
    nyr_base, nmo_base, ndy_base, nhr_base, &
    lulog, jminc )

  kbot = 1
  ktop = ngmax
  nrev = +1
  nstop = 0

  call findilev ( jmin_store_mons, ngmax, jminc, ne, nl, &
    kbot, ktop, ngood, nrev, nstop, lulog )

  ngsum = 0

  if ( ngood == 1 ) then
    if ( jmin_store_mons(ne) == jminc ) then
      ngsum = ne
    else if ( jmin_store_mons(nl) == jminc ) then
      ngsum = nl
    else
      ngood = 0
    end if
  end if

  if ( ngood == 1 ) then
    if ( ngsum < nta ) then
      nta = ngsum
    end if
    if ( ngsum > ntb ) then
      ntb = ngsum
    end if
  end if

  ngood2 = 0

  if ( ngood == 1 ) then
    if ( jminc >= jmin_beg_use .and. jminc <= jmin_end_use ) then
      ngood2 = 1
    end if
  end if

  if ( ngood2 == 1 .and. npassg == 1 ) then
    dflag_store_mons(ngsum) = 1
  end if

!kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk

  if ( npassg == npassg_max .and. ngood2 == 1 ) then
    mon_anom_store%nyr(ngsum) = ihold(1)
    mon_anom_store%nmo(ngsum) = ihold(2)
    mon_anom_store%gb(ngsum) = rhold(1)
    mon_anom_store%nh(ngsum) = rhold(2)
    mon_anom_store%sh(ngsum) = rhold(3)
    mon_anom_store%tp(ngsum) = rhold(4)
    mon_anom_store%ndays(ngsum) = nint( rhold(5) )
    if ( mon_anom_store%ndays(ngsum) < 0 ) then
      mon_anom_store%ndays(ngsum) = -9999
    end if
    mon_anom_store%gb_rm(ngsum) = rhold(6)
    mon_anom_store%nh_rm(ngsum) = rhold(7)
    mon_anom_store%sh_rm(ngsum) = rhold(8)
    mon_anom_store%tp_rm(ngsum) = rhold(9)
    mon_anom_store%ndays_rm(ngsum) = nint( rhold(10) )
    if ( mon_anom_store%ndays_rm(ngsum) < 0 ) then
      mon_anom_store%ndays_rm(ngsum) = -9999
    end if
  end if

  if ( ldebug ) then
    write (lulog,'(/a)') ' subroutine store_mon_data'
    write (lulog,'(a,i10)') ' npassg=', npassg
    write (lulog,'(a,i10)') ' npassg_max=', npassg_max
    write (lulog,'(a,i10)') ' nyrc=', nyrc
    write (lulog,'(a,i10)') ' nmoc=', nmoc
    write (lulog,'(a,i10)') ' ngood=', ngood
    write (lulog,'(a,i10)') ' ngood2=', ngood2
    write (lulog,'(a,i10)') ' ntsum=', ntsum
    write (lulog,'(a,i10)') ' ngsum=', ngsum
  end if

else if ( npassg == npassg_max .and. ndec_flag == 1 ) then

  mon_anom_store%gb_decadal_trend = rhold(1)
  mon_anom_store%nh_decadal_trend = rhold(2)
  mon_anom_store%sh_decadal_trend = rhold(3)
  mon_anom_store%tp_decadal_trend = rhold(4)

end if

end subroutine

end module
!
!***********************************************************************
!
module module_get_cdate
implicit none
contains
 
subroutine get_cdate ( nyrc, nmoc, ndyc, cdate, lulog )
 
!  Global data module

!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nyrc
integer, intent(in) :: nmoc
integer, intent(in) :: ndyc
character ( len = * ), intent(inout) :: cdate
integer, intent(in) :: lulog

! Local Declarations

integer :: nhr
integer :: nmin
integer :: nsec
logical :: ldebug

nhr = 0
nmin = 0
nsec = 0

ldebug = .false.
!       ldebug = .true.

cdate = ''

write (cdate,'(i4,5(a1,i2.2))') &
  nyrc, '-', &
  nmoc, '-', &
  ndyc, '-', &
  nhr, '-', &
  nmin, '-', &
  nsec

if ( ldebug ) then
  write (lulog,'(/a)') ' subroutine get_cdate'
  write (lulog,'(a,i10)') ' nyrc=', nyrc
  write (lulog,'(a,i10)') ' nmoc=', nmoc
  write (lulog,'(a,i10)') ' ndyc=', ndyc
  write (lulog,'(a,i10)') ' nhr=', nhr
  write (lulog,'(a,i10)') ' nmin=', nmin
  write (lulog,'(a,i10)') ' nsec=', nsec
  write (lulog,'(a,a)') ' cdate= ', cdate
end if

end subroutine

end module
!
!***********************************************************************
!
module module_read_mon_gridded_anomaly
implicit none
contains
 
subroutine read_mon_gridded_anomaly ( nv, nf, nta, ntb, npassg, npassg_max, fname, &
  ntsum, ngsum, done_read, ftype )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_get_free_logical_unit
use module_write_error
use module_get_epoch_seconds
use module_get_cdate
use module_calc_julian_minute
use module_findilev
use module_init_arrays_mon_gridded_anom
use module_inv_julian_minute
use module_allocate_arrays_mon_gridded_anom

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
integer, intent(in) :: nf
integer, intent(inout) :: nta
integer, intent(inout) :: ntb
integer, intent(in) :: npassg
integer, intent(in) :: npassg_max
character ( len = * ), intent(in) :: fname
integer, intent(inout) :: ntsum
integer, intent(inout) :: ngsum
logical, intent(inout) :: done_read
character ( len = * ), intent(in) :: ftype

! Local Declarations

integer :: lutemp
integer :: nend
integer :: lnblnk
integer :: ios
character ( len = 1000 ) :: nrec
integer :: nr
integer :: nstrt
integer :: ndate
integer :: isum
integer :: jsum
integer :: n
integer :: nyrc
integer :: nmoc
integer :: ndyc
integer :: nhrc
integer :: nminc
integer :: jminc
integer :: ngood
character ( len = 12 ) :: dtype
character ( len = 19 ) :: cdate
logical :: ldebug
integer :: i
integer :: j
integer :: ne
integer :: nl
integer :: kbot
integer :: ktop
integer :: ngood2
integer :: nrev
integer :: nstop
integer :: jmin_beg_use
integer :: jmin_end_use

integer :: nyr3
integer :: nmo3
integer :: ndy3
integer :: nhr3
integer :: nmin3

! Local Arrays

integer, parameter :: ndval = 200

real, dimension ( ndval ) :: rhold
integer, dimension ( ndval ) :: ihold
character ( len = 500 ), dimension ( ndval ) :: chold

nend = lnblnk ( fname )
if ( nend <= 0 ) then
  nend = 1
end if

write (lulog,'(/a)') ' ENTERING subroutine read_mon_gridded_anomaly'
write (lulog,'(a,i10)') ' nf=', nf
write (lulog,'(a,i10)') ' nv=', nv
write (lulog,'(a,a)') ' fname= ', fname(1:nend)
call flush ( lulog )

if ( .not.allocated(month_store) ) then
  write (lulog,'(a)') ' month_store is NOT ALLOCATED'
  call allocate_arrays_mon_gridded_anom ( .false. )
  month_store = imiss
end if

if ( ngsum == 0 ) then
  month_store = imiss
end if

nta = 2 * ntmax_mons
ntb = 0

call flush ( lulog )

call get_free_logical_unit ( lutemp )

open (unit=lutemp,status='old',form='formatted',file=fname(1:nend),iostat=ios)

if ( ios /= 0 ) then
  call write_error ( lulog, &
  ' subroutine read_mon_gridded_anomaly', &
  ' failed to open monthly anomaly file ???', &
  fname(1:nend) )
end if

nr = 0
nstrt = 1
isum = 0
jsum = 1
ntsum = 0
ndate = 0

if ( jmin_beg_exc(nf) > 0 .and. jmin_end_exc(nf) > 0 ) then
  jmin_beg_use = jmin_beg_exc(nf)
  jmin_end_use = jmin_end_exc(nf)
else
  jmin_beg_use = jmin_beg_date
  jmin_end_use = jmin_end_date
end if

call inv_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_beg_use )

write (lulog,'(a,a)') ' ftype= ', trim ( ftype )
write (lulog,'(a,i10)') ' nyr3=', nyr3
write (lulog,'(a,i10)') ' nv=', nv
write (lulog,'(a,i10)') ' jmin_beg_exc(nf)=', jmin_beg_exc(nf)
write (lulog,'(a,i10)') ' jmin_end_exc(nf)=', jmin_end_exc(nf)
call flush ( lulog )

ldebug = .true.
!       ldebug = .false.

! Each yearly file has up to 12 months of data

do

  read (lutemp,'(a)',iostat=ios) nrec

  if ( ios == 0 ) then

    nr = nr + 1

    if ( nr == 1 ) then
!     month_store = imiss
!     NOLA
      ndate = 1
    else if ( isum == imax .and. jsum == jmax ) then
      ndate = 1
    else
      ndate = 0
    end if

    ihold = imiss
    nend = lnblnk ( nrec )

    if ( ndate == 1 ) then
      read (nrec(1:nend),'(3i12,a12)') ihold(1), ihold(2), ihold(3), dtype
    else
      read (nrec(1:nend),'(16i5)') (ihold(n),n = 1,16)
    end if

!   NOLA-HERE

    if ( ndate == 1 ) then

      if ( trim(ftype) == 'amg' ) then
        nyrc = ihold(2)
        nmoc = ihold(3)
      else if ( trim(ftype) == 'acg' ) then
        nyrc = nyr3
        nmoc = ihold(1)
      end if

      ndyc = 1
      nhrc = 0
      nminc = 0

      ndate = 0
      isum = 0
      jsum = 1

      call calc_julian_minute ( nyrc, nmoc, ndyc, nhrc, nminc, &
        nyr_base, nmo_base, ndy_base, nhr_base, &
        lulog, jminc )

      kbot = 1
      ktop = ngmax
      nrev = +1
      nstop = 0

      call findilev ( jmin_store_mons, ngmax, jminc, ne, nl, &
        kbot, ktop, ngood, nrev, nstop, lulog )

      ngsum = 0

      if ( ngood == 1 ) then
        if ( jmin_store_mons(ne) == jminc ) then
          ngsum = ne
        else if ( jmin_store_mons(nl) == jminc ) then
          ngsum = nl
        else
          ngood = 0
        end if
      end if

      if ( ngood == 1 ) then
        if ( ngsum < nta ) then
          nta = ngsum
        end if
        if ( ngsum > ntb ) then
          ntb = ngsum
        end if
      end if

      ngood2 = 0
      ntsum = 0
      done_read = .false.

      if ( jminc > jmin_end_use ) then
        done_read = .true.
      end if

      if ( ngood == 1 ) then
        if ( jminc >= jmin_beg_use .and. jminc <= jmin_end_use ) then
          ngood2 = 1
          ntsum = nmoc
          if ( ngsum > ngmax ) then
            write (lulog,'(/a)') ' subroutine read_mon_gridded_anomaly'
            write (lulog,'(a)') ' ngsum > ngmax ???'
            write (lulog,'(a,i10)') ' ngsum=', ngsum
            write (lulog,'(a,i10)') ' ngmax=', ngmax
            stop
          end if
          month_store(ngsum) = nmoc
        end if
      end if

      if ( ngood2 == 1 .and. npassg == 1 ) then
        dflag_store_mons(ngsum) = 1
      end if

      if ( ldebug ) then
        write (lulog,'(/a)') ' ESEC: subroutine read_mon_gridded_anomaly'
        write (lulog,'(a)') ' read date record'
        write (lulog,'(a,a)') ' ftype = ', trim ( ftype )
        write (lulog,'(a,i10)') ' npassg=', npassg
        write (lulog,'(a,i10)') ' npassg_max=', npassg_max
        write (lulog,'(a,i10)') ' ntsum=', ntsum
        write (lulog,'(a,i10)') ' ngsum=', ngsum
        write (lulog,'(a,i10)') ' nyrc=', nyrc
        write (lulog,'(a,i10)') ' nmoc=', nmoc
        write (lulog,'(a,i10)') ' ngood=', ngood
        write (lulog,'(a,i10)') ' ngood2=', ngood2
        write (lulog,'(a,i10)') ' nta=', nta
        write (lulog,'(a,i10)') ' ntb=', ntb
        write (lulog,'(a,l10)') ' done_read = ', done_read
        if ( ngsum > 0 ) then
          write (lulog,'(a,a)') ' cdate_monthly_hold(ngsum)= ', &
            cdate_monthly_hold(ngsum)
          write (lulog,'(a,i10)') ' month_store(ngsum)= ', month_store(ngsum)
          write (lulog,'(a,i10)') ' dflag_store_mons(ngsum)= ', dflag_store_mons(ngsum)
        end if

        call inv_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
          nyr_base, nmo_base, ndy_base, nhr_base, &
          lulog, jmin_beg_use )

        write (lulog,'(a,i10)') ' JMIN_BEG_USE: nyr=', nyr3
        write (lulog,'(a,i10)') ' JMIN_BEG_USE: nmo=', nmo3
        write (lulog,'(a,i10)') ' JMIN_BEG_USE: ndy=', ndy3
        write (lulog,'(a,i10)') ' JMIN_BEG_USE: nhr=', nhr3

        call inv_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
          nyr_base, nmo_base, ndy_base, nhr_base, &
          lulog, jmin_end_use )

        write (lulog,'(a,i10)') ' JMIN_END_USE: nyr=', nyr3
        write (lulog,'(a,i10)') ' JMIN_END_USE: nmo=', nmo3
        write (lulog,'(a,i10)') ' JMIN_END_USE: ndy=', ndy3
        write (lulog,'(a,i10)') ' JMIN_END_USE: nhr=', nhr3

        call inv_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
          nyr_base, nmo_base, ndy_base, nhr_base, &
          lulog, jminc )

        write (lulog,'(a,i10)') ' JMINC: nyr=', nyr3
        write (lulog,'(a,i10)') ' JMINC: nmo=', nmo3
        write (lulog,'(a,i10)') ' JMINC: ndy=', ndy3
        write (lulog,'(a,i10)') ' JMINC: nhr=', nhr3

        call flush ( lulog )
      end if

      if ( done_read ) then
        close (unit=lutemp,status='keep')
        return
      end if

      if ( nr == 1 ) then
        call init_arrays_mon_gridded_anom
      end if

    else if ( ndate == 0 ) then

      if ( isum == imax ) then
        jsum = jsum + 1
        if ( jsum > jmax ) then
          call write_error ( lulog, &
          ' subroutine read_mon_gridded_anomaly', &
          ' jsum > jmax ???', &
          fname )
        end if
        isum = 0
      end if

      do n = 1, ndval
        if ( ihold(n) /= imiss ) then
          isum = isum + 1
          if ( isum > imax ) then
            call write_error ( lulog, &
            ' subroutine read_mon_gridded_anomaly', &
            ' isum > imax ???', &
            fname )
          end if
          if ( ngood2 == 1 .and. npassg == npassg_max ) then
            ndata(isum,jsum,ntsum) = ihold(n)
          end if
        else
          exit
        end if
      end do

    end if

  else

    close (unit=lutemp,status='keep')
    exit

  end if

end do

if ( isum /= imax .or. jsum /= jmax ) then
  call write_error ( lulog, &
  ' subroutine read_mon_gridded_anomaly', &
  ' incomplete read ???', &
  fname )
end if

!       write (lulog,'(/a)') ' LEAVING subroutine read_mon_gridded_anomaly'
!       call flush ( lulog )

!       write (lulog,'(/a)') ' STOPPING IN subroutine read_mon_gridded_anomaly'
!       call flush ( lulog )
!       stop

end subroutine

end module
!
!***********************************************************************
!
module module_check_zonal_status
implicit none
contains
 
subroutine check_zonal_status ( ndate, nstore, ihold, ndval, jsum )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: ndate
integer, intent(inout) :: nstore
integer, intent(in), dimension(ndval) :: ihold
integer, intent(in) :: ndval
integer, intent(inout) :: jsum

! Local Declarations

integer :: n

if ( ndate == 0 ) then

  do n = 1, ndval
    if ( ihold(n) == imiss ) then
      exit
    end if
    jsum = jsum + 1
    ndataz(jsum) = ihold(n)
  end do

  if ( jsum == jmaxz ) then
    nstore = 1
  else
    nstore = 0
  end if

else if ( ndate == 1 ) then

  nstore = 1

end if

end subroutine

end module
!
!***********************************************************************
!
module module_read_daily_zonal_anomaly
implicit none
contains
 
subroutine read_daily_zonal_anomaly ( nv, nf, nta, ntb, npassg, npassg_max, fname, ntsum )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_get_free_logical_unit
use module_write_error
use module_allocate_arrays_day_anom
use module_store_daily_zonal_data
use module_check_zonal_status

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
integer, intent(in) :: nf
integer, intent(inout) :: nta
integer, intent(inout) :: ntb
integer, intent(in) :: npassg
integer, intent(in) :: npassg_max
character ( len = * ), intent(in) :: fname
integer, intent(inout) :: ntsum

! Local Declarations

logical :: ldebug
integer :: lutemp
integer :: nend
integer :: lnblnk
integer :: ios
integer :: nr
integer :: nstrt
character ( len = 1000 ) :: nrec
integer :: ndate
integer :: jsum
integer :: nstore
integer :: n
integer :: ngood2
character ( len = 5 ) :: dtype
integer :: ndr

! Local Arrays

integer, parameter :: ndval = 200

real, dimension ( ndval ) :: rhold
integer, dimension ( ndval ) :: ihold
character ( len = 500 ), dimension ( ndval ) :: chold

ldebug = .true.
ldebug = .false.

nend = lnblnk ( fname )
if ( nend <= 0 ) then
  nend = 1
end if

write (lulog,'(/a)') ' ENTERING subroutine read_daily_zonal_anomaly'
write (lulog,'(a,a)') ' fname= ', fname(1:nend)
call flush ( lulog )

!***********************************************************************
!1978  320   319   MT -238  -62 -413 -254
! 203  182  212  213  201  170  146  129   99   75   26  -19  -22  -38  -54  -33
! -25  -25  -23  -40  -51  -44  -44  -41  -40  -30  -29  -30  -27  -29  -26  -28
! -25  -33  -29  -42  -38  -20  -24  -11    1  -12  -27  -56  -74  -58  -97  -86
!-111  -63  -44  -29    2   25   13  -20  -36  -59  -90  -66  -66 -107  -74  -27
! -27  -58  -65
!1978  321   320   MT -243  -52 -434 -236
! 204  198  220  220  211  183  155  133  105   72   34    1  -13  -35  -49  -38
! -39  -43  -51  -48  -51  -48  -40  -35  -35  -29  -29  -21  -21  -25  -24  -31
! -23  -28  -29  -39  -34  -20  -22  -14    0   -9  -22  -56  -73  -64 -102  -99
! -99  -66  -42  -32   -8    9    1  -22  -29  -49  -67  -61  -73  -99  -92  -79
!***********************************************************************

call get_free_logical_unit ( lutemp )

nend = lnblnk ( fname )
if ( nend <= 0 ) then
  nend = 1
end if

open (unit=lutemp,status='old',form='formatted',file=fname(1:nend),iostat=ios)

if ( ios /= 0 ) then
  call write_error ( lulog, &
  ' subroutine read_daily_zonal_anomaly', &
  ' failed to open daily anomaly file ???', &
  fname(1:nend) )
end if

nstrt = 1
nr = 0
ntsum = 0
ngood2 = 0

do

  read (lutemp,'(a)',iostat=ios) nrec

  if ( ios == 0 ) then

    nr = nr + 1

    if ( nr == 1 ) then
      ndate = 1
      jsum = 0
    else if ( jsum == jmaxz ) then
      ndate = 1
      jsum = 0
    else
      ndate = 0
    end if

    ihold = imiss

    if ( ndate == 1 ) then
      nend = lnblnk ( nrec )
      read (nrec(1:nend),'(2i5,i6,a5,4i5)') &
        ihold(1), ihold(2), ihold(3), dtype, &
        ihold(4), ihold(5), ihold(6), ihold(7)
      ndr = 0
    else
      nend = lnblnk ( nrec )
      ndr = ndr + 1
      if ( ndr <= 4 ) then
        read (nrec(1:nend),'(16i5)',iostat=ios) (ihold(n),n=1,16)
      else
        read (nrec(1:nend),'(3i5)',iostat=ios) (ihold(n),n=1,3)
      end if
    end if

    call check_zonal_status ( ndate, nstore, ihold, ndval, jsum )

    if ( nstore == 1 ) then
      call store_daily_zonal_data ( nv, nf, nta, ntb, npassg, npassg_max, &
        ndate, ihold, ndval, ntsum, ngood2 )
    end if
 
    if ( ldebug ) then
      write (lulog,'(/a)') ' subroutine read_daily_zonal_anomaly'
      write (lulog,'(a,i10)') ' npassg=', npassg
      write (lulog,'(a,i10)') ' npassg_max=', npassg_max
      write (lulog,'(a,i10)') ' nr=', nr
      write (lulog,'(a,i10)') ' ndate=', ndate
      write (lulog,'(a,i10)') ' jsum=', jsum
      write (lulog,'(a,i10)') ' nstore=', nstore
      write (lulog,'(a,i10)') ' ngood2=', ngood2
      write (lulog,'(a,i10)') ' ntsum=', ntsum
      write (lulog,'(a,i10)') ' ndr=', ndr
    end if

  else

    close (unit=lutemp,status='keep')
    exit

  end if

end do

end subroutine

end module
!
!***********************************************************************
!
module module_store_daily_zonal_data
implicit none
contains
 
subroutine store_daily_zonal_data ( nv, nf, nta, ntb, npassg, npassg_max, ndate, &
  ihold, ndval, ntsum, ngood2 )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_inverse_julian
use module_calc_julian_minute
use module_findilev

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
integer, intent(in) :: nf
integer, intent(inout) :: nta
integer, intent(inout) :: ntb
integer, intent(in) :: npassg
integer, intent(in) :: npassg_max
integer, intent(in) :: ndate
integer, intent(in), dimension(ndval) :: ihold
integer, intent(in) :: ndval
integer, intent(inout) :: ntsum
integer, intent(inout) :: ngood2

! Local Declarations

integer :: nyrc
integer :: nmoc
integer :: ndyc
integer :: nhrc
integer :: nminc
integer :: jminc
integer :: jday
integer :: j
character ( len = 19 ) :: cdate
logical :: ldebug
integer :: jmin_beg_use
integer :: jmin_end_use
integer :: ne
integer :: nl
integer :: kbot
integer :: ktop
integer :: nrev
integer :: nstop
integer :: ngood

ldebug = .false.
!       ldebug = .true.

!1978  320   319   MT -239  -63 -415 -255
! 204  184  213  214  201  170  146  128   99   75   26  -19  -23  -38  -55  -34
! -26  -26  -24  -40  -52  -44  -44  -41  -40  -30  -29  -30  -27  -29  -26  -28
! -25  -33  -29  -42  -38  -21  -24  -11    1  -12  -27  -56  -73  -58  -97  -86
!-111  -63  -45  -29    2   24   12  -21  -37  -60  -91  -66  -66 -107  -74  -27
! -27  -58  -64

if ( ngood2 == 1 .and. ndate == 0 .and. npassg == npassg_max ) then

  do j = 1, jmaxz
    day_anom_store%za(ntsum,j) = ndataz(j)
  end do

else if ( ndate == 1 ) then

  if ( jmin_beg_exc(nf) > 0 .and. jmin_end_exc(nf) > 0 ) then
    jmin_beg_use = jmin_beg_exc(nf)
    jmin_end_use = jmin_end_exc(nf)
  else
    jmin_beg_use = jmin_beg_date
    jmin_end_use = jmin_end_date
  end if

  nyrc = ihold(1)
  jday = ihold(2)
  call inverse_julian ( nmoc, ndyc, nyrc, jday )
  nhrc = 0
  nminc = 0

!kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk

  call calc_julian_minute ( nyrc, nmoc, ndyc, nhrc, nminc, &
    nyr_base, nmo_base, ndy_base, nhr_base, &
    lulog, jminc )

  kbot = 1
  ktop = ndmax
  nrev = +1
  nstop = 0

  call findilev ( jmin_store_days, ndmax, jminc, ne, nl, &
    kbot, ktop, ngood, nrev, nstop, lulog )

  ntsum = 0

  if ( ngood == 1 ) then
    if ( jmin_store_days(ne) == jminc ) then
      ntsum = ne
    else if ( jmin_store_days(nl) == jminc ) then
      ntsum = nl
    else
      ngood = 0
    end if
  end if

  if ( ngood == 1 ) then
    if ( ntsum < nta ) then
      nta = ntsum
    end if
    if ( ntsum > ntb ) then
      ntb = ntsum
    end if
  end if

  ngood2 = 0

  if ( ngood == 1 ) then
    if ( jminc >= jmin_beg_use .and. jminc <= jmin_end_use ) then
      ngood2 = 1
    end if
  end if

  if ( ngood2 == 1 .and. npassg == 1 ) then
    dflag_store_days(ntsum) = 1
  end if

!kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk

  if ( ngood2 == 1 .and. npassg == npassg_max ) then
    day_anom_store%nyr(ntsum) = ihold(1)
    day_anom_store%jday(ntsum) = ihold(2)
    day_anom_store%jdayc(ntsum) = ihold(3)
    day_anom_store%gb(ntsum) = ihold(4)
    day_anom_store%nh(ntsum) = ihold(5)
    day_anom_store%sh(ntsum) = ihold(6)
    day_anom_store%tp(ntsum) = ihold(7)
  end if

end if

if ( ldebug .and. ndate == 1 ) then
  write (lulog,'(/a)') ' ESEC: subroutine store_daily_zonal_data'
  write (lulog,'(a,i10)') ' npassg=', npassg
  write (lulog,'(a,i10)') ' npassg_max=', npassg_max
  write (lulog,'(a,i10)') ' ndate=', ndate
  write (lulog,'(a,i10)') ' ngood=', ngood
  write (lulog,'(a,i10)') ' ngood2=', ngood2
  write (lulog,'(a,i10)') ' nyrc=', nyrc
  write (lulog,'(a,i10)') ' nmoc=', nmoc
  write (lulog,'(a,i10)') ' ndyc=', ndyc
  write (lulog,'(a,i10)') ' ntsum=', ntsum
end if

end subroutine

end module
!
!***********************************************************************
!
module module_read_mon_reg_anomaly
implicit none
contains
 
subroutine read_mon_reg_anomaly ( nv, nf, nta, ntb, npassg, npassg_max, fname, ntsum, ngsum, &
  done_read, atmos_region )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_get_free_logical_unit
use module_write_error
use module_store_mon_reg_data
use module_init_arrays_mon_regional_anom
use module_handle_special_records_reg
use module_set_mon_reg_data_to_missing

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
integer, intent(in) :: nf
integer, intent(inout) :: nta
integer, intent(inout) :: ntb
integer, intent(in) :: npassg
integer, intent(in) :: npassg_max
character ( len = * ), intent(in) :: fname
integer, intent(inout) :: ntsum
integer, intent(inout) :: ngsum
logical, intent(inout) :: done_read
character ( len = * ), intent(in) :: atmos_region

! Local Declarations

logical :: ldebug
integer :: lutemp
integer :: nend
integer :: lnblnk
integer :: ios
integer :: nr
integer :: n
integer :: nstrt
character ( len = 1000 ) :: nrec
integer :: ncycle_flag
integer :: nysum
integer :: ntrend
integer :: nrsum

character ( len = 1000 ) :: format
character ( len = 1000 ) :: fkeep
integer :: nc
integer :: nendf
integer :: nsum

! Local Arrays

integer, parameter :: ndval = 200

real, dimension ( ndval ) :: rhold
integer, dimension ( ndval ) :: ihold
character ( len = 500 ), dimension ( ndval ) :: chold

ldebug = .true.
ldebug = .false.

if ( nreg_max > ndval ) then
  write (lulog,'(/a)') ' subroutine read_mon_reg_anomaly'
  write (lulog,'(a)') ' nreg_max > ndval ???'
  write (lulog,'(a,i10)') ' nreg_max=', nreg_max
  write (lulog,'(a,i10)') ' ndval=', ndval
  stop
end if

nend = lnblnk ( fname )
if ( nend <= 0 ) then
  nend = 1
end if

!       write (lulog,'(/a)') ' ENTERING subroutine read_mon_reg_anomaly'
!       write (lulog,'(a,a)') ' fname= ', fname(1:nend)
!       call flush ( lulog )

call get_free_logical_unit ( lutemp )

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Year Mo Globe  Land Ocean   NH   Land Ocean   SH   Land Ocean Trpcs  Land Ocean NoExt  Land Ocean SoExt  Land Ocean NoPol  Land Ocean SoPol  Land Ocean USA48 USA49  AUST
!1978 12 -0.28 -0.36 -0.24 -0.30 -0.41 -0.20 -0.26 -0.26 -0.26 -0.21 -0.22 -0.21 -0.37 -0.51 -0.20 -0.26 -0.21 -0.28 -0.61 -1.29  0.47 -0.20 -0.20 -0.21 -1.60 -1.46 -0.50
!1979  1 -0.25 -0.40 -0.17 -0.47 -0.69 -0.26 -0.03  0.17 -0.10 -0.12 -0.11 -0.13 -0.67 -0.89 -0.40  0.03  0.41 -0.07 -0.45 -1.03  0.46  0.25  0.43  0.10 -3.75 -2.77  1.22
!1979  2 -0.26 -0.46 -0.14 -0.41 -0.57 -0.26 -0.10 -0.24 -0.05  0.00  0.07 -0.03 -0.65 -0.82 -0.43 -0.13 -0.39 -0.06 -2.86 -3.15 -2.39 -0.30 -0.68 -0.01 -2.41 -2.48 -0.17
!1979  3 -0.26 -0.34 -0.21 -0.35 -0.36 -0.34 -0.17 -0.30 -0.13 -0.10 -0.15 -0.08 -0.51 -0.46 -0.57 -0.17 -0.31 -0.13 -0.94 -1.14 -0.64 -0.41 -0.89 -0.03 -0.52 -0.34  0.32
! skipped records
!2015 12  0.56  0.61  0.53  0.69  0.72  0.66  0.42  0.40  0.43  0.70  0.34  0.89  0.71  0.89  0.49  0.24  0.30  0.23 -0.35  0.06 -1.00  0.41  0.48  0.35  1.94  1.52  0.14
!2016  1  0.62  0.56  0.66  0.85  0.73  0.96  0.40  0.23  0.45  0.88  0.63  1.00  0.87  0.81  0.95  0.11 -0.22  0.20  3.16  2.95  3.51 -0.45 -0.44 -0.45 -0.43  0.10 -0.60
!Year Mo Globe  Land Ocean   NH   Land Ocean   SH   Land Ocean Trpcs  Land Ocean NoExt  Land Ocean SoExt  Land Ocean NoPol  Land Ocean SoPol  Land Ocean USA48 USA49  AUST
     
!Trend    0.14  0.18  0.12  0.20  0.23  0.17  0.09  0.10  0.09  0.08  0.11  0.07  0.26  0.27  0.25  0.09  0.09  0.09  0.43  0.40  0.48 -0.02  0.00 -0.03  0.23  0.23  0.15
    
! NOTE:  New Reference for annual cycle 1981-2010 
! NOTE:  Version 5.6 as of Jun 2013 
! NOTE:  3/13 More accurate grid weighting for USA48
! NOTE:  3/13 Alaska added in USA49, Australia added
    
! GL 85S-85N, NH 0-85N, SH 85S-0, TRPCS 20S-20N
! NoExt 20N-85N, SoExt 85S-20S, NoPol 60N-85N, SoPol 85S-60S
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

nend = lnblnk ( fname )
if ( nend <= 0 ) then
  nend = 1
end if

open (unit=lutemp,status='old',form='formatted',file=fname(1:nend),iostat=ios)

if ( ios /= 0 ) then
  call write_error ( lulog, &
  ' subroutine read_mon_reg_anomaly', &
  ' failed to open monthly anomaly file ???', &
  fname(1:nend) )
end if

nstrt = 1
nr = 0
ntsum = 0
ngsum = 0

nta = 2 * ntmax_mons
ntb = 0
done_read = .false.

nysum = 0
ntrend = 0
nrsum = 0
rname = cmiss
region_lat1 = rmiss
region_lat2 = rmiss
note_hold = cmiss
note_hold_sum = 0
region_def_hold = cmiss
region_def_hold_sum = 0
thold = rmiss

call init_arrays_mon_regional_anom

! Make dynamic format statement in case number of regions changes

write (format,'(a,i10,a)') '(i5,i3,', nreg_max, 'f6.2)'

fkeep = format
format = ''
nendf = lnblnk ( fkeep )

nsum = 0

do nc = 1, nendf
  if ( fkeep(nc:nc) == ' ' ) then
    cycle
  end if
  nsum = nsum + 1
  format(nsum:nsum) = fkeep(nc:nc)
end do

nendf = lnblnk ( format )

write (lulog,'(/a)') ' subroutine read_mon_reg_anomaly'
write (lulog,'(a,a)') ' format = ', trim ( format )

do

  read (lutemp,'(a)',iostat=ios) nrec

  if ( ios == 0 ) then

    nr = nr + 1
    nend = lnblnk( nrec )

    call handle_special_records_reg ( nrec, nysum, ntrend, ncycle_flag, rname, &
      region_lat1, region_lat2, thold, nreg_max, nrsum, &
      note_hold, note_hold_max, note_hold_sum, &
      region_def_hold, region_def_hold_max, region_def_hold_sum, &
      rmiss, imiss, cmiss, lulog )

    if ( ncycle_flag == 1 ) then
      cycle
    end if

    ihold = imiss
    rhold = rmiss
    nend = lnblnk( nrec )

    read (nrec(1:nend),fmt=format(1:nendf)) &
      ihold(1), ihold(2), (rhold(n),n=1,nreg_max)

    call store_mon_reg_data ( nv, nf, nta, ntb, ihold, rhold, ndval, ntsum, &
      ngsum, npassg, npassg_max )

  else

    close (unit=lutemp,status='keep')
    exit

  end if

end do

if ( trim_tls_2005 ) then
  call set_mon_reg_data_to_missing ( atmos_region )
end if

!       write (lulog,'(/a)') ' LEAVING subroutine read_mon_reg_anomaly'
!       call flush ( lulog )

end subroutine

end module
!
!***********************************************************************
!
module module_set_mon_reg_data_to_missing
implicit none
contains
 
subroutine set_mon_reg_data_to_missing ( atmos_region )

!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_calc_julian_minute
use module_inv_julian_minute

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: atmos_region

! Local Declarations

integer :: nyr3
integer :: nmo3
integer :: ndy3
integer :: nhr3
integer :: nmin3
integer :: jmin_bad
integer :: nt
integer :: nd
integer :: nbad
integer :: nfind
integer :: nend
integer :: lnblnk
logical :: ldebug

!kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk
!REGION REVIEW
!  nr                                              region        lim1      lim2
!   1                                              global       -85.0      85.0
!   2                                         global_land       -85.0      85.0
!   3                                        global_ocean       -85.0      85.0
!   4                                 northern_hemisphere         0.0      85.0
!   5                            northern_hemisphere_land         0.0      85.0
!   6                           northern_hemisphere_ocean         0.0      85.0
!   7                                 southern_hemisphere       -85.0       0.0
!   8                            southern_hemisphere_land       -85.0       0.0
!   9                           southern_hemisphere_ocean       -85.0       0.0
!  10                                             tropics       -20.0      20.0
!  11                                        tropics_land       -20.0      20.0
!  12                                       tropics_ocean       -20.0      20.0
!  13                               northern_extratropics        20.0      85.0
!  14                          northern_extratropics_land        20.0      85.0
!  15                         northern_extratropics_ocean        20.0      85.0
!  16                               southern_extratropics       -85.0     -20.0
!  17                          southern_extratropics_land       -85.0     -20.0
!  18                         southern_extratropics_ocean       -85.0     -20.0
!  19                                         north_polar        60.0      85.0
!  20                                    north_polar_land        60.0      85.0
!  21                                   north_polar_ocean        60.0      85.0
!  22                                         south_polar       -85.0     -60.0
!  23                                    south_polar_land       -85.0     -60.0
!  24                                   south_polar_ocean       -85.0     -60.0
!  25                 contiguous_united_states_of_america     -9999.0   -9999.0
!  26     contiguous_united_states_of_america_plus_alaska     -9999.0   -9999.0
!  27                                           australia     -9999.0   -9999.0
!kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk

! HERE-KGD

if ( trim(atmos_region) /= 'LS' ) then
  return
end if

ldebug = .false.
!       ldebug = .true.

nyr3 = 2005
nmo3 = 1
ndy3 = 1
nhr3 = 0
nmin3 = 0

call calc_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_bad )

! Set values to missing for Jan 1, 2015 onwards for Lower Stratospheric
! data for regions which are not fully zonal.

do nt = 1, ngmax

  if ( jmin_store_mons(nt) < jmin_bad ) then
    cycle
  end if

  do nd = 1, nreg_max
    nbad = 0
    nend = lnblnk ( rname(nd) )
    nfind = index ( rname(nd)(1:nend), 'land' )
    if ( nfind > 0 ) then
      nbad = nbad + 1
    end if
    nfind = index ( rname(nd)(1:nend), 'ocean' )
    if ( nfind > 0 ) then
      nbad = nbad + 1
    end if
    nfind = index ( rname(nd)(1:nend), 'contiguous' )
    if ( nfind > 0 ) then
      nbad = nbad + 1
    end if
    nfind = index ( rname(nd)(1:nend), 'australia' )
    if ( nfind > 0 ) then
      nbad = nbad + 1
    end if
    if ( nbad == 0 ) then
      cycle
    end if
    reg_data_2d(nd,nt) = -99.9
    thold(nd) = -99.9
    if ( ldebug ) then
      call inv_julian_minute ( nyr3, nmo3, ndy3, nhr3, nmin3, &
        nyr_base, nmo_base, ndy_base, nhr_base, &
        lulog, jmin_store_mons(nt) )
      write (lulog,'(/a)') ' subroutine set_mon_reg_data_to_missing'
      write (lulog,'(a,a)') ' atmos_region = ', trim ( atmos_region )
      write (lulog,'(a,a)') ' rname(nd) = ', trim ( rname(nd) )
      write (lulog,'(a,i10)') ' nbad=', nbad
      write (lulog,'(a,i10)') ' nd=', nd
      write (lulog,'(a,i10)') ' nyr3=', nyr3
      write (lulog,'(a,i10)') ' nmo3=', nmo3
      write (lulog,'(a,i10)') ' ndy3=', ndy3
      write (lulog,'(a,i10)') ' nhr3=', nhr3
      write (lulog,'(a,i10)') ' nmin3=', nmin3
    end if
  end do
end do

end subroutine

end module
!
!***********************************************************************
!
module module_store_mon_reg_data
implicit none
contains
 
subroutine store_mon_reg_data ( nv, nf, nta, ntb, ihold, rhold, ndval, ntsum, &
  ngsum, npassg, npassg_max )

!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_calc_julian_minute
use module_findilev

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: nv
integer, intent(in) :: nf
integer, intent(inout) :: nta
integer, intent(inout) :: ntb
integer, intent(in), dimension(ndval) :: ihold
real, intent(in), dimension(ndval) :: rhold
integer, intent(in) :: ndval
integer, intent(inout) :: ntsum
integer, intent(inout) :: ngsum
integer, intent(in) :: npassg
integer, intent(in) :: npassg_max

! Local Declarations

integer :: nyrc
integer :: nmoc
integer :: ndyc
integer :: nhrc
integer :: nminc
integer :: jminc
integer :: ngood
integer :: ngood2
integer :: jmin_beg_use
integer :: jmin_end_use

integer :: ne
integer :: nl
integer :: kbot
integer :: ktop
integer :: nrev
integer :: nstop
integer :: nd

logical :: ldebug

ldebug = .false.
!       ldebug = .true.

if ( jmin_beg_exc(nf) > 0 .and. jmin_end_exc(nf) > 0 ) then
  jmin_beg_use = jmin_beg_exc(nf)
  jmin_end_use = jmin_end_exc(nf)
else
  jmin_beg_use = jmin_beg_date
  jmin_end_use = jmin_end_date
end if

nyrc = ihold(1)
nmoc = ihold(2)
ndyc = 1
nhrc = 0
nminc = 0

call calc_julian_minute ( nyrc, nmoc, ndyc, nhrc, nminc, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jminc )

kbot = 1
ktop = ngmax
nrev = +1
nstop = 0

call findilev ( jmin_store_mons, ngmax, jminc, ne, nl, &
  kbot, ktop, ngood, nrev, nstop, lulog )

ngsum = 0

if ( ngood == 1 ) then
  if ( jmin_store_mons(ne) == jminc ) then
    ngsum = ne
  else if ( jmin_store_mons(nl) == jminc ) then
    ngsum = nl
  else
    ngood = 0
  end if
end if

if ( ngood == 1 ) then
  if ( ngsum < nta ) then
    nta = ngsum
  end if
  if ( ngsum > ntb ) then
    ntb = ngsum
  end if
end if

ngood2 = 0

if ( ngood == 1 ) then
  if ( jminc >= jmin_beg_use .and. jminc <= jmin_end_use ) then
    ngood2 = 1
  end if
end if

if ( ngood2 == 1 .and. npassg == 1 ) then
  dflag_store_mons_reg(ngsum) = 1
end if

if ( npassg == npassg_max .and. ngood2 == 1 ) then
  do nd = 1, nreg_max
    reg_data_2d(nd,ngsum) = rhold(nd)
  end do
end if

if ( ldebug ) then
  write (lulog,'(/a)') ' subroutine store_mon_reg_data'
  write (lulog,'(a,i10)') ' npassg=', npassg
  write (lulog,'(a,i10)') ' npassg_max=', npassg_max
  write (lulog,'(a,i10)') ' nyrc=', nyrc
  write (lulog,'(a,i10)') ' nmoc=', nmoc
  write (lulog,'(a,i10)') ' ngood=', ngood
  write (lulog,'(a,i10)') ' ngood2=', ngood2
  write (lulog,'(a,i10)') ' ntsum=', ntsum
  write (lulog,'(a,i10)') ' ngsum=', ngsum
end if

end subroutine

end module
!
!***********************************************************************
!
module module_handle_special_records_reg
implicit none
contains
 
subroutine handle_special_records_reg ( nrec, nysum, ntrend, ncycle_flag, rname, &
  region_lat1, region_lat2, thold, nrmax, nrsum, &
  note_hold, note_hold_max, note_hold_sum, &
  region_def_hold, region_def_hold_max, region_def_hold_sum, &
  rmiss, imiss, cmiss, lulog )
 
!  Global data module

!  Modules for needed subroutines ...

use module_get_free_logical_unit
use module_write_error
use module_rdrec
use module_trim2

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(inout) :: nrec
integer, intent(inout) :: nysum
integer, intent(inout) :: ntrend
integer, intent(inout) :: ncycle_flag
character ( len = * ), intent(inout), dimension(nrmax) :: rname
real, intent(inout), dimension(nrmax) :: region_lat1
real, intent(inout), dimension(nrmax) :: region_lat2
real, intent(inout), dimension(nrmax) :: thold
integer, intent(in) :: nrmax
integer, intent(inout) :: nrsum

integer, intent(in) :: note_hold_max
character ( len = * ), intent(inout), dimension(note_hold_max) :: note_hold
integer, intent(inout) :: note_hold_sum

integer, intent(in) :: region_def_hold_max
character ( len = * ), intent(inout), dimension(region_def_hold_max) :: region_def_hold
integer, intent(inout) :: region_def_hold_sum

real, intent(in) :: rmiss
integer, intent(in) :: imiss
character ( len = * ), intent(in) :: cmiss
integer, intent(in) :: lulog

! Local Declarations

integer :: nend
integer :: nend2
integer :: lnblnk
integer :: nfind
integer :: nfind1
integer :: nfind2
character ( len = 1000 ) :: command
character ( len = 1000 ) :: rec
character ( len = 1000 ) :: nrec2
logical :: fexist
integer :: lutemp
integer :: ios
integer :: nstrt
integer :: nsum
integer :: ns
integer :: nd
integer :: nc
integer :: nr
integer :: n1
integer :: n2

character ( len = 1000 ), save :: nrec1
character ( len = 1000 ), save :: nrec2
character ( len = 1000 ), save :: nrec3

! Local Arrays

integer, parameter :: ndval = 50

real, dimension ( ndval ) :: rhold
integer, dimension ( ndval ) :: ihold
character ( len = 500 ), dimension ( ndval ) :: chold

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Year Mo Globe  Land Ocean   NH   Land Ocean   SH   Land Ocean Trpcs  Land Ocean NoExt  Land Ocean SoExt  Land Ocean NoPol  Land Ocean SoPol  Land Ocean USA48 USA49  AUST
!1978 12 -0.28 -0.36 -0.24 -0.30 -0.41 -0.20 -0.26 -0.26 -0.26 -0.21 -0.22 -0.21 -0.37 -0.51 -0.20 -0.26 -0.21 -0.28 -0.61 -1.29  0.47 -0.20 -0.20 -0.21 -1.60 -1.46 -0.50
!1979  1 -0.25 -0.40 -0.17 -0.47 -0.69 -0.26 -0.03  0.17 -0.10 -0.12 -0.11 -0.13 -0.67 -0.89 -0.40  0.03  0.41 -0.07 -0.45 -1.03  0.46  0.25  0.43  0.10 -3.75 -2.77  1.22
!1979  2 -0.26 -0.46 -0.14 -0.41 -0.57 -0.26 -0.10 -0.24 -0.05  0.00  0.07 -0.03 -0.65 -0.82 -0.43 -0.13 -0.39 -0.06 -2.86 -3.15 -2.39 -0.30 -0.68 -0.01 -2.41 -2.48 -0.17
!1979  3 -0.26 -0.34 -0.21 -0.35 -0.36 -0.34 -0.17 -0.30 -0.13 -0.10 -0.15 -0.08 -0.51 -0.46 -0.57 -0.17 -0.31 -0.13 -0.94 -1.14 -0.64 -0.41 -0.89 -0.03 -0.52 -0.34  0.32
! skipped records
!2015 12  0.56  0.61  0.53  0.69  0.72  0.66  0.42  0.40  0.43  0.70  0.34  0.89  0.71  0.89  0.49  0.24  0.30  0.23 -0.35  0.06 -1.00  0.41  0.48  0.35  1.94  1.52  0.14
!2016  1  0.62  0.56  0.66  0.85  0.73  0.96  0.40  0.23  0.45  0.88  0.63  1.00  0.87  0.81  0.95  0.11 -0.22  0.20  3.16  2.95  3.51 -0.45 -0.44 -0.45 -0.43  0.10 -0.60
!Year Mo Globe  Land Ocean   NH   Land Ocean   SH   Land Ocean Trpcs  Land Ocean NoExt  Land Ocean SoExt  Land Ocean NoPol  Land Ocean SoPol  Land Ocean USA48 USA49  AUST
     
!Trend    0.14  0.18  0.12  0.20  0.23  0.17  0.09  0.10  0.09  0.08  0.11  0.07  0.26  0.27  0.25  0.09  0.09  0.09  0.43  0.40  0.48 -0.02  0.00 -0.03  0.23  0.23  0.15
    
! NOTE:  New Reference for annual cycle 1981-2010 
! NOTE:  Version 5.6 as of Jun 2013 
! NOTE:  3/13 More accurate grid weighting for USA48
! NOTE:  3/13 Alaska added in USA49, Australia added
    
! GL 85S-85N, NH 0-85N, SH 85S-0, TRPCS 20S-20N
! NoExt 20N-85N, SoExt 85S-20S, NoPol 60N-85N, SoPol 85S-60S
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

ncycle_flag = 0
ntrend = 0

nend = lnblnk ( nrec )
if ( nend <= 0 ) then
  nend = 1
end if

if ( nend == 1 ) then
  ncycle_flag = 1
  return
end if

nfind = index ( nrec(1:nend), 'Year' )

if ( nfind > 0 ) then
  nysum = nysum + 1
  if ( nysum == 1 ) then
    nrec1 = cmiss
    nrec2 = cmiss
    nrec3 = cmiss
    do nc = 1, nend
      if ( iachar(nrec(nc:nc)) == 9 ) then ! tab
        nrec(nc:nc) = '_'
      else if ( iachar(nrec(nc:nc)) == 32 ) then ! space
        nrec(nc:nc) = '_'
      end if
    end do
    nrec1 = nrec
  end if
  ncycle_flag = 1
  return
end if

nfind = index ( nrec(1:nend), 'NOTE' )

if ( nfind > 0 ) then
  ncycle_flag = 1
  call trim2 ( nrec, n1, n2 )
  note_hold_sum = note_hold_sum + 1
  if ( note_hold_sum > note_hold_max ) then
    call write_error ( lulog, &
      ' subroutine handle_special_records_reg', &
      ' note_hold_sum > note_hold_max ???', &
      '' )
  end if
  note_hold(note_hold_sum) = nrec(n1:n2)
  return
end if

nfind = index ( nrec(1:nend), 'Trend' )

if ( nysum == 2 .and. nfind > 0 ) then
  ncycle_flag = 1
  ntrend = 1
  nstrt = 1
  call rdrec ( nstrt, nrec, ndval, chold, rhold, ihold, &
    rmiss, imiss, cmiss, lulog )
  nsum = 0
  do nd = 1, ndval
    if ( nint(rhold(nd)) == nint(rmiss) ) then
      exit
    end if
    nsum = nsum + 1
    if ( nsum > nrmax ) then
      call write_error ( lulog, &
      ' subroutine handle_special_records_reg', &
      ' nsum > nrmax ???', &
      '' )
    end if
    thold(nsum) = rhold(nd)
  end do
  if ( nsum /= nrmax ) then
    call write_error ( lulog, &
    ' subroutine handle_special_records_reg', &
      ' nsum /= nrmax ???', &
    '' )
  end if
  return
end if

! GL 85S-85N, NH 0-85N, SH 85S-0, TRPCS 20S-20N
! NoExt 20N-85N, SoExt 85S-20S, NoPol 60N-85N, SoPol 85S-60S

nfind1 = index ( nrec(1:nend), 'GL' )
nfind2 = index ( nrec(1:nend), 'NoExt' )
nfind = max ( nfind1, nfind2 )

!       write (lulog,'(/a)') ' subroutine handle_special_records_reg'
!       write (lulog,'(a,a)') ' nrec = ', trim ( nrec )
!       write (lulog,'(a,i10)') ' nfind1=', nfind1
!       write (lulog,'(a,i10)') ' nfind2=', nfind2
!       write (lulog,'(a,i10)') ' nfind=', nfind
!       write (lulog,'(a,i10)') ' nysum=', nysum

if ( nysum == 2 .and. nfind > 0 ) then

  ncycle_flag = 1

  do nc = 1, nend
    if ( iachar(nrec(nc:nc)) == 9 ) then ! tab
      nrec(nc:nc) = ' '
    end if
  end do

  region_def_hold_sum = region_def_hold_sum + 1

  if ( region_def_hold_sum > region_def_hold_max ) then
    call write_error ( lulog, &
    ' subroutine handle_special_records_reg', &
    ' region_def_hold_sum > region_def_hold_max ???', &
    '' )
  end if

  region_def_hold(region_def_hold_sum ) = nrec(1:nend)

  do nc = 1, nend
    if ( iachar(nrec(nc:nc)) == 9 ) then ! tab
      nrec(nc:nc) = '_'
    else if ( iachar(nrec(nc:nc)) == 32 ) then ! space
      nrec(nc:nc) = '_'
    end if
  end do

  if ( nfind1 > 0 ) nrec2 = nrec
  if ( nfind2 > 0 ) nrec3 = nrec

  nsum = 0
  if ( trim(nrec1) /= trim(cmiss) ) nsum = nsum + 1
  if ( trim(nrec2) /= trim(cmiss) ) nsum = nsum + 1
  if ( trim(nrec3) /= trim(cmiss) ) nsum = nsum + 1

!       write (lulog,'(/a)') ' subroutine handle_special_records_reg'
!       write (lulog,'(a)') ' after converting tabs and spaces to underscores'
!       write (lulog,'(a,a)') ' nrec = ', trim ( nrec )
!       write (lulog,'(a,i10)') ' nfind1=', nfind1
!       write (lulog,'(a,i10)') ' nfind2=', nfind2
!       write (lulog,'(a,a)') ' nrec1 = ', trim ( nrec1 )
!       write (lulog,'(a,a)') ' nrec2 = ', trim ( nrec2 )
!       write (lulog,'(a,a)') ' nrec3 = ', trim ( nrec3 )
!       write (lulog,'(a,i10)') ' nsum=', nsum

  if ( nsum == 3 ) then

    call get_free_logical_unit ( lutemp )
    inquire ( file='region.txt',exist=fexist )

    if ( fexist ) then
      open (unit=lutemp,file='region.txt',status='old')
      close (unit=lutemp,status='delete')
    end if

    write (command,'(a,a,a,a,a,a,a,a)') &
      './rsplit.pl ', &
      trim ( nrec1 ), ' ', &
      trim ( nrec2 ), ' ', &
      trim ( nrec3 ), ' ', &
      ' > rsplit.LOG'

    nend2 = lnblnk ( command )

!   write (lulog,'(/a)') ' subroutine handle_special_records_reg'
!   write (lulog,'(a)') ' BEFORE rsplit.pl command ...'
!   write (lulog,'(a,a)') ' nrec1 = ', trim ( nrec1 )
!   write (lulog,'(a,a)') ' nrec2 = ', trim ( nrec2 )
!   write (lulog,'(a,a)') ' nrec3 = ', trim ( nrec3 )
!   write (lulog,'(a,a)') ' command = ', trim ( command )
!   call flush ( lulog )

    call system ( command(1:nend2) )

    inquire ( file='region.txt',exist=fexist )

    if ( .not.fexist ) then
      call write_error ( lulog, &
      ' subroutine handle_special_records_reg', &
      ' file region.txt does not exist ???', &
      '' )
    end if

    open (unit=lutemp,file='region.txt',status='old')

    nstrt = 1
    nrsum = 0

    do
      read (lutemp,'(a)',iostat=ios) rec
      if ( ios == 0 ) then
        call rdrec ( nstrt, rec, ndval, chold, rhold, ihold, &
          rmiss, imiss, cmiss, lulog )
        nrsum = nrsum + 1
        if ( nrsum > nrmax ) then
          call write_error ( lulog, &
          ' subroutine handle_special_records_reg', &
          ' nrsum > nrmax ???', &
          '' )
        end if
        rname(nrsum) = chold(1)
        region_lat1(nrsum) = rhold(1)
        region_lat2(nrsum) = rhold(2)
      else
        close (unit=lutemp,status='delete')
        exit
      end if
    end do

    write (lulog,'(/a)') ' subroutine handle_special_records_reg'
    write (lulog,'(a)') ' REGION REVIEW'
    write (lulog,'(a)') &
      '   nr                                              region        lim1      lim2'
    do nr = 1, nrsum
      write (lulog,'(i5,2x,a50,2x,2f10.1)') nr, trim(rname(nr)), region_lat1(nr), &
        region_lat2(nr)
    end do

  end if

end if

end subroutine

end module
!
!***********************************************************************
!
module module_trim2
implicit none
contains

subroutine trim2 ( cs, nbeg, nend )

!  Modules for needed subroutines ...

!  Global data module

implicit none

!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: cs
integer, intent(inout) :: nbeg
integer, intent(inout) :: nend

!  Local declarations

integer :: n
integer :: nn
integer :: lnblnk
character ( len = 1 ) :: tab

nn = lnblnk ( cs )

! Decimal value of tab character in ASCII table is 9

tab = achar ( 9 )

do n = 1, nn
  if ( cs(n:n) /= tab .and. cs(n:n) /= ' ' ) then
    nbeg = n
    exit
  end if
end do

do n = nn, 1, -1
  if ( cs(n:n) /= tab .and. cs(n:n) /= ' ' ) then
    nend = n
    exit
  end if
end do

end subroutine
 
end module
!
!***********************************************************************
!
module module_open_log_file
implicit none
contains
 
subroutine open_log_file
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_get_free_logical_unit
use module_write_error

implicit none
 
!  Procedure dummy argument declarations
 
!  Local declarations
 
integer :: nend
integer :: lnblnk
integer :: lutemp
character ( len = 500 ) :: command
character ( len = 500 ) :: fname
integer :: ios

logical :: fexist

call get_free_logical_unit ( lutemp )

inquire ( file='ERROR',exist=fexist )

if ( fexist ) then
  open (unit=lutemp,status='old',form='formatted',file='ERROR')
  close (unit=lutemp,status='delete')
end if

inquire ( file='DONE',exist=fexist )

if ( fexist ) then
  open (unit=lutemp,status='old',form='formatted',file='DONE')
  close (unit=lutemp,status='delete')
end if

command = 'date -u +"%Y %m %d %H %M %S %s" > date.dmp'
nend = lnblnk ( command )
call system ( command(1:nend) )

open (unit=lutemp,status='old',form='formatted',file='date.dmp',iostat=ios)

if ( ios /= 0 ) then
  lulog = 0
  call write_error ( lulog, &
  ' subroutine open_log_file', &
  ' failed to read file date.dmp from unix date command ???', &
  '' )
end if

read (lutemp,*) nyr, nmo, ndy, nhr, nmin, nsec

write (cdate_string,'(i4,a1,5(i2.2,a1))') &
  nyr, '-', &
  nmo, '-', &
  ndy, 'T', &
  nhr, ':', &
  nmin, ':', &
  nsec, 'Z'

close (unit=lutemp,status='delete')

fname = ''

write (fname,'(a,i4,a1,4(i2.2,a1),i2.2,a)') &
  'pmsu.', nyr, ':', nmo, ':', ndy, ':', &
  nhr, ':', nmin, ':', nsec, '.UTC.log'

nend = lnblnk ( fname )

call get_free_logical_unit ( lulog )

open (unit=lulog,status='unknown',form='formatted',file=fname(1:nend))

end subroutine

end module
!
!***********************************************************************
!
module module_isort
implicit none
contains

subroutine isort ( key, ra, n )

implicit none

!  Procedure dummy argument declarations

integer, intent(inout), dimension (n) :: key
integer, intent(inout), dimension (n) :: ra
integer, intent(inout) :: n

!  Local declarations

integer :: i
integer :: ir
integer :: j
integer :: k
integer :: ktemp
integer :: l
real :: rra

!	Numerical Recipes heap-sort routine

l = (n / 2) + 1
ir = n

do k = 1, n
  key(k) = k
end do

outer: do

  if ( l > 1 ) then
    l = l - 1
    rra = ra(l)
    ktemp = key(l)
  else
    rra = ra(ir)
    ktemp = key(ir)
    ra(ir) = ra(1)
    key(ir) = key(1)
    ir = ir - 1
    if ( ir == 1 ) then
      ra(1) = rra
      key(1) = ktemp
      exit outer
    end if
  end if

  i = l
  j = l + l

  inner: do
    if ( j.le.ir ) then
      if ( j < ir ) then
        if ( ra(j) < ra(j + 1) ) j = j + 1
      end if
      if ( rra < ra(j) ) then
        ra(i) = ra(j)
        key(i) = key(j)
        i = j
        j = j + j
      else
        j = ir + 1
      end if
    else
      exit inner
    end if
  end do inner

  ra(i) = rra
  key(i) = ktemp

end do outer

end subroutine

end module
!
!***********************************************************************
!
module module_rdrec
implicit none
contains

subroutine rdrec ( nstrt, nrec, nmax, chold, rhold, ihold, &
  rmiss, imiss, cmiss, lulog )

implicit none

!  Procedure dummy argument declarations

integer, intent(in) :: nstrt
character ( len = * ), intent(in) :: nrec
integer, intent(in) :: nmax
character ( len = * ), intent(inout), dimension (nmax) :: chold
real, intent(inout), dimension (nmax) :: rhold
integer, intent(inout), dimension (nmax) :: ihold
real, intent(in) :: rmiss
integer, intent(in) :: imiss
character ( len = * ), intent(in) :: cmiss
integer, intent(in) :: lulog

!  Local declarations

character ( len = 1 ), dimension (54), save :: alphabet
character ( len = 1 ) :: tab
character ( len = 20 ) :: fmt
integer :: ios
integer :: n
integer :: na
integer :: nb
integer :: nbb
integer :: nc
integer :: ncsum
integer :: nend
integer :: nfield
integer :: nfindc
integer :: nfindr
integer :: nflag
integer :: nisum
integer :: nn
integer :: nrsum
integer :: nsum
integer :: nnsum
integer :: nval
real :: xval
integer :: lnblnk

logical :: ldebug

alphabet = (/ &
 & 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', &
 & 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', &
 & 'u', 'v', 'w', 'x', 'y', 'z', &
 & 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', &
 & 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', &
 & 'U', 'V', 'W', 'X', 'Y', 'Z', '_', '/' &
 & /)

tab = '	'

nsum = 0
nisum = 0
nrsum = 0
ncsum = 0
na = 0

nflag = 0
ldebug= .false.

nend = lnblnk( nrec )

do n = 1, nmax
  chold(n) = cmiss
  rhold(n) = rmiss
  ihold(n) = imiss
end do

outer: do n = nstrt, nend

  if ( nrec(n:n) /= ' ' .and. nflag == 0 .and. &
    nrec(n:n) /= tab ) then
    nflag = 1
    nsum = nsum + 1
    na = n
  end if

  if ( nsum > nmax ) then
    write (lulog,'(/a)') ' subroutine rdrec'
    write (lulog,'(a)') ' more than nmax data fields ...'
    write (lulog,'(a,i5)') ' nmax=',nmax
    write (lulog,'(a,a)') 'nrec=',nrec(nstrt:nend)
    stop
  end if

  if ( ldebug ) then
    write (lulog,'(/a)') ' subroutine rdrec'
    write (lulog,'(a)') ' NEAR TOP of n=nstrt,nend loop ...'
    write (lulog,'(a,a)') 'nrec=',nrec(nstrt:nend)
    write (lulog,'(a,a)') 'nrec(n:n)=',nrec(n:n)
    write (lulog,'(a,3i5)') ' n,nstrt,nend=',n,nstrt,nend
    write (lulog,'(a,i5)') ' nsum=',nsum
    write (lulog,'(a,i5)') ' na=',na
    write (lulog,'(a,i5)') ' nflag=',nflag
  end if

  if ( ( nrec(n:n) == ' '.and.nflag == 1 ) .or. &
    ( nflag == 1.and.n == nend) .or. &
    ( nflag == 1.and.nrec(n:n) == tab) ) then
    nflag = 0

    if ( n /= nend ) then
      nb = n - 1
    else if ( n == nend ) then
      nb = n
    end if

!	    check for a period for a possible real number ...

    nfindr=index( nrec(na:nb), '.' )

    if ( ldebug ) then
      write (lulog,'(/a)') ' subroutine rdrec'
      write (lulog,'(a)') ' AFTER nfindr=index statement ...'
      write (lulog,'(a,3i5)') ' n,nstrt,nend=',n,nstrt,nend
      write (lulog,'(a,2i5)') ' na,nb=',na,nb
      write (lulog,'(a,i5)') ' nflag=',nflag
      write (lulog,'(a,i5)') ' nfindr=',nfindr
    end if

    if ( nfindr /= 0 ) then

      nfield = nb - na + 1
      fmt=' '
      write (fmt,'(a,i2,a,a)') '(e',nfield,'.','4)'

      nbb = lnblnk( fmt )
      do nn = 1, nbb
        if ( fmt(nn:nn) == ' ' ) fmt(nn:nn)='0'
      end do

      read (nrec(na:nb),fmt(1:nbb),iostat=ios) xval

      if ( ios == 0 ) then
        nrsum = nrsum + 1
        if ( nrsum <= nmax ) then
          rhold(nrsum) = xval
        else
          write (lulog,'(/a)') ' more than nmax real fields ...'
          write (lulog,'(a,i5)') ' nmax=',nmax
          write (lulog,'(a,a)') 'nrec=',nrec(nstrt:nend)
          stop
        end if
        cycle outer
      end if

    end if

!	    check for a character string ...

    if ( ldebug ) then
      write (lulog,'(/a)') ' subroutine rdrec'
      write (lulog,'(a)') ' BEFORE character-string checks ...'
      write (lulog,'(a,3i5)') ' n,nstrt,nend=',n,nstrt,nend
      write (lulog,'(a,i5)') ' ncsum=',ncsum
      write (lulog,'(a,2i5)') ' na,nb=',na,nb
      call flush ( lulog )
    end if

      inner_a: do nc = 1, 54
      nfindc = index( nrec(na:nb), alphabet(nc) )
      if ( nfindc /= 0 ) then
        ncsum = ncsum + 1
        if ( ncsum <= nmax ) then
          chold(ncsum) = nrec(na:nb)
        else
          write (lulog,'(/a)') ' more than nmax character fields ...'
          write (lulog,'(a,i5)') ' nmax=',nmax
          write (lulog,'(a,a)') 'nrec=',nrec(nstrt:nend)
          stop
        end if
        cycle outer
      end if
    end do inner_a

    if ( ldebug ) then
      write (lulog,'(/a)') ' subroutine rdrec'
      write (lulog,'(a)') ' AFTER character-string checks ...'
      write (lulog,'(a,3i5)') ' n,nstrt,nend=',n,nstrt,nend
      write (lulog,'(a,i5)') ' ncsum=',ncsum
      write (lulog,'(a,2i5)') ' na,nb=',na,nb
      call flush ( lulog )
    end if

!	    remaining option is an integer ...

    nisum = nisum + 1
    read (nrec(na:nb),*,iostat=ios) nval
    if ( ios /= 0 ) then
      write (lulog,'(/a)') ' unknown data type in input file ???'
      write (lulog,'(a)') &
      ' error in trying to read field as an integer'
      write (lulog,'(a,2i5)') ' na,nb=',na,nb
      write (lulog,'(a,a)') ' nrec(na:nb)= ',nrec(na:nb)
      stop
    end if
    if ( nisum <= nmax ) then
      ihold(nisum) = nval
    else
      write (lulog,'(/a)') ' more than nmax integer fields ...'
      write (lulog,'(a,i5)') ' nmax=',nmax
      write (lulog,'(a,a)') 'nrec=',nrec(nstrt:nend)
      stop
    end if

    cycle outer

  end if

end do outer

end subroutine

end module
!
!***********************************************************************
!
module module_allocate_arrays_mon_gridded_anom
implicit none
contains
 
subroutine allocate_arrays_mon_gridded_anom ( deallocate_only )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_alloc_status
use module_write_error
use module_init_arrays_mon_gridded_anom

implicit none
 
!  Procedure dummy argument declarations

logical, intent(in) :: deallocate_only
 
!  Local declarations

integer :: istatus
integer :: kdum = 1

if ( ngmax <= 0 ) then
  call write_error ( lulog, &
  ' subroutine allocate_arrays_mon_gridded_anom', &
  ' ngmax <= 0 ???', &
  '' )
end if

if ( allocated ( ndata ) ) then
  deallocate ( ndata, stat = istatus )
end if

if ( allocated ( month_store ) ) then
  deallocate ( month_store, stat = istatus )
end if

if ( allocated ( ndata_2d ) ) then
  deallocate ( ndata_2d, stat = istatus )
end if

if ( allocated ( rdata_2d ) ) then
  deallocate ( rdata_2d, stat = istatus )
end if

if ( allocated ( esec_monthly_hold ) ) then
  deallocate ( esec_monthly_hold, stat = istatus )
end if

if ( allocated ( cdate_monthly_hold ) ) then
  deallocate ( cdate_monthly_hold, stat = istatus )
end if

write (lulog,'(/a)') ' ENTERING subroutine allocate_arrays_mon_gridded_anom'
write (lulog,'(a,i10)') ' ngmax=', ngmax
write (lulog,'(a,l10)') ' deallocate_only=', deallocate_only
call flush ( lulog )

if ( deallocate_only ) then
  return
end if

allocate ( ndata(imax,jmax,nmonths), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_gridded_anom', &
    'ndata', istatus, &
    '3D', imax, jmax, nmonths, lulog )
end if

allocate ( month_store(ngmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_gridded_anom', &
    'month_store', istatus, &
    '1D', ngmax, kdum, kdum, lulog )
end if

allocate ( ndata_2d(imax,jmax,1), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_gridded_anom', &
    'ndata_2d', istatus, &
    '3D', imax, jmax, kdum, lulog )
end if

allocate ( rdata_2d(imax,jmax,1), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_gridded_anom', &
    'rdata_2d', istatus, &
    '3D', imax, jmax, kdum, lulog )
end if

allocate ( esec_monthly_hold(ngmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_gridded_anom', &
    'esec_monthly_hold', istatus, &
    '1D', ngmax, kdum, kdum, lulog )
end if

allocate ( cdate_monthly_hold(ngmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_gridded_anom', &
    'cdate_monthly_hold', istatus, &
    '1D', ngmax, kdum, kdum, lulog )
end if

call init_arrays_mon_gridded_anom

end subroutine

end module
!
!***********************************************************************
!
module module_init_arrays_mon_gridded_anom
implicit none
contains
 
subroutine init_arrays_mon_gridded_anom
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

!  Local declarations

if ( .not.allocated ( ndata ) ) then
write (lulog,'(/a)') ' subroutine init_arrays_mon_gridded_anom'
write (lulog,'(a)') ' ndata not allocated ???'
call flush ( lulog )
stop
end if

if ( .not.allocated ( ndata_2d ) ) then
write (lulog,'(/a)') ' subroutine init_arrays_mon_gridded_anom'
write (lulog,'(a)') ' ndata_2d not allocated ???'
call flush ( lulog )
stop
end if

if ( .not.allocated ( rdata_2d ) ) then
write (lulog,'(/a)') ' subroutine init_arrays_mon_gridded_anom'
write (lulog,'(a)') ' rdata_2d not allocated ???'
call flush ( lulog )
stop
end if

ndata = imiss
ndata_2d = imiss
rdata_2d = rmiss

end subroutine

end module
!
!***********************************************************************
!
module module_allocate_arrays_mon_regional_anom
implicit none
contains
 
subroutine allocate_arrays_mon_regional_anom ( deallocate_only )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_alloc_status
use module_write_error
use module_init_arrays_mon_regional_anom

implicit none
 
!  Procedure dummy argument declarations

logical, intent(in) :: deallocate_only
 
!  Local declarations

integer :: istatus
integer :: kdum = 1

if ( ngmax <= 0 ) then
  call write_error ( lulog, &
  ' subroutine allocate_arrays_mon_regional_anom', &
  ' ngmax <= 0 ???', &
  '' )
end if

if ( allocated ( reg_data_2d ) ) then
  deallocate ( reg_data_2d, stat = istatus )
end if

if ( allocated ( reg_rdata_2d ) ) then
  deallocate ( reg_rdata_2d, stat = istatus )
end if

if ( allocated ( esec_monthly_reg_hold ) ) then
  deallocate ( esec_monthly_reg_hold, stat = istatus )
end if

if ( allocated ( cdate_monthly_reg_hold ) ) then
  deallocate ( cdate_monthly_reg_hold, stat = istatus )
end if

write (lulog,'(/a)') ' ENTERING subroutine allocate_arrays_mon_regional_anom'
write (lulog,'(a,i10)') ' ngmax=', ngmax
write (lulog,'(a,l10)') ' deallocate_only=', deallocate_only
call flush ( lulog )

if ( deallocate_only ) then
  return
end if

allocate ( reg_data_2d(nreg_max,ngmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_regional_anom', &
    'reg_data_2d', istatus, &
    '2D', nreg_max, ngmax, kdum, lulog )
end if

allocate ( reg_rdata_2d(nreg_max,ngmax,1), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_regional_anom', &
    'reg_rdata_2d', istatus, &
    '2D', nreg_max, ngmax, kdum, lulog )
end if

allocate ( esec_monthly_reg_hold(ngmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_regional_anom', &
    'esec_monthly_reg_hold', istatus, &
    '1D', ngmax, kdum, kdum, lulog )
end if

allocate ( cdate_monthly_reg_hold(ngmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_regional_anom', &
    'cdate_monthly_reg_hold', istatus, &
    '1D', ngmax, kdum, kdum, lulog )
end if

call init_arrays_mon_regional_anom

end subroutine

end module
!
!***********************************************************************
!
module module_init_arrays_mon_regional_anom
implicit none
contains
 
subroutine init_arrays_mon_regional_anom
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

!  Local declarations

reg_data_2d = rmiss
esec_monthly_reg_hold = rmiss
cdate_monthly_reg_hold = cmiss

end subroutine

end module
!
!***********************************************************************
!
module module_allocate_arrays_day_anom
implicit none
contains
 
subroutine allocate_arrays_day_anom ( deallocate_only )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_alloc_status
use module_write_error
use module_init_arrays_day_anom

implicit none
 
!  Procedure dummy argument declarations

logical, intent(in) :: deallocate_only
 
!  Local declarations

integer :: istatus
integer :: kdum = 1

if ( ndmax <= 0 ) then
  call write_error ( lulog, &
  ' subroutine allocate_arrays_day_anom', &
  ' ndmax <= 0 ???', &
  '' )
end if

if ( allocated ( day_anom_store%nyr ) ) then
  deallocate ( day_anom_store%nyr, stat = istatus )
end if

if ( allocated ( day_anom_store%jday ) ) then
  deallocate ( day_anom_store%jday, stat = istatus )
end if

if ( allocated ( day_anom_store%jdayc ) ) then
  deallocate ( day_anom_store%jdayc, stat = istatus )
end if

if ( allocated ( day_anom_store%esec ) ) then
  deallocate ( day_anom_store%esec, stat = istatus )
end if

if ( allocated ( day_anom_store%cdate ) ) then
  deallocate ( day_anom_store%cdate, stat = istatus )
end if

if ( allocated ( day_anom_store%gb ) ) then
  deallocate ( day_anom_store%gb, stat = istatus )
end if

if ( allocated ( day_anom_store%nh ) ) then
  deallocate ( day_anom_store%nh, stat = istatus )
end if

if ( allocated ( day_anom_store%sh ) ) then
  deallocate ( day_anom_store%sh, stat = istatus )
end if

if ( allocated ( day_anom_store%tp ) ) then
  deallocate ( day_anom_store%tp, stat = istatus )
end if

if ( allocated ( day_anom_store%za ) ) then
  deallocate ( day_anom_store%za, stat = istatus )
end if

write (lulog,'(/a)') ' subroutine allocate_arrays_day_anom'
write (lulog,'(a,i10)') ' ndmax=', ndmax
write (lulog,'(a,l10)') ' deallocate_only=', deallocate_only
call flush ( lulog )

if ( deallocate_only ) then
  return
end if

allocate ( day_anom_store%nyr(ndmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_day_anom', &
    'day_anom_store%nyr', istatus, &
    '1D', ndmax, kdum, kdum, lulog )
end if

allocate ( day_anom_store%jday(ndmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_day_anom', &
    'day_anom_store%jday', istatus, &
    '1D', ndmax, kdum, kdum, lulog )
end if

allocate ( day_anom_store%jdayc(ndmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_day_anom', &
    'day_anom_store%jdayc', istatus, &
    '1D', ndmax, kdum, kdum, lulog )
end if

allocate ( day_anom_store%esec(ndmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_day_anom', &
    'day_anom_store%esec', istatus, &
    '1D', ndmax, kdum, kdum, lulog )
end if

allocate ( day_anom_store%cdate(ndmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_day_anom', &
    'day_anom_store%cdate', istatus, &
    '1D', ndmax, kdum, kdum, lulog )
end if

allocate ( day_anom_store%gb(ndmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_day_anom', &
    'day_anom_store%gb', istatus, &
    '1D', ndmax, kdum, kdum, lulog )
end if

allocate ( day_anom_store%nh(ndmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_day_anom', &
    'day_anom_store%nh', istatus, &
    '1D', ndmax, kdum, kdum, lulog )
end if

allocate ( day_anom_store%sh(ndmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_day_anom', &
    'day_anom_store%sh', istatus, &
    '1D', ndmax, kdum, kdum, lulog )
end if

allocate ( day_anom_store%tp(ndmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_day_anom', &
    'day_anom_store%tp', istatus, &
    '1D', ndmax, kdum, kdum, lulog )
end if

allocate ( day_anom_store%za(ndmax,jmaxz), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_day_anom', &
    'day_anom_store%za', istatus, &
    '2D', ndmax, jmaxz, kdum, lulog )
end if

call init_arrays_day_anom

end subroutine

end module
!
!***********************************************************************
!
module module_init_arrays_day_anom
implicit none
contains
 
subroutine init_arrays_day_anom
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

!  Local declarations

day_anom_store%nyr = imiss
day_anom_store%jday = imiss
day_anom_store%jdayc = imiss
day_anom_store%gb = imiss
day_anom_store%nh = imiss
day_anom_store%sh = imiss
day_anom_store%tp = imiss
day_anom_store%za = imiss

end subroutine

end module
!
!***********************************************************************
!
module module_allocate_arrays_mon_anom
implicit none
contains
 
subroutine allocate_arrays_mon_anom ( deallocate_only )
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_alloc_status
use module_write_error
use module_init_arrays_mon_anom

implicit none
 
!  Procedure dummy argument declarations

logical, intent(in) :: deallocate_only
 
!  Local declarations

integer :: istatus
integer :: kdum = 1

if ( ntmax <= 0 ) then
  call write_error ( lulog, &
  ' subroutine allocate_arrays_mon_anom', &
  ' ntmax <= 0 ???', &
  '' )
end if

if ( allocated ( mon_anom_store%nyr ) ) then
  deallocate ( mon_anom_store%nyr, stat = istatus )
end if

if ( allocated ( mon_anom_store%nmo ) ) then
  deallocate ( mon_anom_store%nmo, stat = istatus )
end if

if ( allocated ( mon_anom_store%esec ) ) then
  deallocate ( mon_anom_store%esec, stat = istatus )
end if

if ( allocated ( mon_anom_store%cdate ) ) then
  deallocate ( mon_anom_store%cdate, stat = istatus )
end if

if ( allocated ( mon_anom_store%gb ) ) then
  deallocate ( mon_anom_store%gb, stat = istatus )
end if

if ( allocated ( mon_anom_store%nh ) ) then
  deallocate ( mon_anom_store%nh, stat = istatus )
end if

if ( allocated ( mon_anom_store%sh ) ) then
  deallocate ( mon_anom_store%sh, stat = istatus )
end if

if ( allocated ( mon_anom_store%tp ) ) then
  deallocate ( mon_anom_store%tp, stat = istatus )
end if

if ( allocated ( mon_anom_store%ndays ) ) then
  deallocate ( mon_anom_store%ndays, stat = istatus )
end if

if ( allocated ( mon_anom_store%gb_rm ) ) then
  deallocate ( mon_anom_store%gb_rm, stat = istatus )
end if

if ( allocated ( mon_anom_store%nh_rm ) ) then
  deallocate ( mon_anom_store%nh_rm, stat = istatus )
end if

if ( allocated ( mon_anom_store%sh_rm ) ) then
  deallocate ( mon_anom_store%sh_rm, stat = istatus )
end if

if ( allocated ( mon_anom_store%tp_rm ) ) then
  deallocate ( mon_anom_store%tp_rm, stat = istatus )
end if

if ( allocated ( mon_anom_store%ndays_rm ) ) then
  deallocate ( mon_anom_store%ndays_rm, stat = istatus )
end if

write (lulog,'(/a)') ' subroutine allocate_arrays_mon_anom'
write (lulog,'(a,i10)') ' ntmax=', ntmax
write (lulog,'(a,l10)') ' deallocate_only=', deallocate_only
call flush ( lulog )

if ( deallocate_only ) then
  return
end if

allocate ( mon_anom_store%nyr(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%nyr', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

allocate ( mon_anom_store%nmo(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%nmo', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

allocate ( mon_anom_store%esec(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%esec', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

allocate ( mon_anom_store%cdate(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%cdate', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

allocate ( mon_anom_store%gb(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%gb', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

allocate ( mon_anom_store%nh(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%nh', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

allocate ( mon_anom_store%sh(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%sh', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

allocate ( mon_anom_store%tp(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%tp', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

allocate ( mon_anom_store%ndays(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%ndays', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

allocate ( mon_anom_store%gb_rm(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%gb_rm', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

allocate ( mon_anom_store%nh_rm(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%nh_rm', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

allocate ( mon_anom_store%sh_rm(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%sh_rm', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

allocate ( mon_anom_store%tp_rm(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%tp_rm', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

allocate ( mon_anom_store%ndays_rm(ntmax), stat = istatus )

if ( istatus /= 0 ) then
  call write_alloc_status ( 'subroutine allocate_arrays_mon_anom', &
    'mon_anom_store%ndays_rm', istatus, &
    '1D', ntmax, kdum, kdum, lulog )
end if

call init_arrays_mon_anom

end subroutine

end module
!
!***********************************************************************
!
module module_init_arrays_mon_anom
implicit none
contains
 
subroutine init_arrays_mon_anom
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_write_alloc_status
use module_write_error

implicit none
 
!  Procedure dummy argument declarations

!  Local declarations

mon_anom_store%nyr = imiss
mon_anom_store%nmo = imiss
mon_anom_store%gb = imiss
mon_anom_store%nh = imiss
mon_anom_store%sh = imiss
mon_anom_store%tp = imiss
mon_anom_store%ndays = imiss
mon_anom_store%gb_rm = imiss
mon_anom_store%nh_rm = imiss
mon_anom_store%sh_rm = imiss
mon_anom_store%tp_rm = imiss
mon_anom_store%ndays_rm = imiss

end subroutine

end module
!
!***********************************************************************
!
module module_maxmin2d
implicit none
contains

subroutine maxmin2d ( var, imaxd, jmaxd, vmax, vmin, imax, jmax, rmiss )

!  Modules for needed subroutines ...

implicit none

!  Procedure dummy argument declarations

real, intent(inout), dimension (imaxd,jmaxd) :: var
integer, intent(inout) :: imaxd
integer, intent(inout) :: jmaxd
real, intent(inout) :: vmax
real, intent(inout) :: vmin
integer, intent(inout) :: imax
integer, intent(inout) :: jmax
real, intent(in) :: rmiss

!  Local declarations

integer :: i
integer :: j

vmax = - 1.0e+30
vmin = + 1.0e+30

do i = 1, imax
  do j = 1, jmax
    if ( nint(var(i,j)) == nint(rmiss) ) then
      cycle
    end if
    if ( var(i,j) > vmax ) vmax = var(i,j)
    if ( var(i,j) < vmin ) vmin = var(i,j)
  end do
end do

end subroutine

end module
!
!***********************************************************************
!
module module_write_error
implicit none
contains
 
subroutine write_error ( lulog, cs1, cs2, cs3 )
 
!  Global data module

!  Modules for needed subroutines ...

use module_get_free_logical_unit

implicit none
 
!  Procedure dummy argument declarations

integer, intent(in) :: lulog
character ( len = * ), intent(in) :: cs1
character ( len = * ), intent(in) :: cs2
character ( len = * ), intent(in) :: cs3

! Local Declarations

integer :: nend
integer :: lnblnk
integer :: lutemp
character ( len = 1000 ) :: cs
integer :: ne

call get_free_logical_unit ( lutemp )
open (unit=lutemp,status='unknown',form='formatted',file='ERROR')

do ne = 1, 3

  if ( ne == 1 ) cs = cs1
  if ( ne == 2 ) cs = cs2
  if ( ne == 3 ) cs = cs3

  nend = lnblnk ( cs )
  if ( nend > 0 ) then
    write (lutemp,'(a)') cs(1:nend)
    if ( lulog > 0 ) then
      write (lulog,'(a)') cs(1:nend)
    end if
  end if

end do

close (unit=lutemp,status='keep')

if ( lulog > 0 ) then
  close (unit=lulog,status='keep')
end if

stop

end subroutine

end module
!
!***********************************************************************
!
module module_remove_underscore
implicit none
contains
 
subroutine remove_underscore ( cs )
 
!  Global data module

!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(inout) :: cs

! Local Declarations

integer :: nend
integer :: lnblnk
integer :: nc

nend = lnblnk ( cs )

if ( nend <= 0 ) then
  return
end if

do nc = 1, nend
  if ( cs(nc:nc) == '_' ) then
    cs(nc:nc) = ' '
  end if
end do

end subroutine

end module
!
!***********************************************************************
!
module module_read_input_options
implicit none
contains
 
subroutine read_input_options
 
!  Global data module

use module_global_pmsu
 
!  Modules for needed subroutines ...

use module_get_free_logical_unit
use module_replace_commas_with_tabs
use module_rdrec
use module_change_to_caps
use module_write_error
use module_process_versions
use module_process_ftypes
use module_get_epoch_seconds
use module_remove_underscore
use module_process_time_ranges
use module_calc_julian_minute

implicit none
 
!  Procedure dummy argument declarations

! Local Declarations

integer :: luinp
integer :: lutemp
integer :: ios
integer :: nend
integer :: nend2
integer :: nend3
integer :: nfind
integer :: lnblnk
character ( len = 1000 ) nrec
character ( len = 1000 ) nrec_keep
integer :: nstrt
integer :: nv
integer :: nf
integer :: ngood

! Local Arrays

integer, parameter :: ndval = 200

real, dimension ( ndval ) :: rhold
integer, dimension ( ndval ) :: ihold
character ( len = 500 ), dimension ( ndval ) :: chold

write (lulog,'(/a)') ' ENTERING subroutine read_input_options'

call get_free_logical_unit ( luinp )

open (unit=luinp,status='old',form='formatted',file='pmsu_options.inp',iostat=ios)

if ( ios /= 0 ) then
  call write_error ( lulog, &
  ' subroutine read_input_options', &
  ' unable to open file pmsu_options.inp ???', &
  '' )
end if

nstrt = 1

do

  read (luinp,'(a)',iostat=ios) nrec
  nend = lnblnk ( nrec )

  if ( ios == 0 ) then
    nend = lnblnk ( nrec )
    if ( nend <= 1 ) then
      cycle
    end if
    if ( nrec(1:1) == '!' ) then
      cycle
    end if
    if ( nrec(1:1) == '#' ) then
      cycle
    end if
  end if

  if ( ios == 0 ) then

    nrec_keep = nrec
    call replace_commas_with_tabs ( nrec )

    call rdrec ( nstrt, nrec, ndval, chold, rhold, ihold, &
      rmiss, imiss, cmiss, lulog )

    nend = lnblnk ( chold(1) )

    if ( chold(1)(1:nend) == 'TITLE' ) then

      nrec = nrec_keep
      call rdrec ( nstrt, nrec, ndval, chold, rhold, ihold, &
      rmiss, imiss, cmiss, lulog )

      title = chold(2)
      call remove_underscore ( title )

    else if ( chold(1)(1:nend) == 'HISTORY' ) then

      nrec = nrec_keep
      call rdrec ( nstrt, nrec, ndval, chold, rhold, ihold, &
      rmiss, imiss, cmiss, lulog )

      history = chold(2)
      call remove_underscore ( history )

    else if ( chold(1)(1:nend) == 'CONTACT' ) then

      nrec = nrec_keep
      call rdrec ( nstrt, nrec, ndval, chold, rhold, ihold, &
      rmiss, imiss, cmiss, lulog )

      contact = chold(2)
      call remove_underscore ( contact )

    else if ( chold(1)(1:nend) == 'MASTER_FILE_BEGINNING_DATE' ) then

      nyr_beg = ihold(1)
      nmo_beg = ihold(2)
      ndy_beg = ihold(3)

      nhr = 0
      nmin = 0

      call calc_julian_minute ( nyr_beg, nmo_beg, ndy_beg, nhr, nmin, &
        nyr_base, nmo_base, ndy_base, nhr_base, &
        lulog, jmin_beg_date )

    else if ( chold(1)(1:nend) == 'MASTER_FILE_ENDING_DATE' ) then

      nyr_end = ihold(1)
      nmo_end = ihold(2)
      ndy_end = ihold(3)

      if ( nyr_end < 0 .or. nmo_end < 0 .or. ndy_end < 0 ) then
        nyr_end = nyr
        nmo_end = nmo
        ndy_end = ndy
      end if

      nhr = 0
      nmin = 0

      call calc_julian_minute ( nyr_end, nmo_end, ndy_end, nhr, nmin, &
        nyr_base, nmo_base, ndy_base, nhr_base, &
        lulog, jmin_end_date )

    else if ( chold(1)(1:nend) == 'PMODE' ) then

      pmode = chold(2)

      ngood = 0
      if ( trim ( pmode ) == 'OLD_NETCDF' ) ngood = 1
      if ( trim ( pmode ) == 'NEW_NETCDF' ) ngood = 1

      if ( ngood == 0 ) then
        call write_error ( lulog, &
        ' subroutine read_input_options', &
        ' unexpected pmode category ???', &
        chold(2) )
      end if

    else if ( chold(1)(1:nend) == 'DATA_DIRECTORY' ) then

      data_dir = chold(2)

    else if ( chold(1)(1:nend) == 'RUN_DIRECTORY' ) then

      run_dir = chold(2)

    else if ( chold(1)(1:nend) == 'PROCESS_MIDDLE_TROPOSPHERE' ) then

      call change_to_caps ( chold(2) )
      nend2 = lnblnk ( chold(2) )

      if ( chold(2)(1:nend2) == 'YES' ) then
        process_middle_troposphere = .true.
        nend3 = lnblnk ( chold(3) )
        nfind = index ( chold(3)(1:nend3), 'http' )
        if ( nfind == 0 ) then
          call write_error ( lulog, &
          ' subroutine read_input_options', &
          ' expecting web address ???', &
          chold(3) )
        end if
        process_middle_troposphere_web = chold(3)
        call process_versions ( nrec, chold, ndval, process_middle_troposphere_ver, &
          ncvmax, process_middle_troposphere_ncvsum, lulog )
      else if ( chold(2)(1:nend2) == 'NO' ) then
        process_middle_troposphere = .false.
      else
        call write_error ( lulog, &
        ' subroutine read_input_options', &
        ' expecting yes or no ???', &
        chold(2) )
      end if

    else if ( chold(1)(1:nend) == 'PROCESS_MIDDLE_TROPOSPHERE_FTYPES' ) then

      call process_ftypes ( nrec, chold, ihold, ndval, &
        middle_troposphere_ftypes, middle_troposphere_fselect, &
        ncvmax, middle_troposphere_nfsum, cmiss, imiss, lulog )

!     NOLA-HERE

    else if ( chold(1)(1:nend) == 'PROCESS_MIDDLE_TROPOSPHERE_TIMES' ) then

      call process_time_ranges ( nrec, chold, ndval, &
        middle_troposphere_ftypes, middle_troposphere_gfiles, &
        middle_troposphere_jmin_beg, middle_troposphere_jmin_end, &
        nyr_base, nmo_base, ndy_base, nhr_base, &
        ncvmax, middle_troposphere_nfsum, cmiss, imiss, lulog )

    else if ( chold(1)(1:nend) == 'PROCESS_LOWER_TROPOSPHERE' ) then

      call change_to_caps ( chold(2) )
      nend2 = lnblnk ( chold(2) )

      if ( chold(2)(1:nend2) == 'YES' ) then
        process_lower_troposphere = .true.
        nend3 = lnblnk ( chold(3) )
        nfind = index ( chold(3)(1:nend3), 'http' )
        if ( nfind == 0 ) then
          call write_error ( lulog, &
          ' subroutine read_input_options', &
          ' expecting web address ???', &
          chold(3) )
        end if
        process_lower_troposphere_web = chold(3)
        call process_versions ( nrec, chold, ndval, process_lower_troposphere_ver, &
          ncvmax, process_lower_troposphere_ncvsum, lulog )
      else if ( chold(2)(1:nend2) == 'NO' ) then
        process_lower_troposphere = .false.
      else
        call write_error ( lulog, &
        ' subroutine read_input_options', &
        ' expecting yes or no ???', &
        chold(2) )
      end if

    else if ( chold(1)(1:nend) == 'PROCESS_LOWER_TROPOSPHERE_FTYPES' ) then

      call process_ftypes ( nrec, chold, ihold, ndval, &
        lower_troposphere_ftypes, lower_troposphere_fselect, &
        ncvmax, lower_troposphere_nfsum, cmiss, imiss, lulog )

    else if ( chold(1)(1:nend) == 'PROCESS_LOWER_TROPOSPHERE_TIMES' ) then

      call process_time_ranges ( nrec, chold, ndval, &
        lower_troposphere_ftypes, lower_troposphere_gfiles, &
        lower_troposphere_jmin_beg, lower_troposphere_jmin_end, &
        nyr_base, nmo_base, ndy_base, nhr_base, &
        ncvmax, lower_troposphere_nfsum, cmiss, imiss, lulog )

    else if ( chold(1)(1:nend) == 'PROCESS_LOWER_STRATOSPHERE' ) then

      call change_to_caps ( chold(2) )
      nend2 = lnblnk ( chold(2) )

      if ( chold(2)(1:nend2) == 'YES' ) then
        process_lower_stratosphere = .true.
        nend3 = lnblnk ( chold(3) )
        nfind = index ( chold(3)(1:nend3), 'http' )
        if ( nfind == 0 ) then
          call write_error ( lulog, &
          ' subroutine read_input_options', &
          ' expecting web address ???', &
          chold(3) )
        end if
        process_lower_stratosphere_web = chold(3)
        call process_versions ( nrec, chold, ndval, process_lower_stratosphere_ver, &
          ncvmax, process_lower_stratosphere_ncvsum, lulog )
      else if ( chold(2)(1:nend2) == 'NO' ) then
        process_lower_stratosphere = .false.
      else
        call write_error ( lulog, &
        ' subroutine read_input_options', &
        ' expecting yes or no ???', &
        chold(2) )
      end if

    else if ( chold(1)(1:nend) == 'PROCESS_LOWER_STRATOSPHERE_FTYPES' ) then

      call process_ftypes ( nrec, chold, ihold, ndval, &
        lower_stratosphere_ftypes, lower_stratosphere_fselect, &
        ncvmax, lower_stratosphere_nfsum, cmiss, imiss, lulog )

    else if ( chold(1)(1:nend) == 'PROCESS_LOWER_STRATOSPHERE_TIMES' ) then

      call process_time_ranges ( nrec, chold, ndval, &
        lower_stratosphere_ftypes, lower_stratosphere_gfiles, &
        lower_stratosphere_jmin_beg, lower_stratosphere_jmin_end, &
        nyr_base, nmo_base, ndy_base, nhr_base, &
        ncvmax, lower_stratosphere_nfsum, cmiss, imiss, lulog )

    else if ( chold(1)(1:nend) == 'PROCESS_TROPOPAUSE' ) then

      call change_to_caps ( chold(2) )
      nend2 = lnblnk ( chold(2) )

      if ( chold(2)(1:nend2) == 'YES' ) then
        process_tropopause = .true.
        nend3 = lnblnk ( chold(3) )
        nfind = index ( chold(3)(1:nend3), 'http' )
        if ( nfind == 0 ) then
          call write_error ( lulog, &
          ' subroutine read_input_options', &
          ' expecting web address ???', &
          chold(3) )
        end if
        process_tropopause_web = chold(3)
        call process_versions ( nrec, chold, ndval, process_tropopause_ver, &
          ncvmax, process_tropopause_ncvsum, lulog )
      else if ( chold(2)(1:nend2) == 'NO' ) then
        process_tropopause = .false.
      else
        call write_error ( lulog, &
        ' subroutine read_input_options', &
        ' expecting yes or no ???', &
        chold(2) )
      end if

    else if ( chold(1)(1:nend) == 'PROCESS_TROPOPAUSE_FTYPES' ) then

      call process_ftypes ( nrec, chold, ihold, ndval, &
        tropopause_ftypes, tropopause_fselect, &
        ncvmax, tropopause_nfsum, cmiss, imiss, lulog )

    else if ( chold(1)(1:nend) == 'PROCESS_TROPOPAUSE_TIMES' ) then

      call process_time_ranges ( nrec, chold, ndval, &
        tropopause_ftypes, tropopause_gfiles, &
        tropopause_jmin_beg, tropopause_jmin_end, &
        nyr_base, nmo_base, ndy_base, nhr_base, &
        ncvmax, tropopause_nfsum, cmiss, imiss, lulog )

    else if ( chold(1)(1:nend) == 'CHECK_NETCDF_DATA' ) then

      call change_to_caps ( chold(2) )
      nend2 = lnblnk ( chold(2) )

      if ( chold(2)(1:nend2) == 'YES' ) then
        check_netcdf_data = .true.
      else
        check_netcdf_data = .false.
      end if

    else if ( chold(1)(1:nend) == 'TRIM_TLS_2005' ) then

      call change_to_caps ( chold(2) )
      nend2 = lnblnk ( chold(2) )

      if ( chold(2)(1:nend2) == 'YES' ) then
        trim_tls_2005 = .true.
      else
        trim_tls_2005 = .false.
      end if

    else if ( chold(1)(1:nend) == 'REGION_DEF_STYLE' ) then

      call change_to_caps ( chold(2) )
      nend2 = lnblnk ( chold(2) )

      region_def_style = chold(2)(1:nend2)

      call change_to_caps ( chold(3) )
      nend3 = lnblnk ( chold(3) )

      region_only = .false.
!     NOLA-HERE

      if ( chold(3)(1:nend3) == 'YES' ) then
        region_only = .true.
      else if ( chold(3)(1:nend3) == 'NO' ) then
        region_only = .false.
      else
        write (lulog,'(/a)') ' subroutine read_input_options'
        write (lulog,'(a)') ' REGION_DEF_STYLE record'
        write (lulog,'(a)') ' unexpected value of chold(3) ???'
        write (lulog,'(a,a)') ' chold(3) = ', chold(3)(1:nend3)
        stop
      end if

    else if ( chold(1)(1:nend) == 'MISSING_DATA_STYLE' ) then

      call change_to_caps ( chold(2) )
      nend2 = lnblnk ( chold(2) )

      missing_data_style = chold(2)(1:nend2)

    else if ( chold(1)(1:nend) == 'USE_ALTERNATE_ACG_DATE' ) then

      use_alternate = .false.

      call change_to_caps ( chold(2) )
      nend2 = lnblnk ( chold(2) )

      if ( chold(2)(1:nend2) == 'YES' ) then
        use_alternate = .true.
      else if ( chold(2)(1:nend2) == 'NO' ) then
        use_alternate = .false.
      else
        write (lulog,'(/a)') ' subroutine read_input_options'
        write (lulog,'(a)') ' USE_ALTERNATE_ACG_DATE record'
        write (lulog,'(a)') ' unexpected value of chold(1) ???'
        write (lulog,'(a,a)') ' chold(2) = ', chold(2)(1:nend2)
        stop
      end if

    else

      call write_error ( lulog, &
      ' subroutine read_input_options', &
      ' unexpected input category ???', &
      chold(1) )

    end if

  else

    close (unit=luinp,status='keep')
    exit

  end if

end do

write (lulog,'(/a)') ' subroutine read_input_options SUMMARY ...'
write (lulog,'(a,a)') ' pmode = ', trim ( pmode )
write (lulog,'(a,l10)') ' process_lower_troposphere=', process_lower_troposphere
write (lulog,'(a,l10)') ' process_middle_troposphere=', process_middle_troposphere
write (lulog,'(a,l10)') ' process_lower_stratosphere=', process_lower_stratosphere
write (lulog,'(a,i10)') ' nyr_beg=', nyr_beg
write (lulog,'(a,i10)') ' nmo_beg=', nmo_beg
write (lulog,'(a,i10)') ' ndy_beg=', ndy_beg
write (lulog,'(a,i10)') ' nyr_end=', nyr_end
write (lulog,'(a,i10)') ' nmo_end=', nmo_end
write (lulog,'(a,i10)') ' ndy_end=', ndy_end
nend = lnblnk ( data_dir )
write (lulog,'(a,a)') ' data_dir= ', data_dir(1:nend)
nend = lnblnk ( run_dir )
write (lulog,'(a,a)') ' run_dir= ', run_dir(1:nend)
write (lulog,'(a,l10)') ' trim_tls_2005=', trim_tls_2005
write (lulog,'(a,a)') ' region_def_style = ', trim ( region_def_style )
write (lulog,'(a,a)') ' missing_data_style = ', trim ( missing_data_style )
write (lulog,'(a,l10)') ' use_alternate=', use_alternate

ngood = 0
if ( trim ( region_def_style ) == 'GLOBAL' ) ngood = ngood + 1
if ( trim ( region_def_style ) == 'VAR' ) ngood = ngood + 1
if ( trim ( missing_data_style ) == 'GLOBAL' ) ngood = ngood + 1
if ( trim ( missing_data_style ) == 'VAR' ) ngood = ngood + 1

if ( ngood /= 2 ) then
  write (lulog,'(/a)') ' subroutine read_input_options'
  write (lulog,'(a)') ' region_def_style and/or missing_data_style not defined properly ???'
  stop
end if

if ( process_lower_troposphere ) then
  write (lulog,'(/a)') ' LT SUMMARY ...'
  nend = lnblnk ( process_lower_troposphere_web )
  write (lulog,'(a,a)') ' process_lower_troposphere_web= ', process_lower_troposphere_web(1:nend)
  do nv = 1, process_lower_troposphere_ncvsum
    nend = lnblnk ( process_lower_troposphere_ver(nv) )
    write (lulog,'(a,i10,5x,a)') ' nv, process_lower_troposphere_ver(nv)=', &
      nv, process_lower_troposphere_ver(nv)(1:nend)
  end do
  do nf = 1, lower_troposphere_nfsum
    nend = lnblnk ( lower_troposphere_ftypes(nf) )
    write (lulog,'(a,i10,5x,a)') ' nf, lower_troposphere_ftypes(nf)=', &
      nf, lower_troposphere_ftypes(nf)(1:nend)
  end do
end if

if ( process_middle_troposphere ) then
  write (lulog,'(/a)') ' MT SUMMARY ...'
  nend = lnblnk ( process_middle_troposphere_web )
  write (lulog,'(a,a)') ' process_middle_troposphere_web= ', process_middle_troposphere_web(1:nend)
  do nv = 1, process_middle_troposphere_ncvsum
    nend = lnblnk ( process_middle_troposphere_ver(nv) )
    write (lulog,'(a,i10,5x,a)') ' nv, process_middle_troposphere_ver(nv)=', &
      nv, process_middle_troposphere_ver(nv)(1:nend)
  end do
  do nf = 1, middle_troposphere_nfsum
    nend = lnblnk ( middle_troposphere_ftypes(nf) )
    write (lulog,'(a,i10,5x,a)') ' nf, middle_troposphere_ftypes(nf)=', &
      nf, middle_troposphere_ftypes(nf)(1:nend)
  end do
end if

if ( process_lower_stratosphere ) then
  write (lulog,'(/a)') ' LT SUMMARY ...'
  nend = lnblnk ( process_lower_stratosphere_web )
  write (lulog,'(a,a)') ' process_lower_stratosphere_web= ', process_lower_stratosphere_web(1:nend)
  do nv = 1, process_lower_stratosphere_ncvsum
    nend = lnblnk ( process_lower_stratosphere_ver(nv) )
    write (lulog,'(a,i10,5x,a)') ' nv, process_lower_stratosphere_ver(nv)=', &
      nv, process_lower_stratosphere_ver(nv)(1:nend)
  end do
  do nf = 1, lower_stratosphere_nfsum
    nend = lnblnk ( lower_stratosphere_ftypes(nf) )
    write (lulog,'(a,i10,5x,a)') ' nf, lower_stratosphere_ftypes(nf)=', &
      nf, lower_stratosphere_ftypes(nf)(1:nend)
  end do
end if

if ( process_tropopause ) then
  write (lulog,'(/a)') ' TP SUMMARY ...'
  nend = lnblnk ( process_tropopause_web )
  write (lulog,'(a,a)') ' process_tropopause_web= ', process_tropopause_web(1:nend)
  do nv = 1, process_tropopause_ncvsum
    nend = lnblnk ( process_tropopause_ver(nv) )
    write (lulog,'(a,i10,5x,a)') ' nv, process_tropopause_ver(nv)=', &
      nv, process_tropopause_ver(nv)(1:nend)
  end do
  do nf = 1, tropopause_nfsum
    nend = lnblnk ( tropopause_ftypes(nf) )
    write (lulog,'(a,i10,5x,a)') ' nf, tropopause_ftypes(nf)=', &
      nf, tropopause_ftypes(nf)(1:nend)
  end do
end if

end subroutine

end module
!
!***********************************************************************
!
module module_process_time_ranges
implicit none
contains

subroutine process_time_ranges ( nrec, chold, ndval, cv, gfiles, jmin_beg, jmin_end, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  ncvmax, ncvsum, cmiss, imiss, lulog )
 
!  Global data module

!  Modules for needed subroutines ...

use module_write_error
use module_calc_julian_minute
use module_inv_julian_minute

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: nrec
character ( len = * ), intent(in), dimension(ndval) :: chold
integer, intent(in) :: ndval
character ( len = * ), intent(in), dimension(ncvmax) :: cv
character ( len = * ), intent(inout), dimension(ncvmax) :: gfiles
integer, intent(inout), dimension(ncvmax) :: jmin_beg
integer, intent(inout), dimension(ncvmax) :: jmin_end
integer, intent(in) :: nyr_base
integer, intent(in) :: nmo_base
integer, intent(in) :: ndy_base
integer, intent(in) :: nhr_base

integer, intent(in) :: ncvmax
integer, intent(in) :: ncvsum
character ( len = * ), intent(in) :: cmiss
integer, intent(in) :: imiss
integer, intent(in) :: lulog

! Local Declarations

integer :: nend
integer :: nend2
integer :: nend3
integer :: nend4
integer :: nend5
integer :: nendc
integer :: lnblnk
integer :: nc
integer :: nfind
integer :: nyr
integer :: nmo
integer :: ndy
integer :: nhr
integer :: nmin

nendc = lnblnk ( cmiss )
nend2 = lnblnk ( chold(2) )

nfind = 0

! Find specified ftype in ftype name array

! PROCESS_MIDDLE_TROPOSPHERE_TIMES amg 19781116_ 20151031_	global_attributes_amg_mt.txt

do nc = 1, ncvsum
  nend = lnblnk ( cv(nc) )
  if ( cv(nc)(1:nend) == chold(2)(1:nend2) ) then
    nfind = nc
    exit
  end if
end do

if ( nfind == 0 ) then
  call write_error ( lulog, &
  ' subroutine process_time_ranges', &
  ' could not find match: nfind == 0 ???', &
  nrec )
end if

nend3 = lnblnk ( chold(3) )

if ( chold(3)(1:nend3) == cmiss(1:nendc) ) then

  jmin_beg(nfind) = imiss
  jmin_end(nfind) = imiss
  return

else

  if ( chold(3)(nend3:nend3) /= '_' ) then
    call write_error ( lulog, &
    ' subroutine process_time_ranges', &
    ' time field does not end in _ ???', &
    nrec )
  end if

  nend3 = nend3 - 1
! yyyymmdd
  if ( nend3 /= 8 ) then
    call write_error ( lulog, &
    ' subroutine process_time_ranges', &
    ' time field is not of form yyyymmdd ???', &
    nrec )
  end if

  read (chold(3)(1:nend3),'(i4,2i2)') nyr, nmo, ndy

  nhr = 0
  nmin = 0

  call calc_julian_minute ( nyr, nmo, ndy, nhr, nmin, &
    nyr_base, nmo_base, ndy_base, nhr_base, &
    lulog, jmin_beg(nfind) )

end if

nend4 = lnblnk ( chold(4) )

if ( chold(4)(1:nend4) == cmiss(1:nendc) ) then

  jmin_beg(nfind) = imiss
  jmin_end(nfind) = imiss
  return

else

  if ( chold(4)(nend4:nend4) /= '_' ) then
    call write_error ( lulog, &
    ' subroutine process_time_ranges', &
    ' time field does not end in _ ???', &
    nrec )
  end if

  nend4 = nend4 - 1
! yyyymmdd
  if ( nend4 /= 8 ) then
    call write_error ( lulog, &
    ' subroutine process_time_ranges', &
    ' time field is not of form yyyymmdd ???', &
    nrec )
  end if

  read (chold(4)(1:nend4),'(i4,2i2)') nyr, nmo, ndy

  nhr = 0
  nmin = 0

  call calc_julian_minute ( nyr, nmo, ndy, nhr, nmin, &
    nyr_base, nmo_base, ndy_base, nhr_base, &
    lulog, jmin_end(nfind) )

end if

nend5 = lnblnk ( chold(5) )
if ( nend5 <= 0 ) then
  nend5 = 1
end if

if ( chold(5)(1:nend5) == cmiss(1:nendc) .or. chold(5)(1:nend5) == ' ' ) then
    call write_error ( lulog, &
    ' subroutine process_time_ranges', &
    ' gfiles field not defined ???', &
    nrec )
end if

gfiles(nfind) = chold(5)(1:nend5)

write (lulog,'(/a)') ' subroutine process_time_ranges'
nend = lnblnk ( cv(nfind) )
write (lulog,'(a,a)') ' ftype: cv(nfind)= ', cv(nfind)(1:nend)
write (lulog,'(a,i10)') ' nfind=', nfind
write (lulog,'(a,i10)') ' jmin_beg(nfind)=', jmin_beg(nfind)
write (lulog,'(a,a)') ' gfiles(nfind) = ', trim ( gfiles(nfind) )

call inv_julian_minute ( nyr, nmo, ndy, nhr, nmin, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_beg(nfind) )

write (lulog,'(a,t15,i10)') ' beg nyr=', nyr
write (lulog,'(a,t15,i10)') ' beg nmo=', nmo
write (lulog,'(a,t15,i10)') ' beg ndy=', ndy
write (lulog,'(a,t15,i10)') ' beg nhr=', nhr
write (lulog,'(a,t15,i10)') ' beg nmin=', nmin

write (lulog,'(a,i10)') ' jmin_end(nfind)=', jmin_end(nfind)

call inv_julian_minute ( nyr, nmo, ndy, nhr, nmin, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jmin_end(nfind) )

write (lulog,'(a,t15,i10)') ' end nyr=', nyr
write (lulog,'(a,t15,i10)') ' end nmo=', nmo
write (lulog,'(a,t15,i10)') ' end ndy=', ndy
write (lulog,'(a,t15,i10)') ' end nhr=', nhr
write (lulog,'(a,t15,i10)') ' end nmin=', nmin

end subroutine

end module
!
!***********************************************************************
!
module module_get_epoch_seconds
implicit none
contains
 
subroutine get_epoch_seconds ( nyr, nmo, ndy, esec, lulog )
 
!  Global data module

!  Modules for needed subroutines ...

use module_get_free_logical_unit
use module_write_error

implicit none
 
!  Procedure dummy argument declarations

integer, parameter :: double_precision = 8

integer, intent(in) :: nyr
integer, intent(in) :: nmo
integer, intent(in) :: ndy
integer ( kind = double_precision ), intent(inout) :: esec
integer, intent(in) :: lulog

! Local Declarations

character ( len = 500 ) :: command
integer :: nend
integer :: lnblnk
integer :: lutemp
integer :: ios

integer, save :: nhr = 0
integer, save :: nmin = 0
integer, save :: nsec = 0

!       date -u --date="2013-02-02 00:21:03" +"%Y %m %d %H %M %S %s"
!       %s     seconds since 1970-01-01 00:00:00 UTC

command = ''

write (command,'(a,i4,a1,4(i2.2,a1),i2.2,a)') &
'date -u --date="', &
nyr, '-', &
nmo, '-', &
ndy, ' ', &
nhr, ':', &
nmin, ':', &
nsec, '" +"%s" > date.dmp'

nend = lnblnk ( command )

call system ( command(1:nend) )

call get_free_logical_unit ( lutemp )

open (unit=lutemp,status='old',form='formatted',file='date.dmp',iostat=ios)

if ( ios /= 0 ) then
  call write_error ( lulog, &
  ' subroutine get_epoch_seconds', &
  ' failed to read file date.dmp from unix date command ???', &
  '' )
end if

read (lutemp,*) esec

close (unit=lutemp,status='delete')

end subroutine

end module
!
!***********************************************************************
!
module module_process_versions
implicit none
contains
 
subroutine process_versions ( nrec, chold, ndval, cv, ncvmax, ncvsum, lulog )
 
!  Global data module

!  Modules for needed subroutines ...

use module_write_error

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: nrec
character ( len = * ), intent(in), dimension(ndval) :: chold
integer, intent(in) :: ndval
character ( len = * ), intent(inout), dimension(ncvmax) :: cv
integer, intent(in) :: ncvmax
integer, intent(inout) :: ncvsum
integer, intent(in) :: lulog

! Local Declarations

integer :: nend
integer :: lnblnk
integer :: nc

ncvsum = 0

do nc = 4, ndval
  if ( chold(nc)(1:1) == 'V' .or. chold(nc)(1:1) == 'v' ) then
    ncvsum = ncvsum + 1
    if ( ncvsum > ncvmax ) then
      call write_error ( lulog, &
      ' subroutine process_versions', &
      ' too many versions: ncvsum > ncvmax ???', &
      nrec )
    end if
    cv(ncvsum) = chold(nc)
  else
    exit
  end if
end do

if ( ncvsum == 0 ) then
  call write_error ( lulog, &
  ' subroutine process_versions', &
  ' no versions: ncvsum == 0 ???', &
  nrec )
end if

end subroutine

end module
!
!***********************************************************************
!
module module_process_ftypes
implicit none
contains

subroutine process_ftypes ( nrec, chold, ihold, ndval, cv, fselect, &
  ncvmax, ncvsum, cmiss, imiss, lulog )
 
!  Global data module

!  Modules for needed subroutines ...

use module_write_error

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: nrec
character ( len = * ), intent(in), dimension(ndval) :: chold
integer, intent(in), dimension(ndval) :: ihold
integer, intent(in) :: ndval
character ( len = * ), intent(inout), dimension(ncvmax) :: cv
integer, intent(inout), dimension(ncvmax) :: fselect
integer, intent(in) :: ncvmax
integer, intent(inout) :: ncvsum
character ( len = * ), intent(in) :: cmiss
integer, intent(in) :: imiss
integer, intent(in) :: lulog

! Local Declarations

integer :: nend
integer :: nend2
integer :: lnblnk
integer :: nc

ncvsum = 0
nend2 = lnblnk ( cmiss )

do nc = 2, ndval
  nend = lnblnk ( chold(nc) )
  if ( chold(nc)(1:nend) == cmiss(1:nend2) ) then
    exit
  end if
  ncvsum = ncvsum + 1
  if ( ncvsum > ncvmax ) then
    call write_error ( lulog, &
    ' subroutine process_ftypes', &
    ' too many ftypes: ncvsum > ncvmax ???', &
    nrec )
  end if
  cv(ncvsum) = chold(nc)
  if ( ihold(ncvsum) <= 0 ) then
    call write_error ( lulog, &
    ' subroutine process_ftypes', &
    ' fselect field not defined ???', &
    nrec )
  end if
  fselect(ncvsum) = ihold(ncvsum)
end do

if ( ncvsum == 0 ) then
  call write_error ( lulog, &
  ' subroutine process_ftypes', &
  ' no ftypes: ncvsum == 0 ???', &
  nrec )
end if

end subroutine

end module
!
!***********************************************************************
!
module module_replace_commas_with_tabs
implicit none
contains
 
subroutine replace_commas_with_tabs ( nrec )
 
!  Global data module

!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(inout) :: nrec

! Local Declarations

integer :: nend
integer :: lnblnk
character ( len = 1 ), parameter :: tab = '     '
integer :: nc

nend = lnblnk ( nrec )

if ( nend <= 0 ) then
  nrec = ' '
  return
end if

do nc = 1, nend
  if ( nrec(nc:nc) == ',' ) then
    nrec(nc:nc) = tab
  end if
end do

end subroutine

end module
!
!***********************************************************************
!
module module_change_to_caps
implicit none
contains
 
subroutine change_to_caps ( nrec )
 
!  Global data module

!  Modules for needed subroutines ...

implicit none
 
!  Procedure dummy argument declarations

character ( len = * ), intent(inout) :: nrec

! Local Declarations

integer :: nend
integer :: lnblnk
integer :: nc
integer :: na

! Local Arrays

integer, parameter :: namax = 26
character ( len = 1 ), save :: lcase(namax)
character ( len = 1 ), save :: ucase(namax)

lcase = (/ &
 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', &
 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', &
 'u', 'v', 'w', 'x', 'y', 'z' &
 /)

ucase = (/ &
 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', &
 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', &
 'U', 'V', 'W', 'X', 'Y', 'Z' &
 /)

nend = lnblnk ( nrec )

if ( nend <= 0 ) then
  return
end if

do nc = 1, nend
  do na = 1, namax
    if ( nrec(nc:nc) == lcase(na) ) then
      nrec(nc:nc) = ucase(na)
    end if
  end do
end do

end subroutine

end module
!
!***********************************************************************
!
module module_get_jdnum
implicit none
contains
 
subroutine get_jdnum ( nyr, nmo, ndy, nhr, nmin, nsec, jdnum, lulog )

implicit none

integer, parameter :: double_precision = 8

!  Procedure dummy argument declarations

integer, intent(in) :: nyr
integer, intent(in) :: nmo
integer, intent(in) :: ndy
integer, intent(in) :: nhr
integer, intent(in) :: nmin
integer, intent(in) :: nsec
real ( kind = double_precision ), intent(inout) :: jdnum
integer, intent(in) :: lulog

! Local Declaration

integer :: nyr_use
integer :: ndy_use
integer :: nmo_use
integer :: c
integer ( kind = double_precision ) :: a
integer ( kind = double_precision ) :: b
real ( kind = double_precision ) :: dfrac
real ( kind = double_precision ) :: day_use
integer ( kind = double_precision ) :: arg1
integer ( kind = double_precision ) :: arg2
logical :: ldebug

! Formulas taken from:
! Astronomical Algorithms
! Second Edition
! Chapter 7. Julian Day
! Jean Meeus
! William-Bell, Inc.
! 1998, 477p

if ( nmo <= 2 ) then
  nmo_use = nmo + 12
  nyr_use = nyr - 1
else
  nmo_use = nmo
  nyr_use = nyr
end if

a = int8( dble(nyr_use) / 100.0d+00 )
b = 2 - a + int8( dble(a) / 4.0d+00 )

dfrac = ( dble(nhr)/24.0d+00 ) + ( dble(nmin) / dble( 24 * 60 ) ) + &
        ( dble(nsec) / dble( 24 * 60 * 60 ) )

day_use = dble( ndy ) + dfrac

arg1 = int8( 365.25d+00 * ( dfloat(nyr_use) + 4716.00d+00 ) )
arg2 = int8( 30.6001d+00 * ( dfloat(nmo) + 1.00d+00 ) )

jdnum = dble ( arg1 + arg2 + b ) +  day_use - 1524.5d+00

ldebug = .false.
ldebug = .true.

if ( ldebug ) then
  write (lulog,'(/a)') ' subroutine get_jdnum'
  write (lulog,'(a,i10)') ' nyr=', nyr
  write (lulog,'(a,i10)') ' nmo=', nmo
  write (lulog,'(a,i10)') ' ndy=', ndy
  write (lulog,'(a,i10)') ' nhr=', nhr
  write (lulog,'(a,i10)') ' nmin=', nmin
  write (lulog,'(a,i10)') ' nyr_use=', nyr_use
  write (lulog,'(a,i10)') ' nmo_use=', nmo_use
  write (lulog,'(a,f15.3)') ' dfrac=', dfrac
  write (lulog,'(a,f15.3)') ' day_use=', day_use
  write (lulog,'(a,i15)') ' a= ', a
  write (lulog,'(a,i15)') ' b= ', b
  write (lulog,'(a,i15)') ' arg1=', arg1
  write (lulog,'(a,i15)') ' arg2=', arg2
  write (lulog,'(a,f15.3)') ' dfrac=', dfrac
  write (lulog,'(a,f15.5)') ' jdnum=', jdnum
end if

end subroutine

end module
!
!***********************************************************************
!
module module_get_free_logical_unit
implicit none
contains
 
subroutine get_free_logical_unit ( lutemp )
 
!  Modules for needed subroutines ...
 
!  Global data module
 
implicit none
 
!  Procedure dummy argument declarations
 
integer, intent(inout) :: lutemp
 
!  Local declarations
 
logical :: check_for_open
integer :: nlu
integer :: ios
 
do nlu = 10 , 2000
  inquire (unit=nlu, opened=check_for_open )
  if ( .not.check_for_open ) then
    lutemp = nlu
    exit
  end if
end do
 
end subroutine
 
end module
!
!***********************************************************************
!
module module_write_alloc_status
implicit none
contains

subroutine write_alloc_status ( unit_name, vname, istatus, id_1d2d3d, &
  imax, jmax, kmax, lulog )

!  Global data module

!  Modules for needed subroutines ...

implicit none

!  Procedure dummy argument declarations

character ( len = * ), intent(in) :: unit_name
character ( len = * ), intent(in) :: vname
integer, intent(in) :: istatus
character ( len = * ), intent(in) :: id_1d2d3d
integer, intent(in) :: imax
integer, intent(in) :: jmax
integer, intent(in) :: kmax
integer, intent(in) :: lulog

!  Local declarations

write (lulog,'(/a,a)') ' Program unit= ',unit_name
write (lulog,'(a)') ' unable to allocate array ???'
write (lulog,'(a,a)') ' array name= ',vname
write (lulog,'(a,a)') ' dimension id= ',id_1d2d3d
write (lulog,'(a,i10)') ' istatus=',istatus
write (lulog,'(a,i10)') ' imax=',imax
write (lulog,'(a,i10)') ' jmax=',jmax
write (lulog,'(a,i10)') ' kmax=',kmax
stop

end subroutine

end module
!
!***********************************************************************
!
module module_calc_julian_minute
implicit none
contains

subroutine calc_julian_minute ( nyr_in, nmo_in, ndy_in, nhr_in, nmin_in, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jul_min )

!  Modules for needed subroutines ...

use module_julian
use module_getjtime
use module_inv_getjtime
use module_inverse_julian

!  Global data module

implicit none

!  Procedure dummy argument declarations

integer, intent(in) :: nyr_in
integer, intent(in) :: nmo_in
integer, intent(in) :: ndy_in
integer, intent(in) :: nhr_in
integer, intent(in) :: nmin_in

integer, intent(in) :: nyr_base
integer, intent(in) :: nmo_base
integer, intent(in) :: ndy_base
integer, intent(in) :: nhr_base

integer, intent(in) :: lulog

integer, intent(inout) :: jul_min

!  Local declarations

integer :: nyr
integer :: nmo
integer :: ndy
integer :: nhr
integer :: nmin
integer :: jday
integer :: jmin
integer :: jminb
integer :: ny
integer :: nyend
integer :: jtest
integer :: nyr_out
integer :: nmo_out
integer :: ndy_out
integer :: nhr_out
integer :: nmin_out

!
!=======================================================================
! Calculate time in julian minutes from the specified reference date.
!=======================================================================
!

jul_min = 0

if ( nyr_in > nyr_base ) then
  nyend = nyr_in - 1
  nmo = 12
  ndy = 31
  nhr = 24
  nmin = 0
  do ny = nyr_base, nyend
    call julian ( nmo, ndy, ny, jday )
    call getjtime ( jday, nhr, nmin, jmin )
    jul_min = jul_min + jmin
  end do
  call julian ( nmo_in, ndy_in, nyr_in, jday )
  call getjtime ( jday, nhr_in, nmin_in, jmin )
  jul_min = jul_min + jmin
else if ( nyr_in == nyr_base ) then
  call julian ( nmo_in, ndy_in, nyr_in, jday )
  call getjtime ( jday, nhr_in, nmin_in, jmin )
  jul_min = jul_min + jmin
else if ( nyr_in < nyr_base ) then
  write (lulog,'(/a)') ' subroutine calc_julian_minute'
  write (lulog,'(a)') &
  ' requested date is before base time for julian minute calculations'
  write (lulog,'(a,i5)') ' nyr_base=',nyr_base
  write (lulog,'(a,i5)') ' nyr_in=',nyr_in
  close (unit=lulog,status='keep')
  stop
end if

end subroutine

end module
!
!***********************************************************************
!
module module_inverse_julian
implicit none
contains

subroutine inverse_julian ( imo, idy, iyr2, jday )

implicit none

!  Procedure dummy argument declarations

integer, intent(inout) :: imo
integer, intent(inout) :: idy
integer, intent(inout) :: iyr2
integer, intent(in) :: jday

!  Local declarations

integer, parameter :: nmonth = 12
integer, dimension (nmonth), save :: mday
integer :: iyr
integer :: jsum
integer :: jsumold
integer :: l
integer :: n100
integer :: n400
integer :: nadd
integer :: nleap
integer :: nltest

mday = (/ &
 31, 28, 31, 30, 31, 30, &
 31, 31, 30, 31, 30, 31 &
 /)

!	test for leap year
!	a leap year occurs every 4th year with the exception of
!	centenary years not divisible by 400

if ( iyr2 >= 1900 ) then
  iyr = iyr2
else
  if ( iyr2 <= 50 ) then
    iyr = iyr2 + 2000
  else
    iyr = iyr2 + 1900
  end if
end if

nleap = 0
nltest = mod( iyr,4 )

if ( nltest == 0 ) then
  nleap = 1
  n100 = mod( iyr,100 )
  if ( n100 == 0 ) then
    n400 = mod( iyr,400 )
    if ( n400 /= 0 ) nleap = 0
  end if
end if

jsum = 0

do l = 1, nmonth
  if ( l == 2 ) then
    nadd = nleap
  else
    nadd = 0
  end if
  jsumold = jsum
  jsum = jsum + mday(l) + nadd
  if ( jsum >= jday ) then
    imo = l
    idy = jday - jsumold
    exit
  end if
end do

end subroutine

end module
!
!***********************************************************************
!
module module_inv_getjtime
implicit none
contains

subroutine inv_getjtime ( jday, nhr, nmin, jmin )

implicit none

!  Procedure dummy argument declarations

integer, intent(inout) :: jday
integer, intent(inout) :: nhr
integer, intent(inout) :: nmin
integer, intent(inout) :: jmin

!  Local declarations

integer :: jdmin
integer :: jhmin
integer :: jminr
real :: xday
real :: xhour

xday = float(jmin) / 1440.0
jday = int( xday )
jdmin = jday * 1440
jminr = jmin - jdmin
xhour = float(jminr) / 60.0
nhr = int( xhour )
jhmin = nhr * 60
nmin = jminr - jhmin
jday = jday + 1

end subroutine

end module
!
!***********************************************************************
!
module module_inv_julian_minute
implicit none
contains

subroutine inv_julian_minute ( nyr_out, nmo_out, ndy_out, nhr_out, nmin_out, &
  nyr_base, nmo_base, ndy_base, nhr_base, &
  lulog, jul_min )

!  Modules for needed subroutines ...

use module_julian
use module_getjtime
use module_inv_getjtime
use module_inverse_julian

!  Global data module

implicit none

!  Procedure dummy argument declarations

integer, intent(inout) :: nyr_out
integer, intent(inout) :: nmo_out
integer, intent(inout) :: ndy_out
integer, intent(inout) :: nhr_out
integer, intent(inout) :: nmin_out

integer, intent(in) :: nyr_base
integer, intent(in) :: nmo_base
integer, intent(in) :: ndy_base
integer, intent(in) :: nhr_base

integer, intent(in) :: lulog

integer, intent(in) :: jul_min

!  Local declarations

integer :: nyr
integer :: nmo
integer :: ndy
integer :: nhr
integer :: nmin
integer :: jday
integer :: jmin
integer :: jminb
integer :: ny
integer :: nyend
integer :: jtest

nyr = nyr_base
nmo = 12
ndy = 31
nhr = 24
nmin = 0
jtest = 0

do
  call julian ( nmo, ndy, nyr, jday )
  call getjtime ( jday, nhr, nmin, jmin )
  jtest = jtest + jmin
  if ( jtest > jul_min ) then
    jtest = jtest - jmin
    jmin = jul_min - jtest
    nyr_out = nyr
    call inv_getjtime ( jday, nhr_out, nmin_out, jmin )
    call inverse_julian ( nmo_out, ndy_out, nyr, jday )
    exit
  end if
  nyr = nyr + 1
end do

end subroutine

end module
!
!***********************************************************************
!
module module_julian
implicit none
contains

subroutine julian ( imo, idy, iyr2, jday )

implicit none

!  Procedure dummy argument declarations

integer, intent(in) :: imo
integer, intent(in) :: idy
integer, intent(in) :: iyr2
integer, intent(inout) :: jday

!  Local declarations

integer, parameter :: nmonth = 12
integer, dimension (nmonth), save :: mday
integer :: imob
integer :: iyr
integer :: jsum
integer :: l
integer :: n100
integer :: n400
integer :: nadd
integer :: nleap
integer :: nltest

mday = (/ &
 31, 28, 31, 30, 31, 30, &
 31, 31, 30, 31, 30, 31 &
 /)

!	test for leap year
!	a leap year occurs every 4th year with the exception of
!	centenary years not divisible by 400

nleap = 0

if ( iyr2 >= 1900 ) then
  iyr = iyr2
else
  if ( iyr2 <= 50 ) then
    iyr = iyr2 + 2000
  else
    iyr = iyr2 + 1900
  end if
end if

nltest = mod( iyr,4 )

if ( nltest == 0 ) then
  nleap = 1
  n100 = mod( iyr,100 )
  if ( n100 == 0 ) then
    n400 = mod( iyr,400 )
    if ( n400 /= 0 ) nleap = 0
  end if
end if

jsum = 0

imob = imo - 1

if ( imob > 0 ) then
  do l = 1, imob
    if ( l == 2 ) then
      nadd = nleap
    else
      nadd = 0
    end if
    jsum = jsum + mday(l) + nadd
  end do
end if

jday = jsum + idy

end subroutine

end module
!
!***********************************************************************
!
module module_getjtime
implicit none
contains

subroutine getjtime ( jday, nhr, nmin, jmin )

implicit none

!  Procedure dummy argument declarations

integer, intent(in) :: jday
integer, intent(in) :: nhr
integer, intent(in) :: nmin
integer, intent(inout) :: jmin

jmin = (jday - 1) * 1440 + nhr * 60 + nmin

end subroutine

end module
!
!***********************************************************************
!
module module_hsort
implicit none
contains
 
subroutine hsort ( key, ra, n )
 
implicit none
 
!  Procedure dummy argument declarations
 
integer, intent(inout), dimension (n) :: key
real, intent(inout), dimension (n) :: ra
integer, intent(inout) :: n
 
!  Local declarations
 
integer :: i
integer :: ir
integer :: j
integer :: k
integer :: ktemp
integer :: l
real :: rra
 
! Numerical Recipes heap-sort routine
 
l = (n / 2) + 1
ir = n
 
do k = 1, n
  key(k) = k
end do
 
outer: do
 
  if ( l > 1 ) then
    l = l - 1
    rra = ra(l)
    ktemp = key(l)
  else
    rra = ra(ir)
    ktemp = key(ir)
    ra(ir) = ra(1)
    key(ir) = key(1)
    ir = ir - 1
    if ( ir == 1 ) then
      ra(1) = rra
      key(1) = ktemp
      exit outer
    end if
  end if
   
  i = l
  j = l + l
 
  inner: do
    if ( j <= ir ) then
      if ( j < ir ) then
        if ( ra(j) < ra(j + 1) ) j = j + 1
      end if
      if ( rra < ra(j) ) then
        ra(i) = ra(j)
        key(i) = key(j)
        i = j
        j = j + j
      else
        j = ir + 1
      end if
    else
      exit inner
    end if
  end do inner
 
  ra(i) = rra
  key(i) = ktemp

end do outer
 
end subroutine
 
end module
!
!***********************************************************************
!  END OF PMSU CODE
!***********************************************************************
!
module module_global_pmsu

use module_mdstore1_def
use module_ddstore1_def

implicit none
save
!
! START GLOBAL MODULES
!
!***********************************************************************
!
!  Various Parameter Values ...

integer, parameter :: double_precision = 8
integer, parameter :: half_precision = 2
integer, parameter :: imax = 144
integer, parameter :: jmax = 72
integer, parameter :: jmaxz = 67
integer, parameter :: nmonths = 12

!
!***********************************************************************
!
!  Missing Values

real, parameter :: rmiss = -999999.0
integer, parameter :: imiss = -999999
character ( len = 8 ), parameter :: cmiss = '99999999'

!
!***********************************************************************
!
!  Logical Units ...

integer :: lulog

!
!***********************************************************************
!
!  General Declarations

logical :: process_lower_troposphere = .false.
logical :: process_middle_troposphere = .false.
logical :: process_lower_stratosphere = .false.
logical :: process_tropopause = .false.

character ( len = 500 ) :: process_lower_troposphere_web = cmiss
character ( len = 500 ) :: process_middle_troposphere_web = cmiss
character ( len = 500 ) :: process_lower_stratosphere_web = cmiss
character ( len = 500 ) :: process_tropopause_web = cmiss

character ( len = 500 ) :: data_dir = cmiss
character ( len = 500 ) :: run_dir = cmiss
character ( len = 500 ) :: pmode = cmiss

integer, parameter :: ncvmax = 100

character ( len = 10 ), dimension(ncvmax) :: process_lower_troposphere_ver = cmiss
character ( len = 10 ), dimension(ncvmax) :: process_middle_troposphere_ver = cmiss
character ( len = 10 ), dimension(ncvmax) :: process_lower_stratosphere_ver = cmiss
character ( len = 10 ), dimension(ncvmax) :: process_tropopause_ver = cmiss

integer :: process_lower_troposphere_ncvsum = 0
integer :: process_middle_troposphere_ncvsum = 0
integer :: process_lower_stratosphere_ncvsum = 0
integer :: process_tropopause_ncvsum = 0

character ( len = 100 ), dimension(ncvmax) :: lower_troposphere_ftypes = cmiss
character ( len = 100 ), dimension(ncvmax) :: middle_troposphere_ftypes = cmiss
character ( len = 100 ), dimension(ncvmax) :: lower_stratosphere_ftypes = cmiss
character ( len = 100 ), dimension(ncvmax) :: tropopause_ftypes = cmiss

integer, dimension(ncvmax) :: lower_troposphere_jmin_beg = imiss
integer, dimension(ncvmax) :: lower_troposphere_jmin_end = imiss

integer, dimension(ncvmax) :: middle_troposphere_jmin_beg = imiss
integer, dimension(ncvmax) :: middle_troposphere_jmin_end = imiss

integer, dimension(ncvmax) :: lower_stratosphere_jmin_beg = imiss
integer, dimension(ncvmax) :: lower_stratosphere_jmin_end = imiss

integer, dimension(ncvmax) :: tropopause_jmin_beg = imiss
integer, dimension(ncvmax) :: tropopause_jmin_end = imiss

integer, dimension(ncvmax) :: jmin_beg_exc
integer, dimension(ncvmax) :: jmin_end_exc

integer, dimension(ncvmax) :: lower_troposphere_fselect = imiss
integer, dimension(ncvmax) :: middle_troposphere_fselect = imiss
integer, dimension(ncvmax) :: lower_stratosphere_fselect = imiss
integer, dimension(ncvmax) :: tropopause_fselect = imiss

character ( len = 1000 ), dimension(ncvmax) :: lower_troposphere_gfiles = cmiss
character ( len = 1000 ), dimension(ncvmax) :: middle_troposphere_gfiles = cmiss
character ( len = 1000 ), dimension(ncvmax) :: lower_stratosphere_gfiles = cmiss
character ( len = 1000 ), dimension(ncvmax) :: tropopause_gfiles = cmiss

integer :: lower_troposphere_nfsum = 0
integer :: middle_troposphere_nfsum = 0
integer :: lower_stratosphere_nfsum = 0
integer :: tropopause_nfsum = 0

real :: dlatlon
real :: offset

real :: glat(jmax)
real :: glat_bnds(2,jmax)
real :: glon(imax)
real :: glon_bnds(2,imax)

real :: glatz(jmaxz)

integer, dimension(jmaxz) :: ndataz

!
!***********************************************************************
!

! Monthly gridded anomaly declarations

integer, allocatable, dimension(:,:,:) :: ndata
integer, allocatable, dimension(:) :: month_store
integer, target, allocatable, dimension(:,:,:) :: ndata_2d
real, target, allocatable, dimension(:,:,:) :: rdata_2d
real, target, allocatable, dimension(:,:,:) :: reg_rdata_2d
real ( kind = double_precision ), target, allocatable, dimension(:) :: esec_monthly_hold
character ( len = 19 ), target, allocatable, dimension(:) :: cdate_monthly_hold
real ( kind = double_precision ), target, dimension(1) :: esec_temp

!
!***********************************************************************
!

! Monthly regional anomaly declarations

real, target, allocatable, dimension(:,:) :: reg_data_2d
real ( kind = double_precision ), target, allocatable, dimension(:) :: esec_monthly_reg_hold
character ( len = 19 ), target, allocatable, dimension(:) :: cdate_monthly_reg_hold
real ( kind = double_precision ), target, dimension(1) :: esec_reg_temp

integer, parameter :: nreg_max = 27
integer, parameter :: rname_max = 50
character ( len = rname_max ), dimension(nreg_max) :: rname
real, dimension(nreg_max) :: region_lat1
real, dimension(nreg_max) :: region_lat2
real, target, dimension(nreg_max) :: thold

integer, parameter :: note_hold_max = 20
character ( len = 200 ), dimension(note_hold_max) :: note_hold = cmiss
integer :: note_hold_sum = 0

integer, parameter :: region_def_hold_max = 20
character ( len = 200 ), dimension(region_def_hold_max) :: region_def_hold = cmiss
integer :: region_def_hold_sum = 0

!
!***********************************************************************
!

type (dstore_info_1) :: mon_anom_store
type (dstore_info_2) :: day_anom_store

!
!***********************************************************************
!
! Global max and min


real :: amg_min_mt = +1.0e+30
real :: amg_max_mt = -1.0e+30
real :: amg_min_lt = +1.0e+30
real :: amg_max_lt = -1.0e+30
real :: amg_min_ls = +1.0e+30
real :: amg_max_ls = -1.0e+30
real :: amg_min_tp = +1.0e+30
real :: amg_max_tp = -1.0e+30

real :: acg_min_mt = +1.0e+30
real :: acg_max_mt = -1.0e+30
real :: acg_min_lt = +1.0e+30
real :: acg_max_lt = -1.0e+30
real :: acg_min_ls = +1.0e+30
real :: acg_max_ls = -1.0e+30
real :: acg_min_tp = +1.0e+30
real :: acg_max_tp = -1.0e+30

real :: reg_min_mt = +1.0e+30
real :: reg_max_mt = -1.0e+30
real :: reg_min_lt = +1.0e+30
real :: reg_max_lt = -1.0e+30
real :: reg_min_ls = +1.0e+30
real :: reg_max_ls = -1.0e+30
real :: reg_min_tp = +1.0e+30
real :: reg_max_tp = -1.0e+30

real :: reg_trend_min_mt = +1.0e+30
real :: reg_trend_max_mt = -1.0e+30
real :: reg_trend_min_lt = +1.0e+30
real :: reg_trend_max_lt = -1.0e+30
real :: reg_trend_min_ls = +1.0e+30
real :: reg_trend_max_ls = -1.0e+30
real :: reg_trend_min_tp = +1.0e+30
real :: reg_trend_max_tp = -1.0e+30

!
!***********************************************************************
!
!  Date variables and arrays ...

integer, parameter :: nyr_base = 1978
integer, parameter :: nmo_base = 1
integer, parameter :: ndy_base = 1
integer, parameter :: nhr_base = 0

integer :: nyr
integer :: nmo
integer :: ndy
integer :: nhr
integer :: nmin
integer :: nsec

character ( len = 20 ) :: cdate_string
character ( len = 20 ) :: fdate_beg_string
character ( len = 20 ) :: fdate_end_string

integer :: nyr_beg
integer :: nmo_beg
integer :: ndy_beg
integer :: jmin_beg

integer :: nyr_end
integer :: nmo_end
integer :: ndy_end
integer :: jmin_end

integer :: nta_global
integer :: ntb_global

integer :: ntmax_days = 0
integer :: ntmax_mons = 0
integer, allocatable, dimension(:) :: jmin_store_days
integer, allocatable, dimension(:) :: jmin_store_mons
integer, allocatable, dimension(:) :: dflag_store_days
integer, allocatable, dimension(:) :: dflag_store_mons
integer, allocatable, dimension(:) :: dflag_store_mons_reg
integer ( kind = half_precision ), target, allocatable, dimension(:) :: jday_store_mons
integer ( kind = half_precision ), target, allocatable, dimension(:,:) :: time_bnds

integer :: ndaily_esec_done = 0
integer :: ndaily_cdate_done = 0
integer :: nmonthly_esec_done = 0
integer :: nmonthly_cdate_done = 0

integer ( kind = double_precision ) :: esec
integer :: jmin_beg_date
integer :: jmin_end_date

integer :: ntmax = 0
integer :: ndmax = 0
integer :: ngmax = 0
logical :: define_mode = .false.
logical :: check_netcdf_data = .false.
logical :: trim_tls_2005 = .false.
character ( len = 256 ) :: region_def_style = cmiss
logical :: region_only = .false.
character ( len = 256 ) :: missing_data_style = cmiss
logical :: use_alternate = .false.

character ( len = 19 ) :: cdate_beg
character ( len = 19 ) :: cdate_end

character ( len = 256 ) :: title
character ( len = 256 ) :: history
character ( len = 256 ) :: contact

integer, parameter :: netcdf_max_dim = 10
integer :: netcdf_ndim_use = 0
integer, dimension(netcdf_max_dim) :: netcdf_dimid_hold = 0
integer, dimension(netcdf_max_dim) :: netcdf_size_hold = 0
character ( len = 256 ), dimension(netcdf_max_dim) :: netcdf_vname_hold = ''
character ( len = 256 ), dimension(netcdf_max_dim) :: netcdf_desc_hold = ''

integer, parameter :: new_max = 100
character ( len = 256 ), dimension(new_max) :: new_dim_desc
character ( len = 256 ), dimension(new_max) :: new_dim_vname
integer, dimension(new_max) :: new_dim_ndsize

character ( len = 256 ), dimension(new_max) :: dim_desc
character ( len = 256 ), dimension(new_max) :: dim_vname
integer, dimension(new_max) :: dim_ndsize

character ( len = 256 ), dimension(new_max) :: comment_name
character ( len = 256 ), dimension(new_max) :: comment_value

character ( len = 256 ), dimension(new_max) :: var_desc = cmiss
integer, dimension(new_max) :: netcdf_varid_hold = 0
integer :: netcdf_nvar_use = 0

end module
!
!***********************************************************************
!
module module_mdstore1_def
implicit none

type :: dstore_info_1

  integer, allocatable, dimension(:) :: nyr
  integer, allocatable, dimension(:) :: nmo
  real ( kind = 8 ), allocatable, dimension(:) :: esec
  character ( len = 19 ), allocatable, dimension(:) :: cdate
  real, allocatable, dimension(:) :: gb
  real, allocatable, dimension(:) :: nh
  real, allocatable, dimension(:) :: sh
  real, allocatable, dimension(:) :: tp
  integer, allocatable, dimension(:) :: ndays

  real, allocatable, dimension(:) :: gb_rm
  real, allocatable, dimension(:) :: nh_rm
  real, allocatable, dimension(:) :: sh_rm
  real, allocatable, dimension(:) :: tp_rm
  integer, allocatable, dimension(:) :: ndays_rm

  real, dimension(1) :: gb_decadal_trend
  real, dimension(1) :: nh_decadal_trend
  real, dimension(1) :: sh_decadal_trend
  real, dimension(1) :: tp_decadal_trend

end type dstore_info_1

end module module_mdstore1_def
!
!***********************************************************************
!
module module_ddstore1_def
implicit none

type :: dstore_info_2

  integer, allocatable, dimension(:) :: nyr
  integer, allocatable, dimension(:) :: jday
  integer, allocatable, dimension(:) :: jdayc
  real ( kind = 8 ), allocatable, dimension(:) :: esec
  character ( len = 19 ), allocatable, dimension(:) :: cdate

  integer, allocatable, dimension(:) :: gb
  integer, allocatable, dimension(:) :: nh
  integer, allocatable, dimension(:) :: sh
  integer, allocatable, dimension(:) :: tp

  integer, allocatable, dimension(:,:) :: za

end type dstore_info_2

end module module_ddstore1_def
!
!***********************************************************************
!
